@@ -16,22 +16,22 @@
  *   along with this program; if not, write to the Free Software
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- *   $Id: example_module.c 33 2005-10-02 20:50:00Z knight $
+ *   $Id: example_module.c 1268 2012-01-18 08:20:31Z michael $
  */
 
 /* List of ircd includes from ../include/ 
  * These ones are necessary to build THIS module...
  */
 
-#include "stdinc.h" /* includes setup.h, for STATIC_MODULES */
+#include "stdinc.h" /* includes setup.h */
 
 #include "client.h" /* Required for IsClient, etc. */
 
 #include "send.h" /* sendto_one, most useful function of all time */
 
-#include "modules.h" /* includes msg.h; use for the msgtab */
+#include "parse.h"
 
-#include "handlers.h" /* m_ignore */
+#include "modules.h" /* includes msg.h; use for the msgtab */
 
 /* OTHER USEFUL INCLUDES:
  * 
@@ -48,7 +48,8 @@
  * natively for your OS the prototypes in irc_string.h may be required for
  * others. */
 
-/* Declare the void's initially up here, as modules don't have an
+/*
+ * Declare the void's initially up here, as modules don't have an
  * include file, we will normally have client_p, source_p, parc
  * and parv[] where:
  *
@@ -57,81 +58,7 @@
  * parc     == the number of parameters
  * parv     == an array of the parameters
  */
-static void mr_test(struct Client *, struct Client *, int, char *[]);
-static void m_test(struct Client *, struct Client *, int, char *[]);
-static void ms_test(struct Client *, struct Client *, int, char *[]);
-static void mo_test(struct Client *, struct Client *, int, char *[]);
 
-/*
- * Show the commands this module can handle in a msgtab
- * and give the msgtab a name, here its test_msgtab
- */
-struct Message test_msgtab = {
-
- /* Fields are in order:
-  *-> "COMMAND", 0, 0, parc_count, maxparc, MFLG_SLOW, 0,
-  *
-  * where:
-  * COMMAND == the /command you want
-  * parc_count == the number of parameters needed
-  *               (the clients name is one param, parv[0])
-  * maxparc == the maximum parameters we allow
-  * the 0's and MFLG_SLOW should not be changed..
-  */
-
- /* This would add the command "TEST" which requires no additional
-  * parameters
-  */
-  "TEST", 0, 0, 1, 0, MFLG_SLOW, 0,
-
- /* Fields are in order:
-  *-> {unregged, regged, remote, encap, oper, dummy}
-  *
-  * where:
-  * unregged == function to call for unregistered clients
-  * regged == function to call for normal users
-  * remote == function to call for servers/remote users
-  * encap == function to call for encap'd server/remote commands
-  * oper == function to call for operators
-  * dummy == function called when client is quarantined
-  *
-  * There are also some pre-coded functions for use:
-  * m_unregistered: prevent the client using this if unregistered
-  * m_not_oper:     tell the client it requires being an operator
-  * m_ignore:       ignore the command when it comes from certain types
-  * m_error:        give an error when the command comes from certain types
-  */
-  { mr_test, m_test, ms_test, m_ignore, mo_test, m_ignore }
-
- /* It is normal for unregistered functions to be prefixed with mr_
-  *   "      "       normal users to be prefixed with m_
-  *   "      "       remote clients to be prefixed with ms_
-  *   "      "       operators to be prefixed with mo_
-  */
-};
-/* That's the msgtab finished */
-
-#ifndef STATIC_MODULES
-/* Here we tell it what to do when the module is loaded */
-void
-_modinit(void)
-{
-  /* This will add the commands in test_msgtab (which is above) */
-  mod_add_cmd(&test_msgtab);
-}
-
-/* here we tell it what to do when the module is unloaded */
-void
-_moddeinit(void)
-{
-  /* This will remove the commands in test_msgtab (which is above) */
-  mod_del_cmd(&test_msgtab);
-}
-
-/* When we last modified the file (shown in /modlist), this is usually:
- */
-const char *_version = "$Revision: 33 $";
-#endif
 
 /*
  * mr_test
@@ -139,7 +66,8 @@ const char *_version = "$Revision: 33 $";
  *      parv[1] = parameter
  */
 
-/* Here we have the functions themselves that we declared above,
+/*
+ * Here we have the functions themselves that we declared above,
  * and the fairly normal C coding
  */
 static void
@@ -217,4 +145,80 @@ mo_test(struct Client *client_p, struct Client *source_p,
                me.name, source_p->name, parv[1]);
 }
 
+/*
+ * Show the commands this module can handle in a msgtab
+ * and give the msgtab a name, here its test_msgtab
+ */
+static struct Message test_msgtab = {
+
+ /* Fields are in order:
+  *-> "COMMAND", 0, 0, parc_count, maxparc, MFLG_SLOW, 0,
+  *
+  * where:
+  * COMMAND == the /command you want
+  * parc_count == the number of parameters needed
+  *               (the clients name is one param, parv[0])
+  * maxparc == the maximum parameters we allow
+  * the 0's and MFLG_SLOW should not be changed..
+  */
+
+ /*
+  * This would add the command "TEST" which requires no additional
+  * parameters
+  */
+  "TEST", 0, 0, 1, MAXPARA, MFLG_SLOW, 0,
+
+ /* Fields are in order:
+  *-> {unregged, regged, remote, encap, oper, dummy}
+  *
+  * where:
+  * unregged == function to call for unregistered clients
+  * regged == function to call for normal users
+  * remote == function to call for servers/remote users
+  * encap == function to call for encap'd server/remote commands
+  * oper == function to call for operators
+  * dummy == function called when client is quarantined
+  *
+  * There are also some pre-coded functions for use:
+  * m_unregistered: prevent the client using this if unregistered
+  * m_not_oper:     tell the client it requires being an operator
+  * m_ignore:       ignore the command when it comes from certain types
+  * rfc1459_command_send_error: give an error when the command comes from certain types
+  */
+  { mr_test, m_test, ms_test, m_ignore, mo_test, m_ignore }
+
+ /* It is normal for unregistered functions to be prefixed with mr_
+  *   "      "       normal users to be prefixed with m_
+  *   "      "       remote clients to be prefixed with ms_
+  *   "      "       operators to be prefixed with mo_
+  */
+};
+/* That's the msgtab finished */
+
+/* Here we tell it what to do when the module is loaded */
+static void
+module_init(void)
+{
+  /* This will add the commands in test_msgtab (which is above) */
+  mod_add_cmd(&test_msgtab);
+}
+
+/* here we tell it what to do when the module is unloaded */
+static void
+module_exit(void)
+{
+  /* This will remove the commands in test_msgtab (which is above) */
+  mod_del_cmd(&test_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1268 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = 0
+};
+
 /* END OF EXAMPLE MODULE */
