@@ -19,68 +19,26 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_ctrace.c 309 2005-12-09 11:25:44Z adx $
+ *  $Id: m_ctrace.c 1776 2013-01-24 17:26:55Z michael $
  */
 
 #include "stdinc.h"
-#include "handlers.h"
-#include "tools.h"
-#include "hook.h"
+#include "list.h"
 #include "client.h"
 #include "hash.h"
-#include "common.h"
 #include "irc_string.h"
 #include "ircd.h"
 #include "numeric.h"
-#include "fdlist.h"
 #include "s_bsd.h"
-#include "s_conf.h"
+#include "conf.h"
 #include "s_serv.h"
 #include "send.h"
-#include "msg.h"
 #include "parse.h"
 #include "modules.h"
-#include "irc_getnameinfo.h"
 
-static void do_ctrace(struct Client *, int, char **);
-static void mo_ctrace(struct Client *, struct Client *, int, char *[]);
+static void do_ctrace(struct Client *, int, char *[]);
+static void report_this_status(struct Client *, struct Client *);
 
-struct Message ctrace_msgtab = {
-  "CTRACE", 0, 0, 2, 0, MFLG_SLOW, 0,
-  {m_unregistered, m_not_oper, m_ignore, m_ignore, mo_ctrace, m_ignore}
-};
-
-#ifndef STATIC_MODULES
-const char *_version = "$Revision: 309 $";
-static struct Callback *ctrace_cb;
-
-static void *
-va_ctrace(va_list args)
-{
-  struct Client *source_p = va_arg(args, struct Client *);
-  int parc = va_arg(args, int);
-  char **parv = va_arg(args, char **);
-
-  do_ctrace(source_p, parc, parv);
-  return NULL;
-}
-
-void
-_modinit(void)
-{
-  ctrace_cb = register_callback("doing_ctrace", va_ctrace);
-  mod_add_cmd(&ctrace_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&ctrace_msgtab);
-  uninstall_hook(ctrace_cb, va_ctrace);
-}
-#endif
-
-static int report_this_status(struct Client *, struct Client *);
 
 /*
 ** mo_ctrace
@@ -94,35 +52,32 @@ mo_ctrace(struct Client *client_p, struct Client *source_p,
   if (EmptyString(parv[1]))
   {
     sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS),
-               me.name, parv[0], "CTRACE");
+               me.name, source_p->name, "CTRACE");
     return;
   }
 
-#ifdef STATIC_MODULES
   do_ctrace(source_p, parc, parv);
-#else
-  execute_callback(ctrace_cb, source_p, parc, parv);
-#endif
 }
 
 /*
  * do_ctrace
  */
 static void
-do_ctrace(struct Client *source_p, int parc, char **parv)
+do_ctrace(struct Client *source_p, int parc, char *parv[])
 {
-  struct Client *target_p = NULL;
-  char *class_looking_for;
-  const char *class_name;
+  char *class_looking_for = parv[1];
+  const char *class_name = NULL;
   dlink_node *ptr;
 
-  class_looking_for = parv[1];
+  sendto_realops_flags(UMODE_SPY, L_ALL,
+                       "CTRACE requested by %s (%s@%s) [%s]",
+                       source_p->name, source_p->username,
+                       source_p->host, source_p->servptr->name);
 
   /* report all direct connections */
-
   DLINK_FOREACH(ptr, local_client_list.head)
   {
-    target_p = ptr->data;
+    struct Client *target_p = ptr->data;
 
     class_name = get_client_class(target_p);
     if ((class_name != NULL) && match(class_looking_for, class_name))
@@ -130,7 +85,7 @@ do_ctrace(struct Client *source_p, int parc, char **parv)
   }
 
   sendto_one(source_p, form_str(RPL_ENDOFTRACE), me.name,
-             parv[0], class_looking_for);
+             source_p->name, class_looking_for);
 }
 
 /*
@@ -141,18 +96,12 @@ do_ctrace(struct Client *source_p, int parc, char **parv)
  * output	- counter of number of hits
  * side effects - NONE
  */
-static int
+static void
 report_this_status(struct Client *source_p, struct Client *target_p)
 {
   const char *name = NULL;
   const char *class_name = NULL;
-  char ip[HOSTIPLEN];
-  int cnt = 0;
 
-  /* Should this be sockhost? - stu */
-  irc_getnameinfo((struct sockaddr*)&target_p->localClient->ip, 
-        target_p->localClient->ip.ss_len, ip, HOSTIPLEN, NULL, 0, 
-        NI_NUMERICHOST);
   name = get_client_name(target_p, HIDE_IP);
   class_name = get_client_class(target_p);
 
@@ -160,67 +109,89 @@ report_this_status(struct Client *source_p, struct Client *target_p)
   {
     case STAT_CLIENT:
 
-      if ((IsOper(source_p) &&
-          (MyClient(source_p) || !IsInvisible(target_p)))
-          || IsOper(target_p))
+      if ((HasUMode(source_p, UMODE_OPER) &&
+          (MyClient(source_p) || !HasUMode(target_p, UMODE_INVISIBLE)))
+          || HasUMode(target_p, UMODE_OPER))
       {
-        if (IsAdmin(target_p) && !ConfigFileEntry.hide_spoof_ips)
+        if (HasUMode(target_p, UMODE_ADMIN) && !ConfigFileEntry.hide_spoof_ips)
           sendto_one(source_p, form_str(RPL_TRACEOPERATOR),
                        me.name, source_p->name, class_name, name,
-                       IsAdmin(source_p) ? ip : "255.255.255.255",
-                       CurrentTime - target_p->lasttime,
-                       CurrentTime - target_p->localClient->last);
-          else if (IsOper(target_p))
+                       HasUMode(source_p, UMODE_ADMIN) ? target_p->sockhost : "255.255.255.255",
+                       CurrentTime - target_p->localClient->lasttime,
+                       idle_time_get(source_p, target_p));
+          else if (HasUMode(target_p, UMODE_OPER))
           {
             if (ConfigFileEntry.hide_spoof_ips)
 	      sendto_one(source_p, form_str(RPL_TRACEOPERATOR),
 		         me.name, source_p->name, class_name, name, 
-		         IsIPSpoof(target_p) ? "255.255.255.255" : ip,
-		         CurrentTime - target_p->lasttime,
-		         CurrentTime - target_p->localClient->last);
+		         IsIPSpoof(target_p) ? "255.255.255.255" : target_p->sockhost,
+		         CurrentTime - target_p->localClient->lasttime,
+		         idle_time_get(source_p, target_p));
 	    else   
               sendto_one(source_p, form_str(RPL_TRACEOPERATOR),
                          me.name, source_p->name, class_name, name,
-                         (IsIPSpoof(target_p) ? "255.255.255.255" : ip),
-                         CurrentTime - target_p->lasttime,
-                         CurrentTime - target_p->localClient->last);
+                         (IsIPSpoof(target_p) ? "255.255.255.255" : target_p->sockhost),
+                         CurrentTime - target_p->localClient->lasttime,
+                         idle_time_get(source_p, target_p));
           }
 	  else
           {
             if (ConfigFileEntry.hide_spoof_ips)
 	      sendto_one(source_p, form_str(RPL_TRACEUSER),
 		         me.name, source_p->name, class_name, name,
-                         IsIPSpoof(target_p) ? "255.255.255.255" : ip,
-		         CurrentTime - target_p->lasttime,
-		         CurrentTime - target_p->localClient->last);
+                         IsIPSpoof(target_p) ? "255.255.255.255" : target_p->sockhost,
+		         CurrentTime - target_p->localClient->lasttime,
+		         idle_time_get(source_p, target_p));
             else
               sendto_one(source_p, form_str(RPL_TRACEUSER),
                          me.name, source_p->name, class_name, name,
-                         (IsIPSpoof(target_p) ? "255.255.255.255" : ip),
-                         CurrentTime - target_p->lasttime,
-                         CurrentTime - target_p->localClient->last);
+                         (IsIPSpoof(target_p) ? "255.255.255.255" : target_p->sockhost),
+                         CurrentTime - target_p->localClient->lasttime,
+                         idle_time_get(source_p, target_p));
           }
-	  cnt++;
         }
       break;
     case STAT_SERVER:
-      if(!IsAdmin(source_p))
+      if (!HasUMode(source_p, UMODE_ADMIN))
         name = get_client_name(target_p, MASK_IP);
 
       sendto_one(source_p, form_str(RPL_TRACESERVER),
 		 me.name, source_p->name, class_name, 0,
 		 0, name, *(target_p->serv->by) ?
 		 target_p->serv->by : "*", "*",
-		 me.name, CurrentTime - target_p->lasttime);
-      cnt++;
+		 me.name, CurrentTime - target_p->localClient->lasttime);
       break;
       
     default: /* ...we actually shouldn't come here... --msa */
       sendto_one(source_p, form_str(RPL_TRACENEWTYPE), me.name,
                  source_p->name, name);
-      cnt++;
       break;
-    }
-
-  return cnt;
+  }
 }
+
+static struct Message ctrace_msgtab = {
+  "CTRACE", 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
+  {m_unregistered, m_not_oper, m_ignore, m_ignore, mo_ctrace, m_ignore}
+};
+
+static void
+module_init(void)
+{
+  mod_add_cmd(&ctrace_msgtab);
+}
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&ctrace_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1776 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = 0
+};
