@@ -25,58 +25,25 @@
  *  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  *  POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: m_force.c 690 2006-06-17 11:34:25Z michael $
+ * $Id: m_force.c 1474 2012-07-22 14:44:07Z michael $
  */
 
 #include "stdinc.h"
-#include "handlers.h"
 #include "client.h"
-#include "common.h"     /* FALSE bleah */
 #include "ircd.h"
 #include "irc_string.h"
 #include "numeric.h"
 #include "fdlist.h"
 #include "hash.h"
 #include "s_bsd.h"
-#include "s_conf.h"
+#include "conf.h"
 #include "s_serv.h"
 #include "send.h"
-#include "msg.h"
 #include "parse.h"
 #include "modules.h"
 #include "channel.h"
 #include "channel_mode.h"
 
-static void mo_forcejoin(struct Client *, struct Client *, int parc, char *[]);
-static void mo_forcepart(struct Client *, struct Client *, int parc, char *[]);
-
-struct Message forcejoin_msgtab = {
-  "FORCEJOIN", 0, 0, 3, 0, MFLG_SLOW, 0,
-  { m_ignore, m_not_oper, mo_forcejoin, mo_forcejoin, mo_forcejoin, m_ignore }
-};
-
-struct Message forcepart_msgtab = {
-  "FORCEPART", 0, 0, 3, 0, MFLG_SLOW, 0,
-  { m_ignore, m_not_oper, mo_forcepart, mo_forcepart, mo_forcepart, m_ignore }
-};
-
-#ifndef STATIC_MODULES
-void
-_modinit(void)
-{
-  mod_add_cmd(&forcejoin_msgtab);
-  mod_add_cmd(&forcepart_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&forcejoin_msgtab);
-  mod_del_cmd(&forcepart_msgtab);
-}
-
-const char *_version = "$Revision: 690 $";
-#endif
 
 /* m_forcejoin()
  *  parv[0] = sender prefix
@@ -95,14 +62,14 @@ mo_forcejoin(struct Client *client_p, struct Client *source_p,
   char *newch = NULL;
   dlink_node *ptr = NULL;
 
-  if (!IsAdmin(source_p))
+  if (!HasUMode(source_p, UMODE_ADMIN))
   {
     sendto_one(source_p, form_str(ERR_NOPRIVILEGES),
                me.name, source_p->name);
     return;
   }
 
-  if ((target_p = find_client(parv[1])) == NULL || !IsClient(target_p))
+  if ((target_p = hash_find_client(parv[1])) == NULL || !IsClient(target_p))
   {
     sendto_one(source_p, form_str(ERR_NOSUCHNICK),
                me.name, source_p->name, parv[1]);
@@ -163,14 +130,14 @@ mo_forcejoin(struct Client *client_p, struct Client *source_p,
       return;
     }
 
-    add_user_to_channel(chptr, target_p, type, NO);
+    add_user_to_channel(chptr, target_p, type, 0);
 
-    sendto_channel_local(ALL_MEMBERS, NO, chptr, ":%s!%s@%s JOIN :%s",
+    sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s!%s@%s JOIN :%s",
                          target_p->name, target_p->username,
                          target_p->host, chptr->chname);
 
     if (sjmode)
-      sendto_channel_local(ALL_MEMBERS, NO, chptr, ":%s MODE %s +%c %s",
+      sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s MODE %s +%c %s",
                            me.name, chptr->chname, mode, target_p->name);
 
     if (chptr->chname[0] == '#')
@@ -192,18 +159,18 @@ mo_forcejoin(struct Client *client_p, struct Client *source_p,
       }
       else
       {
-        sendto_server(target_p, target_p, chptr, CAP_TS6, NOCAPS, LL_ICLIENT,
+        sendto_server(target_p, CAP_TS6, NOCAPS,
                       ":%s SJOIN %lu %s + :%s",
                       me.id, (unsigned long)chptr->channelts,
                       chptr->chname, target_p->id);
-        sendto_server(target_p, target_p, chptr, NOCAPS, CAP_TS6, LL_ICLIENT,
+        sendto_server(target_p, NOCAPS, CAP_TS6,
                       ":%s SJOIN %lu %s + :%s",
                       me.name, (unsigned long)chptr->channelts,
                       chptr->chname, target_p->name);
       }
     }
 
-    if (chptr->topic != NULL)
+    if (chptr->topic[0])
     {
       sendto_one(target_p, form_str(RPL_TOPIC),
                  me.name, target_p->name,
@@ -227,43 +194,25 @@ mo_forcejoin(struct Client *client_p, struct Client *source_p,
       return;
     }
 
-    /*
-     * it would be interesting here to allow an oper
-     * to force target_p into a channel that doesn't exist
-     * even more so, into a local channel when we disable
-     * local channels... but...
-     * I don't want to break anything - scuzzy
-     */
-    if (ConfigChannel.disable_local_channels && (*newch == '&'))
-    {
-      sendto_one(source_p, form_str(ERR_NOSUCHCHANNEL),
-                 me.name, source_p->name, newch);
-      return;
-    }
-
     chptr = make_channel(newch);
-    add_user_to_channel(chptr, target_p, CHFL_CHANOP, NO);
+    add_user_to_channel(chptr, target_p, CHFL_CHANOP, 0);
 
-    /* send out a join, make target_p join chptr */
-    if (chptr->chname[0] == '#')
-    {
-      sendto_server(target_p, target_p, chptr, CAP_TS6, NOCAPS, LL_ICLIENT,
-                    ":%s SJOIN %lu %s +nt :@%s",
-                    me.id, (unsigned long)chptr->channelts,
-                    chptr->chname, ID(target_p));
-      sendto_server(target_p, target_p, chptr, NOCAPS, CAP_TS6, LL_ICLIENT,
-                    ":%s SJOIN %lu %s +nt :@%s",
-                    me.name, (unsigned long)chptr->channelts,
-                    chptr->chname, target_p->name);
-    }
+    sendto_server(target_p, CAP_TS6, NOCAPS,
+                  ":%s SJOIN %lu %s +nt :@%s",
+                  me.id, (unsigned long)chptr->channelts,
+                  chptr->chname, ID(target_p));
+    sendto_server(target_p, NOCAPS, CAP_TS6,
+                  ":%s SJOIN %lu %s +nt :@%s",
+                  me.name, (unsigned long)chptr->channelts,
+                  chptr->chname, target_p->name);
 
-    sendto_channel_local(ALL_MEMBERS, NO, chptr, ":%s!%s@%s JOIN :%s",
+    sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s!%s@%s JOIN :%s",
                          target_p->name, target_p->username,
                          target_p->host, chptr->chname);
 
     chptr->mode.mode |= MODE_TOPICLIMIT | MODE_NOPRIVMSGS;
 
-    sendto_channel_local(ALL_MEMBERS, NO, chptr, ":%s MODE %s +nt",
+    sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s MODE %s +nt",
                          me.name, chptr->chname);
 
     target_p->localClient->last_join_time = CurrentTime;
@@ -286,7 +235,7 @@ mo_forcepart(struct Client *client_p, struct Client *source_p,
   struct Channel *chptr = NULL;
   struct Membership *member = NULL;
 
-  if (!IsAdmin(source_p))
+  if (!HasUMode(source_p, UMODE_ADMIN))
   {
     sendto_one(source_p, form_str(ERR_NOPRIVILEGES),
                me.name, source_p->name);
@@ -294,7 +243,7 @@ mo_forcepart(struct Client *client_p, struct Client *source_p,
   }
 
   /* if target_p == NULL then let the oper know */
-  if ((target_p = find_client(parv[1])) == NULL || !IsClient(target_p))
+  if ((target_p = hash_find_client(parv[1])) == NULL || !IsClient(target_p))
   {
     sendto_one(source_p, form_str(ERR_NOSUCHNICK),
                me.name, source_p->name, parv[1]);
@@ -331,19 +280,50 @@ mo_forcepart(struct Client *client_p, struct Client *source_p,
     return;
   }
 
-  if (chptr->chname[0] == '#')
-  {
-    sendto_server(target_p, target_p, chptr, CAP_TS6, NOCAPS, LL_ICLIENT,
-                  ":%s PART %s :%s", ID(target_p),
-                  chptr->chname, target_p->name);
-    sendto_server(target_p, target_p, chptr, NOCAPS, CAP_TS6, LL_ICLIENT,
-                  ":%s PART %s :%s", target_p->name,
-                  chptr->chname, target_p->name);
-  }
+  sendto_server(target_p, CAP_TS6, NOCAPS,
+                ":%s PART %s :%s", ID(target_p),
+                chptr->chname, target_p->name);
+  sendto_server(target_p, NOCAPS, CAP_TS6,
+                ":%s PART %s :%s", target_p->name,
+                chptr->chname, target_p->name);
 
-  sendto_channel_local(ALL_MEMBERS, NO, chptr, ":%s!%s@%s PART %s :%s",
+  sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s!%s@%s PART %s :%s",
                        target_p->name, target_p->username,
                        target_p->host, chptr->chname,
                        target_p->name);
   remove_user_from_channel(member);
 }
+
+static struct Message forcejoin_msgtab = {
+  "FORCEJOIN", 0, 0, 3, MAXPARA, MFLG_SLOW, 0,
+  { m_ignore, m_not_oper, mo_forcejoin, mo_forcejoin, mo_forcejoin, m_ignore }
+};
+
+static struct Message forcepart_msgtab = {
+  "FORCEPART", 0, 0, 3, MAXPARA, MFLG_SLOW, 0,
+  { m_ignore, m_not_oper, mo_forcepart, mo_forcepart, mo_forcepart, m_ignore }
+};
+
+static void
+module_init(void)
+{
+  mod_add_cmd(&forcejoin_msgtab);
+  mod_add_cmd(&forcepart_msgtab);
+}
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&forcejoin_msgtab);
+  mod_del_cmd(&forcepart_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1474 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = 0
+};
