@@ -19,15 +19,12 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_ltrace.c 33 2005-10-02 20:50:00Z knight $
+ *  $Id: m_ltrace.c 1776 2013-01-24 17:26:55Z michael $
  */
 
 #include "stdinc.h"
-#include "handlers.h"
-#include "tools.h"
-#include "hook.h"
+#include "list.h"
 #include "client.h"
-#include "common.h"
 #include "hash.h"
 #include "irc_string.h"
 #include "ircd.h"
@@ -35,55 +32,28 @@
 #include "fdlist.h"
 #include "s_bsd.h"
 #include "s_serv.h"
-#include "s_conf.h"
+#include "conf.h"
 #include "send.h"
-#include "msg.h"
 #include "parse.h"
 #include "modules.h"
-#include "irc_getnameinfo.h"
 
-static void do_ltrace(struct Client *, int, char **);
-static void m_ltrace(struct Client *, struct Client *, int, char **);
-static void mo_ltrace(struct Client *, struct Client *, int, char **);
+static void do_ltrace(struct Client *, int, char *[]);
+static void report_this_status(struct Client *, struct Client *, int);
 
-struct Message ltrace_msgtab = {
-  "LTRACE", 0, 0, 0, 0, MFLG_SLOW, 0,
-  {m_unregistered, m_ltrace, mo_ltrace, m_ignore, mo_ltrace, m_ignore}
-};
 
-#ifndef STATIC_MODULES
-const char *_version = "$Revision: 33 $";
-static struct Callback *ltrace_cb;
-
-static void *
-va_ltrace(va_list args)
+static void
+trace_get_dependent(int *const server,
+                    int *const client, const struct Client *target_p)
 {
-  struct Client *source_p = va_arg(args, struct Client *);
-  int parc = va_arg(args, int);
-  char **parv = va_arg(args, char **);
+  const dlink_node *ptr = NULL;
 
-  do_ltrace(source_p, parc, parv);
-  return NULL;
+  (*server)++;
+  (*client) += dlink_list_length(&target_p->serv->client_list);
+
+  DLINK_FOREACH(ptr, target_p->serv->server_list.head)
+    trace_get_dependent(server, client, ptr->data);
 }
 
-void
-_modinit(void)
-{
-  ltrace_cb = register_callback("doing_ltrace", va_ltrace);
-  mod_add_cmd(&ltrace_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&ltrace_msgtab);
-  uninstall_hook(ltrace_cb, va_ltrace);
-}
-#endif
-
-static int report_this_status(struct Client *source_p, struct Client *target_p,int dow,
-                              int link_u_p, int link_u_s);
-
 /*
  * m_ltrace()
  *
@@ -94,21 +64,21 @@ static void
 m_ltrace(struct Client *client_p, struct Client *source_p,
          int parc, char *parv[])
 {
-  char *tname;
+  const char *tname = NULL;
 
   if (parc > 1)
     tname = parv[1];
   else
     tname = me.name;
   sendto_one(source_p, form_str(RPL_ENDOFTRACE),
-             me.name, parv[0], tname);
+             me.name, source_p->name, tname);
 }
 
 /*
  * do_ltrace
  */
 static void
-do_ltrace(struct Client *source_p, int parc, char **parv)
+do_ltrace(struct Client *source_p, int parc, char *parv[])
 {
   struct Client *target_p = NULL;
   int   doall;
@@ -123,12 +93,12 @@ do_ltrace(struct Client *source_p, int parc, char **parv)
     {
       struct Client *ac2ptr = NULL;
 
-      if ((ac2ptr = find_client(tname)) == NULL)
+      if ((ac2ptr = hash_find_client(tname)) == NULL)
         DLINK_FOREACH(ptr, global_client_list.head)
         {
           ac2ptr = ptr->data;
 
-          if (match(tname, ac2ptr->name) || match(ac2ptr->name, tname))
+          if (match(tname, ac2ptr->name))
             break;
           else
             ac2ptr = NULL;
@@ -148,7 +118,12 @@ do_ltrace(struct Client *source_p, int parc, char **parv)
       return;
   }
 
-  doall = (parv[1] && (parc > 1)) ? match(tname, me.name): TRUE;
+  sendto_realops_flags(UMODE_SPY, L_ALL,
+                       "LTRACE requested by %s (%s@%s) [%s]",
+                       source_p->name, source_p->username,
+                       source_p->host, source_p->servptr->name);
+
+  doall = (parv[1] && (parc > 1)) ? match(tname, me.name) : 1;
   wilds = !parv[1] || strchr(tname, '*') || strchr(tname, '?');
   dow = wilds || doall;
   
@@ -157,33 +132,28 @@ do_ltrace(struct Client *source_p, int parc, char **parv)
   {
     const char* name;
     const char* class_name;
-    char ipaddr[HOSTIPLEN];
 
-    target_p = find_client(tname);
+    target_p = hash_find_client(tname);
 
     if (target_p && IsClient(target_p)) 
     {
       name = get_client_name(target_p, HIDE_IP);
-      /* Should this be sockhost? - stu */
-      irc_getnameinfo((struct sockaddr*)&target_p->localClient->ip,
-                  target_p->localClient->ip.ss_len, ipaddr,
-                  HOSTIPLEN, NULL, 0, NI_NUMERICHOST);
       class_name = get_client_class(target_p);
 
-      if (IsOper(target_p))
+      if (HasUMode(target_p, UMODE_OPER))
       {
         if (ConfigFileEntry.hide_spoof_ips)
           sendto_one(source_p, form_str(RPL_TRACEOPERATOR),
                      me.name, source_p->name, class_name, name, 
-                     (IsIPSpoof(target_p) ? "255.255.255.255" : ipaddr),
-                     CurrentTime - target_p->lasttime,
-                     CurrentTime - target_p->localClient->last);
+                     (IsIPSpoof(target_p) ? "255.255.255.255" : target_p->sockhost),
+                     CurrentTime - target_p->localClient->lasttime,
+                     idle_time_get(source_p, target_p));
         else
           sendto_one(source_p, form_str(RPL_TRACEOPERATOR),
                      me.name, source_p->name, class_name, name,
-                     (IsIPSpoof(target_p) ? "255.255.255.255" : ipaddr),
-                     CurrentTime - target_p->lasttime,
-                     CurrentTime - target_p->localClient->last);
+                     (IsIPSpoof(target_p) ? "255.255.255.255" : target_p->sockhost),
+                     CurrentTime - target_p->localClient->lasttime,
+                     idle_time_get(source_p, target_p));
       }
     }
 
@@ -197,7 +167,7 @@ do_ltrace(struct Client *source_p, int parc, char **parv)
   {
     target_p = ptr->data;
 
-    if (!IsOper(target_p))
+    if (!HasUMode(target_p, UMODE_OPER))
       continue;
 
     if (!doall && wilds && !match(tname, target_p->name))
@@ -206,7 +176,7 @@ do_ltrace(struct Client *source_p, int parc, char **parv)
     if (!dow && irccmp(tname, target_p->name))
       continue;
 
-    report_this_status(source_p, target_p, dow, 0, 0);
+    report_this_status(source_p, target_p, dow);
   }
 
   /* report all servers */
@@ -219,8 +189,7 @@ do_ltrace(struct Client *source_p, int parc, char **parv)
     if (!dow && irccmp(tname, target_p->name))
       continue;
 
-    report_this_status(source_p, target_p, dow, target_p->serv->dep_users,
-                       target_p->serv->dep_servers);
+    report_this_status(source_p, target_p, dow);
   }
 
   sendto_one(source_p, form_str(RPL_ENDOFTRACE), me.name, parv[0], tname);
@@ -235,7 +204,7 @@ static void
 mo_ltrace(struct Client *client_p, struct Client *source_p,
           int parc, char *parv[])
 {
-  if (!IsOper(source_p))
+  if (!HasUMode(source_p, UMODE_OPER))
   {
     sendto_one(source_p, form_str(RPL_ENDOFTRACE), me.name, parv[0],
                parc > 1 ? parv[1] : me.name);
@@ -246,11 +215,7 @@ mo_ltrace(struct Client *client_p, struct Client *source_p,
     if (hunt_server(client_p, source_p, ":%s LTRACE %s :%s", 2, parc, parv))
       return;
 
-#ifdef STATIC_MODULES
   do_ltrace(source_p, parc, parv);
-#else
-  execute_callback(ltrace_cb, source_p, parc, parv);
-#endif
 }
 
 /*
@@ -261,18 +226,12 @@ mo_ltrace(struct Client *client_p, struct Client *source_p,
  * output	- counter of number of hits
  * side effects - NONE
  */
-static int
+static void
 report_this_status(struct Client *source_p, struct Client *target_p,
-                   int dow, int link_u_p, int link_s_p)
+                   int dow)
 {
   const char *name = NULL;
   const char *class_name = NULL;
-  char ip[HOSTIPLEN];
-
-  /* Should this be sockhost? - stu */
-  irc_getnameinfo((struct sockaddr *)&target_p->localClient->ip,
-              target_p->localClient->ip.ss_len, ip,
-              HOSTIPLEN, NULL, 0, NI_NUMERICHOST);
 
   name = get_client_name(target_p, HIDE_IP);
   class_name = get_client_class(target_p);
@@ -282,59 +241,66 @@ report_this_status(struct Client *source_p, struct Client *target_p,
     case STAT_CONNECTING:
       sendto_one(source_p, form_str(RPL_TRACECONNECTING), me.name,
                  source_p->name, class_name, 
-                 IsAdmin(source_p) ? name : target_p->name);
+                 HasUMode(source_p, UMODE_ADMIN) ? name : target_p->name);
       break;
 
     case STAT_HANDSHAKE:
       sendto_one(source_p, form_str(RPL_TRACEHANDSHAKE), me.name,
                  source_p->name, class_name, 
-                 IsAdmin(source_p) ? name : target_p->name);
+                 HasUMode(source_p, UMODE_ADMIN) ? name : target_p->name);
       break;
 
     case STAT_CLIENT:
-      if (IsAdmin(target_p))
+      if (HasUMode(target_p, UMODE_ADMIN))
       {
         if (ConfigFileEntry.hide_spoof_ips)
           sendto_one(source_p, form_str(RPL_TRACEOPERATOR),
                      me.name, source_p->name, class_name, name,
-                     (IsIPSpoof(target_p) ? "255.255.255.255" : ip),
-                     CurrentTime - target_p->lasttime,
-                     CurrentTime - target_p->localClient->last);
+                     (IsIPSpoof(target_p) ? "255.255.255.255" : target_p->sockhost),
+                     CurrentTime - target_p->localClient->lasttime,
+                     idle_time_get(source_p, target_p));
         else
           sendto_one(source_p, form_str(RPL_TRACEOPERATOR),
                      me.name, source_p->name, class_name, name,
-                     IsAdmin(source_p) ? ip :
-                     (IsIPSpoof(target_p) ? "255.255.255.255" : ip),
-                     CurrentTime - target_p->lasttime,
-                     CurrentTime - target_p->localClient->last);
+                     HasUMode(source_p, UMODE_ADMIN) ? target_p->sockhost :
+                     (IsIPSpoof(target_p) ? "255.255.255.255" : target_p->sockhost),
+                     CurrentTime - target_p->localClient->lasttime,
+                     idle_time_get(source_p, target_p));
       }
-      else if (IsOper(target_p))
+      else if (HasUMode(target_p, UMODE_OPER))
       {
         if (ConfigFileEntry.hide_spoof_ips)
           sendto_one(source_p, form_str(RPL_TRACEOPERATOR),
                      me.name, source_p->name, class_name, name,
-                     (IsIPSpoof(target_p) ? "255.255.255.255" : ip),
-                     CurrentTime - target_p->lasttime,
-                     CurrentTime - target_p->localClient->last);
+                     (IsIPSpoof(target_p) ? "255.255.255.255" : target_p->sockhost),
+                     CurrentTime - target_p->localClient->lasttime,
+                     idle_time_get(source_p, target_p));
         else
           sendto_one(source_p, form_str(RPL_TRACEOPERATOR),
                      me.name, source_p->name, class_name, name, 
-                     (IsIPSpoof(target_p) ? "255.255.255.255" : ip),
-                     CurrentTime - target_p->lasttime,
-                     CurrentTime - target_p->localClient->last);
+                     (IsIPSpoof(target_p) ? "255.255.255.255" : target_p->sockhost),
+                     CurrentTime - target_p->localClient->lasttime,
+                     idle_time_get(source_p, target_p));
       }
       break;
 
     case STAT_SERVER:
-      if(!IsAdmin(source_p))
+    {
+      int clients = 0;
+      int servers = 0;
+
+      trace_get_dependent(&servers, &clients, target_p);
+
+      if (!HasUMode(source_p, UMODE_ADMIN))
         name = get_client_name(target_p, MASK_IP);
 
       sendto_one(source_p, form_str(RPL_TRACESERVER),
-                 me.name, source_p->name, class_name, link_s_p,
-                 link_u_p, name, *(target_p->serv->by) ?
+                 me.name, source_p->name, class_name, servers,
+                 clients, name, *(target_p->serv->by) ?
                  target_p->serv->by : "*", "*",
-                 me.name, CurrentTime - target_p->lasttime);
+                 me.name, CurrentTime - target_p->localClient->lasttime);
       break;
+    }
 
     case STAT_ME:
     case STAT_UNKNOWN:
@@ -345,6 +311,31 @@ report_this_status(struct Client *source_p, struct Client *target_p,
                  source_p->name, name);
       break;
   }
-
-  return 0;
 }
+
+static struct Message ltrace_msgtab = {
+  "LTRACE", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  {m_unregistered, m_ltrace, mo_ltrace, m_ignore, mo_ltrace, m_ignore}
+};
+
+static void
+module_init(void)
+{
+  mod_add_cmd(&ltrace_msgtab);
+}
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&ltrace_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1776 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = 0
+};
