@@ -19,50 +19,24 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_opme.c 33 2005-10-02 20:50:00Z knight $
+ *  $Id: m_opme.c 1474 2012-07-22 14:44:07Z michael $
  */
 
 #include "stdinc.h"
-#include "tools.h"
-#include "handlers.h"
+#include "list.h"
 #include "channel.h"
 #include "channel_mode.h"
 #include "client.h"
 #include "ircd.h"
 #include "numeric.h"
-#include "s_log.h"
+#include "log.h"
 #include "s_serv.h"
 #include "send.h"
-#include "whowas.h"
 #include "irc_string.h"
 #include "hash.h"
-#include "msg.h"
 #include "parse.h"
 #include "modules.h"
-#include "common.h"
 
-static void mo_opme(struct Client *, struct Client *, int, char *[]);
-
-struct Message opme_msgtab = {
-  "OPME", 0, 0, 2, 0, MFLG_SLOW, 0,
-  { m_unregistered, m_not_oper, m_ignore, m_ignore, mo_opme, m_ignore }
-};
-
-#ifndef STATIC_MODULES
-void
-_modinit(void)
-{
-  mod_add_cmd(&opme_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&opme_msgtab);
-}
-
-const char *_version = "$Revision: 33 $";
-#endif
 
 static int
 chan_is_opless(const struct Channel *const chptr)
@@ -71,9 +45,9 @@ chan_is_opless(const struct Channel *const chptr)
 
   DLINK_FOREACH(ptr, chptr->members.head)
     if (((struct Membership *)ptr->data)->flags & CHFL_CHANOP)
-      return(0);
+      return 0;
 
-  return(1);
+  return 1;
 }
 
 /*
@@ -88,15 +62,13 @@ mo_opme(struct Client *client_p, struct Client *source_p,
   struct Channel *chptr = NULL;
   struct Membership *member = NULL;
 
-  if (!IsAdmin(source_p))
+  if (!HasUMode(source_p, UMODE_ADMIN))
   {
     sendto_one(source_p, form_str(ERR_NOPRIVILEGES),
                me.name, source_p->name);
     return;
   }
 
-  /* XXX - we might not have CBURSTed this channel if we are a lazylink
-   * yet. */
   if ((chptr = hash_find_channel(parv[1])) == NULL)
   {
     sendto_one(source_p, form_str(ERR_NOSUCHCHANNEL),
@@ -111,7 +83,7 @@ mo_opme(struct Client *client_p, struct Client *source_p,
     return;
   }
 
-  if (chan_is_opless(chptr) == 0)
+  if (!chan_is_opless(chptr))
   {
     sendto_one(source_p, ":%s NOTICE %s :%s Channel is not opless",
                me.name, source_p->name, chptr->chname);
@@ -120,42 +92,59 @@ mo_opme(struct Client *client_p, struct Client *source_p,
 
   AddMemberFlag(member, CHFL_CHANOP);
 
-  if (*parv[1] == '&')
-  {
-    sendto_wallops_flags(UMODE_LOCOPS, &me,
-                         "OPME called for [%s] by %s!%s@%s",
-                         chptr->chname, source_p->name, source_p->username,
-                         source_p->host);
-  }
-  else
-  {
-    sendto_wallops_flags(UMODE_WALLOP, &me,
-                         "OPME called for [%s] by %s!%s@%s",
-                         chptr->chname, source_p->name, source_p->username,
-                         source_p->host);
-    sendto_server(NULL, source_p, NULL, NOCAPS, NOCAPS, LL_ICLIENT,
-                  ":%s WALLOPS :OPME called for [%s] by %s!%s@%s",
-                  me.name, chptr->chname, source_p->name, source_p->username,
-                  source_p->host);
-  }
+  sendto_wallops_flags(UMODE_WALLOP, &me, "OPME called for [%s] by %s!%s@%s",
+                       chptr->chname, source_p->name, source_p->username,
+                       source_p->host);
+  sendto_server(NULL, NOCAPS, NOCAPS,
+                ":%s WALLOPS :OPME called for [%s] by %s!%s@%s",
+                me.name, chptr->chname, source_p->name, source_p->username,
+                source_p->host);
 
-  ilog(L_NOTICE, "OPME called for [%s] by %s!%s@%s",
+  ilog(LOG_TYPE_IRCD, "OPME called for [%s] by %s!%s@%s",
        chptr->chname, source_p->name, source_p->username,
        source_p->host);
 
-  sendto_server(NULL, source_p, chptr, CAP_TS6, NOCAPS, NOFLAGS,
-                 ":%s PART %s", ID(source_p), chptr->chname);
-  sendto_server(NULL, source_p, chptr, NOCAPS, CAP_TS6, NOFLAGS,
+  sendto_server(NULL, CAP_TS6, NOCAPS,
+                ":%s PART %s", ID(source_p), chptr->chname);
+  sendto_server(NULL, NOCAPS, CAP_TS6,
                 ":%s PART %s", source_p->name, chptr->chname);
-  sendto_server(NULL, source_p, chptr, CAP_TS6, NOCAPS, NOFLAGS,
+
+  sendto_server(NULL, CAP_TS6, NOCAPS,
                 ":%s SJOIN %lu %s + :@%s",
                 me.id, (unsigned long)chptr->channelts,
                 chptr->chname, ID(source_p));
-  sendto_server(NULL, source_p, chptr, NOCAPS, CAP_TS6, NOFLAGS,
+  sendto_server(NULL, NOCAPS, CAP_TS6,
                 ":%s SJOIN %lu %s + :@%s",
                 me.name, (unsigned long)chptr->channelts,
                 chptr->chname, source_p->name);
 
-  sendto_channel_local(ALL_MEMBERS, NO, chptr, ":%s MODE %s +o %s",
+  sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s MODE %s +o %s",
                        me.name, chptr->chname, source_p->name);
 }
+
+static struct Message opme_msgtab = {
+  "OPME", 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
+  { m_unregistered, m_not_oper, m_ignore, m_ignore, mo_opme, m_ignore }
+};
+
+static void
+module_init(void)
+{
+  mod_add_cmd(&opme_msgtab);
+}
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&opme_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1474 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = 0
+};
