@@ -21,14 +21,13 @@
 
 /*! \file channel.h
  * \brief Responsible for managing channels, members, bans and topics
- * \version $Id: channel.h 632 2006-06-01 10:53:00Z michael $
+ * \version $Id: channel.h 1750 2013-01-16 14:07:08Z michael $
  */
 
 #ifndef INCLUDED_channel_h
 #define INCLUDED_channel_h
 
 #include "ircd_defs.h"        /* KEYLEN, CHANNELLEN */
-#include "tools.h"
 
 struct Client;
 
@@ -37,7 +36,7 @@ struct Mode
 {
   unsigned int mode;   /*!< simple modes */
   unsigned int limit;  /*!< +l userlimit */
-  char key[KEYLEN];    /*!< +k key */
+  char key[KEYLEN + 1];    /*!< +k key */
 };
 
 /*! \brief Channel structure */
@@ -46,13 +45,18 @@ struct Channel
   dlink_node node;
 
   struct Channel *hnextch;
-
   struct Mode mode;
-  char *topic;
-  char *topic_info;
+
+  char topic[TOPICLEN + 1];
+  char topic_info[USERHOST_REPLYLEN];
+
+  time_t channelts;
   time_t topic_time;
-  unsigned long lazyLinkChannelExists;
   time_t last_knock; /*!< don't allow knock to flood */
+  time_t last_join_time;
+  time_t first_received_message_time; /*!< channel flood control */
+  unsigned int flags;
+  int received_number_of_privmsgs;
 
   dlink_list members;
   dlink_list invites;
@@ -60,13 +64,8 @@ struct Channel
   dlink_list exceptlist;
   dlink_list invexlist;
 
-  time_t first_received_message_time; /*!< channel flood control */
-  int received_number_of_privmsgs;
-  int flags;
   float number_joined;
-  time_t last_join_time;
 
-  time_t channelts;
   char chname[CHANNELLEN + 1];
 };
 
@@ -84,11 +83,11 @@ struct Membership
 struct Ban
 {
   dlink_node node;
-  size_t len;
   char *name;
   char *username;
   char *host;
   char *who;
+  size_t len;
   time_t when;
   struct irc_ssaddr addr;
   int bits;
@@ -99,7 +98,7 @@ extern dlink_list global_channel_list;
 
 extern int check_channel_name(const char *, int);
 extern int can_send(struct Channel *, struct Client *, struct Membership *);
-extern int is_banned(struct Channel *, struct Client *);
+extern int is_banned(const struct Channel *, const struct Client *);
 extern int can_join(struct Client *, struct Channel *, const char *);
 extern int has_member_flags(struct Membership *, unsigned int);
 
@@ -116,9 +115,8 @@ extern void channel_modes(struct Channel *, struct Client *, char *, char *);
 extern void check_spambot_warning(struct Client *, const char *);
 extern void check_splitmode(void *);
 extern void free_channel_list(dlink_list *);
-extern void free_topic(struct Channel *);
 extern void destroy_channel(struct Channel *);
-extern void set_channel_topic(struct Channel *, const char *, const char *, time_t);
+extern void set_channel_topic(struct Channel *, const char *, const char *, time_t, int);
 
 extern const char *get_member_status(const struct Membership *, int);
 
