@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: channel_mode.h 33 2005-10-02 20:50:00Z knight $
+ *  $Id: channel_mode.h 1175 2011-08-14 10:47:48Z michael $
  */
 
 
@@ -35,8 +35,8 @@
 
 /* can_send results */
 #define CAN_SEND_NO	0
-#define CAN_SEND_NONOP  1
-#define CAN_SEND_OPV	2
+#define CAN_SEND_NONOP  -1
+#define CAN_SEND_OPV	-2
 
 
 /* Channel related flags */
@@ -55,6 +55,10 @@
 #define MODE_TOPICLIMIT 0x0008
 #define MODE_INVITEONLY 0x0010
 #define MODE_NOPRIVMSGS 0x0020
+#define MODE_SSLONLY    0x0040
+#define MODE_OPERONLY   0x0080
+#define MODE_REGISTERED 0x0100 /* Channel has been registered with ChanServ */
+#define MODE_REGONLY    0x0200
 
 /* cache flags for silence on ban */
 #define CHFL_BAN_CHECKED  0x0080
@@ -82,8 +86,8 @@ struct ChModeChange
   const char *arg;
   const char *id;
   int dir;
-  int caps;
-  int nocaps;
+  unsigned int caps;
+  unsigned int nocaps;
   int mems;
   struct Client *client;
 };
@@ -91,14 +95,22 @@ struct ChModeChange
 struct ChCapCombo
 {
   int count;
-  int cap_yes;
-  int cap_no;
+  unsigned int cap_yes;
+  unsigned int cap_no;
 };
 
+struct mode_letter
+{
+  const unsigned int mode;
+  const unsigned char letter;
+};
+
+extern const struct mode_letter chan_modes[];
 extern int add_id(struct Client *, struct Channel *, char *, int);
 extern void set_channel_mode(struct Client *, struct Client *, struct Channel *,
                              struct Membership *, int, char **, char *);
 extern void clear_ban_cache(struct Channel *);
+extern void clear_ban_cache_client(struct Client *);
 extern void init_chcap_usage_counts(void);
 extern void set_chcap_usage_counts(struct Client *);
 extern void unset_chcap_usage_counts(struct Client *);
