@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: hash.h 33 2005-10-02 20:50:00Z knight $
+ *  $Id: hash.h 1169 2011-08-12 18:45:03Z michael $
  */
 
 #ifndef INCLUDED_hash_h
@@ -57,14 +57,14 @@ extern void hash_del_userhost(struct UserHost *);
 
 extern struct UserHost *hash_find_userhost(const char *);
 extern struct Client *hash_find_id(const char *);
-extern struct Client *find_client(const char *);
-extern struct Client *find_server(const char *);
+extern struct Client *hash_find_client(const char *);
+extern struct Client *hash_find_server(const char *);
 extern struct Channel *hash_find_channel(const char *);
 extern void *hash_get_bucket(int, unsigned int);
 extern struct ResvChannel *hash_find_resv(const char *);
 
 extern void free_list_task(struct ListTask *, struct Client *);
-extern void safe_list_channels(struct Client *, struct ListTask *, int, int);
+extern void safe_list_channels(struct Client *, struct ListTask *, int);
 
 extern unsigned int strhash(const char *);
 #endif  /* INCLUDED_hash_h */
