@@ -19,17 +19,18 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: irc_string.h 633 2006-06-01 12:30:29Z db $
+ *  $Id: irc_string.h 1400 2012-05-07 20:08:32Z michael $
  */
 
 #ifndef INCLUDED_irc_string_h
 #define INCLUDED_irc_string_h
 
-#include "setup.h"
-#include "pcre.h"
+#include "config.h"
 
-extern int ircd_pcre_exec(const pcre *, const char *);
-extern pcre *ircd_pcre_compile(const char *, const char **);
+
+extern int has_wildcards(const char *);
+extern int ircd_pcre_exec(const void *, const char *);
+extern void *ircd_pcre_compile(const char *, const char **);
 
 /*
  * match - compare name with mask, mask may contain * and ? as wildcards
@@ -66,18 +67,6 @@ extern int irccmp(const char *, const char *);
  */
 extern int ircncmp(const char *, const char *, size_t);
 
-/*
- * inetntoa - optimized inet_ntoa
- */
-extern const char *inetntoa(const char *);
-
-/* XXX
- * inetntop() 
- * portable interface for inet_ntop(), kludge; please use inet_ntop if possible
- * since inet_misc has a more conformant one
- */
-extern const char *inetntop(int, const void *, char *, unsigned int);
-   
 #ifndef HAVE_STRLCPY
 extern size_t strlcpy(char *, const char *, size_t);
 #endif
@@ -86,23 +75,7 @@ extern size_t strlcpy(char *, const char *, size_t);
 extern size_t strlcat(char *, const char *, size_t);
 #endif
 
-#ifndef HAVE_SNPRINTF
-extern int snprintf(char *, size_t, const char *,...);
-#endif
-
-#ifndef HAVE_VSNPRINTF
-extern int vsnprintf(char *, size_t, const char *, va_list);
-#endif
-
-#ifndef HAVE_BASENAME
-extern char *basename(char *);
-#endif
-
-/*
- * clean_string - cleanup control and high ascii characters
- * -Dianora
- */
-extern char *clean_string(char *, const unsigned char *, size_t);
+extern const char *libio_basename(const char *);
 
 /*
  * strip_tabs - convert tabs to spaces
