@@ -19,39 +19,36 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: listener.h,v 7.10 2002/05/24 23:34:07 androsyn Exp $
+ *  $Id: listener.h 1123 2011-02-06 21:57:50Z michael $
  */
 
 #ifndef INCLUDED_listener_h
 #define INCLUDED_listener_h
 
+#define LISTENER_SSL    0x1
+#define LISTENER_HIDDEN 0x2
+#define LISTENER_SERVER 0x4
+
 #include "ircd_defs.h"  
+#include "fdlist.h"
 
 struct Client;
 
-struct Listener {
-  struct Listener* next;               /* list node pointer */
-  const char*      name;               /* listener name */
-  int              fd;                 /* file descriptor */
+struct Listener
+{
+  dlink_node	   listener_node;      /* list node pointer */
+  fde_t            fd;                 /* file descriptor */
   int              port;               /* listener IP port */
   int              ref_count;          /* number of connection references */
   int              active;             /* current state of listener */
-  int              index;              /* index into poll array */
-  time_t           last_accept;        /* last time listener accepted */
-/* jdc -- this seems to be incorrect in comparison to src/listener.c */
-/*
-  struct in_addr    addr;
-*/
-  struct irc_inaddr addr;              /* virtual address or INADDR_ANY */
-  struct DNSQuery   *dns_query;
-  char             vhost[HOSTLEN + 1]; /* virtual name of listener */
+  struct irc_ssaddr addr;              /* virtual address or INADDR_ANY */
+  char              name[HOSTLEN + 1]; /* virtual name of listener */
+  unsigned int      flags;
 };
 
-extern void        add_listener(int port, const char* vaddr_ip);
-extern void        close_listener(struct Listener* listener);
-extern void        close_listeners(void);
-extern const char* get_listener_name(const struct Listener* listener);
-extern void        show_ports(struct Client* client);
-extern void free_listener(struct Listener * );
-
+extern void add_listener(int, const char *, unsigned int);
+extern void close_listeners(void);
+extern const char *get_listener_name(const struct Listener *const);
+extern void show_ports(struct Client *);
+extern void free_listener(struct Listener *);
 #endif /* INCLUDED_listener_h */
