@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: motd.h 33 2005-10-02 20:50:00Z knight $
+ *  $Id: motd.h 1738 2013-01-14 17:38:34Z michael $
  */
 
 #ifndef INCLUDED_motd_h
@@ -31,8 +31,8 @@
 
 struct MessageFileLine
 {
-  char line[MESSAGELINELEN + 1];
   struct MessageFileLine *next;
+  char line[MESSAGELINELEN + 1];
 };
 
 typedef struct MessageFileLine MessageFileLine;
@@ -40,15 +40,14 @@ typedef struct MessageFileLine MessageFileLine;
 typedef enum {
   USER_MOTD,
   USER_LINKS,
-  OPER_MOTD,
   ISSUPPORT
 } MotdType;
   
 struct MessageFile
 {
-  MotdType motdType;
   MessageFileLine *contentsOfFile;
-  char fileName[PATH_MAX + 1];
+  MotdType motdType;
+  char fileName[HYB_PATH_MAX + 1];
   char lastChangedDate[MAX_DATE_STRING + 1];
 };
 
