@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: numeric.h 660 2006-06-09 18:51:22Z adx $
+ *  $Id: numeric.h 1549 2012-10-01 18:11:11Z michael $
  */
 
 #ifndef INCLUDED_numeric_h
@@ -78,7 +78,6 @@ extern const char *get_locale(void);
 #define RPL_TRACEOPERATOR    204
 #define RPL_TRACEUSER        205
 #define RPL_TRACESERVER      206
-#define RPL_TRACECAPTURED    207
 #define RPL_TRACENEWTYPE     208
 #define RPL_TRACECLASS       209
 
@@ -134,6 +133,7 @@ extern const char *get_locale(void);
 #define RPL_STATSHLINE       244
 /* 245 No longer used in ircd-hybrid */
 #define RPL_STATSSLINE       245
+#define RPL_STATSSERVICE     246
 #define RPL_STATSXLINE       247
 #define RPL_STATSULINE       248
 #define RPL_STATSDEBUG       249
@@ -163,8 +163,6 @@ extern const char *get_locale(void);
 #define RPL_LOCALUSERS       265
 #define RPL_GLOBALUSERS      266
 
-#define RPL_WHOISSSL         275
-
 #define RPL_ACCEPTLIST	     281
 #define RPL_ENDOFACCEPT      282
 
@@ -201,8 +199,8 @@ extern const char *get_locale(void);
 #define RPL_TEXT             304
 #define RPL_UNAWAY           305
 #define RPL_NOWAWAY          306
-#define	RPL_USERIP	     307	/* Undernet extension */
-/*	RPL_WHOISREGNICK     307 	Numeric List: Dalnet */
+/*	RPL_USERIP	     307	Undernet extension */
+#define RPL_WHOISREGNICK     307
 /*	RPL_SUSERHOST	     307	austnet */
 /*      RPL_NOTIFYACTION     308         aircd */
 #define RPL_WHOISADMIN	     308	/* Numeric List: Dalnet */
@@ -367,21 +365,21 @@ extern const char *get_locale(void);
 #define ERR_ALREADYREGISTRED 462
 #define ERR_PASSWDMISMATCH   464
 #define ERR_YOUREBANNEDCREEP 465
-/* 	ERR_ONLYSERVERSCANCHANGE 468	   Dalnet,unreal */
+#define ERR_ONLYSERVERSCANCHANGE 468
 /*	ERR_LINKSET	     469	unreal */
 /*	ERR_LINKCHANNEL	     470	unreal */
 /*      ERR_KICKEDFROMCHAN   470         aircd */
+#define ERR_OPERONLYCHAN     470
 #define ERR_CHANNELISFULL    471
 #define ERR_UNKNOWNMODE      472
 #define ERR_INVITEONLYCHAN   473
 #define ERR_BANNEDFROMCHAN   474
 #define ERR_BADCHANNELKEY    475
-/*      ERR_MODELESS         477           ircu numeric -db */
+#define ERR_NEEDREGGEDNICK   477
 #define ERR_BANLISTFULL      478        /* I stole the numeric from ircu -db */
 #define ERR_BADCHANNAME      479
 /* 	ERR_LINKFAIL	     479	unreal */
-/*	ERR_CANNOTKNOCK	     480	unreal */
-/*	ERR_NOULINE	     480	austnet */
+#define ERR_SSLONLYCHAN      480
 #define ERR_NOPRIVILEGES     481
 #define ERR_CHANOPRIVSNEEDED 482
 #define ERR_CANTKILLSERVER   483
@@ -392,7 +390,7 @@ extern const char *get_locale(void);
 /*      ERR_UNIQOPRIVSNEEDED 485           IRCnet extension */
 /*	ERR_KILLDENY         485	   unreal */
 /*	ERR_CANTKICKADMIN    485	   PTlink */
-/*	ERR_HTMDISABLED      486	   unreal */
+#define ERR_NONONREG         486
 /*      ERR_CHANTOORECENT    487           IRCnet extension (?) */
 /*      ERR_TSLESSCHAN       488           IRCnet extension (?) */
 /*	ERR_VOICENEEDED      489           Undernet extension */
@@ -411,10 +409,8 @@ extern const char *get_locale(void);
  * moved to 999
  */
 
-/*      ERR_NOTIFYFULL       512           aircd */
-/*	ERR_TOOMANYWATCH     512           Numeric List: Dalnet */
-/*      ERR_NEEDPONG	     512           Numeric List: Dalnet */
 
+#define ERR_TOOMANYWATCH     512
 #define ERR_WRONGPONG	     513
 /*	ERR_TOOMANYDCC	     514	dalnet */
 /*	ERR_NOINVITE	     518	unreal */
@@ -431,16 +427,16 @@ extern const char *get_locale(void);
 
 #define ERR_HELPNOTFOUND     524
 
-/*	RPL_LOGON	     600	dalnet,unreal
-	RPL_LOGOFF           601	dalnet,unreal
-	RPL_WATCHOFF         602	dalnet,unreal
-	RPL_WATCHSTAT        603	dalnet,unreal
-	RPL_NOWON            604	dalnet,unreal
-	RPL_NOWOFF           605	dalnet,unreal
-	RPL_WATCHLIST        606	dalnet,unreal
-	RPL_ENDOFWATCHLIST   607	dalnet,unreal
+#define RPL_LOGON            600
+#define RPL_LOGOFF           601
+#define RPL_WATCHOFF         602
+#define RPL_WATCHSTAT        603
+#define RPL_NOWON            604
+#define RPL_NOWOFF           605
+#define RPL_WATCHLIST        606
+#define RPL_ENDOFWATCHLIST   607
 
-	RPL_MAPMORE          610	unreal
+/*	RPL_MAPMORE          610	unreal
 
 	RPL_MAPMORE          615	PTlink
 	
@@ -454,6 +450,7 @@ extern const char *get_locale(void);
 	RPL_EODUMP	     642	unreal
 */
 
+#define RPL_WHOISSECURE      671
 #define RPL_MODLIST          702
 #define RPL_ENDOFMODLIST     703
 
@@ -461,29 +458,25 @@ extern const char *get_locale(void);
 #define RPL_HELPTXT          705
 #define RPL_ENDOFHELP        706
 
+#define RPL_ETRACE_FULL      708
+#define RPL_ETRACE           709
+
 #define RPL_KNOCK	     710
 #define RPL_KNOCKDLVR	     711
 
 #define ERR_TOOMANYKNOCK     712
 #define ERR_CHANOPEN         713
 #define ERR_KNOCKONCHAN      714
-#define ERR_KNOCKDISABLED    715
 
 #define RPL_TARGUMODEG       716
 #define RPL_TARGNOTIFY       717
 #define RPL_UMODEGMSG        718
 
-#define RPL_OMOTDSTART	     720
-#define RPL_OMOTD	     721
-#define RPL_ENDOFOMOTD	     722
-
 #define ERR_NOPRIVS	     723
 
 #define RPL_TESTMASK	     724
 #define RPL_TESTLINE	     725
 #define RPL_NOTESTLINE	     726
-#define RPL_ISCAPTURED	     727
-#define RPL_ISUNCAPTURED     728
 
 #define ERR_LAST_ERR_MSG     999
 
