@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: packet.h 163 2005-10-20 21:09:02Z adx $
+ *  $Id: packet.h 1321 2012-03-30 11:23:16Z michael $
  */
 
 #ifndef INCLUDED_packet_h
@@ -50,9 +50,7 @@ struct Callback;
 
 void *iorecv_default(va_list);
 extern struct Callback *iorecv_cb;
-extern struct Callback *iorecvctrl_cb;
 
-PF read_ctrl_packet;
 PF read_packet;
 PF flood_recalc;
 void flood_endgrace(struct Client *);
