@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: s_user.h 573 2006-05-01 09:40:37Z michael $
+ *  $Id: s_user.h 1815 2013-04-12 09:33:33Z michael $
  */
 
 #ifndef INCLUDED_s_user_h
@@ -30,13 +30,9 @@
 #define TOTALSIDUID (IRC_MAXSID + IRC_MAXUID)
 
 struct Client;
-struct AccessItem;
 
-extern int MaxClientCount;     /* GLOBAL - highest number of clients     */
-extern int MaxConnectionCount; /* GLOBAL - highest number of connections */
 extern struct Callback *entering_umode_cb;
 extern struct Callback *umode_cb;
-extern struct Callback *uid_get_cb;
 extern unsigned int user_modes[];
 
 extern void assemble_umode_buffer(void);
@@ -48,21 +44,18 @@ extern void show_lusers(struct Client *);
 extern void show_isupport(struct Client *);
 extern void oper_up(struct Client *);
 
-extern void register_local_user(struct Client *, struct Client *,
-                                const char *, const char *);
-extern void register_remote_user(struct Client *, struct Client *,
+extern void register_local_user(struct Client *);
+extern void register_remote_user(struct Client *,
                                  const char *, const char *,
                                  const char *, const char *);
-extern void do_local_user(const char *, struct Client *, struct Client *,
-                          const char *, const char *, const char *,
-                          const char *);
 extern void init_uid(void);
 extern int valid_sid(const char *);
 extern int valid_hostname(const char *);
 extern int valid_username(const char *);
+extern int valid_nickname(const char *, const int);
 extern void add_isupport(const char *, const char *, int);
 extern void delete_isupport(const char *);
 extern void init_isupport(void);
 extern void rebuild_isupport_message_line(void);
-
+extern void user_set_hostmask(struct Client *, const char *);
 #endif
