@@ -19,7 +19,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: sprintf_irc.h,v 7.8 2002/05/24 23:34:08 androsyn Exp $
+ *  $Id: sprintf_irc.h 1015 2009-10-25 00:08:06Z michael $
  */
 
 #ifndef SPRINTF_IRC
@@ -30,23 +30,10 @@
  * Proto types
  */
 
-extern int vsprintf_irc(char *str, const char *format, va_list);
-extern int vsnprintf_irc(char *, int, const char*, va_list);
-/* old */
-/* extern int ircsprintf(char *str, char *format, ...); */
-/* */
+extern int vsprintf_irc(char *, const char *, va_list);
 
 /*
  * ircsprintf - optimized sprintf
  */
-#ifdef __GNUC__
-extern int ircsprintf(char*, const char*, ...)
-               __attribute__ ((format(printf, 2, 3)));
-extern int ircsnprintf(char*, int, const char*, ...)
-                __attribute__ ((format(printf, 3, 4)));
-#else
-extern int ircsprintf(char *str, const char *format, ...);
-extern int ircsnprintf(char*, int, const char*);
-#endif
-
+extern int ircsprintf(char *, const char *, ...);
 #endif /* SPRINTF_IRC */
