@@ -19,34 +19,22 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- * $Id: stdinc.h 536 2006-03-23 09:59:02Z michael $
+ * $Id: stdinc.h 1871 2013-04-26 08:53:04Z michael $
  *
  */
 
 #ifndef STDINC_H /* prevent multiple #includes */
 #define STDINC_H
  
-#ifndef IN_AUTOCONF
-#include "setup.h"
-#endif
+#include "config.h"
 
 #include "defaults.h"
 
-#ifdef HAVE_STDLIB_H
+#include <stddef.h>
 #include <stdlib.h>
-#endif
-#ifdef STRING_WITH_STRINGS
-# include <string.h>
-# include <strings.h>
-#else
-# ifdef HAVE_STRING_H
-#  include <string.h>
-# else
-#  ifdef HAVE_STRINGS_H
-#   include <strings.h>
-#  endif
-# endif 
-#endif  
+#include <string.h>
+#include <stdint.h>
+#include <errno.h>
 
 #ifdef HAVE_STRTOK_R
 # define strtoken(x, y, z) strtok_r(y, z, x)
@@ -54,25 +42,6 @@
 
 #include <sys/types.h>
 
-#ifdef HAVE_INTTYPES_H
-#include <inttypes.h>
-#else
-#ifdef HAVE_STDINT_H
-#include <stdint.h>
-#endif
-#endif
-
-#ifdef HAVE_STDDEF_H
-# include <stddef.h>
-#else /* This is basically what stddef.h provides on most systems */
-# ifndef NULL
-#  define NULL ((void*)0)
-# endif
-# ifndef offsetof
-#  define offsetof(TYPE, MEMBER) ((size_t) &((TYPE *)0)->MEMBER)
-# endif
-#endif
-
 #ifdef HAVE_CRYPT_H
 #include <crypt.h>
 #endif
@@ -87,20 +56,10 @@
 #include <time.h>
 #include <fcntl.h>
 
-#ifdef HAVE_LIBGEN_H
-#include <libgen.h>
-#endif
-
 #include <stdarg.h>
 #include <signal.h>
 #include <ctype.h>
 
-#ifdef _WIN32
-#define PATH_MAX (MAX_PATH - 1)
-#define WIN32_LEAN_AND_MEAN
-#include <windows.h>
-#include <winsock.h>
-#else
 #include <dirent.h>
 #include <netdb.h>
 #include <sys/socket.h>
@@ -108,7 +67,6 @@
 #include <arpa/inet.h>
 #include <sys/time.h>
 #include <sys/file.h>
-#endif
 
 #include <limits.h>
 
@@ -129,17 +87,16 @@
 #include <sys/param.h>
 #endif
 
-#ifdef HAVE_ERRNO_H
-#include <errno.h>
+#ifdef PATH_MAX
+#define HYB_PATH_MAX PATH_MAX
 #else
-extern int errno;
+#define HYB_PATH_MAX 4096
 #endif
 
-#include "inet_misc.h"
-
-#ifdef _WIN32
-#undef strcasecmp
-#define strcasecmp stricmp
+#if 0 && __GNUC__
+#define AFP(a,b) __attribute__((format (printf, a, b)))
+#else
+#define AFP(a,b)
 #endif
 
 #endif
