@@ -19,52 +19,21 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_die.c 33 2005-10-02 20:50:00Z knight $
+ *  $Id: m_die.c 1309 2012-03-25 11:24:18Z michael $
  */
 
 #include "stdinc.h"
-#include "tools.h"
-#include "handlers.h"
 #include "client.h"
-#include "common.h"
 #include "ircd.h"
 #include "irc_string.h"
 #include "numeric.h"
-#include "fdlist.h"
-#include "s_bsd.h"
-#include "s_log.h"
-#include "s_conf.h"
 #include "send.h"
-#include "msg.h"
 #include "parse.h"
 #include "modules.h"
 #include "restart.h"
-#include "sprintf_irc.h"
+#include "conf.h"
 
 
-static void mo_die(struct Client *, struct Client *, int, char *[]);
-
-struct Message die_msgtab = {
-  "DIE", 0, 0, 1, 0, MFLG_SLOW, 0,
-  {m_unregistered, m_not_oper, m_ignore, m_ignore, mo_die, m_ignore}
-};
-
-#ifndef STATIC_MODULES
-void
-_modinit(void)
-{
-  mod_add_cmd(&die_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&die_msgtab);
-}
-
-const char *_version = "$Revision: 33 $";
-#endif
-
 /*
  * mo_die - DIE command handler
  */
@@ -74,7 +43,7 @@ mo_die(struct Client *client_p, struct Client *source_p,
 {
   char buf[IRCD_BUFSIZE];
 
-  if (!IsOperDie(source_p))
+  if (!HasOFlag(source_p, OPER_FLAG_DIE))
   {
     sendto_one(source_p, form_str(ERR_NOPRIVS),
                me.name, source_p->name, "die");
@@ -83,19 +52,46 @@ mo_die(struct Client *client_p, struct Client *source_p,
 
   if (parc < 2 || EmptyString(parv[1]))
   {
-    sendto_one(source_p,":%s NOTICE %s :Need server name /die %s",
+    sendto_one(source_p, ":%s NOTICE %s :Need server name /die %s",
                me.name, source_p->name, me.name);
     return;
   }
 
   if (irccmp(parv[1], me.name))
   {
-    sendto_one(source_p,":%s NOTICE %s :Mismatch on /die %s",
-               me.name,source_p->name, me.name);
+    sendto_one(source_p, ":%s NOTICE %s :Mismatch on /die %s",
+               me.name, source_p->name, me.name);
     return;
   }
 
-  ircsprintf(buf, "received DIE command from %s",
-             get_oper_name(source_p));
-  server_die(buf, NO);
+  snprintf(buf, sizeof(buf), "received DIE command from %s",
+           get_oper_name(source_p));
+  server_die(buf, 0);
 }
+
+static struct Message die_msgtab = {
+  "DIE", 0, 0, 1, MAXPARA, MFLG_SLOW, 0,
+  {m_unregistered, m_not_oper, m_ignore, m_ignore, mo_die, m_ignore}
+};
+
+static void
+module_init(void)
+{
+  mod_add_cmd(&die_msgtab);
+}
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&die_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1309 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = MODULE_FLAG_CORE
+};
