@@ -19,33 +19,28 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_join.c 632 2006-06-01 10:53:00Z michael $
+ *  $Id: m_join.c 1871 2013-04-26 08:53:04Z michael $
  */
 
 #include "stdinc.h"
-#include "tools.h"
-#include "handlers.h"
+#include "list.h"
 #include "channel.h"
 #include "channel_mode.h"
 #include "client.h"
-#include "common.h"   /* bleah */
 #include "hash.h"
 #include "irc_string.h"
 #include "sprintf_irc.h"
 #include "ircd.h"
-#include "list.h"
 #include "numeric.h"
 #include "send.h"
 #include "s_serv.h"
-#include "s_conf.h"
-#include "msg.h"
+#include "conf.h"
 #include "parse.h"
 #include "modules.h"
+#include "resv.h"
 
 
-static void m_join(struct Client *, struct Client *, int, char **);
-static void ms_join(struct Client *, struct Client *, int, char **);
-static void do_join_0(struct Client *client_p, struct Client *source_p);
+static void do_join_0(struct Client *, struct Client *);
 
 static void set_final_mode(struct Mode *, struct Mode *);
 static void remove_our_modes(struct Channel *, struct Client *);
@@ -56,27 +51,6 @@ static char parabuf[MODEBUFLEN];
 static char sendbuf[MODEBUFLEN];
 static char *mbuf;
 
-struct Message join_msgtab = {
-  "JOIN", 0, 0, 2, 0, MFLG_SLOW, 0,
-  {m_unregistered, m_join, ms_join, m_ignore, m_join, m_ignore}
-};
-
-#ifndef STATIC_MODULES
-void
-_modinit(void)
-{
-  mod_add_cmd(&join_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&join_msgtab);
-}
-
-const char *_version = "$Revision: 632 $";
-#endif
-
 /* last0() stolen from ircu */
 static char *
 last0(struct Client *client_p, struct Client *source_p, char *chanlist)
@@ -124,6 +98,7 @@ m_join(struct Client *client_p, struct Client *source_p,
   char *chan_list = NULL;
   char *chan = NULL;
   struct Channel *chptr = NULL;
+  struct ResvChannel *resv_cp = NULL;
   int i = 0;
   unsigned int flags = 0;
 
@@ -159,28 +134,24 @@ m_join(struct Client *client_p, struct Client *source_p,
       continue;
     }
 
-    if (ConfigChannel.disable_local_channels && (*chan == '&'))
-    {
-      sendto_one(source_p, form_str(ERR_NOSUCHCHANNEL),
-                 me.name, source_p->name, chan);
-      continue;
-    }
-
     if (!IsExemptResv(source_p) &&
-        !(IsOper(source_p) && ConfigFileEntry.oper_pass_resv) &&
-        (!hash_find_resv(chan) == ConfigChannel.restrict_channels))
+        !(HasUMode(source_p, UMODE_OPER) && ConfigFileEntry.oper_pass_resv) &&
+        (!(resv_cp = hash_find_resv(chan)) == ConfigChannel.restrict_channels))
     {
+      if (resv_cp)
+        ++resv_cp->count;
       sendto_one(source_p, form_str(ERR_BADCHANNAME),
                  me.name, source_p->name, chan);
       sendto_realops_flags(UMODE_SPY, L_ALL,
-                           "User %s (%s@%s) is attempting to join locally juped channel %s",
-                           source_p->name, source_p->username, source_p->host, chan);
+                           "Forbidding reserved channel [%s] from user %s",
+                           chan, get_client_name(source_p, HIDE_IP));
       continue;
     }
 
-    if ((dlink_list_length(&source_p->channel) >= ConfigChannel.max_chans_per_user) &&
-        (!IsOper(source_p) || (dlink_list_length(&source_p->channel) >=
-                               ConfigChannel.max_chans_per_user * 3)))
+    if (dlink_list_length(&source_p->channel) >=
+        (HasUMode(source_p, UMODE_OPER) ?
+         ConfigChannel.max_chans_per_oper :
+         ConfigChannel.max_chans_per_user))
     {
       sendto_one(source_p, form_str(ERR_TOOMANYCHANNELS),
                  me.name, source_p->name, chan);
@@ -192,7 +163,7 @@ m_join(struct Client *client_p, struct Client *source_p,
       if (IsMember(source_p, chptr))
         continue;
 
-      if (splitmode && !IsOper(source_p) && (*chan != '&') &&
+      if (splitmode && !HasUMode(source_p, UMODE_OPER) &&
           ConfigChannel.no_join_on_split)
       {
         sendto_one(source_p, form_str(ERR_UNAVAILRESOURCE),
@@ -200,6 +171,16 @@ m_join(struct Client *client_p, struct Client *source_p,
         continue;
       }
 
+      /*
+       * can_join checks for +i key, bans.
+       */
+      if ((i = can_join(source_p, chptr, key)))
+      {
+        sendto_one(source_p, form_str(i), me.name,
+                   source_p->name, chptr->chname);
+        continue;
+      }
+
       /*
        * This should never be the case unless there is some sort of
        * persistant channels.
@@ -211,7 +192,7 @@ m_join(struct Client *client_p, struct Client *source_p,
     }
     else
     {
-      if (splitmode && !IsOper(source_p) && (*chan != '&') &&
+      if (splitmode && !HasUMode(source_p, UMODE_OPER) &&
           (ConfigChannel.no_create_on_split || ConfigChannel.no_join_on_split))
       {
         sendto_one(source_p, form_str(ERR_UNAVAILRESOURCE),
@@ -223,20 +204,10 @@ m_join(struct Client *client_p, struct Client *source_p,
       chptr = make_channel(chan);
     }
 
-    if (!IsOper(source_p))
+    if (!HasUMode(source_p, UMODE_OPER))
       check_spambot_warning(source_p, chptr->chname);
 
-    /*
-     * can_join checks for +i key, bans.
-     */
-    if ((i = can_join(source_p, chptr, key)))
-    {
-      sendto_one(source_p, form_str(i), me.name,
-                 source_p->name, chptr->chname);
-      continue;
-    }
-
-    add_user_to_channel(chptr, source_p, flags, YES);
+    add_user_to_channel(chptr, source_p, flags, 1);
 
     /*
      *  Set timestamp if appropriate, and propagate
@@ -247,42 +218,42 @@ m_join(struct Client *client_p, struct Client *source_p,
       chptr->mode.mode |= MODE_TOPICLIMIT;
       chptr->mode.mode |= MODE_NOPRIVMSGS;
 
-      sendto_server(client_p, source_p, chptr, CAP_TS6, NOCAPS, LL_ICLIENT,
+      sendto_server(client_p, CAP_TS6, NOCAPS,
                     ":%s SJOIN %lu %s +nt :@%s",
                     me.id, (unsigned long)chptr->channelts,
                     chptr->chname, source_p->id);
-      sendto_server(client_p, source_p, chptr, NOCAPS, CAP_TS6, LL_ICLIENT,
+      sendto_server(client_p, NOCAPS, CAP_TS6,
                     ":%s SJOIN %lu %s +nt :@%s",
                     me.name, (unsigned long)chptr->channelts,
                     chptr->chname, source_p->name);
       /*
        * notify all other users on the new channel
        */
-      sendto_channel_local(ALL_MEMBERS, NO, chptr, ":%s!%s@%s JOIN :%s",
+      sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s!%s@%s JOIN :%s",
                            source_p->name, source_p->username,
                            source_p->host, chptr->chname);
-      sendto_channel_local(ALL_MEMBERS, NO, chptr, ":%s MODE %s +nt",
+      sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s MODE %s +nt",
                            me.name, chptr->chname);
     }
     else
     {
-      sendto_server(client_p, source_p, chptr, CAP_TS6, NOCAPS, LL_ICLIENT,
+      sendto_server(client_p, CAP_TS6, NOCAPS,
                     ":%s JOIN %lu %s +",
                     source_p->id, (unsigned long)chptr->channelts,
                     chptr->chname);
-      sendto_server(client_p, source_p, chptr, NOCAPS, CAP_TS6, LL_ICLIENT,
+      sendto_server(client_p, NOCAPS, CAP_TS6,
                     ":%s SJOIN %lu %s + :%s",
                     me.name, (unsigned long)chptr->channelts,
                     chptr->chname, source_p->name);
 
-      sendto_channel_local(ALL_MEMBERS, NO, chptr, ":%s!%s@%s JOIN :%s",
+      sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s!%s@%s JOIN :%s",
                            source_p->name, source_p->username,
                            source_p->host, chptr->chname);
     }
 
     del_invite(chptr, source_p);
 
-    if (chptr->topic != NULL)
+    if (chptr->topic[0])
     {
       sendto_one(source_p, form_str(RPL_TOPIC), me.name,
                  source_p->name, chptr->chname, chptr->topic);
@@ -303,7 +274,7 @@ m_join(struct Client *client_p, struct Client *source_p,
  * inputs	- parv[0] = uid
  *		  parv[1] = ts
  *		  parv[2] = channel name
- *		  parv[3] = modes
+ *		  parv[3] = modes (Deprecated)
  * output	- none
  * side effects	- handles remote JOIN's sent by servers. In TSora
  *		  remote clients are joined using SJOIN, hence a 
@@ -317,11 +288,9 @@ ms_join(struct Client *client_p, struct Client *source_p,
 {
   time_t newts = 0;
   time_t oldts = 0;
-  int args = 0;
   int keep_our_modes = 1;
   int keep_new_modes = 1;
   int isnew = 0;
-  const char *s = NULL;
   const char *servername = NULL;
   struct Channel *chptr = NULL;
   struct Mode mode, *oldmode;
@@ -332,7 +301,7 @@ ms_join(struct Client *client_p, struct Client *source_p,
     return;
   }
 
-  if (parc < 4 || *parv[2] == '&')
+  if (parc < 4)
     return;
 
   if (!check_channel_name(parv[2], 0))
@@ -347,46 +316,6 @@ ms_join(struct Client *client_p, struct Client *source_p,
   mode.mode = mode.limit = 0;
   mode.key[0] = '\0';
 
-  for (s = parv[3]; *s; ++s)
-  {
-    switch (*s)
-    {
-      case 't':
-        mode.mode |= MODE_TOPICLIMIT;
-        break;
-      case 'n':
-        mode.mode |= MODE_NOPRIVMSGS;
-        break;
-      case 's':
-        mode.mode |= MODE_SECRET;
-        break;
-      case 'm':
-        mode.mode |= MODE_MODERATED;
-        break;
-      case 'i':
-        mode.mode |= MODE_INVITEONLY;
-        break;
-      case 'p':
-        mode.mode |= MODE_PRIVATE;
-        break;
-      case 'k':
-        if (parc < 5 + args)
-          return;
-
-        strlcpy(mode.key, parv[4 + args], sizeof(mode.key));
-        args++;
-        break;
-      case 'l':
-        if (parc < 5 + args)
-          return;
-
-        mode.limit = atoi(parv[4 + args]);
-        args++;
-        break;
-    }
-  }
-
-
   if ((chptr = hash_find_channel(parv[2])) == NULL)
   {
     isnew = 1;
@@ -413,9 +342,9 @@ ms_join(struct Client *client_p, struct Client *source_p,
   {
     if (!newts && !isnew && oldts)
     {
-      sendto_channel_local(ALL_MEMBERS, NO, chptr,
-                             ":%s NOTICE %s :*** Notice -- TS for %s changed from %lu to 0",
-                             me.name, chptr->chname, chptr->chname, (unsigned long)oldts);
+      sendto_channel_local(ALL_MEMBERS, 0, chptr,
+                           ":%s NOTICE %s :*** Notice -- TS for %s changed from %lu to 0",
+                           me.name, chptr->chname, chptr->chname, (unsigned long)oldts);
       sendto_realops_flags(UMODE_ALL, L_ALL,
                            "Server %s changing TS on %s from %lu to 0",
                            source_p->name, chptr->chname, (unsigned long)oldts);
@@ -430,11 +359,11 @@ ms_join(struct Client *client_p, struct Client *source_p,
     ;
   else if (newts < oldts)
   {
-    keep_our_modes = NO;
+    keep_our_modes = 0;
     chptr->channelts = newts;
   }
   else
-    keep_new_modes = NO;
+    keep_new_modes = 0;
 
   if (!keep_new_modes)
     mode = *oldmode;
@@ -454,7 +383,17 @@ ms_join(struct Client *client_p, struct Client *source_p,
   if (!keep_our_modes)
   {
     remove_our_modes(chptr, source_p);
-    sendto_channel_local(ALL_MEMBERS, NO, chptr,
+
+    if (chptr->topic[0])
+    {
+      set_channel_topic(chptr, "", "", 0, 0);
+      sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s TOPIC %s :",
+                           (IsHidden(source_p) ||
+                           ConfigServerHide.hide_servers) ?
+                           me.name : source_p->name, chptr->chname);
+    }
+
+    sendto_channel_local(ALL_MEMBERS, 0, chptr,
                          ":%s NOTICE %s :*** Notice -- TS for %s changed from %lu to %lu",
                           me.name, chptr->chname, chptr->chname,
                          (unsigned long)oldts, (unsigned long)newts);
@@ -467,22 +406,22 @@ ms_join(struct Client *client_p, struct Client *source_p,
 
     /* This _SHOULD_ be to ALL_MEMBERS
      * It contains only +imnpstlk, etc */
-    sendto_channel_local(ALL_MEMBERS, NO, chptr, ":%s MODE %s %s %s",
+    sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s MODE %s %s %s",
                          servername, chptr->chname, modebuf, parabuf);
   }
 
   if (!IsMember(source_p, chptr))
   {
-    add_user_to_channel(chptr, source_p, 0, YES);
-    sendto_channel_local(ALL_MEMBERS, NO, chptr, ":%s!%s@%s JOIN :%s",
+    add_user_to_channel(chptr, source_p, 0, 1);
+    sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s!%s@%s JOIN :%s",
                          source_p->name, source_p->username,
                          source_p->host, chptr->chname);
   }
 
-  sendto_server(client_p, NULL, chptr, CAP_TS6, NOCAPS, NOFLAGS,
+  sendto_server(client_p, CAP_TS6, NOCAPS,
                 ":%s JOIN %lu %s +",
                 ID(source_p), (unsigned long)chptr->channelts, chptr->chname);
-  sendto_server(client_p, NULL, chptr, NOCAPS, CAP_TS6, NOFLAGS,
+  sendto_server(client_p, NOCAPS, CAP_TS6,
                 ":%s SJOIN %lu %s + :%s",
                 source_p->servptr->name, (unsigned long)chptr->channelts,
                 chptr->chname, source_p->name);
@@ -503,18 +442,19 @@ do_join_0(struct Client *client_p, struct Client *source_p)
   struct Channel *chptr = NULL;
   dlink_node *ptr = NULL, *ptr_next = NULL;
 
-  if (source_p->channel.head && MyConnect(source_p) && !IsOper(source_p))
-    check_spambot_warning(source_p, NULL);
+  if (source_p->channel.head)
+    if (MyConnect(source_p) && !HasUMode(source_p, UMODE_OPER))
+      check_spambot_warning(source_p, NULL);
 
   DLINK_FOREACH_SAFE(ptr, ptr_next, source_p->channel.head)
   {
     chptr = ((struct Membership *)ptr->data)->chptr;
 
-    sendto_server(client_p, NULL, chptr, CAP_TS6, NOCAPS, NOFLAGS,
+    sendto_server(client_p, CAP_TS6, NOCAPS,
                   ":%s PART %s", ID(source_p), chptr->chname);
-    sendto_server(client_p, NULL, chptr, NOCAPS, CAP_TS6, NOFLAGS,
+    sendto_server(client_p, NOCAPS, CAP_TS6,
                   ":%s PART %s", source_p->name, chptr->chname);
-    sendto_channel_local(ALL_MEMBERS, NO, chptr, ":%s!%s@%s PART %s",
+    sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s!%s@%s PART %s",
                          source_p->name, source_p->username,
                          source_p->host, chptr->chname);
 
@@ -522,60 +462,49 @@ do_join_0(struct Client *client_p, struct Client *source_p)
   }
 }
 
-/* set_final_mode()
+/* set_final_mode
  *
- * inputs       - pointer to mode to setup
- *              - pointer to old mode
+ * inputs       - channel mode
+ *              - old channel mode
  * output       - NONE
- * side effects -
+ * side effects - walk through all the channel modes turning off modes
+ *                that were on in oldmode but aren't on in mode.
+ *                Then walk through turning on modes that are on in mode
+ *                but were not set in oldmode.
  */
-static const struct mode_letter
-{
-  unsigned int mode;
-  unsigned char letter;
-} flags[] = {
-  { MODE_NOPRIVMSGS, 'n' },
-  { MODE_TOPICLIMIT, 't' },
-  { MODE_SECRET,     's' },
-  { MODE_MODERATED,  'm' },
-  { MODE_INVITEONLY, 'i' },
-  { MODE_PRIVATE,    'p' },
-  { 0, '\0' }
-};
-
 static void
 set_final_mode(struct Mode *mode, struct Mode *oldmode)
 {
+  const struct mode_letter *tab;
   char *pbuf = parabuf;
   int what   = 0;
   int len;
-  int i;
 
-  for (i = 0; flags[i].letter; i++)
+  for (tab = chan_modes; tab->letter; ++tab)
   {
-    if ((flags[i].mode & mode->mode) &&
-        !(flags[i].mode & oldmode->mode))
+    if ((tab->mode & mode->mode) &&
+        !(tab->mode & oldmode->mode))
     {
       if (what != 1)
       {
         *mbuf++ = '+';
         what = 1;
       }
-      *mbuf++ = flags[i].letter;
+      *mbuf++ = tab->letter;
     }
   }
 
-  for (i = 0; flags[i].letter; i++)
+  for (tab = chan_modes; tab->letter; ++tab)
   {
-    if ((flags[i].mode & oldmode->mode) &&
-        !(flags[i].mode & mode->mode))
+    if ((tab->mode & oldmode->mode) &&
+        !(tab->mode & mode->mode))
     {
       if (what != -1)
       {
         *mbuf++ = '-';
         what = -1;
       }
-      *mbuf++ = flags[i].letter;
+      *mbuf++ = tab->letter;
     }
   }
 
@@ -695,7 +624,7 @@ remove_a_mode(struct Channel *chptr, struct Client *source_p,
       }
 
       *mbuf = '\0';
-      sendto_channel_local(ALL_MEMBERS, NO, chptr,
+      sendto_channel_local(ALL_MEMBERS, 0, chptr,
                            ":%s MODE %s %s%s",
                            (IsHidden(source_p) ||
                            ConfigServerHide.hide_servers) ?
@@ -719,7 +648,7 @@ remove_a_mode(struct Channel *chptr, struct Client *source_p,
       strlcat(sendbuf, " ", sizeof(sendbuf));
       strlcat(sendbuf, lpara[lcount], sizeof(sendbuf));
     }
-    sendto_channel_local(ALL_MEMBERS, NO, chptr,
+    sendto_channel_local(ALL_MEMBERS, 0, chptr,
                          ":%s MODE %s %s%s",
                          (IsHidden(source_p) || ConfigServerHide.hide_servers) ?
                          me.name : source_p->name,
@@ -727,3 +656,29 @@ remove_a_mode(struct Channel *chptr, struct Client *source_p,
   }
 }
 
+static struct Message join_msgtab = {
+  "JOIN", 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
+  { m_unregistered, m_join, ms_join, m_ignore, m_join, m_ignore }
+};
+
+static void
+module_init(void)
+{
+  mod_add_cmd(&join_msgtab);
+}
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&join_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1871 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = MODULE_FLAG_CORE
+};
