@@ -19,21 +19,18 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_kick.c 33 2005-10-02 20:50:00Z knight $
+ *  $Id: m_kick.c 1894 2013-04-27 14:53:18Z michael $
  */
 
 #include "stdinc.h"
-#include "tools.h"
-#include "handlers.h"
+#include "list.h"
 #include "channel.h"
 #include "channel_mode.h"
-#include "common.h"
 #include "client.h"
 #include "irc_string.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "send.h"
-#include "msg.h"
 #include "modules.h"
 #include "parse.h"
 #include "hash.h"
@@ -41,29 +38,6 @@
 #include "s_serv.h"
 
 
-static void m_kick(struct Client *, struct Client *, int, char *[]);
-
-struct Message kick_msgtab = {
-  "KICK", 0, 0, 3, 0, MFLG_SLOW, 0,
-  {m_unregistered, m_kick, m_kick, m_ignore, m_kick, m_ignore}
-};
-
-#ifndef STATIC_MODULES
-void
-_modinit(void)
-{
-  mod_add_cmd(&kick_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&kick_msgtab);
-}
-
-const char *_version = "$Revision: 33 $";
-#endif
-
 /* m_kick()
  *  parv[0] = sender prefix
  *  parv[1] = channel
@@ -96,7 +70,7 @@ m_kick(struct Client *client_p, struct Client *source_p,
     to = source_p->name;
   }
 
-  if (*parv[2] == '\0')
+  if (EmptyString(parv[2]))
   {
     sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS),
                from, to, "KICK");
@@ -106,7 +80,7 @@ m_kick(struct Client *client_p, struct Client *source_p,
   if (MyClient(source_p) && !IsFloodDone(source_p))
     flood_endgrace(source_p);
 
-  comment = (EmptyString(parv[3])) ? parv[2] : parv[3];
+  comment = (EmptyString(parv[3])) ? source_p->name : parv[3];
   if (strlen(comment) > (size_t)KICKLEN)
     comment[KICKLEN] = '\0';
 
@@ -126,7 +100,7 @@ m_kick(struct Client *client_p, struct Client *source_p,
     return;
   }
 
-  if (!IsServer(source_p))
+  if (!IsServer(source_p) && !HasFlag(source_p, FLAGS_SERVICE))
   {
     if ((ms = find_channel_link(source_p, chptr)) == NULL)
     {
@@ -218,17 +192,17 @@ m_kick(struct Client *client_p, struct Client *source_p,
      *   be sent anyways.  Just waiting for some oper to abuse it...
      */
     if (IsServer(source_p))
-      sendto_channel_local(ALL_MEMBERS, NO, chptr, ":%s KICK %s %s :%s",
+      sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s KICK %s %s :%s",
                            source_p->name, name, who->name, comment);
     else
-      sendto_channel_local(ALL_MEMBERS, NO, chptr, ":%s!%s@%s KICK %s %s :%s",
+      sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s!%s@%s KICK %s %s :%s",
                            source_p->name, source_p->username,
                            source_p->host, name, who->name, comment);
 
-    sendto_server(client_p, NULL, chptr, CAP_TS6, NOCAPS, NOFLAGS,
+    sendto_server(client_p, CAP_TS6, NOCAPS,
                   ":%s KICK %s %s :%s",
                   ID(source_p), chptr->chname, ID(who), comment);
-    sendto_server(client_p, NULL, chptr, NOCAPS, CAP_TS6, NOFLAGS,
+    sendto_server(client_p, NOCAPS, CAP_TS6,
                   ":%s KICK %s %s :%s", source_p->name, chptr->chname,
                   who->name, comment);
 
@@ -238,3 +212,30 @@ m_kick(struct Client *client_p, struct Client *source_p,
     sendto_one(source_p, form_str(ERR_USERNOTINCHANNEL),
                from, to, user, name);
 }
+
+static struct Message kick_msgtab = {
+  "KICK", 0, 0, 3, MAXPARA, MFLG_SLOW, 0,
+  {m_unregistered, m_kick, m_kick, m_ignore, m_kick, m_ignore}
+};
+
+static void
+module_init(void)
+{
+  mod_add_cmd(&kick_msgtab);
+}
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&kick_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1894 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = MODULE_FLAG_CORE
+};
