@@ -19,29 +19,23 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_message.c,v 1.107.2.2 2004/06/16 04:56:02 erik Exp $
+ *  $Id: m_message.c 1717 2012-12-27 18:52:00Z michael $
  */
 
 #include "stdinc.h"
-#include "handlers.h"
+#include "list.h"
 #include "client.h"
 #include "ircd.h"
 #include "numeric.h"
-#include "common.h"
-#include "s_conf.h"
+#include "conf.h"
 #include "s_serv.h"
 #include "send.h"
-#include "msg.h"
 #include "parse.h"
 #include "modules.h"
-
 #include "channel.h"
 #include "channel_mode.h"
-#include "vchannel.h"
 #include "irc_string.h"
 #include "hash.h"
-#include "class.h"
-#include "msg.h"
 #include "packet.h"
 
 
@@ -52,7 +46,7 @@ struct entity
   int flags;
 };
 
-static int build_target_list(int p_or_n, char *command,
+static int build_target_list(int p_or_n, const char *command,
                              struct Client *client_p,
                              struct Client *source_p,
                              char *nicks_channels, char *text);
@@ -60,7 +54,7 @@ static int build_target_list(int p_or_n, char *command,
 static int flood_attack_client(int p_or_n, struct Client *source_p,
                                struct Client *target_p);
 static int flood_attack_channel(int p_or_n, struct Client *source_p,
-                                struct Channel *chptr, char *chname);
+                                struct Channel *chptr);
 static struct Client* find_userhost (char *, char *, int *);
 
 #define ENTITY_NONE    0
@@ -73,59 +67,27 @@ static int ntargets = 0;
 
 static int duplicate_ptr(void *);
 
-static void m_message(int, char *, struct Client *,
+static void m_message(int, const char *, struct Client *,
                       struct Client *, int, char **);
 
-static void m_privmsg(struct Client *, struct Client *, int, char **);
-static void m_notice(struct Client *, struct Client *, int, char **);
-
-static void msg_channel(int p_or_n, char *command,
+static void msg_channel(int p_or_n, const char *command,
                         struct Client *client_p,
                         struct Client *source_p,
                         struct Channel *chptr, char *text);
 
-static void msg_channel_flags(int p_or_n, char *command,
+static void msg_channel_flags(int p_or_n, const char *command,
                               struct Client *client_p,
                               struct Client *source_p,
                               struct Channel *chptr, int flags, char *text);
 
-static void msg_client(int p_or_n, char *command,
+static void msg_client(int p_or_n, const char *command,
                        struct Client *source_p, struct Client *target_p,
                        char *text);
 
-static void handle_special(int p_or_n, char *command,
+static void handle_special(int p_or_n, const char *command,
 			   struct Client *client_p,
 			   struct Client *source_p, char *nick, char *text);
 
-struct Message privmsg_msgtab = {
-  "PRIVMSG", 0, 0, 1, 0, MFLG_SLOW | MFLG_UNREG, 0L,
-  {m_unregistered, m_privmsg, m_privmsg, m_ignore, m_privmsg}
-};
-
-struct Message notice_msgtab = {
-  "NOTICE", 0, 0, 1, 0, MFLG_SLOW, 0L,
-  {m_unregistered, m_notice, m_notice, m_ignore, m_notice}
-};
-
-#ifndef STATIC_MODULES
-
-void
-_modinit(void)
-{
-  mod_add_cmd(&privmsg_msgtab);
-  mod_add_cmd(&notice_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&privmsg_msgtab);
-  mod_del_cmd(&notice_msgtab);
-}
-
-const char *_version = "$Revision: 1.107.2.2 $";
-#endif
-
 /*
 ** m_privmsg
 **
@@ -149,21 +111,21 @@ const char *_version = "$Revision: 1.107.2.2 $";
 #define NOTICE  1
 
 static void
-m_privmsg(struct Client *client_p,
-          struct Client *source_p, int parc, char *parv[])
+m_privmsg(struct Client *client_p, struct Client *source_p,
+          int parc, char *parv[])
 {
   /* servers have no reason to send privmsgs, yet sometimes there is cause
    * for a notice.. (for example remote kline replies) --fl_
    */
-  if (!IsPerson(source_p))
+  if (!IsClient(source_p))
     return;
 
   m_message(PRIVMSG, "PRIVMSG", client_p, source_p, parc, parv);
 }
 
 static void
-m_notice(struct Client *client_p,
-         struct Client *source_p, int parc, char *parv[])
+m_notice(struct Client *client_p, struct Client *source_p,
+         int parc, char *parv[])
 {
   m_message(NOTICE, "NOTICE", client_p, source_p, parc, parv);
 }
@@ -176,46 +138,36 @@ m_notice(struct Client *client_p,
  *		- pointer to channel
  */
 static void
-m_message(int p_or_n,
-          char *command,
-          struct Client *client_p,
+m_message(int p_or_n, const char *command, struct Client *client_p,
           struct Client *source_p, int parc, char *parv[])
 {
   int i;
 
-  if (parc < 2 || *parv[1] == '\0')
+  if (parc < 2 || EmptyString(parv[1]))
   {
     if (p_or_n != NOTICE)
-      sendto_one(source_p, form_str(ERR_NORECIPIENT), me.name,
-                 source_p->name, command);
+      sendto_one(source_p, form_str(ERR_NORECIPIENT),
+                 ID_or_name(&me, client_p),
+                 ID_or_name(source_p, client_p), command);
     return;
   }
 
-  if (parc < 3 || *parv[2] == '\0')
+  if (parc < 3 || EmptyString(parv[2]))
   {
     if (p_or_n != NOTICE)
-      sendto_one(source_p, form_str(ERR_NOTEXTTOSEND), me.name,
-                 source_p->name);
+      sendto_one(source_p, form_str(ERR_NOTEXTTOSEND),
+                 ID_or_name(&me, client_p),
+                 ID_or_name(source_p, client_p));
     return;
   }
 
   /* Finish the flood grace period... */
-  if(MyClient(source_p) && !IsFloodDone(source_p) &&
-        irccmp(source_p->name, parv[1]) != 0) /* some dumb clients msg/notice themself
-                                                 to determine lag to the server BEFORE
-                                                 sending JOIN commands, and then flood
-                                                 off because they left gracemode. -wiz */
+  if (MyClient(source_p) && !IsFloodDone(source_p))
     flood_endgrace(source_p);
 
   if (build_target_list(p_or_n, command, client_p, source_p, parv[1],
                         parv[2]) < 0)
-  {
-    /* Sigh.  We need to relay this command to the hub */
-    if (!ServerInfo.hub && (uplink != NULL))
-      sendto_one(uplink, ":%s %s %s :%s",
-		 source_p->name, command, parv[1], parv[2]);
     return;
-  }
 
   for (i = 0; i < ntargets; i++)
   {
@@ -240,8 +192,7 @@ m_message(int p_or_n,
   }
 }
 
-/*
- * build_target_list
+/* build_target_list()
  *
  * inputs	- pointer to given client_p (server)
  *		- pointer to given source (oper/client etc.)
@@ -257,30 +208,21 @@ m_message(int p_or_n,
  *		  to disambiguate.
  *
  */
-
 static int
-build_target_list(int p_or_n, char *command, struct Client *client_p,
-                  struct Client *source_p, char *nicks_channels,
-                  char *text)
+build_target_list(int p_or_n, const char *command, struct Client *client_p,
+                  struct Client *source_p, char *nicks_channels, char *text)
 {
   int type;
-  char *p, *nick, *target_list, ncbuf[BUFSIZE];
-  struct Channel *chptr=NULL;
-  struct Client *target_p;
+  char *p = NULL, *nick, *target_list;
+  struct Channel *chptr = NULL;
+  struct Client *target_p = NULL;
 
-  /* Sigh, we can't mutilate parv[1] incase we need it to send to a hub */
-  if (!ServerInfo.hub && (uplink != NULL) && IsCapable(uplink, CAP_LL))
-  {
-    strncpy(ncbuf, nicks_channels, BUFSIZE);
-    target_list = ncbuf;
-  }
-  else
-    target_list = nicks_channels; /* skip strcpy for non-lazyleafs */
+  target_list = nicks_channels;
 
   ntargets = 0;
 
   for (nick = strtoken(&p, target_list, ","); nick;
-       nick = strtoken(&p, (char *)NULL, ","))
+       nick = strtoken(&p, NULL, ","))
   {
     char *with_prefix;
     /*
@@ -290,44 +232,43 @@ build_target_list(int p_or_n, char *command, struct Client *client_p,
 
     if (IsChanPrefix(*nick))
     {
-      /* ignore send of local channel to a server (should not happen) */
-      if (IsServer(client_p) && *nick == '&')
-        continue;
-
       if ((chptr = hash_find_channel(nick)) != NULL)
       {
         if (!duplicate_ptr(chptr))
         {
           if (ntargets >= ConfigFileEntry.max_targets)
-	    {
-	      sendto_one(source_p, form_str(ERR_TOOMANYTARGETS),
-			 me.name, source_p->name, nick);
-	      return (1);
-	    }
+          {
+            sendto_one(source_p, form_str(ERR_TOOMANYTARGETS),
+                       ID_or_name(&me, client_p),
+                       ID_or_name(source_p, client_p), nick,
+		       ConfigFileEntry.max_targets);
+            return (1);
+          }
           targets[ntargets].ptr = (void *)chptr;
           targets[ntargets++].type = ENTITY_CHANNEL;
         }
       }
       else
       {
-        if (!ServerInfo.hub && (uplink != NULL) && IsCapable(uplink, CAP_LL))
-          return -1;
-        else if (p_or_n != NOTICE)
-          sendto_one(source_p, form_str(ERR_NOSUCHNICK), me.name,
-                     source_p->name, nick);
+        if (p_or_n != NOTICE)
+          sendto_one(source_p, form_str(ERR_NOSUCHNICK),
+                     ID_or_name(&me, client_p),
+                     ID_or_name(source_p, client_p), nick);
       }
       continue;
     }
 
     /* look for a privmsg to another client */
-    if ((target_p = find_person(nick)) != NULL)
+    if ((target_p = find_person(client_p, nick)) != NULL)
     {
       if (!duplicate_ptr(target_p))
       {
         if (ntargets >= ConfigFileEntry.max_targets)
 	  {
 	    sendto_one(source_p, form_str(ERR_TOOMANYTARGETS),
-		       me.name, source_p->name, nick);
+                       ID_or_name(&me, client_p),
+                       ID_or_name(source_p, client_p), nick,
+		       ConfigFileEntry.max_targets);
 	    return (1);
 	  }
         targets[ntargets].ptr = (void *)target_p;
@@ -342,14 +283,16 @@ build_target_list(int p_or_n, char *command, struct Client *client_p,
     type = 0;
     with_prefix = nick;
     /*  allow %+@ if someone wants to do that */
-    for (;;)
+    for (; ;)
     {
       if (*nick == '@')
-        type |= MODE_CHANOP;
+        type |= CHFL_CHANOP;
+#ifdef HALFOPS
       else if (*nick == '%')
-        type |= MODE_CHANOP | MODE_HALFOP;
+        type |= CHFL_CHANOP | CHFL_HALFOP;
+#endif
       else if (*nick == '+')
-        type |= MODE_CHANOP | MODE_HALFOP | MODE_VOICE;
+        type |= CHFL_CHANOP | CHFL_HALFOP | CHFL_VOICE;
       else
         break;
       nick++;
@@ -359,68 +302,77 @@ build_target_list(int p_or_n, char *command, struct Client *client_p,
     {
       /* suggested by Mortiis */
       if (*nick == '\0')      /* if its a '\0' dump it, there is no recipient */
-	{
-	  sendto_one(source_p, form_str(ERR_NORECIPIENT),
-		     me.name, source_p->name, command);
-	  continue;
-	}
+      {
+        sendto_one(source_p, form_str(ERR_NORECIPIENT),
+                   ID_or_name(&me, client_p),
+                   ID_or_name(source_p, client_p), command);
+        continue;
+      }
 
       /* At this point, nick+1 should be a channel name i.e. #foo or &foo
        * if the channel is found, fine, if not report an error
        */
 
       if ((chptr = hash_find_channel(nick)) != NULL)
-	{
-	  if(!is_any_op(chptr, source_p) && !is_voiced(chptr, source_p))
-	    {
-	      sendto_one(source_p, form_str(ERR_CHANOPRIVSNEEDED), me.name,
-			 source_p->name, with_prefix);
-	      return(-1);
-	    }
+      {
+        if (IsClient(source_p) && !HasFlag(source_p, FLAGS_SERVICE))
+        {
+          if (!has_member_flags(find_channel_link(source_p, chptr),
+                                CHFL_CHANOP|CHFL_HALFOP|CHFL_VOICE))
+          {
+            sendto_one(source_p, form_str(ERR_CHANOPRIVSNEEDED),
+                       ID_or_name(&me, client_p),
+                       ID_or_name(source_p, client_p), with_prefix);
+            return(-1);
+          }
+        }
 
-	  if (!duplicate_ptr(chptr))
-	    {
-	      if (ntargets >= ConfigFileEntry.max_targets)
-		{
-		  sendto_one(source_p, form_str(ERR_TOOMANYTARGETS),
-			     me.name, source_p->name, nick);
-		  return (1);
-		}
-	      targets[ntargets].ptr = (void *)chptr;
-	      targets[ntargets].type = ENTITY_CHANOPS_ON_CHANNEL;
-	      targets[ntargets++].flags = type;
-	    }
-	}
+        if (!duplicate_ptr(chptr))
+        {
+          if (ntargets >= ConfigFileEntry.max_targets)
+          {
+	    sendto_one(source_p, form_str(ERR_TOOMANYTARGETS),
+                       ID_or_name(&me, client_p),
+                       ID_or_name(source_p, client_p), nick,
+		       ConfigFileEntry.max_targets);
+            return(1);
+          }
+          targets[ntargets].ptr = (void *)chptr;
+          targets[ntargets].type = ENTITY_CHANOPS_ON_CHANNEL;
+          targets[ntargets++].flags = type;
+        }
+      }
       else
-	{
-	  if (!ServerInfo.hub && (uplink != NULL) && IsCapable(uplink, CAP_LL))
-	    return -1;
-	  else if (p_or_n != NOTICE)
-	    sendto_one(source_p, form_str(ERR_NOSUCHNICK), me.name,
-		       source_p->name, nick);
-	}
+      {
+        if (p_or_n != NOTICE)
+          sendto_one(source_p, form_str(ERR_NOSUCHNICK),
+                     ID_or_name(&me, client_p),
+                     ID_or_name(source_p, client_p), nick);
+      }
       continue;
     }
 
-    if(((*nick == '$') || strchr(nick, '@')))
+    if ((*nick == '$') || strchr(nick, '@') != NULL)
     {
       handle_special(p_or_n, command, client_p, source_p, nick, text);
     }
     else
     {
-      if (!ServerInfo.hub && (uplink != NULL) && IsCapable(uplink, CAP_LL))
-        return -1;
-      else if(p_or_n != NOTICE)
-        sendto_one(source_p, form_str(ERR_NOSUCHNICK),
-	           me.name, source_p->name, nick);
+      if (p_or_n != NOTICE)
+      {
+        if (!IsDigit(*nick) || MyClient(source_p))
+	  sendto_one(source_p, form_str(ERR_NOSUCHNICK),
+		     ID_or_name(&me, client_p),
+		     ID_or_name(source_p, client_p), nick);
+      }
     }
     /* continue; */
   }
-  return (1);
+
+  return(1);
 }
 
-/*
- * duplicate_ptr
+/* duplicate_ptr()
  *
  * inputs	- pointer to check
  *		- pointer to table of entities
@@ -433,14 +385,17 @@ static int
 duplicate_ptr(void *ptr)
 {
   int i;
+
   for (i = 0; i < ntargets; i++)
+  {
     if (targets[i].ptr == ptr)
-      return YES;
-  return NO;
+      return(1);
+  }
+
+  return(0);
 }
 
-/*
- * msg_channel
+/* msg_channel()
  *
  * inputs	- flag privmsg or notice
  * 		- pointer to command "PRIVMSG" or "NOTICE"
@@ -451,50 +406,36 @@ duplicate_ptr(void *ptr)
  * side effects	- message given channel
  */
 static void
-msg_channel(int p_or_n, char *command,
-            struct Client *client_p,
+msg_channel(int p_or_n, const char *command, struct Client *client_p,
             struct Client *source_p, struct Channel *chptr, char *text)
 {
-  struct Channel *vchan = NULL;
-  char *chname = NULL;
   int result;
 
-  chname = RootChan(chptr)->chname;
-
-#ifdef VCHANS
-  if (HasVchans(chptr))
-    vchan = map_vchan(chptr, source_p);
-#endif
-  
-  if (!vchan)
-    vchan = chptr;
-
   if (MyClient(source_p))
   {
     /* idle time shouldnt be reset by notices --fl */
-    if ((p_or_n != NOTICE) && source_p->user)
-      source_p->user->last = CurrentTime;
+    if (p_or_n != NOTICE)
+      source_p->localClient->last_privmsg = CurrentTime;
   }
 
   /* chanops and voiced can flood their own channel with impunity */
-  if ((result = can_send(vchan, source_p)))
+  if ((result = can_send(chptr, source_p, NULL)) < 0)
   {
     if (result == CAN_SEND_OPV ||
-        !flood_attack_channel(p_or_n, source_p, vchan, chname))
-    {
-      sendto_channel_butone(client_p, source_p, vchan, command, ":%s", text);
-    }
+        !flood_attack_channel(p_or_n, source_p, chptr))
+      sendto_channel_butone(client_p, source_p, chptr, 0, "%s %s :%s",
+                            command, chptr->chname, text);
   }
   else
   {
     if (p_or_n != NOTICE)
       sendto_one(source_p, form_str(ERR_CANNOTSENDTOCHAN),
-                 me.name, source_p->name, chname);
+                 ID_or_name(&me, client_p),
+                 ID_or_name(source_p, client_p), chptr->chname);
   }
 }
 
-/*
- * msg_channel_flags
+/* msg_channel_flags()
  *
  * inputs	- flag 0 if PRIVMSG 1 if NOTICE. RFC 
  *		  say NOTICE must not auto reply
@@ -508,77 +449,39 @@ msg_channel(int p_or_n, char *command,
  * side effects	- message given channel either chanop or voice
  */
 static void
-msg_channel_flags(int p_or_n, char *command, struct Client *client_p,
+msg_channel_flags(int p_or_n, const char *command, struct Client *client_p,
                   struct Client *source_p, struct Channel *chptr,
                   int flags, char *text)
 {
-  struct Channel *vchan = NULL;
-  char *chname = NULL;
-  int type;
+  unsigned int type;
   char c;
 
-  if (flags & MODE_VOICE)
+  if (flags & CHFL_VOICE)
   {
-    type = ONLY_CHANOPS_HALFOPS_VOICED;
+    type = CHFL_VOICE|CHFL_HALFOP|CHFL_CHANOP;
     c = '+';
   }
-  else if (flags & MODE_HALFOP)
+#ifdef HALFOPS
+  else if (flags & CHFL_HALFOP)
   {
-    type = ONLY_CHANOPS_HALFOPS;
+    type = CHFL_HALFOP|CHFL_CHANOP;
     c = '%';
-  }
+   }
+#endif
   else
   {
-    type = ONLY_CHANOPS;
+    type = CHFL_CHANOP;
     c = '@';
   }
 
-  chname = RootChan(chptr)->chname;
+  if (MyClient(source_p) && p_or_n != NOTICE)
+    source_p->localClient->last_privmsg = CurrentTime;
 
-#ifdef VCHANS
-  if (HasVchans(chptr))
-    vchan = map_vchan(chptr, source_p);
-#endif
-
-  if (!vchan)
-    vchan = chptr;
-
-  if (MyClient(source_p))
-  {
-    /* idletime shouldnt be reset by notice --fl */
-    if ((p_or_n != NOTICE) && source_p->user)
-      source_p->user->last = CurrentTime;
-
-    sendto_channel_local_butone(source_p, type, vchan, ":%s!%s@%s %s %c%s :%s",
-                                source_p->name, source_p->username,
-                                source_p->host, command, c, chname, text);
-  }
-  else
-  {
-    /*
-     * another good catch, lee.  we never would echo to remote clients anyway,
-     * so use slightly less intensive sendto_channel_local()
-     */
-    sendto_channel_local(type, vchan, ":%s!%s@%s %s %c%s :%s",
-                         source_p->name, source_p->username,
-                         source_p->host, command, c, chname, text);
-  }
-
-  if (chptr->chname[0] == '&')
-    return;
-
-  sendto_channel_remote(source_p, client_p, type, CAP_CHW, CAP_UID, vchan,
-                ":%s %s %c%s :%s", source_p->name, command, c,
-                vchan->chname, text);
-  sendto_channel_remote(source_p, client_p, type, CAP_CHW|CAP_UID, NOCAPS, vchan,
-                ":%s %s %c%s :%s", ID(source_p), command, c,
-                vchan->chname, text);
-
-  /* non CAP_CHW servers? */
+  sendto_channel_butone(client_p, source_p, chptr, type, "%s %c%s :%s",
+                        command, c, chptr->chname, text);
 }
 
-/*
- * msg_client
+/* msg_client()
  *
  * inputs	- flag 0 if PRIVMSG 1 if NOTICE. RFC 
  *		  say NOTICE must not auto reply
@@ -590,52 +493,62 @@ msg_channel_flags(int p_or_n, char *command, struct Client *client_p,
  * side effects	- message given channel either chanop or voice
  */
 static void
-msg_client(int p_or_n, char *command,
-           struct Client *source_p, struct Client *target_p, char *text)
+msg_client(int p_or_n, const char *command, struct Client *source_p,
+           struct Client *target_p, char *text)
 {
-  if (MyClient(source_p))
+  if (MyConnect(source_p))
   {
-    /* reset idle time for message only if its not to self 
-     * and its not a notice */
-    if ((p_or_n != NOTICE) && (source_p != target_p) && source_p->user)
-      source_p->user->last = CurrentTime;
-  }
+    /*
+     * reset idle time for message only if it's not a notice
+     */
+    if ((p_or_n != NOTICE))
+      source_p->localClient->last_privmsg = CurrentTime;
 
-  if (MyConnect(source_p) && (p_or_n != NOTICE) &&
-      target_p->user && target_p->user->away)
-    sendto_one(source_p, form_str(RPL_AWAY), me.name,
-               source_p->name, target_p->name, target_p->user->away);
+    if ((p_or_n != NOTICE) && target_p->away[0])
+      sendto_one(source_p, form_str(RPL_AWAY), me.name,
+                 source_p->name, target_p->name, target_p->away);
+
+    if (HasUMode(target_p, UMODE_REGONLY) && target_p != source_p)
+    {
+      if (!HasUMode(source_p, UMODE_REGISTERED|UMODE_OPER))
+      {
+        if (p_or_n != NOTICE)
+          sendto_one(source_p, form_str(ERR_NONONREG), me.name, source_p->name,
+                     target_p->name);
+        return;
+      }
+    }
+  }
 
   if (MyClient(target_p))
   {
-    if (!IsServer(source_p) && IsSetCallerId(target_p))
+    if (!IsServer(source_p) && HasUMode(target_p, UMODE_CALLERID|UMODE_SOFTCALLERID))
     {
       /* Here is the anti-flood bot/spambot code -db */
-      if (accept_message(source_p, target_p))
+      if (accept_message(source_p, target_p) || HasFlag(source_p, FLAGS_SERVICE) ||
+         (HasUMode(source_p, UMODE_OPER) && (ConfigFileEntry.opers_bypass_callerid == 1)))
       {
         sendto_one(target_p, ":%s!%s@%s %s %s :%s",
-                   source_p->name,
-                   source_p->username,
+                   source_p->name, source_p->username,
                    source_p->host, command, target_p->name, text);
       }
       else
       {
         /* check for accept, flag recipient incoming message */
         if (p_or_n != NOTICE)
-          sendto_anywhere(source_p, target_p,
-                          "NOTICE %s :*** I'm in +g mode (server side ignore).",
-                          source_p->name);
+	  sendto_one(source_p, form_str(RPL_TARGUMODEG),
+		     ID_or_name(&me, source_p->from),
+		     ID_or_name(source_p, source_p->from), target_p->name);
 
         if ((target_p->localClient->last_caller_id_time +
              ConfigFileEntry.caller_id_wait) < CurrentTime)
         {
           if (p_or_n != NOTICE)
-            sendto_anywhere(source_p, target_p,
-                            "NOTICE %s :*** I've been informed you messaged me.",
-                            source_p->name);
+	    sendto_one(source_p, form_str(RPL_TARGNOTIFY),
+		       ID_or_name(&me, source_p->from),
+		       ID_or_name(source_p, source_p->from), target_p->name);
 
-          sendto_one(target_p,
-                     ":%s NOTICE %s :*** Client %s is messaging you and you are +g",
+          sendto_one(target_p, form_str(RPL_UMODEGMSG),
                      me.name, target_p->name,
                      get_client_name(source_p, HIDE_IP));
 
@@ -643,7 +556,7 @@ msg_client(int p_or_n, char *command,
 
         }
         /* Only so opers can watch for floods */
-        (void)flood_attack_client(p_or_n, source_p, target_p);
+        flood_attack_client(p_or_n, source_p, target_p);
       }
     }
     else
@@ -653,7 +566,7 @@ msg_client(int p_or_n, char *command,
        * we dont give warnings.. we then check if theyre opered 
        * (to avoid flood warnings), lastly if theyre our client
        * and flooding    -- fl */
-      if (!MyClient(source_p) || IsOper(source_p) ||
+      if (!MyClient(source_p) || HasUMode(source_p, UMODE_OPER) ||
           (MyClient(source_p) &&
            !flood_attack_client(p_or_n, source_p, target_p)))
         sendto_anywhere(target_p, source_p, "%s %s :%s",
@@ -662,16 +575,15 @@ msg_client(int p_or_n, char *command,
   }
   else
     /* The target is a remote user.. same things apply  -- fl */
-  if (!MyClient(source_p) || IsOper(source_p) ||
+  if (!MyClient(source_p) || HasUMode(source_p, UMODE_OPER) ||
         (MyClient(source_p)
            && !flood_attack_client(p_or_n, source_p, target_p)))
     sendto_anywhere(target_p, source_p, "%s %s :%s", command, target_p->name,
                     text);
-  return;
 }
 
-/*
- * flood_attack_client
+/* flood_attack_client()
+ *
  * inputs       - flag 0 if PRIVMSG 1 if NOTICE. RFC
  *                say NOTICE must not auto reply
  *              - pointer to source Client 
@@ -686,7 +598,7 @@ flood_attack_client(int p_or_n, struct Client *source_p,
   int delta;
 
   if (GlobalSetOptions.floodcount && MyConnect(target_p)
-      && IsClient(source_p) && !IsConfCanFlood(source_p))
+      && IsClient(source_p) && !IsCanFlood(source_p))
   {
     if ((target_p->localClient->first_received_message_time + 1)
         < CurrentTime)
@@ -695,41 +607,43 @@ flood_attack_client(int p_or_n, struct Client *source_p,
         CurrentTime - target_p->localClient->first_received_message_time;
       target_p->localClient->received_number_of_privmsgs -= delta;
       target_p->localClient->first_received_message_time = CurrentTime;
+
       if (target_p->localClient->received_number_of_privmsgs <= 0)
       {
         target_p->localClient->received_number_of_privmsgs = 0;
-        target_p->localClient->flood_noticed = 0;
+        DelFlag(target_p, FLAGS_FLOOD_NOTICED);
       }
     }
 
     if ((target_p->localClient->received_number_of_privmsgs >=
-         GlobalSetOptions.floodcount) || target_p->localClient->flood_noticed)
+         GlobalSetOptions.floodcount) || HasFlag(target_p, FLAGS_FLOOD_NOTICED))
     {
-      if (target_p->localClient->flood_noticed == 0)
+      if (!HasFlag(target_p, FLAGS_FLOOD_NOTICED))
       {
-        sendto_realops_flags(FLAGS_BOTS, L_ALL,
+        sendto_realops_flags(UMODE_BOTS, L_ALL,
                              "Possible Flooder %s on %s target: %s",
                              get_client_name(source_p, HIDE_IP),
-                             source_p->user->server, target_p->name);
-        target_p->localClient->flood_noticed = 1;
+                             source_p->servptr->name, target_p->name);
+        AddFlag(target_p, FLAGS_FLOOD_NOTICED);
         /* add a bit of penalty */
         target_p->localClient->received_number_of_privmsgs += 2;
       }
+
       if (MyClient(source_p) && (p_or_n != NOTICE))
         sendto_one(source_p,
                    ":%s NOTICE %s :*** Message to %s throttled due to flooding",
                    me.name, source_p->name, target_p->name);
-      return 1;
+      return(1);
     }
     else
       target_p->localClient->received_number_of_privmsgs++;
   }
 
-  return 0;
+  return(0);
 }
 
-/*
- * flood_attack_channel
+/* flood_attack_channel()
+ *
  * inputs       - flag 0 if PRIVMSG 1 if NOTICE. RFC
  *                says NOTICE must not auto reply
  *              - pointer to source Client 
@@ -739,11 +653,11 @@ flood_attack_client(int p_or_n, struct Client *source_p,
  */
 static int
 flood_attack_channel(int p_or_n, struct Client *source_p,
-                     struct Channel *chptr, char *chname)
+                     struct Channel *chptr)
 {
   int delta;
 
-  if (GlobalSetOptions.floodcount && !IsConfCanFlood(source_p))
+  if (GlobalSetOptions.floodcount && !IsCanFlood(source_p))
   {
     if ((chptr->first_received_message_time + 1) < CurrentTime)
     {
@@ -753,20 +667,20 @@ flood_attack_channel(int p_or_n, struct Client *source_p,
       if (chptr->received_number_of_privmsgs <= 0)
       {
         chptr->received_number_of_privmsgs = 0;
-        chptr->flood_noticed = 0;
+        ClearFloodNoticed(chptr);
       }
     }
 
     if ((chptr->received_number_of_privmsgs >= GlobalSetOptions.floodcount)
-        || chptr->flood_noticed)
+        || IsSetFloodNoticed(chptr))
     {
-      if (chptr->flood_noticed == 0)
+      if (!IsSetFloodNoticed(chptr))
       {
-        sendto_realops_flags(FLAGS_BOTS, L_ALL,
+        sendto_realops_flags(UMODE_BOTS, L_ALL,
                              "Possible Flooder %s on %s target: %s",
                              get_client_name(source_p, HIDE_IP),
-                             source_p->user->server, chptr->chname);
-        chptr->flood_noticed = 1;
+                             source_p->servptr->name, chptr->chname);
+        SetFloodNoticed(chptr);
 
         /* Add a bit of penalty */
         chptr->received_number_of_privmsgs += 2;
@@ -774,19 +688,17 @@ flood_attack_channel(int p_or_n, struct Client *source_p,
       if (MyClient(source_p) && (p_or_n != NOTICE))
         sendto_one(source_p,
                    ":%s NOTICE %s :*** Message to %s throttled due to flooding",
-                   me.name, source_p->name, chname);
-      return 1;
+                   me.name, source_p->name, chptr->chname);
+      return(1);
     }
     else
       chptr->received_number_of_privmsgs++;
   }
 
-  return 0;
+  return(0);
 }
 
-
-/*
- * handle_special
+/* handle_special()
  *
  * inputs	- server pointer
  *		- client pointer
@@ -808,7 +720,7 @@ flood_attack_channel(int p_or_n, struct Client *source_p,
  *		- Dianora
  */
 static void
-handle_special(int p_or_n, char *command, struct Client *client_p,
+handle_special(int p_or_n, const char *command, struct Client *client_p,
 	       struct Client *source_p, char *nick, char *text)
 {
   struct Client *target_p;
@@ -824,24 +736,26 @@ handle_special(int p_or_n, char *command, struct Client *client_p,
   {
     count = 0;
 
-    if ((host = strchr(nick, '%')) && !IsOper(source_p))
+    if ((host = strchr(nick, '%')) && !HasUMode(source_p, UMODE_OPER))
     {
       sendto_one(source_p, form_str(ERR_NOPRIVILEGES),
-		 me.name, source_p->name);
+                 ID_or_name(&me, client_p),
+                 ID_or_name(source_p, client_p));
       return;
     }
-    if ((target_p = find_server(server + 1)) != NULL)
+
+    if ((target_p = hash_find_server(server + 1)) != NULL)
     {
       if (!IsMe(target_p))
       {
 	/*
 	 * Not destined for a user on me :-(
 	 */
-
-	sendto_one(target_p, ":%s %s %s :%s", source_p->name,
+	sendto_one(target_p, ":%s %s %s :%s",
+                   ID_or_name(source_p, target_p->from),
 		   command, nick, text);
-	if ((p_or_n != NOTICE) && source_p->user)
-	  source_p->user->last = CurrentTime;
+	if ((p_or_n != NOTICE) && MyClient(source_p))
+	  source_p->localClient->last_privmsg = CurrentTime;
 	return;
       }
 
@@ -850,18 +764,6 @@ handle_special(int p_or_n, char *command, struct Client *client_p,
       if (host != NULL)
 	*host++ = '\0';
 
-      /* Check if someones msg'ing opers@our.server */
-      if (strcmp(nick, "opers") == 0)
-      {
-	if (!IsOper(source_p))
-	  sendto_one(source_p, form_str(ERR_NOPRIVILEGES),
-		     me.name, source_p->name);
-	else
-	  sendto_realops_flags(FLAGS_ALL, L_ALL, "To opers: From: %s: %s",
-			       source_p->name, text);
-	return;
-      }
-
       /*
        * Look for users which match the destination host
        * (no host == wildcard) and if one and one only is
@@ -878,76 +780,84 @@ handle_special(int p_or_n, char *command, struct Client *client_p,
 
 	if (count == 1)
 	{
-	  sendto_one(target_p, ":%s %s %s :%s",
-		     source_p->name, command, nick, text);
-	  if ((p_or_n != NOTICE) && source_p->user)
-	    source_p->user->last = CurrentTime;
+	  sendto_one(target_p, ":%s!%s@%s %s %s :%s",
+		     source_p->name, source_p->username, source_p->host,
+                     command, nick, text);
+	  if ((p_or_n != NOTICE) && MyClient(source_p))
+	    source_p->localClient->last_privmsg = CurrentTime;
 	}
 	else
 	  sendto_one(source_p, form_str(ERR_TOOMANYTARGETS),
-		     me.name, source_p->name, nick);
+                     ID_or_name(&me, client_p),
+                     ID_or_name(source_p, client_p), nick,
+		     ConfigFileEntry.max_targets);
       }
     }
     else if (server && *(server+1) && (target_p == NULL))
-	sendto_one(source_p, form_str(ERR_NOSUCHSERVER), me.name,
-		   source_p->name, server+1);
+	sendto_one(source_p, form_str(ERR_NOSUCHSERVER),
+                   ID_or_name(&me, client_p),
+                   ID_or_name(source_p, client_p), server+1);
     else if (server && (target_p == NULL))
-	sendto_one(source_p, form_str(ERR_NOSUCHNICK), me.name,
-		   source_p->name, nick);
+	sendto_one(source_p, form_str(ERR_NOSUCHNICK),
+                   ID_or_name(&me, client_p),
+                   ID_or_name(source_p, client_p), nick);
     return;
   }
 
-  if (!IsOper(source_p))
+  if (!HasUMode(source_p, UMODE_OPER))
   {
     sendto_one(source_p, form_str(ERR_NOPRIVILEGES),
-	       me.name, source_p->name);
+               ID_or_name(&me, client_p),
+               ID_or_name(source_p, client_p));
     return;
   }
+
   /*
    * the following two cases allow masks in NOTICEs
    * (for OPERs only)
    *
    * Armin, 8Jun90 (gruner@informatik.tu-muenchen.de)
    */
-  if(*nick == '$')
+  if (*nick == '$')
   {
-    if((*(nick+1) == '$' || *(nick+1) == '#'))
+    if ((*(nick+1) == '$' || *(nick+1) == '#'))
       nick++;
-    else if(MyOper(source_p))
+    else if (MyClient(source_p) && HasUMode(source_p, UMODE_OPER))
     {
       sendto_one(source_p, 
                  ":%s NOTICE %s :The command %s %s is no longer supported, please use $%s",
 		 me.name, source_p->name, command, nick, nick);
       return;
     }
-      
+
     if ((s = strrchr(nick, '.')) == NULL)
     {
       sendto_one(source_p, form_str(ERR_NOTOPLEVEL),
                  me.name, source_p->name, nick);
       return;
     }
+
     while (*++s)
       if (*s == '.' || *s == '*' || *s == '?')
         break;
+
     if (*s == '*' || *s == '?')
     {
       sendto_one(source_p, form_str(ERR_WILDTOPLEVEL),
-                 me.name, source_p->name, nick);
+                 ID_or_name(&me, client_p),
+                 ID_or_name(source_p, client_p), nick);
       return;
     }
     
     sendto_match_butone(IsServer(client_p) ? client_p : NULL, source_p,
-                        nick + 1,
-                        (*nick == '#') ? MATCH_HOST : MATCH_SERVER,
+                        nick + 1, (*nick == '#') ? MATCH_HOST : MATCH_SERVER,
                         "%s $%s :%s", command, nick, text);
 
-    if ((p_or_n != NOTICE) && source_p->user)
-      source_p->user->last = CurrentTime;
+    if ((p_or_n != NOTICE) && MyClient(source_p))
+      source_p->localClient->last_privmsg = CurrentTime;
 
     return;
   }
-
 }
 
 /*
@@ -965,21 +875,61 @@ find_userhost(char *user, char *host, int *count)
 {
   struct Client *c2ptr;
   struct Client *res = NULL;
+  dlink_node *lc2ptr;
 
   *count = 0;
+
   if (collapse(user) != NULL)
+  {
+    DLINK_FOREACH(lc2ptr, local_client_list.head)
     {
-      for (c2ptr = GlobalClientList; c2ptr; c2ptr = c2ptr->next) 
-	{
-	  if (!MyClient(c2ptr)) /* implies mine and an user */
-	    continue;
-	  if ((!host || match(host, c2ptr->host)) &&
-	      irccmp(user, c2ptr->username) == 0)
-	    {
-	      (*count)++;
-	      res = c2ptr;
-	    }
-	}
+      c2ptr = lc2ptr->data;
+
+      if (!IsClient(c2ptr)) /* something other than a client */
+        continue;
+
+      if ((!host || match(host, c2ptr->host)) &&
+          irccmp(user, c2ptr->username) == 0)
+      {
+        (*count)++;
+        res = c2ptr;
+      }
     }
-  return (res);
+  }
+
+  return(res);
 }
+
+static struct Message privmsg_msgtab = {
+  "PRIVMSG", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  {m_unregistered, m_privmsg, m_privmsg, m_ignore, m_privmsg, m_ignore}
+};
+
+static struct Message notice_msgtab = {
+  "NOTICE", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  {m_unregistered, m_notice, m_notice, m_ignore, m_notice, m_ignore}
+};
+
+static void
+module_init(void)
+{
+  mod_add_cmd(&privmsg_msgtab);
+  mod_add_cmd(&notice_msgtab);
+}
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&privmsg_msgtab);
+  mod_del_cmd(&notice_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1717 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = MODULE_FLAG_CORE
+};
