@@ -19,52 +19,25 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_part.c 454 2006-02-12 19:01:25Z michael $
+ *  $Id: m_part.c 1474 2012-07-22 14:44:07Z michael $
  */
 
 #include "stdinc.h"
-#include "tools.h"
-#include "handlers.h"
+#include "list.h"
 #include "channel.h"
 #include "channel_mode.h"
 #include "client.h"
-#include "common.h"  
 #include "hash.h"
 #include "irc_string.h"
 #include "ircd.h"
-#include "list.h"
 #include "numeric.h"
 #include "send.h"
 #include "s_serv.h"
-#include "msg.h"
 #include "parse.h"
 #include "modules.h"
-#include "s_conf.h"
+#include "conf.h"
 #include "packet.h"
 
-static void m_part(struct Client *, struct Client *, int, char *[]);
-
-struct Message part_msgtab = {
-  "PART", 0, 0, 2, 0, MFLG_SLOW, 0,
-  { m_unregistered, m_part, m_part, m_ignore, m_part, m_ignore }
-};
-
-#ifndef STATIC_MODULES
-void
-_modinit(void)
-{
-  mod_add_cmd(&part_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&part_msgtab);
-}
-
-const char *_version = "$Revision: 454 $";
-#endif
-
 
 /* part_one_client()
  *
@@ -76,7 +49,7 @@ const char *_version = "$Revision: 454 $";
  */
 static void
 part_one_client(struct Client *client_p, struct Client *source_p,
-                char *name, char *reason)
+                const char *name, const char *reason)
 {
   struct Channel *chptr = NULL;
   struct Membership *ms = NULL;
@@ -95,7 +68,7 @@ part_one_client(struct Client *client_p, struct Client *source_p,
     return;
   }
 
-  if (MyConnect(source_p) && !IsOper(source_p))
+  if (MyConnect(source_p) && !HasUMode(source_p, UMODE_OPER))
     check_spambot_warning(source_p, NULL);
 
   /*
@@ -104,26 +77,26 @@ part_one_client(struct Client *client_p, struct Client *source_p,
    */
   if (reason[0] && (!MyConnect(source_p) ||
       ((can_send(chptr, source_p, ms) &&
-       (source_p->firsttime + ConfigFileEntry.anti_spam_exit_message_time)
+       (source_p->localClient->firsttime + ConfigFileEntry.anti_spam_exit_message_time)
         < CurrentTime))))
   {
-    sendto_server(client_p, NULL, chptr, CAP_TS6, NOCAPS, NOFLAGS,
+    sendto_server(client_p, CAP_TS6, NOCAPS,
                   ":%s PART %s :%s", ID(source_p), chptr->chname,
                   reason);
-    sendto_server(client_p, NULL, chptr, NOCAPS, CAP_TS6, NOFLAGS,
+    sendto_server(client_p, NOCAPS, CAP_TS6,
                   ":%s PART %s :%s", source_p->name, chptr->chname,
                   reason);
-    sendto_channel_local(ALL_MEMBERS, NO, chptr, ":%s!%s@%s PART %s :%s",
+    sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s!%s@%s PART %s :%s",
                          source_p->name, source_p->username,
                          source_p->host, chptr->chname, reason);
   }
   else
   {
-    sendto_server(client_p, NULL, chptr, CAP_TS6, NOCAPS, NOFLAGS,
+    sendto_server(client_p, CAP_TS6, NOCAPS,
                   ":%s PART %s", ID(source_p), chptr->chname);
-    sendto_server(client_p, NULL, chptr, NOCAPS, CAP_TS6, NOFLAGS,
+    sendto_server(client_p, NOCAPS, CAP_TS6,
                   ":%s PART %s", source_p->name, chptr->chname);
-    sendto_channel_local(ALL_MEMBERS, NO, chptr, ":%s!%s@%s PART %s",
+    sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s!%s@%s PART %s",
                          source_p->name, source_p->username,
                          source_p->host, chptr->chname);
   }
@@ -141,33 +114,54 @@ static void
 m_part(struct Client *client_p, struct Client *source_p,
        int parc, char *parv[])
 {
-  char *p, *name;
-  char reason[KICKLEN + 1];
+  char *p = NULL, *name = NULL;
+  char reason[KICKLEN + 1] = { '\0' };
 
   if (IsServer(source_p))
     return;
 
-  if (*parv[1] == '\0')
+  if (EmptyString(parv[1]))
   {
     sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS),
                me.name, source_p->name, "PART");
     return;
   }
 
-  reason[0] = '\0';
-
   if (parc > 2)
     strlcpy(reason, parv[2], sizeof(reason));
 
-  name = strtoken(&p, parv[1], ",");
-
   /* Finish the flood grace period... */
   if (MyClient(source_p) && !IsFloodDone(source_p))
     flood_endgrace(source_p);
 
-  while (name)
-  {
+  for (name = strtoken(&p, parv[1], ","); name;
+       name = strtoken(&p,    NULL, ","))
     part_one_client(client_p, source_p, name, reason);
-    name = strtoken(&p, NULL, ",");
-  }
 }
+
+static struct Message part_msgtab = {
+  "PART", 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
+  { m_unregistered, m_part, m_part, m_ignore, m_part, m_ignore }
+};
+
+static void
+module_init(void)
+{
+  mod_add_cmd(&part_msgtab);
+}
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&part_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1474 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = MODULE_FLAG_CORE
+};
