@@ -19,44 +19,20 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_quit.c,v 1.28.2.1 2004/06/16 04:56:03 erik Exp $
+ *  $Id: m_quit.c 1891 2013-04-27 14:25:18Z michael $
  */
 
 #include "stdinc.h"
-#include "handlers.h"
 #include "client.h"
 #include "ircd.h"
-#include "numeric.h"
+#include "irc_string.h"
 #include "s_serv.h"
 #include "send.h"
-#include "msg.h"
 #include "parse.h"
 #include "modules.h"
-#include "s_conf.h"
+#include "conf.h"
 
-static void m_quit(struct Client*, struct Client*, int, char**);
-static void ms_quit(struct Client*, struct Client*, int, char**);
 
-struct Message quit_msgtab = {
-  "QUIT", 0, 0, 0, 0, MFLG_SLOW | MFLG_UNREG, 0,
-  {m_quit, m_quit, ms_quit, m_ignore, m_quit}
-};
-
-#ifndef STATIC_MODULES
-void
-_modinit(void)
-{
-  mod_add_cmd(&quit_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&quit_msgtab);
-}
-
-const char *_version = "$Revision: 1.28.2.1 $";
-#endif
 /*
 ** m_quit
 **      parv[0] = sender prefix
@@ -66,31 +42,14 @@ static void
 m_quit(struct Client *client_p, struct Client *source_p,
        int parc, char *parv[])
 {
-  char *comment = (parc > 1 && parv[1]) ? parv[1] : client_p->name;
-  char reason [TOPICLEN + 1];
+  char reason[KICKLEN + 1] = "Quit: ";
 
-  source_p->flags |= FLAGS_NORMALEX;
-  if (strlen(comment) > (size_t) TOPICLEN)
-    comment[TOPICLEN] = '\0';
+  if (!EmptyString(parv[1]) && (HasUMode(source_p, UMODE_OPER) ||
+      (source_p->localClient->firsttime + ConfigFileEntry.anti_spam_exit_message_time)
+      < CurrentTime))
+    strlcpy(reason + 6, parv[1], sizeof(reason) - 6);
 
-  if (ConfigFileEntry.client_exit && comment[0])
-    {
-#ifndef VMS
-      snprintf(reason, TOPICLEN, "Quit: %s", comment);
-#else
-      sprintf(reason, "Quit: %s", comment);
-#endif
-      comment = reason;
-    }
-  
-  if(!IsOper(source_p) && 
-     (source_p->firsttime + ConfigFileEntry.anti_spam_exit_message_time)
-     > CurrentTime)
-    {
-      comment = "Client Quit";
-    }
-
-  exit_client(client_p, source_p, source_p, comment);
+  exit_client(source_p, source_p, reason);
 }
 
 /*
@@ -100,14 +59,41 @@ m_quit(struct Client *client_p, struct Client *source_p,
 */
 static void
 ms_quit(struct Client *client_p, struct Client *source_p,
-	int parc, char *parv[])
+        int parc, char *parv[])
 {
-  char *comment = (parc > 1 && parv[1]) ? parv[1] : client_p->name;
+  char reason[KICKLEN + 1] = { '\0' };
 
-  source_p->flags |= FLAGS_NORMALEX;
-  if (strlen(comment) > (size_t) TOPICLEN)
-    comment[TOPICLEN] = '\0';
+  if (!EmptyString(parv[1]))
+    strlcpy(reason, parv[1], sizeof(reason));
+  else
+    strlcpy(reason, client_p->name, sizeof(reason));
 
-  exit_client(client_p, source_p, source_p, comment);
+  exit_client(source_p, source_p, reason);
 }
 
+static struct Message quit_msgtab = {
+  "QUIT", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  {m_quit, m_quit, ms_quit, m_ignore, m_quit, m_ignore}
+};
+
+static void
+module_init(void)
+{
+  mod_add_cmd(&quit_msgtab);
+}
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&quit_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1891 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = MODULE_FLAG_CORE
+};
