@@ -19,67 +19,27 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_server.c 33 2005-10-02 20:50:00Z knight $
+ *  $Id: m_server.c 1569 2012-10-16 18:46:53Z michael $
  */
 
 #include "stdinc.h"
-#include "tools.h"
-#include "handlers.h"    /* m_server prototype */
+#include "list.h"
 #include "client.h"      /* client struct */
-#include "common.h"      /* TRUE bleah */
 #include "event.h"
 #include "hash.h"        /* add_to_client_hash_table */
 #include "irc_string.h" 
 #include "ircd.h"        /* me */
-#include "list.h"        /* make_server */
 #include "numeric.h"     /* ERR_xxx */
-#include "s_conf.h"      /* struct AccessItem */
-#include "s_log.h"       /* log level defines */
-#include "s_serv.h"      /* server_estab, check_server, my_name_for_link */
-#include "s_stats.h"     /* ServerStats */
+#include "conf.h"      /* struct AccessItem */
+#include "log.h"       /* log level defines */
+#include "s_serv.h"      /* server_estab, check_server */
+#include "s_user.h"
 #include "send.h"        /* sendto_one */
-#include "motd.h"
-#include "msg.h"
 #include "parse.h"
 #include "modules.h"
 
 
-static void mr_server(struct Client *, struct Client *, int, char **);
-static void ms_server(struct Client *, struct Client *, int, char **);
-static void ms_sid(struct Client *, struct Client *, int, char **);
-
-static int bogus_host(char *host);
-static void set_server_gecos(struct Client *, char *);
-static struct Client *server_exists(char *);
-
-struct Message server_msgtab = {
-  "SERVER", 0, 0, 4, 0, MFLG_SLOW | MFLG_UNREG, 0,
-  {mr_server, m_registered, ms_server, m_ignore, m_registered, m_ignore}
-};
-
-struct Message sid_msgtab = {
-  "SID", 0, 0, 5, 0, MFLG_SLOW, 0,
-  {m_error, m_ignore, ms_sid, m_ignore, m_ignore, m_ignore}
-};
-
-#ifndef STATIC_MODULES
-void 
-_modinit(void)
-{
-  mod_add_cmd(&server_msgtab);
-  mod_add_cmd(&sid_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&server_msgtab);
-  mod_del_cmd(&sid_msgtab);
-}
-
-const char *_version = "$Revision: 33 $";
-#endif
-
+static void set_server_gecos(struct Client *, const char *);
 
 /* mr_server()
  *  parv[0] = sender prefix
@@ -91,12 +51,11 @@ static void
 mr_server(struct Client *client_p, struct Client *source_p,
           int parc, char *parv[])
 {
-  char info[REALLEN + 1];
   char *name;
   struct Client *target_p;
   int hop;
 
-  if (parc < 4)
+  if (EmptyString(parv[3]))
   {
     sendto_one(client_p, "ERROR :No servername");
     exit_client(client_p, client_p, "Wrong number of args");
@@ -105,22 +64,30 @@ mr_server(struct Client *client_p, struct Client *source_p,
 
   name = parv[1];
   hop  = atoi(parv[2]);
-  strlcpy(info, parv[3], sizeof(info));
 
-  /* Reject a direct nonTS server connection if we're TS_ONLY -orabidoo
+  /*
+   * Reject a direct nonTS server connection if we're TS_ONLY -orabidoo
    */
   if (!DoesTS(client_p))
   {
-    sendto_realops_flags(UMODE_ALL, L_ADMIN, "Link %s dropped, non-TS server",
-                         get_client_name(client_p, HIDE_IP));
-    sendto_realops_flags(UMODE_ALL, L_OPER,  "Link %s dropped, non-TS server",
-                         get_client_name(client_p, MASK_IP));
+    sendto_realops_flags(UMODE_ALL, L_ADMIN,
+          "Unauthorized server connection attempt from %s: Non-TS server "
+          "for server %s", get_client_name(client_p, HIDE_IP), name);
+    sendto_realops_flags(UMODE_ALL, L_OPER,
+          "Unauthorized server connection attempt from %s: Non-TS server "
+          "for server %s", get_client_name(client_p, MASK_IP), name);
     exit_client(client_p, client_p, "Non-TS server");
     return;
   }
 
-  if (bogus_host(name))
+  if (!valid_servname(name))
   {
+    sendto_realops_flags(UMODE_ALL, L_ADMIN,
+          "Unauthorized server connection attempt from %s: Bogus server name "
+          "for server %s", get_client_name(client_p, HIDE_IP), name);
+    sendto_realops_flags(UMODE_ALL, L_OPER,
+          "Unauthorized server connection attempt from %s: Bogus server name "
+          "for server %s", get_client_name(client_p, MASK_IP), name);
     exit_client(client_p, client_p, "Bogus server name");
     return;
   }
@@ -128,7 +95,7 @@ mr_server(struct Client *client_p, struct Client *source_p,
   /* Now we just have to call check_server and everything should
    * be check for us... -A1kmm.
    */
-  switch (check_server(name, client_p, CHECK_SERVER_NOCRYPTLINK))
+  switch (check_server(name, client_p))
   {
     case -1:
       if (ConfigFileEntry.warn_no_nline)
@@ -174,24 +141,10 @@ mr_server(struct Client *client_p, struct Client *source_p,
       return;
       /* NOT REACHED */
       break;
-
-    /* servername is > HOSTLEN */
-    case -4:
-      sendto_realops_flags(UMODE_ALL, L_ADMIN,
-                           "Invalid servername %s from %s",
-			   name, get_client_name(client_p, HIDE_IP));
-      sendto_realops_flags(UMODE_ALL, L_OPER,
-		           "Invalid servername %s from %s",
-			   name, get_client_name(client_p, MASK_IP));
-
-      exit_client(client_p, client_p, "Invalid servername.");
-      return;
-      /* NOT REACHED */
-      break;
   }
 
-  if ((me.id[0] && client_p->id[0] && (target_p = hash_find_id(client_p->id)))
-      || (target_p = server_exists(name)))
+  if ((client_p->id[0] && (target_p = hash_find_id(client_p->id)))
+      || (target_p = hash_find_server(name)))
   {
     /* This link is trying feed me a server that I already have
      * access through another path -- multiple paths not accepted
@@ -224,41 +177,11 @@ mr_server(struct Client *client_p, struct Client *source_p,
     if (target_p != client_p)
       exit_client(target_p, &me, "Overridden");
 
-  if (ServerInfo.hub && IsCapable(client_p, CAP_LL))
-  {
-    if (IsCapable(client_p, CAP_HUB))
-    {
-      ClearCap(client_p, CAP_LL);
-      sendto_realops_flags(UMODE_ALL, L_ALL,
-               "*** LazyLinks to a hub from a hub, that's a no-no.");
-    }
-    else
-    {
-      client_p->localClient->serverMask = nextFreeMask();
-
-      if (!client_p->localClient->serverMask)
-      {
-        sendto_realops_flags(UMODE_ALL, L_ALL, "serverMask is full!");
-        /* try and negotiate a non LL connect */
-        ClearCap(client_p, CAP_LL);
-      }
-    }
-  }
-  else if (IsCapable(client_p, CAP_LL))
-  {
-    if (!IsCapable(client_p, CAP_HUB))
-    {
-      ClearCap(client_p, CAP_LL);
-      sendto_realops_flags(UMODE_ALL, L_ALL,
-               "*** LazyLinks to a leaf from a leaf, that's a no-no.");
-    }
-  }
-
   /* if we are connecting (Handshake), we already have the name from the
    * connect{} block in client_p->name
    */
   strlcpy(client_p->name, name, sizeof(client_p->name));
-  set_server_gecos(client_p, info);
+  set_server_gecos(client_p, parv[3]);
   client_p->hopcount = hop;
   server_estab(client_p);
 }
@@ -273,22 +196,19 @@ static void
 ms_server(struct Client *client_p, struct Client *source_p,
           int parc, char *parv[])
 {
-  char info[REALLEN + 1];
   char *name;
   struct Client *target_p;
-  struct Client *bclient_p;
-  struct ConfItem *conf;
-  struct MatchItem *match_item;
+  struct AccessItem *aconf;
   int hop;
   int hlined = 0;
   int llined = 0;
-  dlink_node *ptr, *ptr_next;
+  dlink_node *ptr = NULL;
 
   /* Just to be sure -A1kmm. */
   if (!IsServer(source_p))
     return;
 
-  if (parc < 4)
+  if (EmptyString(parv[3]))
   {
     sendto_one(client_p, "ERROR :No servername");
     return;
@@ -296,9 +216,21 @@ ms_server(struct Client *client_p, struct Client *source_p,
 
   name = parv[1];
   hop  = atoi(parv[2]);
-  strlcpy(info, parv[3], sizeof(info));
 
-  if ((target_p = server_exists(name)))
+  if (!valid_servname(name))
+  {
+    sendto_realops_flags(UMODE_ALL, L_ADMIN,
+                         "Link %s introduced server with bogus server name %s",
+                         get_client_name(client_p, SHOW_IP), name);
+    sendto_realops_flags(UMODE_ALL, L_OPER,
+                         "Link %s introduced server with bogus server name %s",
+                         get_client_name(client_p, MASK_IP), name);
+    sendto_one(client_p, "ERROR :Bogus server name introduced");
+    exit_client(client_p, &me, "Bogus server name intoduced");
+    return;
+  }
+
+  if ((target_p = hash_find_server(name)))
   {
     /* This link is trying feed me a server that I already have
      * access through another path -- multiple paths not accepted
@@ -317,13 +249,6 @@ ms_server(struct Client *client_p, struct Client *source_p,
      * doesnt exist, although ircd can handle it, its not a realistic
      * solution.. --fl_ 
      */
-    /* It is behind a host-masked server. Completely ignore the
-     * server message(don't propagate or we will delink from whoever
-     * we propagate to). -A1kmm
-     */
-    if (irccmp(target_p->name, name) && target_p->from == client_p)
-      return;
-
     sendto_one(client_p, "ERROR :Server %s already exists", name);
     sendto_realops_flags(UMODE_ALL, L_ADMIN,
 			 "Link %s cancelled, server %s already exists",
@@ -343,76 +268,24 @@ ms_server(struct Client *client_p, struct Client *source_p,
     if (target_p != client_p)
       exit_client(target_p, &me, "Overridden");
 
-  /* User nicks never have '.' in them and server names
-   * must always have '.' in them.
-   */
-  if (strchr(name, '.') == NULL)
-  {
-    /* Server trying to use the same name as a person. Would
-     * cause a fair bit of confusion. Enough to make it hellish
-     * for a while and servers to send stuff to the wrong place.
-     */
-    sendto_one(client_p,"ERROR :Nickname %s already exists!", name);
-    sendto_realops_flags(UMODE_ALL, L_ADMIN,
-			   "Link %s cancelled: Server/nick collision on %s",
-		/* inpath */ get_client_name(client_p, HIDE_IP), name);
-    sendto_realops_flags(UMODE_ALL, L_OPER,
-          "Link %s cancelled: Server/nick collision on %s",
-	  get_client_name(client_p, MASK_IP), name);
-    exit_client(client_p, client_p, "Nick as Server");
-    return;
-  }
-
-  if (strlen(name) > HOSTLEN)
-  {
-    sendto_realops_flags(UMODE_ALL, L_ADMIN,
-                         "Link %s introduced server with invalid servername %s",
-                         get_client_name(client_p, HIDE_IP), name);
-    sendto_realops_flags(UMODE_ALL, L_OPER,
-                         "Link %s introduced server with invalid servername %s",
-                         client_p->name, name);
-    exit_client(client_p, &me, "Invalid servername introduced.");
-    return;
-  }
-
-  /* Server is informing about a new server behind
-   * this link. Create REMOTE server structure,
-   * add it to list and propagate word to my other
-   * server links...
-   */
-  if (parc == 1 || info[0] == '\0')
-  {
-    sendto_one(client_p, "ERROR :No server info specified for %s", name);
-    return;
-  }
+  aconf = map_to_conf(client_p->localClient->confs.head->data);
 
   /* See if the newly found server is behind a guaranteed
    * leaf. If so, close the link.
    */
-  DLINK_FOREACH(ptr, leaf_items.head)
-  {
-    conf = ptr->data;
-
-    if (match(conf->name, client_p->name))
+  DLINK_FOREACH(ptr, aconf->leaf_list.head)
+    if (match(ptr->data, name))
     {
-      match_item = (struct MatchItem *)map_to_conf(conf);
-      if (match(match_item->host, name))
-	llined++;
+      llined = 1;
+      break;
     }
-  }
 
-  DLINK_FOREACH(ptr, hub_items.head)
-  {
-    conf = ptr->data;
-
-    if (match(conf->name, client_p->name))
+  DLINK_FOREACH(ptr, aconf->hub_list.head)
+    if (match(ptr->data, name))
     {
-      match_item = (struct MatchItem *)map_to_conf(conf);
-
-      if (match(match_item->host, name))
-	hlined++;
+      hlined = 1;
+      break;
     }
-  }
 
   /* Ok, this way this works is
    *
@@ -437,8 +310,8 @@ ms_server(struct Client *client_p, struct Client *source_p,
    * .edu's
    */
 
-  /* Ok, check client_p can hub the new server, and make sure it's not a LL */
-  if (!hlined || (IsCapable(client_p, CAP_LL) && !IsCapable(client_p, CAP_HUB)))
+  /* Ok, check client_p can hub the new server */
+  if (!hlined)
   {
     /* OOOPs nope can't HUB */
     sendto_realops_flags(UMODE_ALL, L_ADMIN, "Non-Hub link %s introduced %s.",
@@ -458,95 +331,43 @@ ms_server(struct Client *client_p, struct Client *source_p,
                          get_client_name(client_p, HIDE_IP), name);
     sendto_realops_flags(UMODE_ALL, L_OPER,
 			 "Link %s introduced leafed server %s.",
-                         client_p->name, name);
-      /* If it is new, we are probably misconfigured, so split the
-       * non-hub server introducing this. Otherwise, split the new
-       * server. -A1kmm.
-       */
-      /* wastes too much bandwidth, generates too many errors on
-       * larger networks, dont bother. --fl_
-       */
-      exit_client(client_p, &me, "Leafed Server.");
-      return;
+                         get_client_name(client_p, MASK_IP), name);
+    /* If it is new, we are probably misconfigured, so split the
+     * non-hub server introducing this. Otherwise, split the new
+     * server. -A1kmm.
+     */
+    /* wastes too much bandwidth, generates too many errors on
+     * larger networks, dont bother. --fl_
+     */
+    exit_client(client_p, &me, "Leafed Server.");
+    return;
   }
 
   target_p = make_client(client_p);
   make_server(target_p);
   target_p->hopcount = hop;
+  target_p->servptr = source_p;
 
   strlcpy(target_p->name, name, sizeof(target_p->name));
 
-  set_server_gecos(target_p, info);
-
-  target_p->servptr = source_p;
-
+  set_server_gecos(target_p, parv[3]);
   SetServer(target_p);
 
-  if ((target_p->node.prev != NULL) || (target_p->node.next != NULL))
-  {
-    sendto_realops_flags(UMODE_ALL, L_OPER,
-			 "already linked %s at %s:%d", target_p->name,
-			 __FILE__, __LINE__);
-    ilog(L_ERROR, "already linked client %s at %s:%d", target_p->name,
-	 __FILE__, __LINE__);
-    assert(0==1);
-  }
-  else
-  {
-    dlinkAdd(target_p, &target_p->node, &global_client_list);
-    dlinkAdd(target_p, make_dlink_node(), &global_serv_list);
-  }
+  if (HasFlag(source_p, FLAGS_SERVICE) || find_matching_name_conf(SERVICE_TYPE, target_p->name, NULL, NULL, 0))
+    AddFlag(target_p, FLAGS_SERVICE);
+
+  dlinkAdd(target_p, &target_p->node, &global_client_list);
+  dlinkAdd(target_p, make_dlink_node(), &global_serv_list);
+  dlinkAdd(target_p, &target_p->lnode, &target_p->servptr->serv->server_list);
 
   hash_add_client(target_p);
-  /* XXX test that target_p->lnode.prev and .next are NULL as well? */
-  if ((target_p->lnode.prev != NULL) || (target_p->lnode.next != NULL))
-  {
-    sendto_realops_flags(UMODE_ALL, L_OPER,
-			 "already lnode linked %s at %s:%d", target_p->name,
-			 __FILE__, __LINE__);
-    ilog(L_ERROR, "already lnode linked %s at %s:%d", target_p->name,
-	 __FILE__, __LINE__);
-    assert(0==2);
-  }
-  else
-    dlinkAdd(target_p, &target_p->lnode, &target_p->servptr->serv->servers);
 
-  client_p->serv->dep_servers++;
-
-  /* Old sendto_serv_but_one() call removed because we now
-   * need to send different names to different servers
-   * (domain name matching)
-   */
-  DLINK_FOREACH_SAFE(ptr, ptr_next, serv_list.head)
-  {
-    bclient_p = ptr->data;
-
-    if (bclient_p == client_p)
-      continue;
-
-    if ((conf = bclient_p->serv->sconf) == NULL)
-    {
-      sendto_realops_flags(UMODE_ALL, L_ADMIN,
-			   "Lost connect{} block for %s on %s. Closing",
-                           get_client_name(client_p, HIDE_IP), name);
-      sendto_realops_flags(UMODE_ALL, L_OPER,
-			   "Lost connect{} block for %s on %s. Closing",
-                           get_client_name(client_p, MASK_IP), name);
-      exit_client(client_p, client_p, "Lost connect{} block");
-      continue;
-    }
-
-    if (match(my_name_for_link(conf), target_p->name))
-      continue;
-
-    sendto_one(bclient_p, ":%s SERVER %s %d :%s%s",
-               ID_or_name(source_p, bclient_p), target_p->name, hop + 1,
-               IsHidden(target_p) ? "(H) " : "",
-               target_p->info);
-  }
+  sendto_server(client_p, NOCAPS, NOCAPS, ":%s SERVER %s %d :%s%s",
+                source_p->name, target_p->name, hop + 1,
+                IsHidden(target_p) ? "(H) " : "", target_p->info);
 
   sendto_realops_flags(UMODE_EXTERNAL, L_ALL,
-		       "Server %s being introduced by %s",
+                       "Server %s being introduced by %s",
                        target_p->name, source_p->name);
 }
 
@@ -561,52 +382,75 @@ static void
 ms_sid(struct Client *client_p, struct Client *source_p,
        int parc, char *parv[])
 {
-  char info[REALLEN + 1];
   struct Client *target_p;
-  struct Client *bclient_p;
-  struct ConfItem *conf;
-  struct MatchItem *match_item;
+  struct AccessItem *aconf = NULL;
   int hlined = 0;
   int llined = 0;
-  dlink_node *ptr, *ptr_next;
+  dlink_node *ptr = NULL;
   int hop;
-#define SID_NAME	parv[1]
-#define SID_HOP		parv[2]
-#define SID_SID		parv[3]
-#define SID_GECOS	parv[4]
-
-  hop = atoi(SID_HOP);
 
   /* Just to be sure -A1kmm. */
   if (!IsServer(source_p))
     return;
 
-  strlcpy(info, SID_GECOS, sizeof(info));
+  if (EmptyString(parv[4]))
+  {
+    sendto_one(client_p, "ERROR :No servername");
+    return;
+  }
+
+  hop = atoi(parv[2]);
+
+  if (!valid_servname(parv[1]))
+  {
+    sendto_realops_flags(UMODE_ALL, L_ADMIN,
+                         "Link %s introduced server with bogus server name %s",
+                         get_client_name(client_p, SHOW_IP), parv[1]);
+    sendto_realops_flags(UMODE_ALL, L_OPER,
+                         "Link %s introduced server with bogus server name %s",
+                         get_client_name(client_p, MASK_IP), parv[1]);
+    sendto_one(client_p, "ERROR :Bogus server name introduced");
+    exit_client(client_p, &me, "Bogus server name intoduced");
+    return;
+  }
+
+  if (!valid_sid(parv[3]))
+  {
+    sendto_realops_flags(UMODE_ALL, L_ADMIN,
+                         "Link %s introduced server with bogus server ID %s",
+                         get_client_name(client_p, SHOW_IP), parv[3]);
+    sendto_realops_flags(UMODE_ALL, L_OPER,
+                         "Link %s introduced server with bogus server ID %s",
+                         get_client_name(client_p, MASK_IP), parv[3]);
+    sendto_one(client_p, "ERROR :Bogus server ID introduced");
+    exit_client(client_p, &me, "Bogus server ID intoduced");
+    return;
+  }
 
   /* collision on SID? */
-  if ((target_p = hash_find_id(SID_SID)) != NULL)
+  if ((target_p = hash_find_id(parv[3])))
   {
-    sendto_one(client_p, "ERROR :SID %s already exists", SID_SID);
+    sendto_one(client_p, "ERROR :SID %s already exists", parv[3]);
     sendto_realops_flags(UMODE_ALL, L_ADMIN,
 			 "Link %s cancelled, SID %s already exists",
-                         get_client_name(client_p, SHOW_IP), SID_SID);
+                         get_client_name(client_p, SHOW_IP), parv[3]);
     sendto_realops_flags(UMODE_ALL, L_OPER,
 			 "Link %s cancelled, SID %s already exists",
-                         client_p->name, SID_SID);
-    exit_client(client_p, &me, "Server Exists");
+                         client_p->name, parv[3]);
+    exit_client(client_p, &me, "SID Exists");
     return;
   }
 
   /* collision on name? */
-  if ((target_p = server_exists(SID_NAME)) != NULL)
+  if ((target_p = hash_find_server(parv[1])))
   {
-    sendto_one(client_p, "ERROR :Server %s already exists", SID_NAME);
+    sendto_one(client_p, "ERROR :Server %s already exists", parv[1]);
     sendto_realops_flags(UMODE_ALL, L_ADMIN,
                          "Link %s cancelled, server %s already exists",   
-                         get_client_name(client_p, SHOW_IP), SID_NAME);
+                         get_client_name(client_p, SHOW_IP), parv[1]);
     sendto_realops_flags(UMODE_ALL, L_OPER,
                          "Link %s cancelled, server %s already exists",   
-                         client_p->name, SID_NAME);       
+                         client_p->name, parv[1]);
     exit_client(client_p, &me, "Server Exists");
     return;
   }
@@ -615,69 +459,29 @@ ms_sid(struct Client *client_p, struct Client *source_p,
    * a connect comes in with same name toss the pending one,
    * but only if it's not the same client! - Dianora
    */
-  if ((target_p = find_servconn_in_progress(SID_NAME)))
+  if ((target_p = find_servconn_in_progress(parv[1])))
     if (target_p != client_p)
       exit_client(target_p, &me, "Overridden");
 
-  /* User nicks never have '.' in them and server names
-   * must always have '.' in them.
-   */
-  if (strchr(SID_NAME, '.') == NULL || /* servernames must have a '.' and nicks must not */
-      strlen(SID_NAME) > HOSTLEN)      /* ensure the name of the server is not too long  */
-  {
-    /* Server trying to use the same name as a person. Would
-     * cause a fair bit of confusion. Enough to make it hellish
-     * for a while and servers to send stuff to the wrong place.
-     */
-    sendto_one(client_p, "ERROR :Invalid servername");
-    sendto_realops_flags(UMODE_ALL, L_ADMIN,
-			 "Link %s cancelled: servername name %s invalid",
-		         get_client_name(client_p, SHOW_IP), SID_NAME);
-    sendto_realops_flags(UMODE_ALL, L_OPER,
-                         "Link %s cancelled: servername name %s invalid",
-	                 get_client_name(client_p, MASK_IP), SID_NAME);
-    exit_client(client_p, client_p, "Nick as Server");
-    return;
-  }
-
-  /* Server is informing about a new server behind
-   * this link. Create REMOTE server structure,
-   * add it to list and propagate word to my other
-   * server links...
-   */
-  if (parc == 1 || info[0] == '\0')
-  {
-    sendto_one(client_p, "ERROR :No server info specified for %s", SID_NAME);
-    return;
-  }
+  aconf = map_to_conf(client_p->localClient->confs.head->data);
 
   /* See if the newly found server is behind a guaranteed
    * leaf. If so, close the link.
    */
-  DLINK_FOREACH(ptr, leaf_items.head)
-  {
-    conf = ptr->data;
-
-    if (match(conf->name, client_p->name))
+  DLINK_FOREACH(ptr, aconf->leaf_list.head)
+    if (match(ptr->data, parv[1]))
     {
-      match_item = (struct MatchItem *)map_to_conf(conf);
-      if (match(match_item->host, SID_NAME))
-	llined++;
+      llined = 1;
+      break;
     }
-  }
 
-  DLINK_FOREACH(ptr, hub_items.head)
-  {
-    conf = ptr->data;
-
-    if (match(conf->name, client_p->name))
+  DLINK_FOREACH(ptr, aconf->hub_list.head)
+    if (match(ptr->data, parv[1]))
     {
-      match_item = (struct MatchItem *)map_to_conf(conf);
-
-      if (match(match_item->host, SID_NAME))
-	hlined++;
+      hlined = 1;
+      break;
     }
-  }
+
 
   /* Ok, this way this works is
    *
@@ -703,13 +507,13 @@ ms_sid(struct Client *client_p, struct Client *source_p,
    */
 
   /* Ok, check client_p can hub the new server, and make sure it's not a LL */
-  if (!hlined || (IsCapable(client_p, CAP_LL) && !IsCapable(client_p, CAP_HUB)))
+  if (!hlined)
   {
     /* OOOPs nope can't HUB */
     sendto_realops_flags(UMODE_ALL, L_ADMIN, "Non-Hub link %s introduced %s.",
-                         get_client_name(client_p, SHOW_IP), SID_NAME);
+                         get_client_name(client_p, SHOW_IP), parv[1]);
     sendto_realops_flags(UMODE_ALL, L_OPER,  "Non-Hub link %s introduced %s.",
-                         get_client_name(client_p, MASK_IP), SID_NAME);
+                         get_client_name(client_p, MASK_IP), parv[1]);
     exit_client(source_p, &me, "No matching hub_mask.");
     return;
   }
@@ -720,10 +524,10 @@ ms_sid(struct Client *client_p, struct Client *source_p,
     /* OOOPs nope can't HUB this leaf */
     sendto_realops_flags(UMODE_ALL, L_ADMIN,
 			 "Link %s introduced leafed server %s.",
-                         get_client_name(client_p, SHOW_IP), SID_NAME);
+                         get_client_name(client_p, SHOW_IP), parv[1]);
     sendto_realops_flags(UMODE_ALL, L_OPER,  
 			 "Link %s introduced leafed server %s.",
-                         client_p->name, SID_NAME);
+                         get_client_name(client_p, MASK_IP), parv[1]);
     exit_client(client_p, &me, "Leafed Server.");
     return;
   }
@@ -731,85 +535,33 @@ ms_sid(struct Client *client_p, struct Client *source_p,
   target_p = make_client(client_p);
   make_server(target_p);
   target_p->hopcount = hop;
-
-  strlcpy(target_p->name, SID_NAME, sizeof(target_p->name));
-  strlcpy(target_p->id, SID_SID, sizeof(target_p->id));
-
-  set_server_gecos(target_p, info);
-
   target_p->servptr = source_p;
 
+  strlcpy(target_p->name, parv[1], sizeof(target_p->name));
+  strlcpy(target_p->id, parv[3], sizeof(target_p->id));
+
+  set_server_gecos(target_p, parv[4]);
   SetServer(target_p);
 
-  if ((target_p->node.prev != NULL) || (target_p->node.next != NULL))
-  {
-    sendto_realops_flags(UMODE_ALL, L_ADMIN,
-			 "already linked %s at %s:%d", target_p->name,
-			 __FILE__, __LINE__);
-    ilog(L_ERROR, "already linked %s at %s:%d", target_p->name,
-	 __FILE__, __LINE__);
-    assert(0==3);
-  }
-  else
-  {
-    dlinkAdd(target_p, &target_p->node, &global_client_list);
-    dlinkAdd(target_p, make_dlink_node(), &global_serv_list);
-  }
+  if (HasFlag(source_p, FLAGS_SERVICE) || find_matching_name_conf(SERVICE_TYPE, target_p->name, NULL, NULL, 0))
+    AddFlag(target_p, FLAGS_SERVICE);
+
+  dlinkAdd(target_p, &target_p->node, &global_client_list);
+  dlinkAdd(target_p, make_dlink_node(), &global_serv_list);
+  dlinkAdd(target_p, &target_p->lnode, &target_p->servptr->serv->server_list);
 
   hash_add_client(target_p);
-  /* XXX test that target_p->lnode.prev and next are NULL as well? */
-  if ((target_p->lnode.prev != NULL) || (target_p->lnode.next != NULL))
-  {
-    sendto_realops_flags(UMODE_ALL, L_OPER,
-			 "already lnode linked %s at %s:%d", target_p->name,
-			 __FILE__, __LINE__);
-    ilog(L_ERROR, "already lnode linked %s at %s:%d", target_p->name,
-	 __FILE__, __LINE__);
-    assert(0==4);
-  }
-  else
-    dlinkAdd(target_p, &target_p->lnode, &target_p->servptr->serv->servers);
-
   hash_add_id(target_p);
 
-  client_p->serv->dep_servers++;
-
-  DLINK_FOREACH_SAFE(ptr, ptr_next, serv_list.head)
-  {
-    bclient_p = ptr->data;
-    
-    if (bclient_p == client_p)
-      continue;
-    
-    if ((conf = bclient_p->serv->sconf) == NULL)
-    {
-      sendto_realops_flags(UMODE_ALL, L_ADMIN,
-                           "Lost connect{} block for %s on %s. Closing",
-                           get_client_name(client_p, HIDE_IP), SID_NAME);
-      sendto_realops_flags(UMODE_ALL, L_OPER,
-                           "Lost connect{} block for %s on %s. Closing",
-                           get_client_name(client_p, MASK_IP), SID_NAME);
-      exit_client(client_p, client_p, "Lost connect{} block");
-      continue;
-    }
-
-    if (match(my_name_for_link(conf), target_p->name))
-      continue;
-    
-    if (IsCapable(bclient_p, CAP_TS6))
-      sendto_one(bclient_p, ":%s SID %s %d %s :%s%s",
-                 ID_or_name(source_p, client_p), target_p->name, hop + 1,
-                 SID_SID, IsHidden(target_p) ? "(H) " : "",
-                 target_p->info);
-    else
-      sendto_one(bclient_p, ":%s SERVER %s %d :%s%s",
-                 source_p->name, target_p->name, hop + 1,
-                 IsHidden(target_p) ? "(H) " : "",
-                 target_p->info);
-  }
+  sendto_server(client_p, CAP_TS6, NOCAPS, ":%s SID %s %d %s :%s%s",
+                ID_or_name(source_p, client_p), target_p->name, hop + 1,
+                target_p->id, IsHidden(target_p) ? "(H) " : "", target_p->info);
+  sendto_server(client_p, NOCAPS, CAP_TS6, ":%s SERVER %s %d :%s%s",
+                source_p->name, target_p->name, hop + 1,
+                IsHidden(target_p) ? "(H) " : "", target_p->info);
 
   sendto_realops_flags(UMODE_EXTERNAL, L_ALL, 
-		       "Server %s being introduced by %s",
+                       "Server %s being introduced by %s",
                        target_p->name, source_p->name);
 }
 
@@ -820,113 +572,53 @@ ms_sid(struct Client *client_p, struct Client *source_p,
  * side effects - servers gecos field is set
  */
 static void
-set_server_gecos(struct Client *client_p, char *info)
+set_server_gecos(struct Client *client_p, const char *info)
 {
-  /* check the info for [IP] */
-  if (info[0])
+  const char *s = info;
+
+  /* check for (H) which is a hidden server */
+  if (!strncmp(s, "(H) ", 4))
   {
-    char *p;
-    char *s;
-    char *t;
-    
-    s = info;
-    
-    /* we should only check the first word for an ip */
-    if ((p = strchr(s, ' ')) != NULL)
-      *p = '\0';
-      
-    /* check for a ] which would symbolise an [IP] */
-    if ((t = strchr(s, ']')) != NULL)
-    {
-      /* set s to after the first space */
-      if (p)
-        s = ++p;
-      else
-        s = NULL;
-    }
-    /* no ], put the space back */
-    else if (p)
-      *p = ' ';
-
-    /* p may have been set to a trailing space, so check s exists and that
-     * it isnt \0 */
-    if (s && (*s != '\0'))
-    {
-      /* a space? if not (H) could be the last part of info.. */
-      if ((p = strchr(s, ' ')))
-        *p = '\0';
-      
-      /* check for (H) which is a hidden server */
-      if (!strcmp(s, "(H)"))
-      {
-        SetHidden(client_p);
-
-        /* if there was no space.. theres nothing to set info to */
-        if (p)
-	  s = ++p;
-	else
-	  s = NULL;
-      }
-      else if (p)
-        *p = ' ';
-
-      /* if there was a trailing space, s could point to \0, so check */
-      if (s && (*s != '\0'))
-        strlcpy(client_p->info, s, sizeof(client_p->info));
-      else
-        strlcpy(client_p->info, "(Unknown Location)", sizeof(client_p->info));
-    }
-    else
-      strlcpy(client_p->info, "(Unknown Location)", sizeof(client_p->info));
+    SetHidden(client_p);
+    s = s + 4;
   }
+
+  if (!EmptyString(s))
+    strlcpy(client_p->info, s, sizeof(client_p->info));
   else
     strlcpy(client_p->info, "(Unknown Location)", sizeof(client_p->info));
 }
 
-/* bogus_host()
- *
- * inputs	- hostname
- * output	- 1 if a bogus hostname input,
- *              - 0 if its valid
- * side effects	- none
- */
-static int
-bogus_host(char *host)
+static struct Message server_msgtab = {
+  "SERVER", 0, 0, 4, MAXPARA, MFLG_SLOW, 0,
+  {mr_server, m_registered, ms_server, m_ignore, m_registered, m_ignore}
+};
+
+static struct Message sid_msgtab = {
+  "SID", 0, 0, 5, MAXPARA, MFLG_SLOW, 0,
+  {rfc1459_command_send_error, m_ignore, ms_sid, m_ignore, m_ignore, m_ignore}
+};
+
+static void
+module_init(void)
 {
-  unsigned int dots = 0;
-  char *s;
-
-  for (s = host; *s; s++)
-  {
-    if (!IsServChar(*s))
-      return(1);
-
-    if ('.' == *s)
-      ++dots;
-  }
-
-  return(!dots);
+  mod_add_cmd(&sid_msgtab);
+  mod_add_cmd(&server_msgtab);
 }
 
-/* server_exists()
- *
- * inputs	- servername
- * output	- 1 if server exists, 0 if doesnt exist
- */
-static struct Client *
-server_exists(char *servername)
+static void
+module_exit(void)
 {
-  struct Client *target_p;
-  dlink_node *ptr;
-
-  DLINK_FOREACH(ptr, global_serv_list.head)
-  {
-    target_p = ptr->data;
-
-    if (match(target_p->name, servername) || 
-        match(servername, target_p->name))
-      return(target_p);
-  }
-
-  return(NULL);
+  mod_del_cmd(&sid_msgtab);
+  mod_del_cmd(&server_msgtab);
 }
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1569 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = MODULE_FLAG_CORE
+};
