@@ -19,12 +19,11 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_sjoin.c 632 2006-06-01 10:53:00Z michael $
+ *  $Id: m_sjoin.c 1750 2013-01-16 14:07:08Z michael $
  */
 
 #include "stdinc.h"
-#include "tools.h"
-#include "handlers.h"
+#include "list.h"
 #include "channel.h"
 #include "channel_mode.h"
 #include "client.h"
@@ -32,38 +31,14 @@
 #include "irc_string.h"
 #include "sprintf_irc.h"
 #include "ircd.h"
-#include "list.h"
 #include "numeric.h"
 #include "send.h"
-#include "common.h"
-#include "msg.h"
 #include "parse.h"
 #include "modules.h"
 #include "s_serv.h"
-#include "s_conf.h"
+#include "conf.h"
+#include "s_misc.h"
 
-static void ms_sjoin(struct Client *, struct Client *, int, char *[]);
-
-struct Message sjoin_msgtab = {
-  "SJOIN", 0, 0, 0, 0, MFLG_SLOW, 0,
-  {m_unregistered, m_ignore, ms_sjoin, m_ignore, m_ignore, m_ignore}
-};
-
-#ifndef STATIC_MODULES
-void
-_modinit(void)
-{
-  mod_add_cmd(&sjoin_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&sjoin_msgtab);
-}
-
-const char *_version = "$Revision: 632 $";
-#endif
 
 static char modebuf[MODEBUFLEN];
 static char parabuf[MODEBUFLEN];
@@ -76,8 +51,7 @@ static void set_final_mode(struct Mode *, struct Mode *);
 static void remove_our_modes(struct Channel *, struct Client *);
 static void remove_a_mode(struct Channel *, struct Client *, int, char);
 static void remove_ban_list(struct Channel *, struct Client *, dlink_list *, char, int);
-static void introduce_lazy_link_clients(struct Client *,struct Client *,
-					struct Channel *);
+
 
 /* ms_sjoin()
  *
@@ -102,9 +76,9 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
   time_t         tstosend;
   struct Mode mode, *oldmode;
   int            args = 0;
-  char           keep_our_modes = YES;
-  char           keep_new_modes = YES;
-  char           have_many_nicks = NO;
+  char           keep_our_modes = 1;
+  char           keep_new_modes = 1;
+  char           have_many_nicks = 0;
   int            lcount;
   char           nick_prefix[4];
   char           uid_prefix[4];
@@ -128,10 +102,6 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
   if (IsClient(source_p) || parc < 5)
     return;
 
-  /* SJOIN's for local channels can't happen. */
-  if (*parv[2] != '#')
-    return;
-
   if (!check_channel_name(parv[2], 0))
   {
     sendto_realops_flags(UMODE_DEBUG, L_ALL,
@@ -171,6 +141,15 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
       case 'p':
         mode.mode |= MODE_PRIVATE;
         break;
+      case 'r':
+        mode.mode |= MODE_REGISTERED;
+        break;
+      case 'O':
+        mode.mode |= MODE_OPERONLY;
+        break;
+      case 'S':
+        mode.mode |= MODE_SSLONLY;
+        break;
       case 'k':
         strlcpy(mode.key, parv[4 + args], sizeof(mode.key));
         args++;
@@ -214,7 +193,7 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
   {
     if (!newts && !isnew && oldts)
     {
-      sendto_channel_local(ALL_MEMBERS, NO, chptr,
+      sendto_channel_local(ALL_MEMBERS, 0, chptr,
  		  	   ":%s NOTICE %s :*** Notice -- TS for %s changed from %lu to 0",
 		  	   me.name, chptr->chname, chptr->chname, (unsigned long)oldts);
       sendto_realops_flags(UMODE_ALL, L_ALL,
@@ -231,12 +210,12 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
     tstosend = oldts;
   else if (newts < oldts)
   {
-    keep_our_modes = NO;
+    keep_our_modes = 0;
     chptr->channelts = tstosend = newts;
   }
   else
   {
-    keep_new_modes = NO;
+    keep_new_modes = 0;
     tstosend = oldts;
   }
 
@@ -257,7 +236,17 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
   if (!keep_our_modes)
   {
     remove_our_modes(chptr, source_p);
-    sendto_channel_local(ALL_MEMBERS, NO, chptr,
+
+    if (chptr->topic[0])
+    {
+      set_channel_topic(chptr, "", "", 0, 0);
+      sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s TOPIC %s :",
+                           (IsHidden(source_p) ||
+                           ConfigServerHide.hide_servers) ?
+                           me.name : source_p->name, chptr->chname);
+    }
+
+    sendto_channel_local(ALL_MEMBERS, 0, chptr,
    		         ":%s NOTICE %s :*** Notice -- TS for %s changed from %lu to %lu",
 	 		 me.name, chptr->chname, chptr->chname,
 			 (unsigned long)oldts, (unsigned long)newts);
@@ -267,7 +256,7 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
   {
     /* This _SHOULD_ be to ALL_MEMBERS
      * It contains only +imnpstlk, etc */
-    sendto_channel_local(ALL_MEMBERS, NO, chptr, ":%s MODE %s %s %s",
+    sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s MODE %s %s %s",
 	                 servername, chptr->chname, modebuf, parabuf);
   }
 
@@ -321,7 +310,7 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
 
   while (*s)
   {
-    int valid_mode = YES;
+    int valid_mode = 1;
     fl = 0;
 
     do
@@ -343,7 +332,7 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
         s++;
 	break;
       default:
-	valid_mode = NO;
+	valid_mode = 0;
 	break;
       }
     } while (valid_mode);
@@ -406,18 +395,19 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
 
     if ((nick_ptr - nick_buf + len_nick) > (IRCD_BUFSIZE  - 2))
     {
-      sendto_server(client_p, NULL, chptr, 0, CAP_TS6, 0, "%s", nick_buf);
+      sendto_server(client_p, 0, CAP_TS6, "%s", nick_buf);
       
       buflen = ircsprintf(nick_buf, ":%s SJOIN %lu %s %s %s:",
                           source_p->name, (unsigned long)tstosend,
                           chptr->chname, modebuf, parabuf);
       nick_ptr = nick_buf + buflen;
     }
+
     nick_ptr += ircsprintf(nick_ptr, "%s%s ", nick_prefix, target_p->name);
     
     if ((uid_ptr - uid_buf + len_uid) > (IRCD_BUFSIZE - 2))
     {
-      sendto_server(client_p, NULL, chptr, CAP_TS6, 0, 0, "%s", uid_buf);
+      sendto_server(client_p, CAP_TS6, 0, "%s", uid_buf);
       
       buflen = ircsprintf(uid_buf, ":%s SJOIN %lu %s %s %s:",
                           ID(source_p), (unsigned long)tstosend,
@@ -427,14 +417,10 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
 
     uid_ptr  += ircsprintf(uid_ptr,  "%s%s ", uid_prefix, ID(target_p));
 	
-    /* LazyLinks - Introduce unknown clients before sending the sjoin */
-    if (ServerInfo.hub)
-      introduce_lazy_link_clients(client_p, target_p, chptr);
-
     if (!IsMember(target_p, chptr))
     {
       add_user_to_channel(chptr, target_p, fl, !have_many_nicks);
-      sendto_channel_local(ALL_MEMBERS, NO, chptr, ":%s!%s@%s JOIN :%s",
+      sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s!%s@%s JOIN :%s",
                            target_p->name, target_p->username,
                            target_p->host, chptr->chname);
     }
@@ -462,7 +448,7 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
           slen = ircsprintf(sptr, " %s", para[lcount]);	/* see? */
 	  sptr += slen;					/* ready for next */
         }
-        sendto_channel_local(ALL_MEMBERS, NO, chptr, ":%s MODE %s %s%s",
+        sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s MODE %s %s%s",
                             servername, chptr->chname, modebuf, sendbuf);
         mbuf = modebuf;
         *mbuf++ = '+';
@@ -486,7 +472,7 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
           slen = ircsprintf(sptr, " %s", para[lcount]);
           sptr += slen;
         }
-        sendto_channel_local(ALL_MEMBERS, NO, chptr, ":%s MODE %s %s%s",
+        sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s MODE %s %s%s",
                              servername, chptr->chname, modebuf, sendbuf);
 
         mbuf = modebuf;
@@ -511,7 +497,7 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
           slen = ircsprintf(sptr, " %s", para[lcount]);
 	  sptr += slen;
         }
-        sendto_channel_local(ALL_MEMBERS, NO, chptr, ":%s MODE %s %s%s",
+        sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s MODE %s %s%s",
                              servername, chptr->chname, modebuf, sendbuf);
 
         mbuf = modebuf;
@@ -557,7 +543,7 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
       sptr += slen;
     }
 
-    sendto_channel_local(ALL_MEMBERS, NO, chptr, ":%s MODE %s %s%s",
+    sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s MODE %s %s%s",
                         servername, chptr->chname, modebuf, sendbuf);
   }
 
@@ -618,35 +604,20 @@ ms_sjoin(struct Client *client_p, struct Client *source_p,
  *		  Then walk through turning on modes that are on in mode
  *		  but were not set in oldmode.
  */
-
-static const struct mode_letter
-{
-  unsigned int mode;
-  unsigned char letter;
-} flags[] = {
-  { MODE_NOPRIVMSGS, 'n' },
-  { MODE_TOPICLIMIT, 't' },
-  { MODE_SECRET,     's' },
-  { MODE_MODERATED,  'm' },
-  { MODE_INVITEONLY, 'i' },
-  { MODE_PRIVATE,    'p' },
-  { 0, '\0' }
-};
-
 static void
 set_final_mode(struct Mode *mode, struct Mode *oldmode)
 {
+  const struct mode_letter *tab;
   char *pbuf = parabuf;
   int len;
-  int i;
 
   *mbuf++ = '-';
 
-  for (i = 0; flags[i].letter; i++)
+  for (tab = chan_modes; tab->letter; ++tab)
   {
-    if ((flags[i].mode & oldmode->mode) &&
-        !(flags[i].mode & mode->mode))
-      *mbuf++ = flags[i].letter;
+    if ((tab->mode & oldmode->mode) &&
+        !(tab->mode & mode->mode))
+      *mbuf++ = tab->letter;
   }
 
   if (oldmode->limit != 0 && mode->limit == 0)
@@ -665,11 +636,11 @@ set_final_mode(struct Mode *mode, struct Mode *oldmode)
   else
     *mbuf++ = '+';
 
-  for (i = 0; flags[i].letter; i++)
+  for (tab = chan_modes; tab->letter; ++tab)
   {
-    if ((flags[i].mode & mode->mode) &&
-        !(flags[i].mode & oldmode->mode))
-      *mbuf++ = flags[i].letter;
+    if ((tab->mode & mode->mode) &&
+        !(tab->mode & oldmode->mode))
+      *mbuf++ = tab->letter;
   }
 
   if (mode->limit != 0 && oldmode->limit != mode->limit)
@@ -759,7 +730,7 @@ remove_a_mode(struct Channel *chptr, struct Client *source_p,
       }
 
       *mbuf = '\0';
-      sendto_channel_local(ALL_MEMBERS, NO, chptr,
+      sendto_channel_local(ALL_MEMBERS, 0, chptr,
                            ":%s MODE %s %s%s",
 			   (IsHidden(source_p) ||
 			   ConfigServerHide.hide_servers) ?
@@ -781,7 +752,7 @@ remove_a_mode(struct Channel *chptr, struct Client *source_p,
       l = ircsprintf(sp, " %s", lpara[i]);
       sp += l;
     }
-    sendto_channel_local(ALL_MEMBERS, NO, chptr,
+    sendto_channel_local(ALL_MEMBERS, 0, chptr,
 			 ":%s MODE %s %s%s",
 			 (IsHidden(source_p) || ConfigServerHide.hide_servers) ?
 			 me.name : source_p->name,
@@ -825,9 +796,9 @@ remove_ban_list(struct Channel *chptr, struct Client *source_p,
     {
       /* NUL-terminate and remove trailing space */
       *mbuf = *(pbuf - 1) = '\0';
-      sendto_channel_local(ALL_MEMBERS, NO, chptr, "%s %s",
+      sendto_channel_local(ALL_MEMBERS, 0, chptr, "%s %s",
                lmodebuf, lparabuf);
-      sendto_server(source_p, NULL, chptr, cap, CAP_TS6, NOFLAGS,
+      sendto_server(source_p, cap, CAP_TS6,
 		    "%s %s", lmodebuf, lparabuf);
 
       cur_len = mlen;
@@ -846,52 +817,34 @@ remove_ban_list(struct Channel *chptr, struct Client *source_p,
   }
 
   *mbuf = *(pbuf - 1) = '\0';
-  sendto_channel_local(ALL_MEMBERS, NO, chptr, "%s %s", lmodebuf, lparabuf);
-  sendto_server(source_p, NULL, chptr, cap, CAP_TS6, NOFLAGS,
+  sendto_channel_local(ALL_MEMBERS, 0, chptr, "%s %s", lmodebuf, lparabuf);
+  sendto_server(source_p, cap, CAP_TS6,
 		"%s %s", lmodebuf, lparabuf);
 }
 
-/*
- * introduce_lazy_link_clients
- *
- * inputs	- pointer to client 
- *		- pointer to server having clients introduce -to-
- *		- pointer to channel being introduced
- * output	- NONE
- * side effects	-
- */
+static struct Message sjoin_msgtab = {
+  "SJOIN", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  {m_unregistered, m_ignore, ms_sjoin, m_ignore, m_ignore, m_ignore}
+};
+
 static void
-introduce_lazy_link_clients(struct Client *client_p, 
-			    struct Client *target_p, struct Channel *chptr)
+module_init(void)
 {
-  struct Client  *lclient_p;
-  dlink_node     *m;
-
-  DLINK_FOREACH(m, serv_list.head)
-  {
-    lclient_p = m->data;
-    
-    /* Hopefully, the server knows about it's own clients. */
-    if (client_p == lclient_p)
-      continue;
-
-    /* Ignore non lazylinks */
-    if (!IsCapable(lclient_p,CAP_LL))
-      continue;
-
-    /* Ignore servers we won't tell anyway */
-    if (!chptr->lazyLinkChannelExists &
-	(lclient_p->localClient->serverMask))
-      continue;
-
-    /* Ignore servers that already know target_p */
-    if (!(target_p->lazyLinkClientExists &
-	  lclient_p->localClient->serverMask))
-    {
-      /* Tell LazyLink Leaf about client_p,
-       * as the leaf is about to get a SJOIN */
-      sendnick_TS(lclient_p, target_p);
-      add_lazylinkclient(lclient_p,target_p);
-    }
-  }
+  mod_add_cmd(&sjoin_msgtab);
 }
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&sjoin_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1750 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = MODULE_FLAG_CORE
+};
