@@ -2,7 +2,7 @@
  *  ircd-hybrid: an advanced Internet Relay Chat Daemon(ircd).
  *
  *  Copyright (C) 2004 Kevin L. Mitchell <klmitch@mit.edu>
- *  Copyright (C) 2006 Hybrid Development Team
+ *  Copyright (C) 2006-2012 Hybrid Development Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,53 +19,28 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_cap.c 575 2006-05-01 11:41:09Z michael $
+ *  $Id: m_cap.c 1309 2012-03-25 11:24:18Z michael $
  */
 
 /** @file
  * @brief Capability negotiation commands
- * @version $Id: m_cap.c 575 2006-05-01 11:41:09Z michael $
+ * @version $Id: m_cap.c 1309 2012-03-25 11:24:18Z michael $
  */
 
 #include "stdinc.h"
-#include "handlers.h"
 #include "client.h"
 #include "hash.h"
 #include "ircd.h"
 #include "numeric.h"
-#include "s_conf.h"
+#include "conf.h"
 #include "s_user.h"
 #include "s_serv.h"
 #include "send.h"
-#include "msg.h"
 #include "parse.h"
 #include "modules.h"
-#include "common.h"
 #include "packet.h"
 #include "irc_string.h"
 
-static void m_cap(struct Client *, struct Client *, int, char *[]);
-
-struct Message cap_msgtab = {
-  "CAP", 0, 0, 2, 0, MFLG_SLOW, 0,
-  { m_cap, m_cap, m_ignore, m_ignore, m_cap, m_ignore }
-};
-
-#ifndef STATIC_MODULES
-void
-_modinit(void)
-{
-  mod_add_cmd(&cap_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&cap_msgtab);
-}
-
-const char *_version = "$Revision: 33 $";
-#endif
 
 #define CAPFL_HIDDEN    0x0001  /**< Do not advertize this capability */
 #define CAPFL_PROHIBIT  0x0002  /**< Client may not set this capability */
@@ -106,7 +81,8 @@ capab_search(const char *key, const struct capabilities *cap)
     else    /* OK, let's move on... */
       rb++;
 
-  /* If the character they differ on happens to be a space, and it happens
+  /*
+   * If the character they differ on happens to be a space, and it happens
    * to be the same length as the capability name, then we've found a
    * match; otherwise, return the difference of the two.
    */
@@ -118,7 +94,7 @@ find_cap(const char **caplist_p, int *neg_p)
 {
   static int inited = 0;
   const char *caplist = *caplist_p;
-  struct capabilities *cap = 0;
+  struct capabilities *cap = NULL;
 
   *neg_p = 0;    /* clear negative flag... */
 
@@ -134,13 +110,15 @@ find_cap(const char **caplist_p, int *neg_p)
     ++caplist;
 
   /* We are now at the beginning of an element of the list; is it negative? */
-  if (*caplist == '-') {
+  if (*caplist == '-')
+  {
     ++caplist;    /* yes; step past the flag... */
     *neg_p = 1;    /* remember that it is negative... */
   }
 
   /* OK, now see if we can look up the capability... */
-  if (*caplist) {
+  if (*caplist)
+  {
     if (!(cap = bsearch(caplist, capab_list, CAPAB_LIST_LEN,
                         sizeof(struct capabilities),
                         (bqcmp)capab_search)))
@@ -161,16 +139,16 @@ find_cap(const char **caplist_p, int *neg_p)
   return cap;    /* and return the capability (if any) */
 }
 
-/** Send a CAP \a subcmd list of capability changes to \a sptr.
+/** Send a CAP \a subcmd list of capability changes to \a source_p.
  * If more than one line is necessary, each line before the last has
  * an added "*" parameter before that line's capability list.
- * @param[in] sptr Client receiving capability list.
+ * @param[in] source_p Client receiving capability list.
  * @param[in] set Capabilities to show as set (with ack and sticky modifiers).
  * @param[in] rem Capabalities to show as removed (with no other modifier).
  * @param[in] subcmd Name of capability subcommand.
  */
 static int
-send_caplist(struct Client *sptr, unsigned int set,
+send_caplist(struct Client *source_p, unsigned int set,
              unsigned int rem, const char *subcmd)
 {
   char capbuf[IRCD_BUFSIZE] = "", pfx[16];
@@ -179,7 +157,7 @@ send_caplist(struct Client *sptr, unsigned int set,
 
   /* set up the buffer for the final LS message... */
   clen = snprintf(cmdbuf, sizeof(capbuf), ":%s CAP %s %s ", me.name,
-                  sptr->name[0] ? sptr->name : "*", subcmd);
+                  source_p->name[0] ? source_p->name : "*", subcmd);
 
   for (i = 0, loc = 0; i < CAPAB_LIST_LEN; ++i)
   {
@@ -218,7 +196,7 @@ send_caplist(struct Client *sptr, unsigned int set,
     if (sizeof(capbuf) < (clen + loc + len + 15))
     {
       /* would add too much; must flush */
-      sendto_one(sptr, "%s* :%s", cmdbuf, capbuf);
+      sendto_one(source_p, "%s* :%s", cmdbuf, capbuf);
       capbuf[(loc = 0)] = '\0';    /* re-terminate the buffer... */
     }
 
@@ -226,39 +204,39 @@ send_caplist(struct Client *sptr, unsigned int set,
                     "%s%s", pfx, capab_list[i].name);
   }
 
-  sendto_one(sptr, "%s:%s", cmdbuf, capbuf);
+  sendto_one(source_p, "%s:%s", cmdbuf, capbuf);
 
   return 0;    /* convenience return */
 }
 
 static int
-cap_ls(struct Client *sptr, const char *caplist)
+cap_ls(struct Client *source_p, const char *caplist)
 {
-  if (IsUnknown(sptr)) /* registration hasn't completed; suspend it... */
-    sptr->localClient->registration |= REG_NEED_CAP;
+  if (IsUnknown(source_p)) /* registration hasn't completed; suspend it... */
+    source_p->localClient->registration |= REG_NEED_CAP;
 
-  return send_caplist(sptr, 0, 0, "LS"); /* send list of capabilities */
+  return send_caplist(source_p, 0, 0, "LS"); /* send list of capabilities */
 }
 
 static int
-cap_req(struct Client *sptr, const char *caplist)
+cap_req(struct Client *source_p, const char *caplist)
 {
   const char *cl = caplist;
-  struct capabilities *cap;
+  struct capabilities *cap = NULL;
   unsigned int set = 0, rem = 0;
-  unsigned int cs = sptr->localClient->cap_client; /* capability set */
-  unsigned int as = sptr->localClient->cap_active; /* active set */
-  int neg;
+  unsigned int cs = source_p->localClient->cap_client; /* capability set */
+  unsigned int as = source_p->localClient->cap_active; /* active set */
+  int neg = 0;
 
-  if (IsUnknown(sptr)) /* registration hasn't completed; suspend it... */
-    sptr->localClient->registration |= REG_NEED_CAP;
+  if (IsUnknown(source_p)) /* registration hasn't completed; suspend it... */
+    source_p->localClient->registration |= REG_NEED_CAP;
 
   while (cl) { /* walk through the capabilities list... */
     if (!(cap = find_cap(&cl, &neg)) /* look up capability... */
 	|| (!neg && (cap->flags & CAPFL_PROHIBIT)) /* is it prohibited? */
         || (neg && (cap->flags & CAPFL_STICKY))) { /* is it sticky? */
-      sendto_one(sptr, ":%s CAP %s NAK :%s", me.name,
-                 sptr->name[0] ? sptr->name : "*", caplist);
+      sendto_one(source_p, ":%s CAP %s NAK :%s", me.name,
+                 source_p->name[0] ? source_p->name : "*", caplist);
       return 0; /* can't complete requested op... */
     }
 
@@ -284,20 +262,20 @@ cap_req(struct Client *sptr, const char *caplist)
   }
 
   /* Notify client of accepted changes and copy over results. */
-  send_caplist(sptr, set, rem, "ACK");
+  send_caplist(source_p, set, rem, "ACK");
 
-  sptr->localClient->cap_client = cs;
-  sptr->localClient->cap_active = as;
+  source_p->localClient->cap_client = cs;
+  source_p->localClient->cap_active = as;
 
   return 0;
 }
 
 static int
-cap_ack(struct Client *sptr, const char *caplist)
+cap_ack(struct Client *source_p, const char *caplist)
 {
   const char *cl = caplist;
   struct capabilities *cap = NULL;
-  int neg;
+  int neg = 0;
 
   /*
    * Coming from the client, this generally indicates that the client
@@ -308,23 +286,23 @@ cap_ack(struct Client *sptr, const char *caplist)
   {
     /* walk through the capabilities list... */
     if (!(cap = find_cap(&cl, &neg)) || /* look up capability... */
-	(neg ? (sptr->localClient->cap_active & cap->cap) :
-              !(sptr->localClient->cap_active & cap->cap))) /* uh... */
+	(neg ? (source_p->localClient->cap_active & cap->cap) :
+              !(source_p->localClient->cap_active & cap->cap))) /* uh... */
       continue;
 
     if (neg)    /* set or clear the active capability... */
-      sptr->localClient->cap_active &= ~cap->cap;
+      source_p->localClient->cap_active &= ~cap->cap;
     else
-      sptr->localClient->cap_active |=  cap->cap;
+      source_p->localClient->cap_active |=  cap->cap;
   }
 
   return 0;
 }
 
 static int
-cap_clear(struct Client *sptr, const char *caplist)
+cap_clear(struct Client *source_p, const char *caplist)
 {
-  struct capabilities *cap;
+  struct capabilities *cap = NULL;
   unsigned int ii;
   unsigned int cleared = 0;
 
@@ -333,35 +311,32 @@ cap_clear(struct Client *sptr, const char *caplist)
     cap = &capab_list[ii];
 
     /* Only clear active non-sticky capabilities. */
-    if (!(sptr->localClient->cap_active & cap->cap) || (cap->flags & CAPFL_STICKY))
+    if (!(source_p->localClient->cap_active & cap->cap) || (cap->flags & CAPFL_STICKY))
       continue;
 
     cleared |= cap->cap;
-    sptr->localClient->cap_client &= ~cap->cap;
+    source_p->localClient->cap_client &= ~cap->cap;
 
     if (!(cap->flags & CAPFL_PROTO))
-      sptr->localClient->cap_active &= ~cap->cap;
+      source_p->localClient->cap_active &= ~cap->cap;
   }
 
-  return send_caplist(sptr, 0, cleared, "ACK");
+  return send_caplist(source_p, 0, cleared, "ACK");
 }
 
 static int
-cap_end(struct Client *sptr, const char *caplist)
+cap_end(struct Client *source_p, const char *caplist)
 {
-  if (!IsUnknown(sptr))    /* registration has completed... */
+  if (!IsUnknown(source_p))    /* registration has completed... */
     return 0;    /* so just ignore the message... */
 
   /* capability negotiation is now done... */
-  sptr->localClient->registration &= ~REG_NEED_CAP;
+  source_p->localClient->registration &= ~REG_NEED_CAP;
 
   /* if client is now done... */
-  if (!sptr->localClient->registration)
+  if (!source_p->localClient->registration)
   {
-    char buf[USERLEN + 1];
-
-    strlcpy(buf, sptr->username, sizeof(buf));
-    register_local_user(sptr, sptr, sptr->name, buf);
+    register_local_user(source_p);
     return 0;
   }
 
@@ -369,16 +344,16 @@ cap_end(struct Client *sptr, const char *caplist)
 }
 
 static int
-cap_list(struct Client *sptr, const char *caplist)
+cap_list(struct Client *source_p, const char *caplist)
 {
   /* Send the list of the client's capabilities */
-  return send_caplist(sptr, sptr->localClient->cap_client, 0, "LIST");
+  return send_caplist(source_p, source_p->localClient->cap_client, 0, "LIST");
 }
 
 static struct subcmd
 {
   const char *cmd;
-  int (*proc)(struct Client *sptr, const char *caplist);
+  int (*proc)(struct Client *, const char *);
 } cmdlist[] = {
   { "ACK",   cap_ack   },
   { "CLEAR", cap_clear },
@@ -396,18 +371,18 @@ subcmd_search(const char *cmd, const struct subcmd *elem)
 }
 
 /** Handle a capability request or response from a client.
- * \param cptr Client that sent us the message.
- * \param sptr Original source of message.
- * \param parc Number of arguments.
- * \param parv Argument vector.
+ * \param client_p Client that sent us the message.
+ * \param source_p Original source of message.
+ * \param parc     Number of arguments.
+ * \param parv     Argument vector.
  */
 static void
-m_cap(struct Client *cptr, struct Client *sptr, int parc, char *parv[])
+m_cap(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
 {
-  char *subcmd, *caplist = 0;
+  const char *subcmd = NULL, *caplist = NULL;
   struct subcmd *cmd = NULL;
 
-  if (parc < 2)    /* a subcommand is required */
+  if (EmptyString(parv[1]))    /* a subcommand is required */
     return;
 
   subcmd = parv[1];
@@ -420,12 +395,39 @@ m_cap(struct Client *cptr, struct Client *sptr, int parc, char *parv[])
                       sizeof(cmdlist) / sizeof(struct subcmd),
                       sizeof(struct subcmd), (bqcmp)subcmd_search)))
   {
-    sendto_one(sptr, form_str(ERR_INVALIDCAPCMD),
-               me.name, sptr->name, subcmd);
+    sendto_one(source_p, form_str(ERR_INVALIDCAPCMD), me.name,
+               source_p->name[0] ? source_p->name : "*", subcmd);
     return;
   }
 
   /* then execute it... */
   if (cmd->proc)
-    (cmd->proc)(sptr, caplist);
+    (cmd->proc)(source_p, caplist);
 }
+
+static struct Message cap_msgtab = {
+  "CAP", 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
+  { m_cap, m_cap, m_ignore, m_ignore, m_cap, m_ignore }
+};
+
+static void
+module_init(void)
+{
+  mod_add_cmd(&cap_msgtab);
+}
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&cap_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1309 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = 0
+};
