@@ -19,43 +19,18 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_close.c 33 2005-10-02 20:50:00Z knight $
+ *  $Id: m_close.c 1243 2011-09-30 10:47:53Z michael $
  */
 
 #include "stdinc.h"
-#include "tools.h"
-#include "handlers.h"
+#include "list.h"
 #include "client.h"
 #include "ircd.h"
 #include "numeric.h"
-#include "fdlist.h"
-#include "s_bsd.h"
 #include "send.h"
-#include "msg.h"
 #include "parse.h"
 #include "modules.h"
 
-static void mo_close(struct Client *, struct Client *, int, char **);
-
-struct Message close_msgtab = {
-  "CLOSE", 0, 0, 0, 0, MFLG_SLOW, 0,
-  {m_unregistered, m_not_oper, m_ignore, m_ignore, mo_close, m_ignore}
-};
-#ifndef STATIC_MODULES
-void
-_modinit(void)
-{
-  mod_add_cmd(&close_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&close_msgtab);
-}
-
-const char *_version = "$Revision: 33 $";
-#endif
 
 /*
  * mo_close - CLOSE message handler
@@ -65,31 +40,51 @@ static void
 mo_close(struct Client *client_p, struct Client *source_p,
          int parc, char *parv[])
 {
-  struct Client *target_p;
-  dlink_node *ptr;
-  dlink_node *ptr_next;
-  unsigned int closed = 0;
+  dlink_node *ptr = NULL, *ptr_next = NULL;
+  unsigned int closed = dlink_list_length(&unknown_list);
+
 
   DLINK_FOREACH_SAFE(ptr, ptr_next, unknown_list.head)
   {
-    target_p = ptr->data;
+    struct Client *target_p = ptr->data;
 
-  /* Which list would connecting servers be found in? serv_list ? */
-#if 0
-      if (!IsUnknown(target_p) && !IsConnecting(target_p) &&
-          !IsHandshake(target_p) && !IsDoingKauth(target_p))
-        continue;
-#endif
-    sendto_one(source_p, form_str(RPL_CLOSING), me.name, parv[0],
+    sendto_one(source_p, form_str(RPL_CLOSING), me.name, source_p->name,
                get_client_name(target_p, SHOW_IP), target_p->status);
-    /* exit here is safe, because it is guaranteed not to be source_p
+
+    /*
+     * exit here is safe, because it is guaranteed not to be source_p
      * because it is unregistered and source_p is an oper.
      */
     exit_client(target_p, target_p, "Oper Closing");
-    closed++;
   }
 
   sendto_one(source_p, form_str(RPL_CLOSEEND),
              me.name, source_p->name, closed);
 }
 
+static struct Message close_msgtab = {
+  "CLOSE", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  { m_unregistered, m_not_oper, m_ignore, m_ignore, mo_close, m_ignore }
+};
+
+static void
+module_init(void)
+{
+  mod_add_cmd(&close_msgtab);
+}
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&close_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1243 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = 0
+};
