@@ -19,49 +19,24 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_connect.c 33 2005-10-02 20:50:00Z knight $
+ *  $Id: m_connect.c 1474 2012-07-22 14:44:07Z michael $
  */
 
 #include "stdinc.h"
-#include "handlers.h"
 #include "client.h"
 #include "ircd.h"
 #include "irc_string.h"
 #include "numeric.h"
 #include "fdlist.h"
 #include "s_bsd.h"
-#include "s_conf.h"
-#include "s_log.h"
+#include "conf.h"
+#include "log.h"
 #include "s_serv.h"
 #include "send.h"
-#include "msg.h"
 #include "parse.h"
 #include "hash.h"
 #include "modules.h"
 
-static void mo_connect(struct Client *, struct Client *, int, char **);
-static void ms_connect(struct Client *, struct Client *, int, char **);
-
-struct Message connect_msgtab = {
-  "CONNECT", 0, 0, 2, 0, MFLG_SLOW, 0,
-  {m_unregistered, m_not_oper, ms_connect, m_ignore, mo_connect, m_ignore}
-};
-
-#ifndef STATIC_MODULES
-void
-_modinit(void)
-{
-  mod_add_cmd(&connect_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&connect_msgtab);
-}
-
-const char *_version = "$Revision: 33 $";
-#endif
 
 /*
  * mo_connect - CONNECT command handler
@@ -75,35 +50,37 @@ const char *_version = "$Revision: 33 $";
  *      parv[3] = remote server
  */
 static void
-mo_connect(struct Client* client_p, struct Client* source_p,
-           int parc, char* parv[])
+mo_connect(struct Client *client_p, struct Client *source_p,
+           int parc, char *parv[])
 {
   int port;
   int tmpport;
   struct ConfItem *conf = NULL;
   struct AccessItem *aconf = NULL;
-  struct Client *target_p;
+  const struct Client *target_p = NULL;
 
-  /* always privileged with handlers */
-  if (MyConnect(source_p) && !IsOperRemote(source_p) && parc > 3)
-  {
-    sendto_one(source_p, form_str(ERR_NOPRIVS),
-               me.name, source_p->name, "connect");
-    return;
-  }
-
-  if (hunt_server(client_p, source_p,
-                  ":%s CONNECT %s %s :%s", 3, parc, parv) != HUNTED_ISME)
-    return;
-
-  if (*parv[1] == '\0')
+  if (EmptyString(parv[1]))
   {
     sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS),
                me.name, source_p->name, "CONNECT");
     return;
   }
 
-  if ((target_p = find_server(parv[1])))
+  if (parc > 3)
+  {
+    if (!HasOFlag(source_p, OPER_FLAG_REMOTE))
+    {
+      sendto_one(source_p, form_str(ERR_NOPRIVS),
+                 me.name, source_p->name, "connect");
+      return;
+    }
+
+    if (hunt_server(client_p, source_p, ":%s CONNECT %s %s :%s", 3,
+                    parc, parv) != HUNTED_ISME)
+      return;
+  }
+
+  if ((target_p = hash_find_server(parv[1])))
   {
     sendto_one(source_p,
 	       ":%s NOTICE %s :Connect: Server %s already exists from %s.",
@@ -121,14 +98,14 @@ mo_connect(struct Client* client_p, struct Client* source_p,
 					   NULL, NULL, parv[1], 0)) != NULL)
     aconf = (struct AccessItem *)map_to_conf(conf);
   
-  if (conf == NULL)
+  if (conf == NULL || aconf == NULL)
   {
     sendto_one(source_p,
 	       ":%s NOTICE %s :Connect: Host %s not listed in ircd.conf",
 	       me.name, source_p->name, parv[1]);
     return;
   }
-    
+
   /* Get port number from user, if given. If not specified,
    * use the default form configuration structure. If missing
    * from there, then use the precompiled default.
@@ -161,7 +138,7 @@ mo_connect(struct Client* client_p, struct Client* source_p,
   /*
    * Notify all operators about remote connect requests
    */
-  ilog(L_TRACE, "CONNECT From %s : %s %s", 
+  ilog(LOG_TYPE_IRCD, "CONNECT From %s : %s %s", 
        source_p->name, parv[1], parv[2] ? parv[2] : "");
 
   aconf->port = port;
@@ -171,7 +148,7 @@ mo_connect(struct Client* client_p, struct Client* source_p,
    */
   if (serv_connect(aconf, source_p))
   {
-    if (!ConfigServerHide.hide_server_ips && IsAdmin(source_p))
+    if (!ConfigServerHide.hide_server_ips && HasUMode(source_p, UMODE_ADMIN))
       sendto_one(source_p, ":%s NOTICE %s :*** Connecting to %s[%s].%d",
                  me.name, source_p->name, aconf->host,
                  conf->name, aconf->port);
@@ -210,20 +187,20 @@ ms_connect(struct Client *client_p, struct Client *source_p,
   int tmpport;
   struct ConfItem *conf = NULL;
   struct AccessItem *aconf = NULL;
-  struct Client *target_p;
+  const struct Client *target_p = NULL;
 
   if (hunt_server(client_p, source_p,
                   ":%s CONNECT %s %s :%s", 3, parc, parv) != HUNTED_ISME)
     return;
 
-  if (*parv[1] == '\0')
+  if (EmptyString(parv[1]))
   {
     sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS),
                me.name, source_p->name, "CONNECT");
     return;
   }
 
-  if ((target_p = find_server(parv[1])))
+  if ((target_p = hash_find_server(parv[1])))
   {
     sendto_one(source_p,
 	       ":%s NOTICE %s :Connect: Server %s already exists from %s.",
@@ -236,12 +213,12 @@ ms_connect(struct Client *client_p, struct Client *source_p,
    */
   if ((conf = find_matching_name_conf(SERVER_TYPE,
 				      parv[1], NULL, NULL, 0)) != NULL)
-    aconf = (struct AccessItem *)map_to_conf(conf);
+    aconf = map_to_conf(conf);
   else if ((conf = find_matching_name_conf(SERVER_TYPE,
 					   NULL, NULL, parv[1], 0)) != NULL)
-    aconf = (struct AccessItem *)map_to_conf(conf);
+    aconf = map_to_conf(conf);
 
-  if (aconf == NULL)
+  if (conf == NULL || aconf == NULL)
   {
     sendto_one(source_p,
 	       ":%s NOTICE %s :Connect: Host %s not listed in ircd.conf",
@@ -249,8 +226,6 @@ ms_connect(struct Client *client_p, struct Client *source_p,
     return;
   }
 
-  assert(aconf != NULL);
-
   /* Get port number from user, if given. If not specified,
    * use the default form configuration structure. If missing
    * from there, then use the precompiled default.
@@ -290,16 +265,20 @@ ms_connect(struct Client *client_p, struct Client *source_p,
    */
   sendto_wallops_flags(UMODE_WALLOP, &me, "Remote CONNECT %s %d from %s",
                        parv[1], port, source_p->name);
-  sendto_server(NULL, NULL, NULL, NOCAPS, NOCAPS, NOFLAGS,
+  sendto_server(NULL, NOCAPS, CAP_TS6,
                 ":%s WALLOPS :Remote CONNECT %s %d from %s",
                 me.name, parv[1], port, source_p->name);
+  sendto_server(NULL, CAP_TS6, NOCAPS,
+                ":%s WALLOPS :Remote CONNECT %s %d from %s",
+                me.id, parv[1], port, source_p->name);
 
-  ilog(L_TRACE, "CONNECT From %s : %s %d", 
+  ilog(LOG_TYPE_IRCD, "CONNECT From %s : %s %d", 
        source_p->name, parv[1], port);
 
   aconf->port = port;
 
-  /* at this point we should be calling connect_server with a valid
+  /*
+   * At this point we should be calling connect_server with a valid
    * C:line and a valid port in the C:line
    */
   if (serv_connect(aconf, source_p))
@@ -308,9 +287,36 @@ ms_connect(struct Client *client_p, struct Client *source_p,
   else
     sendto_one(source_p, ":%s NOTICE %s :*** Couldn't connect to %s.%d",
                me.name, source_p->name, conf->name, aconf->port);
-  /* client is either connecting with all the data it needs or has been
+  /*
+   * Client is either connecting with all the data it needs or has been
    * destroyed
    */
   aconf->port = tmpport;
 }
 
+static struct Message connect_msgtab = {
+  "CONNECT", 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
+  { m_unregistered, m_not_oper, ms_connect, m_ignore, mo_connect, m_ignore }
+};
+
+static void
+module_init(void)
+{
+  mod_add_cmd(&connect_msgtab);
+}
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&connect_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1474 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = 0
+};
