@@ -19,60 +19,24 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_help.c 575 2006-05-01 11:41:09Z michael $
+ *  $Id: m_help.c 1738 2013-01-14 17:38:34Z michael $
  */
 
 #include "stdinc.h"
-#include "handlers.h"
 #include "client.h"
 #include "ircd.h"
-#include "ircd_handler.h"
-#include "msg.h"
 #include "numeric.h"
 #include "send.h"
-#include "s_conf.h"
-#include "s_log.h"
+#include "conf.h"
 #include "parse.h"
 #include "modules.h"
 #include "irc_string.h"
 
-#define HPATH  IRCD_PREFIX "/help/opers"
-#define UHPATH IRCD_PREFIX "/help/users"
 #define HELPLEN 400
 
-static void m_help(struct Client *, struct Client *, int, char *[]);
-static void mo_help(struct Client *, struct Client *, int, char *[]);
-static void mo_uhelp(struct Client *, struct Client *, int, char *[]);
 static void dohelp(struct Client *, const char *, char *);
 static void sendhelpfile(struct Client *, const char *, const char *);
 
-struct Message help_msgtab = {
-  "HELP", 0, 0, 0, 0, MFLG_SLOW, 0,
-  {m_unregistered, m_help, m_ignore, m_ignore, mo_help, m_ignore}
-};
-
-struct Message uhelp_msgtab = {
-  "UHELP", 0, 0, 0, 0, MFLG_SLOW, 0,
-  {m_unregistered, m_help, m_ignore, m_ignore, mo_uhelp, m_ignore}
-};
-
-#ifndef STATIC_MODULES
-void
-_modinit(void)
-{
-  mod_add_cmd(&help_msgtab);
-  mod_add_cmd(&uhelp_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&help_msgtab);
-  mod_del_cmd(&uhelp_msgtab);
-}
-
-const char *_version = "$Revision: 575 $";
-#endif
 
 /*
  * m_help - HELP message handler
@@ -125,7 +89,7 @@ static void
 dohelp(struct Client *source_p, const char *hpath, char *topic)
 {
   char h_index[] = "index";
-  char path[PATH_MAX + 1];
+  char path[HYB_PATH_MAX + 1];
   struct stat sb;
   int i;
 
@@ -150,7 +114,7 @@ dohelp(struct Client *source_p, const char *hpath, char *topic)
     return;
   }
 
-  if (strlen(hpath) + strlen(topic) + 1 > PATH_MAX)
+  if (strlen(hpath) + strlen(topic) + 1 > HYB_PATH_MAX)
   {
     sendto_one(source_p, form_str(ERR_HELPNOTFOUND),
                me.name, source_p->name, topic);
@@ -161,21 +125,17 @@ dohelp(struct Client *source_p, const char *hpath, char *topic)
 
   if (stat(path, &sb) < 0)
   {
-    ilog(L_NOTICE, "help file %s not found", path);
     sendto_one(source_p, form_str(ERR_HELPNOTFOUND),
                me.name, source_p->name, topic);
     return;
   }
 
-#ifndef _WIN32
   if (!S_ISREG(sb.st_mode))
   {
-    ilog(L_NOTICE, "help file %s not found", path);
     sendto_one(source_p, form_str(ERR_HELPNOTFOUND),
                me.name, source_p->name, topic);
     return;
   }
-#endif
 
   sendhelpfile(source_p, path, topic);
 }
@@ -183,19 +143,19 @@ dohelp(struct Client *source_p, const char *hpath, char *topic)
 static void 
 sendhelpfile(struct Client *source_p, const char *path, const char *topic)
 {
-  FBFILE *file;
+  FILE *file;
   char line[HELPLEN];
   char started = 0;
   int type;
 
-  if ((file = fbopen(path, "r")) == NULL)
+  if ((file = fopen(path, "r")) == NULL)
   {
     sendto_one(source_p, form_str(ERR_HELPNOTFOUND),
                me.name, source_p->name, topic);
     return;
   }
 
-  if (fbgets(line, sizeof(line), file) == NULL)
+  if (fgets(line, sizeof(line), file) == NULL)
   {
     sendto_one(source_p, form_str(ERR_HELPNOTFOUND),
                me.name, source_p->name, topic);
@@ -210,15 +170,15 @@ sendhelpfile(struct Client *source_p, const char *path, const char *topic)
     started = 1;
   }
 
-  while (fbgets(line, sizeof(line), file))
+  while (fgets(line, sizeof(line), file))
   {
     line[strlen(line) - 1] = '\0';
-    if(line[0] != '#')
+    if (line[0] != '#')
     {
       if (!started)
       {
         type = RPL_HELPSTART;
-	started = 1;
+        started = 1;
       }
       else
         type = RPL_HELPTXT;
@@ -228,9 +188,43 @@ sendhelpfile(struct Client *source_p, const char *path, const char *topic)
     }
   }
 
-  fbclose(file);
+  fclose(file);
   sendto_one(source_p, form_str(RPL_HELPTXT),
              me.name, source_p->name, topic, "");
   sendto_one(source_p, form_str(RPL_ENDOFHELP),
              me.name, source_p->name, topic);
 }
+
+static struct Message help_msgtab = {
+  "HELP", 0, 0, 0, 0, MFLG_SLOW, 0,
+  {m_unregistered, m_help, m_ignore, m_ignore, mo_help, m_ignore}
+};
+
+static struct Message uhelp_msgtab = {
+  "UHELP", 0, 0, 0, 0, MFLG_SLOW, 0,
+  {m_unregistered, m_help, m_ignore, m_ignore, mo_uhelp, m_ignore}
+};
+
+static void
+module_init(void)
+{
+  mod_add_cmd(&help_msgtab);
+  mod_add_cmd(&uhelp_msgtab);
+}
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&help_msgtab);
+  mod_del_cmd(&uhelp_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1738 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = 0
+};
