@@ -19,68 +19,28 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_info.c 201 2005-11-01 11:41:52Z adx $
+ *  $Id: m_info.c 1766 2013-01-19 18:22:39Z michael $
  */
 
 #include "stdinc.h"
-#include "tools.h"
-#include "m_info.h"
+#include "list.h"
 #include "channel.h"
 #include "client.h"
-#include "common.h"
 #include "irc_string.h"
 #include "ircd.h"
-#include "hook.h"
 #include "numeric.h"
-#include "s_log.h"
 #include "s_serv.h"
 #include "s_user.h"
 #include "send.h"
-#include "s_conf.h"
-#include "handlers.h"
-#include "msg.h"
+#include "conf.h"
 #include "parse.h"
 #include "modules.h"
 
+
 static void send_conf_options(struct Client *);
 static void send_birthdate_online_time(struct Client *);
 static void send_info_text(struct Client *);
 
-static void m_info(struct Client *, struct Client *, int, char *[]);
-static void ms_info(struct Client *, struct Client *, int, char *[]);
-static void mo_info(struct Client *, struct Client *, int, char *[]);
-
-struct Message info_msgtab = {
-  "INFO", 0, 0, 0, 0, MFLG_SLOW, 0,
-  { m_unregistered, m_info, ms_info, m_ignore, mo_info, m_ignore }
-};
-
-#ifndef STATIC_MODULES
-const char *_version = "$Revision: 201 $";
-static struct Callback *info_cb;
-
-static void *
-va_send_info_text(va_list args)
-{
-  send_info_text(va_arg(args, struct Client *));
-  return NULL;
-}
-
-void
-_modinit(void)
-{
-  info_cb = register_callback("doing_info", va_send_info_text);
-  mod_add_cmd(&info_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&info_msgtab);
-  uninstall_hook(info_cb, va_send_info_text);
-}
-#endif
-
 /*
  * jdc -- Structure for our configuration value table
  */
@@ -103,6 +63,31 @@ struct InfoStruct
 static const struct InfoStruct info_table[] =
 {
   /* --[  START OF TABLE  ]-------------------------------------------- */
+
+  {
+    "CPATH",
+    OUTPUT_STRING,
+    &ConfigFileEntry.configfile,
+    "Path to Main Configuration File"
+  },
+  {
+    "DPATH",
+    OUTPUT_STRING,
+    &ConfigFileEntry.dpath,
+    "Directory Containing Configuration Files"
+  },
+  {
+    "DLPATH",
+    OUTPUT_STRING,
+    &ConfigFileEntry.dlinefile,
+    "Path to D-line File"
+  },
+  {
+    "KPATH",
+    OUTPUT_STRING,
+    &ConfigFileEntry.klinefile,
+    "Path to K-line File"
+  },
   {
     "network_name",
     OUTPUT_STRING,
@@ -121,6 +106,24 @@ static const struct InfoStruct info_table[] =
     &ServerInfo.hub,
     "Server is a hub"
   },
+  {
+    "max_clients",
+    OUTPUT_DECIMAL,
+    &ServerInfo.max_clients,
+    "Maximum number of clients permitted simultaneously on this server"
+  },
+  {
+    "max_nick_length",
+    OUTPUT_DECIMAL,
+    &ServerInfo.max_nick_length,
+    "Maximum nickname length"
+  },
+  {
+    "max_topic_length",
+    OUTPUT_DECIMAL,
+    &ServerInfo.max_topic_length,
+    "Maximum topic length"
+  },
   {
     "use_logging",
     OUTPUT_BOOLEAN_YN,
@@ -128,34 +131,10 @@ static const struct InfoStruct info_table[] =
     "Enable logging"
   },
   {
-    "fuserlog",
-    OUTPUT_STRING_PTR,
-    &ConfigLoggingEntry.userlog,
-    "User log file"
-  },
-  {
-    "foperlog",
-    OUTPUT_STRING_PTR,
-    &ConfigLoggingEntry.operlog,
-    "Operator log file"
-  },
-  {
-    "fkilllog",
-    OUTPUT_STRING_PTR,
-    &ConfigLoggingEntry.killlog,
-    "Kill log file"
-  },
-  {
-    "fklinelog",
-    OUTPUT_STRING_PTR,
-    &ConfigLoggingEntry.klinelog,
-    "K-Line log file"
-  },
-  {
-    "fglinelog",
-    OUTPUT_STRING_PTR,
-    &ConfigLoggingEntry.glinelog,
-    "G-Line log file"
+    "disable_fake_channels",
+    OUTPUT_BOOLEAN_YN,
+    &ConfigChannel.disable_fake_channels,
+    "Forbids channels with special ASCII characters in their name"
   },
   {
     "restrict_channels",
@@ -163,30 +142,6 @@ static const struct InfoStruct info_table[] =
     &ConfigChannel.restrict_channels,
     "Only reserved channels are allowed"
   },
-  {
-    "disable_local_channels",
-    OUTPUT_BOOLEAN_YN,
-    &ConfigChannel.disable_local_channels,
-    "Prevent users from joining &channels"
-  },
- {
-    "use_invex",
-    OUTPUT_BOOLEAN_YN,
-    &ConfigChannel.use_invex,
-    "Enable chanmode +I (invite exceptions)"
-  },
-  {
-    "use_except",
-    OUTPUT_BOOLEAN_YN,
-    &ConfigChannel.use_except,
-    "Enable chanmode +e (ban exceptions)"
-  },
-  {
-    "use_knock",
-    OUTPUT_BOOLEAN_YN,
-    &ConfigChannel.use_knock,
-    "Enable /KNOCK"
-  },
   {
     "knock_delay",
     OUTPUT_DECIMAL,
@@ -205,6 +160,12 @@ static const struct InfoStruct info_table[] =
     &ConfigChannel.max_chans_per_user,
     "Maximum number of channels a user can join"
   },
+  {
+    "max_chans_per_oper",
+    OUTPUT_DECIMAL,
+    &ConfigChannel.max_chans_per_oper,
+    "Maximum number of channels an oper can join"
+  },
   {
     "quiet_on_ban",
     OUTPUT_BOOLEAN_YN,
@@ -241,12 +202,6 @@ static const struct InfoStruct info_table[] =
     &ConfigChannel.no_join_on_split,
     "Disallow joining channels when split"
   },
-  {
-    "burst_topicwho",
-    OUTPUT_BOOLEAN_YN,
-    &ConfigChannel.burst_topicwho,
-    "Enable sending of who set topic on topicburst"
-  },
   {
     "flatten_links",
     OUTPUT_BOOLEAN_YN,
@@ -265,12 +220,6 @@ static const struct InfoStruct info_table[] =
     &ConfigServerHide.hidden,
     "Hide this server from a flattened /links on remote servers"
   },
-  {
-    "disable_hidden",
-    OUTPUT_BOOLEAN_YN,
-    &ConfigServerHide.disable_hidden,
-    "Prevent servers from hiding themselves from a flattened /links"
-  },
   {
     "hide_servers",
     OUTPUT_BOOLEAN_YN,
@@ -307,18 +256,6 @@ static const struct InfoStruct info_table[] =
     &ConfigFileEntry.invisible_on_connect,
     "Automatically set mode +i on connecting users"
   },
-  {
-    "burst_away",
-    OUTPUT_BOOLEAN_YN,
-    &ConfigFileEntry.burst_away,
-    "Send /away string that users have set on the server burst"
-  },
-  {
-    "use_whois_actually",
-    OUTPUT_BOOLEAN_YN,
-    &ConfigFileEntry.use_whois_actually,
-    "Show IP address on /WHOIS when possible"
-  },
   {
     "kill_chase_time_limit",
     OUTPUT_DECIMAL,
@@ -373,12 +310,6 @@ static const struct InfoStruct info_table[] =
     &ConfigFileEntry.dots_in_ident,
     "Number of permissable dots in an ident"
   },
-  {
-    "dot_in_ip6_addr",
-    OUTPUT_BOOLEAN,
-    &ConfigFileEntry.dot_in_ip6_addr,
-    "Suffix a . to ip6 addresses (for linked servers still running hybrid-6)"
-  },
   {
     "min_nonwildcard",
     OUTPUT_DECIMAL,
@@ -433,24 +364,18 @@ static const struct InfoStruct info_table[] =
     &ConfigFileEntry.ts_max_delta,
     "Maximum permitted TS delta from another server"
   },
-  {
-    "kline_with_reason",
-    OUTPUT_BOOLEAN_YN,
-    &ConfigFileEntry.kline_with_reason,
-    "Display K-line reason to client on disconnect"
-  },
-  {
-    "kline_reason",
-    OUTPUT_STRING,
-    &ConfigFileEntry.kline_reason,
-    "Reason given to K-lined clients on sign off"
-  },
   {
     "warn_no_nline",
     OUTPUT_BOOLEAN,
     &ConfigFileEntry.warn_no_nline,
     "Display warning if connecting server lacks N-line"
   },
+  {
+    "stats_e_disabled",
+    OUTPUT_BOOLEAN_YN,
+    &ConfigFileEntry.stats_e_disabled,
+    "Whether or not STATS e is disabled"
+  },
   {
     "stats_o_oper_only",
     OUTPUT_BOOLEAN_YN,
@@ -529,24 +454,12 @@ static const struct InfoStruct info_table[] =
     &ConfigFileEntry.oper_pass_resv,
     "Opers can over-ride RESVs"
   },
-  {
-    "idletime",
-    OUTPUT_DECIMAL,
-    &ConfigFileEntry.idletime,
-    "Number of seconds before a client is considered idle"
-  },
   {
     "max_targets",
     OUTPUT_DECIMAL,
     &ConfigFileEntry.max_targets,
     "The maximum number of PRIVMSG/NOTICE targets"
   },
-  {
-    "client_flood",
-    OUTPUT_DECIMAL,
-    &ConfigFileEntry.client_flood,
-    "Maximum amount of data in a client's queue before they are disconnected"
-  },
   {
     "throttle_time",
     OUTPUT_DECIMAL,
@@ -560,11 +473,19 @@ static const struct InfoStruct info_table[] =
     "G-line (network-wide K-line) support"
   },
   {
-    "duration",
+    "gline_duration",
     OUTPUT_DECIMAL,
     &ConfigFileEntry.gline_time,
     "Expiry time for G-lines"
   },
+
+  {
+    "gline_request_duration",
+    OUTPUT_DECIMAL,
+    &ConfigFileEntry.gline_request_time,
+    "Expiry time for pending G-lines"
+  },
+
   /* --[  END OF TABLE  ]---------------------------------------------- */
   {
     NULL,
@@ -592,23 +513,15 @@ m_info(struct Client *client_p, struct Client *source_p,
                me.name, source_p->name);
     return;
   }
-  else
-    last_used = CurrentTime;
+
+  last_used = CurrentTime;
 
   if (!ConfigFileEntry.disable_remote)
-  {
-    if (hunt_server(client_p,source_p, ":%s INFO :%s",
-                    1, parc, parv) != HUNTED_ISME)
-    {
+    if (hunt_server(client_p,source_p, ":%s INFO :%s", 1,
+                    parc, parv) != HUNTED_ISME)
       return;
-    }
-  }
 
-#ifdef STATIC_MODULES
   send_info_text(source_p);
-#else
-  execute_callback(info_cb, source_p, parc, parv);
-#endif
 }
 
 /*
@@ -624,11 +537,7 @@ mo_info(struct Client *client_p, struct Client *source_p,
                   parc, parv) != HUNTED_ISME)
     return;
 
-#ifdef STATIC_MODULES
   send_info_text(source_p);
-#else
-  execute_callback(info_cb, source_p, parc, parv);
-#endif
 }
 
 /*
@@ -643,15 +552,11 @@ ms_info(struct Client *client_p, struct Client *source_p,
   if (!IsClient(source_p))
       return;
 
-  if (hunt_server(client_p, source_p, ":%s INFO :%s",
-                  1, parc, parv) != HUNTED_ISME)
+  if (hunt_server(client_p, source_p, ":%s INFO :%s", 1,
+                  parc, parv) != HUNTED_ISME)
     return;
 
-#ifdef STATIC_MODULES
   send_info_text(source_p);
-#else
-  execute_callback(info_cb, source_p, parc, parv);
-#endif
 }
 
 /* send_info_text()
@@ -665,7 +570,12 @@ send_info_text(struct Client *source_p)
 {
   const char **text = infotext;
   char *source, *target;
-  
+
+  sendto_realops_flags(UMODE_SPY, L_ALL,
+                       "INFO requested by %s (%s@%s) [%s]",
+                       source_p->name, source_p->username,
+                       source_p->host, source_p->servptr->name);
+
   if (!MyClient(source_p) && IsCapable(source_p->from, CAP_TS6) &&
       HasID(source_p))
     source = me.id, target = source_p->id;
@@ -683,7 +593,7 @@ send_info_text(struct Client *source_p)
                source, target, line);
   }
 
-  if (IsOper(source_p))
+  if (HasUMode(source_p, UMODE_OPER))
     send_conf_options(source_p);
 
   send_birthdate_online_time(source_p);
@@ -705,13 +615,13 @@ send_birthdate_online_time(struct Client *source_p)
   {
     sendto_one(source_p, ":%s %d %s :On-line since %s",
                me.id, RPL_INFO, source_p->id,
-               myctime(me.firsttime));
+               myctime(me.localClient->firsttime));
   }
   else
   {
     sendto_one(source_p, ":%s %d %s :On-line since %s",
                me.name, RPL_INFO, source_p->name,
-               myctime(me.firsttime));
+               myctime(me.localClient->firsttime));
   }
 }
 
@@ -724,12 +634,11 @@ send_birthdate_online_time(struct Client *source_p)
 static void
 send_conf_options(struct Client *source_p)
 {
-  Info *infoptr;
   const char *from, *to;
   const struct InfoStruct *iptr = NULL;
 
   /* Now send them a list of all our configuration options
-   * (mostly from defaults.h and setup.h)
+   * (mostly from defaults.h and config.h)
    */
   if (!MyClient(source_p) && IsCapable(source_p->from, CAP_TS6) && HasID(source_p))
   {
@@ -742,22 +651,6 @@ send_conf_options(struct Client *source_p)
     to = source_p->name;
   }
 
-  for (infoptr = MyInformation; infoptr->name; infoptr++)
-  {
-    if (infoptr->intvalue)
-    {
-      sendto_one(source_p, ":%s %d %s :%-30s %-5d [%-30s]",
-                 from, RPL_INFO, to, infoptr->name,
-                 infoptr->intvalue, infoptr->desc);
-    }
-    else
-    {
-      sendto_one(source_p, ":%s %d %s :%-30s %-5s [%-30s]",
-                 from, RPL_INFO, to, infoptr->name,
-                 infoptr->strvalue, infoptr->desc);
-    }
-  }
-
   /*
    * Parse the info_table[] and do the magic.
    */
@@ -838,15 +731,33 @@ send_conf_options(struct Client *source_p)
     }
   }
 
-  /* Don't send oper_only_umodes...it's a bit mask, we will have to decode it
-   * in order for it to show up properly to opers who issue INFO
-   */
-#ifndef EFNET
-  /* jdc -- Only send compile information to admins. */
-  if (IsAdmin(source_p))
-    sendto_one(source_p, ":%s %d %s :Running on [%s]",
-               from, RPL_INFO, to, ircd_platform); 
-#endif
   sendto_one(source_p, form_str(RPL_INFO),
              from, to, "");
 }
+
+static struct Message info_msgtab = {
+  "INFO", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  { m_unregistered, m_info, ms_info, m_ignore, mo_info, m_ignore }
+};
+
+static void
+module_init(void)
+{
+  mod_add_cmd(&info_msgtab);
+}
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&info_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1766 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = 0
+};
