@@ -19,51 +19,25 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_invite.c 322 2005-12-24 22:04:58Z michael $
+ *  $Id: m_invite.c 1722 2012-12-29 22:15:10Z michael $
  */
 
 #include "stdinc.h"
-#include "tools.h"
-#include "handlers.h"
-#include "common.h"
+#include "list.h"
 #include "channel.h"
 #include "channel_mode.h"
-#include "list.h"
 #include "client.h"
 #include "hash.h"
 #include "irc_string.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "send.h"
-#include "s_conf.h"
+#include "conf.h"
 #include "s_serv.h"
-#include "msg.h"
 #include "parse.h"
 #include "modules.h"
 #include "packet.h"
 
-static void m_invite(struct Client *, struct Client *, int, char *[]);
-
-struct Message invite_msgtab = {
-  "INVITE", 0, 0, 3, 0, MFLG_SLOW, 0,
-  { m_unregistered, m_invite, m_invite, m_ignore, m_invite, m_ignore }
-};
-
-#ifndef STATIC_MODULES
-void
-_modinit(void)
-{
-  mod_add_cmd(&invite_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&invite_msgtab);
-}
-
-const char *_version = "$Revision: 322 $";
-#endif
 
 /*
 ** m_invite
@@ -83,7 +57,7 @@ m_invite(struct Client *client_p, struct Client *source_p,
   if (IsServer(source_p))
     return;
 
-  if (*parv[2] == '\0')
+  if (EmptyString(parv[2]))
   {
     sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS),
                me.name, source_p->name, "INVITE");
@@ -100,21 +74,6 @@ m_invite(struct Client *client_p, struct Client *source_p,
     return;
   }
 
-  /* Do not send local channel invites to users if they are not on the
-   * same server as the person sending the INVITE message. 
-   */
-  /* Possibly should be an error sent to source_p */
-  /* done .. there should be no problem because MyConnect(source_p) should
-   * always be true if parse() and such is working correctly --is
-   */
-  if (!MyConnect(target_p) && (*parv[2] == '&'))
-  {
-    if (ConfigServerHide.hide_servers == 0)
-      sendto_one(source_p, form_str(ERR_USERNOTONSERV),
-                 me.name, source_p->name, target_p->name);
-    return;
-  }
-
   if ((chptr = hash_find_channel(parv[2])) == NULL)
   {
     sendto_one(source_p, form_str(ERR_NOSUCHCHANNEL),
@@ -129,14 +88,11 @@ m_invite(struct Client *client_p, struct Client *source_p,
     return;
   }
 
-  if ((chptr->mode.mode & (MODE_INVITEONLY | MODE_PRIVATE)))
+  if (MyConnect(source_p) && !has_member_flags(ms, CHFL_CHANOP))
   {
-    if (MyConnect(source_p) && !has_member_flags(ms, CHFL_CHANOP|CHFL_HALFOP))
-    {
-      sendto_one(source_p, form_str(ERR_CHANOPRIVSNEEDED),
-                 me.name, source_p->name, chptr->chname);
-      return;
-    }
+    sendto_one(source_p, form_str(ERR_CHANOPRIVSNEEDED),
+               me.name, source_p->name, chptr->chname);
+    return;
   }
 
   if (IsMember(target_p, chptr))
@@ -151,7 +107,7 @@ m_invite(struct Client *client_p, struct Client *source_p,
     sendto_one(source_p, form_str(RPL_INVITING), me.name,
                source_p->name, target_p->name, chptr->chname);
 
-    if (target_p->away)
+    if (target_p->away[0])
       sendto_one(source_p, form_str(RPL_AWAY),
                  me.name, source_p->name, target_p->name,
                  target_p->away);
@@ -160,18 +116,6 @@ m_invite(struct Client *client_p, struct Client *source_p,
     if (atoi(parv[3]) > chptr->channelts)
       return;
 
-  if (!MyConnect(target_p) && ServerInfo.hub &&
-      IsCapable(target_p->from, CAP_LL))
-  {
-    /* target_p is connected to a LL leaf, connected to us */
-    if (IsClient(source_p))
-      client_burst_if_needed(target_p->from, source_p);
-
-    if ((chptr->lazyLinkChannelExists &
-         target_p->from->localClient->serverMask) == 0)
-      burst_channel(target_p->from, chptr);
-  }
-
   if (MyConnect(target_p))
   {
     sendto_one(target_p, ":%s!%s@%s INVITE %s :%s",
@@ -181,19 +125,10 @@ m_invite(struct Client *client_p, struct Client *source_p,
 
     if (chptr->mode.mode & MODE_INVITEONLY)
     {
-      if (chptr->mode.mode & MODE_PRIVATE)
-      {
-        /* Only do this if channel is set +i AND +p */
-        sendto_channel_local(CHFL_CHANOP|CHFL_HALFOP, NO, chptr,
-                             ":%s NOTICE %s :%s is inviting %s to %s.",
-                             me.name, chptr->chname, source_p->name,
-                             target_p->name, chptr->chname);
-        sendto_channel_remote(source_p, client_p, CHFL_CHANOP|CHFL_HALFOP,
-                              NOCAPS, NOCAPS, chptr,
-                              ":%s NOTICE %s :%s is inviting %s to %s.",
-                              source_p->name, chptr->chname, source_p->name,
-                              target_p->name, chptr->chname);
-      }
+      sendto_channel_butone(NULL, &me, chptr, CHFL_CHANOP,
+                            "NOTICE @%s :%s is inviting %s to %s.",
+                            chptr->chname, source_p->name,
+                            target_p->name, chptr->chname);
 
       /* Add the invite if channel is +i */
       add_invite(chptr, target_p);
@@ -205,3 +140,30 @@ m_invite(struct Client *client_p, struct Client *source_p,
                ID_or_name(target_p, target_p->from),
                chptr->chname, (unsigned long)chptr->channelts);
 }
+
+static struct Message invite_msgtab = {
+  "INVITE", 0, 0, 3, MAXPARA, MFLG_SLOW, 0,
+  { m_unregistered, m_invite, m_invite, m_ignore, m_invite, m_ignore }
+};
+
+static void
+module_init(void)
+{
+  mod_add_cmd(&invite_msgtab);
+}
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&invite_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1722 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = 0
+};
