@@ -19,57 +19,32 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_locops.c 33 2005-10-02 20:50:00Z knight $
+ *  $Id: m_locops.c 1474 2012-07-22 14:44:07Z michael $
  */
 
 #include "stdinc.h"
-#include "handlers.h"
 #include "client.h"
 #include "ircd.h"
 #include "irc_string.h"
 #include "numeric.h"
 #include "send.h"
-#include "s_conf.h"
+#include "conf.h"
 #include "s_user.h"
 #include "s_serv.h"
 #include "hash.h"
-#include "msg.h"
 #include "parse.h"
 #include "modules.h"
 
-static void m_locops(struct Client *, struct Client *, int, char *[]);
-static void ms_locops(struct Client *, struct Client *, int, char *[]);
-
-struct Message locops_msgtab = {
-  "LOCOPS", 0, 0, 2, 0, MFLG_SLOW, 0,
-  { m_unregistered, m_not_oper, ms_locops, m_ignore, m_locops, m_ignore }
-};
-
-#ifndef STATIC_MODULES
-void
-_modinit(void)
-{
-  mod_add_cmd(&locops_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&locops_msgtab);
-}
-
-const char *_version = "$Revision: 33 $";
-#endif
 
 /*
- * m_locops - LOCOPS message handler
+ * mo_locops - LOCOPS message handler
  * (write to *all* local opers currently online)
  *      parv[0] = sender prefix
  *      parv[1] = message text
  */
 static void
-m_locops(struct Client *client_p, struct Client *source_p,
-         int parc, char *parv[])
+mo_locops(struct Client *client_p, struct Client *source_p,
+          int parc, char *parv[])
 {
   const char *message = parv[1];
 
@@ -93,7 +68,7 @@ ms_locops(struct Client *client_p, struct Client *source_p,
   if (parc != 3 || EmptyString(parv[2]))
     return;
 
-  sendto_server(client_p, NULL, NULL, CAP_CLUSTER, 0, 0, "LOCOPS %s :%s",
+  sendto_server(client_p, CAP_CLUSTER, 0, "LOCOPS %s :%s",
                 parv[1], parv[2]);
 
   if (!IsClient(source_p) || !match(parv[1], me.name))
@@ -103,3 +78,30 @@ ms_locops(struct Client *client_p, struct Client *source_p,
                               "*", "*", SHARED_LOCOPS))
     sendto_wallops_flags(UMODE_LOCOPS, source_p, "SLOCOPS - %s", parv[2]);
 }
+
+static struct Message locops_msgtab = {
+  "LOCOPS", 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
+  { m_unregistered, m_not_oper, ms_locops, m_ignore, mo_locops, m_ignore }
+};
+
+static void
+module_init(void)
+{
+  mod_add_cmd(&locops_msgtab);
+}
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&locops_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1474 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = 0
+};
