@@ -19,47 +19,21 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_lusers.c 269 2005-11-14 19:39:09Z adx $
+ *  $Id: m_lusers.c 1309 2012-03-25 11:24:18Z michael $
  */
 
 #include "stdinc.h"
-#include "handlers.h"
 #include "client.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "s_serv.h"    /* hunt_server */
 #include "s_user.h"    /* show_lusers */
 #include "send.h"
-#include "s_conf.h"
-#include "msg.h"
+#include "conf.h"
 #include "parse.h"
 #include "modules.h"
 
-/* XXX LazyLinks ? */
 
-static void m_lusers(struct Client*, struct Client*, int, char**);
-static void ms_lusers(struct Client*, struct Client*, int, char**);
-
-struct Message lusers_msgtab = {
-  "LUSERS", 0, 0, 0, 0, MFLG_SLOW, 0,
-  {m_unregistered, m_lusers, ms_lusers, m_ignore, ms_lusers, m_ignore}
-};
-#ifndef STATIC_MODULES
-
-void
-_modinit(void)
-{
-  mod_add_cmd(&lusers_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&lusers_msgtab);
-}
-
-const char *_version = "$Revision: 269 $";
-#endif
 /* m_lusers - LUSERS message handler
  * parv[0] = sender
  * parv[1] = host/server mask.
@@ -72,25 +46,23 @@ const char *_version = "$Revision: 269 $";
  */
 static void
 m_lusers(struct Client *client_p, struct Client *source_p,
-	 int parc, char *parv[])
+         int parc, char *parv[])
 {
   static time_t last_used = 0;
 
   if ((last_used + ConfigFileEntry.pace_wait_simple) > CurrentTime)
   {
     /* safe enough to give this on a local connect only */
-    if (MyClient(source_p))
-      sendto_one(source_p, form_str(RPL_LOAD2HI), me.name, parv[0]);
+    sendto_one(source_p, form_str(RPL_LOAD2HI), me.name, source_p->name);
     return;
   }
-  else
-    last_used = CurrentTime;
+
+  last_used = CurrentTime;
 
   if (parc > 2 && !ConfigFileEntry.disable_remote)
-  {   
-    if (hunt_server(client_p, source_p, ":%s LUSERS %s :%s", 2, parc, parv) != HUNTED_ISME)
+    if (hunt_server(client_p, source_p, ":%s LUSERS %s :%s", 2,
+                    parc, parv) != HUNTED_ISME)
       return;
-  }
 
   show_lusers(source_p);
 }
@@ -102,15 +74,40 @@ m_lusers(struct Client *client_p, struct Client *source_p,
  */
 static void
 ms_lusers(struct Client *client_p, struct Client *source_p,
-	  int parc, char *parv[])
+          int parc, char *parv[])
 {
   if (parc > 2)
-  {
-    if(hunt_server(client_p, source_p, ":%s LUSERS %s :%s", 2, parc, parv)
-     != HUNTED_ISME)
+    if (hunt_server(client_p, source_p, ":%s LUSERS %s :%s", 2,
+                    parc, parv) != HUNTED_ISME)
         return;
-  }
 
   if (IsClient(source_p))
     show_lusers(source_p);
 }
+
+static struct Message lusers_msgtab = {
+  "LUSERS", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  {m_unregistered, m_lusers, ms_lusers, m_ignore, ms_lusers, m_ignore}
+};
+
+static void
+module_init(void)
+{
+  mod_add_cmd(&lusers_msgtab);
+}
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&lusers_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1309 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = 0
+};
