@@ -19,80 +19,30 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_motd.c 589 2006-05-08 18:40:00Z michael $
+ *  $Id: m_motd.c 1309 2012-03-25 11:24:18Z michael $
  */
 
 #include "stdinc.h"
+#include "list.h"
 #include "client.h"
-#include "tools.h"
 #include "motd.h"
 #include "ircd.h"
 #include "send.h"
 #include "numeric.h"
-#include "handlers.h"
-#include "hook.h"
-#include "msg.h"
 #include "s_serv.h"     /* hunt_server */
 #include "parse.h"
 #include "modules.h"
-#include "s_conf.h"
+#include "conf.h"
 
-static void mr_motd(struct Client *, struct Client *, int, char *[]);
-static void m_motd(struct Client*, struct Client*, int, char *[]);
-static void mo_motd(struct Client*, struct Client*, int, char *[]);
 
-/*
- * note regarding mo_motd being used twice:
- * this is not a kludge.  any rate limiting, shide, or whatever
- * other access restrictions should be done by the source's server.
- * for security's sake, still check that the source is an oper
- * for 'oper only' information in the mo_ function(s).
- */
-struct Message motd_msgtab = {
-  "MOTD", 0, 0, 0, 1, MFLG_SLOW, 0,
-  {mr_motd, m_motd, mo_motd, m_ignore, mo_motd, m_ignore}
-};
-
-#ifndef STATIC_MODULES
-const char *_version = "$Revision: 589 $";
-static struct Callback *motd_cb;
-
-static void *
-do_motd(va_list args)
-{
-  struct Client *source_p = va_arg(args, struct Client *);
-
-  send_message_file(source_p, &ConfigFileEntry.motd);
-  return NULL;
-}
-
-void
-_modinit(void)
-{
-  motd_cb = register_callback("doing_motd", do_motd);
-  mod_add_cmd(&motd_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&motd_msgtab);
-  uninstall_hook(motd_cb, do_motd);
-}
-#endif
-
-/* mr_motd()
- *
- * parv[0] = sender prefix
- */
 static void
-mr_motd(struct Client *client_p, struct Client *source_p,
-        int parc, char *parv[])
+do_motd(struct Client *source_p)
 {
-  ClearCap(client_p, CAP_TS6);
-  /* allow unregistered clients to see the motd, but exit them */
+  sendto_realops_flags(UMODE_SPY, L_ALL,
+                       "MOTD requested by %s (%s@%s) [%s]",
+                       source_p->name, source_p->username,
+                       source_p->host, source_p->servptr->name);
   send_message_file(source_p, &ConfigFileEntry.motd);
-  exit_client(source_p, source_p, "Client Exit after MOTD");
 }
 
 /*
@@ -118,17 +68,21 @@ m_motd(struct Client *client_p, struct Client *source_p,
 
   /* This is safe enough to use during non hidden server mode */
   if (!ConfigFileEntry.disable_remote && !ConfigServerHide.hide_servers)
-    if (hunt_server(client_p, source_p, ":%s MOTD :%s", 1, parc, parv)
-                    != HUNTED_ISME)
+    if (hunt_server(client_p, source_p, ":%s MOTD :%s", 1,
+                    parc, parv) != HUNTED_ISME)
       return;
 
-#ifdef STATIC_MODULES
-  send_message_file(source_p, &ConfigFileEntry.motd);
-#else
-  execute_callback(motd_cb, source_p, parc, parv);
-#endif
+  do_motd(source_p);
 }
 
+/*
+ * note regarding mo_motd being used twice:
+ * this is not a kludge.  any rate limiting, shide, or whatever
+ * other access restrictions should be done by the source's server.
+ * for security's sake, still check that the source is an oper
+ * for 'oper only' information in the mo_ function(s).
+ */
+
 /*
 ** mo_motd
 **      parv[0] = sender prefix
@@ -141,12 +95,36 @@ mo_motd(struct Client *client_p, struct Client *source_p,
   if (!IsClient(source_p))
     return;
 
-  if (hunt_server(client_p, source_p, ":%s MOTD :%s",1,parc,parv)!=HUNTED_ISME)
+  if (hunt_server(client_p, source_p, ":%s MOTD :%s", 1,
+                  parc, parv) != HUNTED_ISME)
     return;
 
-#ifdef STATIC_MODULES
-  send_message_file(source_p, &ConfigFileEntry.motd);
-#else
-  execute_callback(motd_cb, source_p, parc, parv);
-#endif
+  do_motd(source_p);
 }
+
+static struct Message motd_msgtab = {
+  "MOTD", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  { m_unregistered, m_motd, mo_motd, m_ignore, mo_motd, m_ignore }
+};
+
+static void
+module_init(void)
+{
+  mod_add_cmd(&motd_msgtab);
+}
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&motd_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1309 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = 0
+};
