@@ -19,46 +19,21 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_pong.c 503 2006-03-03 19:53:47Z michael $
+ *  $Id: m_pong.c 1569 2012-10-16 18:46:53Z michael $
  */
 
 #include "stdinc.h"
 #include "ircd.h"
-#include "handlers.h"
 #include "s_user.h"
 #include "client.h"
 #include "hash.h"       /* for find_client() */
 #include "numeric.h"
-#include "s_conf.h"
+#include "conf.h"
 #include "send.h"
 #include "irc_string.h"
-#include "msg.h"
 #include "parse.h"
 #include "modules.h"
 
-static void mr_pong(struct Client *, struct Client *, int, char **);
-static void ms_pong(struct Client *, struct Client *, int, char **);
-
-struct Message pong_msgtab = {
-  "PONG", 0, 0, 1, 0, MFLG_SLOW | MFLG_UNREG, 0,
-  {mr_pong, m_ignore, ms_pong, m_ignore, m_ignore, m_ignore}
-};
-
-#ifndef STATIC_MODULES
-void
-_modinit(void)
-{
-  mod_add_cmd(&pong_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&pong_msgtab);
-}
-
-const char *_version = "$Revision: 503 $";
-#endif
 
 static void
 ms_pong(struct Client *client_p, struct Client *source_p,
@@ -67,10 +42,10 @@ ms_pong(struct Client *client_p, struct Client *source_p,
   struct Client *target_p;
   const char *origin, *destination;
 
-  if (parc < 2 || *parv[1] == '\0')
+  if (parc < 2 || EmptyString(parv[1]))
   {
     sendto_one(source_p, form_str(ERR_NOORIGIN),
-               me.name, parv[0]);
+               me.name, source_p->name);
     return;
   }
 
@@ -86,16 +61,16 @@ ms_pong(struct Client *client_p, struct Client *source_p,
   if (!EmptyString(destination) && !match(destination, me.name) &&
       irccmp(destination, me.id))
   {
-      if ((target_p = find_client(destination)) ||
-          (target_p = find_server(destination)))
-        sendto_one(target_p,":%s PONG %s %s",
-                   parv[0], origin, destination);
-      else
-        {
-          sendto_one(source_p, form_str(ERR_NOSUCHSERVER),
-                     me.name, parv[0], destination);
-          return;
-        }
+    if ((target_p = hash_find_client(destination)) ||
+        (target_p = hash_find_server(destination)))
+      sendto_one(target_p, ":%s PONG %s %s",
+                 source_p->name, origin, destination);
+    else
+    {
+      sendto_one(source_p, form_str(ERR_NOSUCHSERVER),
+                 me.name, source_p->name, destination);
+      return;
+    }
   }
 }
 
@@ -109,17 +84,14 @@ mr_pong(struct Client *client_p, struct Client *source_p,
   {
     if (ConfigFileEntry.ping_cookie && !source_p->localClient->registration)
     {
-      unsigned long incoming_ping = strtoul(parv[1], NULL, 10);
+      unsigned int incoming_ping = strtoul(parv[1], NULL, 10);
 
       if (incoming_ping)
       {
         if (source_p->localClient->random_ping == incoming_ping)
         {
-          char buf[USERLEN + 1];
-
-          strlcpy(buf, source_p->username, sizeof(buf));
           SetPingCookie(source_p);
-          register_local_user(client_p, source_p, source_p->name, buf);
+          register_local_user(source_p);
         }
         else
         {
@@ -131,6 +103,33 @@ mr_pong(struct Client *client_p, struct Client *source_p,
     }
   }
   else
-    sendto_one(source_p, form_str(ERR_NOORIGIN), me.name, parv[0]);
+    sendto_one(source_p, form_str(ERR_NOORIGIN),
+               me.name, source_p->name);
 }
 
+static struct Message pong_msgtab = {
+  "PONG", 0, 0, 1, MAXPARA, MFLG_SLOW, 0,
+  {mr_pong, m_ignore, ms_pong, m_ignore, m_ignore, m_ignore}
+};
+
+static void
+module_init(void)
+{
+  mod_add_cmd(&pong_msgtab);
+}
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&pong_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1569 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = 0
+};
