@@ -19,46 +19,20 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_restart.c 588 2006-05-08 13:22:59Z michael $
+ *  $Id: m_restart.c 1309 2012-03-25 11:24:18Z michael $
  */
 
 #include "stdinc.h"
-#include "handlers.h"
 #include "client.h"
-#include "common.h"
 #include "irc_string.h"
-#include "sprintf_irc.h"
 #include "ircd.h"
 #include "numeric.h"
-#include "s_conf.h"
 #include "restart.h"
 #include "send.h"
-#include "msg.h"
 #include "parse.h"
 #include "modules.h"
+#include "conf.h"
 
-static void mo_restart(struct Client *, struct Client *, int, char *[]);
-
-struct Message restart_msgtab = {
-  "RESTART", 0, 0, 0, 0, MFLG_SLOW, 0,
-  { m_unregistered, m_not_oper, m_ignore, m_ignore, mo_restart, m_ignore }
-};
-
-#ifndef STATIC_MODULES
-void
-_modinit(void)
-{
-  mod_add_cmd(&restart_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&restart_msgtab);
-}
-
-const char *_version = "$Revision: 588 $";
-#endif
 
 /*
  * mo_restart
@@ -70,7 +44,7 @@ mo_restart(struct Client *client_p, struct Client *source_p,
 {
   char buf[IRCD_BUFSIZE]; 
 
-  if (!IsOperDie(source_p))
+  if (!HasOFlag(source_p, OPER_FLAG_RESTART))
   {
     sendto_one(source_p, form_str(ERR_NOPRIVS),
                me.name, source_p->name, "restart");
@@ -91,7 +65,34 @@ mo_restart(struct Client *client_p, struct Client *source_p,
     return;
   }
 
-  ircsprintf(buf, "received RESTART command from %s",
-             get_oper_name(source_p));
-  server_die(buf, YES);
+  snprintf(buf, sizeof(buf), "received RESTART command from %s",
+           get_oper_name(source_p));
+  server_die(buf, 1);
 }
+
+static struct Message restart_msgtab = {
+  "RESTART", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  { m_unregistered, m_not_oper, m_ignore, m_ignore, mo_restart, m_ignore }
+};
+
+static void
+module_init(void)
+{
+  mod_add_cmd(&restart_msgtab);
+}
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&restart_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1309 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = 0
+};
