@@ -19,45 +19,21 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_svinfo.c 33 2005-10-02 20:50:00Z knight $
+ *  $Id: m_svinfo.c 1309 2012-03-25 11:24:18Z michael $
  */
+
 #include "stdinc.h"
-#include "handlers.h"
 #include "client.h"
-#include "common.h"     /* TRUE bleah */
 #include "irc_string.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "send.h"
-#include "s_conf.h"
-#include "s_log.h"
-#include "msg.h"
+#include "conf.h"
+#include "log.h"
 #include "parse.h"
 #include "modules.h"
 
 
-static void ms_svinfo(struct Client*, struct Client*, int, char**);
-
-struct Message svinfo_msgtab = {
-  "SVINFO", 0, 0, 4, 0, MFLG_SLOW, 0,
-  {m_unregistered, m_ignore, ms_svinfo, m_ignore, m_ignore, m_ignore}
-};
-
-#ifndef STATIC_MODULES
-void
-_modinit(void)
-{
-  mod_add_cmd(&svinfo_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&svinfo_msgtab);
-}
-
-const char *_version = "$Revision: 33 $";
-#endif
 /*
  * ms_svinfo - SVINFO message handler
  *      parv[0] = sender prefix
@@ -68,7 +44,7 @@ const char *_version = "$Revision: 33 $";
  */
 static void
 ms_svinfo(struct Client *client_p, struct Client *source_p,
-	  int parc, char *parv[])
+          int parc, char *parv[])
 {
   time_t deltat;
   time_t theirtime;
@@ -83,21 +59,21 @@ ms_svinfo(struct Client *client_p, struct Client *source_p,
     return;
 
   if (TS_CURRENT < atoi(parv[2]) || atoi(parv[1]) < TS_MIN)
-    {
-      /*
-       * a server with the wrong TS version connected; since we're
-       * TS_ONLY we can't fall back to the non-TS protocol so
-       * we drop the link  -orabidoo
-       */
-      sendto_realops_flags(UMODE_ALL, L_ADMIN,
+  {
+    /*
+     * a server with the wrong TS version connected; since we're
+     * TS_ONLY we can't fall back to the non-TS protocol so
+     * we drop the link  -orabidoo
+     */
+    sendto_realops_flags(UMODE_ALL, L_ADMIN,
             "Link %s dropped, wrong TS protocol version (%s,%s)",
             get_client_name(source_p, SHOW_IP), parv[1], parv[2]);
-      sendto_realops_flags(UMODE_ALL, L_OPER,
+    sendto_realops_flags(UMODE_ALL, L_OPER,
                  "Link %s dropped, wrong TS protocol version (%s,%s)",
                  get_client_name(source_p, MASK_IP), parv[1], parv[2]);
-      exit_client(source_p, source_p, "Incompatible TS version");
-      return;
-    }
+    exit_client(source_p, source_p, "Incompatible TS version");
+    return;
+  }
 
   /*
    * since we're here, might as well set CurrentTime while we're at it
@@ -107,36 +83,61 @@ ms_svinfo(struct Client *client_p, struct Client *source_p,
   deltat = abs(theirtime - CurrentTime);
 
   if (deltat > ConfigFileEntry.ts_max_delta)
-    {
-      sendto_realops_flags(UMODE_ALL, L_ADMIN,
+  {
+    sendto_realops_flags(UMODE_ALL, L_ADMIN,
           "Link %s dropped, excessive TS delta (my TS=%lu, their TS=%lu, delta=%d)",
           get_client_name(source_p, SHOW_IP),
           (unsigned long) CurrentTime,
           (unsigned long) theirtime,
           (int) deltat);
-      sendto_realops_flags(UMODE_ALL, L_OPER,
+    sendto_realops_flags(UMODE_ALL, L_OPER,
           "Link %s dropped, excessive TS delta (my TS=%lu, their TS=%lu, delta=%d)",
            get_client_name(source_p, MASK_IP),
            (unsigned long) CurrentTime,
            (unsigned long) theirtime,
            (int) deltat);
-      ilog(L_NOTICE,
-          "Link %s dropped, excessive TS delta (my TS=%lu, their TS=%lu, delta=%d)",
-          get_client_name(source_p, SHOW_IP),
-          (unsigned long) CurrentTime,
-          (unsigned long) theirtime,
-          (int) deltat);
-      exit_client(source_p, source_p, "Excessive TS delta");
-      return;
-    }
+    ilog(LOG_TYPE_IRCD,
+         "Link %s dropped, excessive TS delta (my TS=%lu, their TS=%lu, delta=%d)",
+         get_client_name(source_p, SHOW_IP),
+         (unsigned long) CurrentTime,
+         (unsigned long) theirtime,
+         (int) deltat);
+    exit_client(source_p, source_p, "Excessive TS delta");
+    return;
+  }
 
   if (deltat > ConfigFileEntry.ts_warn_delta)
-    { 
-      sendto_realops_flags(UMODE_ALL, L_ALL,
+    sendto_realops_flags(UMODE_ALL, L_ALL,
                 "Link %s notable TS delta (my TS=%lu, their TS=%lu, delta=%d)",
                 source_p->name,
                 (unsigned long) CurrentTime,
                 (unsigned long) theirtime,
                 (int) deltat);
-    }
 }
+
+static struct Message svinfo_msgtab = {
+  "SVINFO", 0, 0, 4, MAXPARA, MFLG_SLOW, 0,
+  {m_unregistered, m_ignore, ms_svinfo, m_ignore, m_ignore, m_ignore}
+};
+
+static void
+module_init(void)
+{
+  mod_add_cmd(&svinfo_msgtab);
+}
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&svinfo_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1309 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = 0
+};
