@@ -19,20 +19,17 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_testline.c 593 2006-05-12 05:47:32Z michael $
+ *  $Id: m_testline.c 1871 2013-04-26 08:53:04Z michael $
  */
 
 #include "stdinc.h"
-#include "handlers.h"
 #include "client.h"
-#include "common.h"
 #include "irc_string.h"
 #include "ircd_defs.h"
 #include "ircd.h"
 #include "restart.h"
-#include "s_conf.h"
+#include "conf.h"
 #include "send.h"
-#include "msg.h"
 #include "hostmask.h"
 #include "numeric.h"
 #include "parse.h"
@@ -40,36 +37,6 @@
 #include "hash.h"
 #include "modules.h"
 
-static void mo_testline(struct Client *, struct Client *, int, char *[]);
-static void mo_testgecos(struct Client *, struct Client *, int, char *[]);
-
-struct Message testline_msgtab = {
-  "TESTLINE", 0, 0, 0, 0, MFLG_SLOW, 0,
-  { m_unregistered, m_not_oper, m_ignore, m_ignore, mo_testline, m_ignore }
-};
-
-struct Message testgecos_msgtab = {
-  "TESTGECOS", 0, 0, 0, 0, MFLG_SLOW, 0,
-  { m_unregistered, m_not_oper, m_ignore, m_ignore, mo_testgecos, m_ignore }
-};
- 
-#ifndef STATIC_MODULES
-void
-_modinit(void)
-{
-  mod_add_cmd(&testline_msgtab);
-  mod_add_cmd(&testgecos_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&testline_msgtab);
-  mod_del_cmd(&testgecos_msgtab);
-}
- 
-const char *_version = "$Revision: 593 $";
-#endif
 
 /* mo_testline()
  *
@@ -88,10 +55,10 @@ static void
 mo_testline(struct Client *client_p, struct Client *source_p,
             int parc, char *parv[])
 {
-  char *orig_parv1 = NULL;
   /* IRCD_BUFSIZE to allow things like *u*s*e*r*n*a*m*e* etc. */
   char given_name[IRCD_BUFSIZE];
   char given_host[IRCD_BUFSIZE];
+  char parv1_copy[IRCD_BUFSIZE];
   struct ConfItem *conf;
   struct AccessItem *aconf;
   struct irc_ssaddr ip;
@@ -116,12 +83,12 @@ mo_testline(struct Client *client_p, struct Client *source_p,
     {
       sendto_one(source_p, form_str(RPL_TESTLINE),
                  me.name, source_p->name, 'Q', 0, chptr->name, 
-                 chptr->reason ? chptr->reason : "No reason", "");
+                 chptr->reason ? chptr->reason : CONF_NOREASON, "");
       return;
     }
   }
 
-  DupString(orig_parv1,parv[1]);
+  strlcpy(parv1_copy, parv[1], sizeof(parv1_copy));
 
   nuh.nuhmask  = parv[1];
   nuh.nickptr  = NULL;
@@ -147,44 +114,22 @@ mo_testline(struct Client *client_p, struct Client *source_p,
                             );
     if (aconf != NULL)
     {
-      conf = unmap_conf_item(aconf);
-
+      ++matches;
       if (aconf->status & CONF_EXEMPTDLINE)
-      {
         sendto_one(source_p,
                    ":%s NOTICE %s :Exempt D-line host [%s] reason [%s]",
                    me.name, source_p->name, aconf->host, aconf->reason);
-        ++matches;
-      }
       else
-      {
         sendto_one(source_p, form_str(RPL_TESTLINE),
                    me.name, source_p->name,
                    IsConfTemporary(aconf) ? 'd' : 'D',
                    IsConfTemporary(aconf) ? ((aconf->hold - CurrentTime) / 60)
                    : 0L,
-                   aconf->host, aconf->reason, aconf->oper_reason);
-        ++matches;
-      }
+                   aconf->host, aconf->reason,
+                   aconf->oper_reason ? aconf->oper_reason : "");
     }
   }
 
-  aconf = find_kline_conf(given_host, given_name, &ip, t);
-  if ((aconf != NULL) && (aconf->status & CONF_KILL))
-  {
-    snprintf(userhost, sizeof(userhost), "%s@%s", aconf->user, aconf->host);
-    sendto_one(source_p, form_str(RPL_TESTLINE),
-               me.name, source_p->name,
-               IsConfTemporary(aconf) ? 'k' : 'K',
-               IsConfTemporary(aconf) ? ((aconf->hold - CurrentTime) / 60)
-               : 0L,
-               userhost, 
-               aconf->passwd ? aconf->passwd : "No reason",
-               aconf->oper_reason ? aconf->oper_reason : "");
-    ++matches;
-  }
-
-
   if (t != HM_HOST)
     aconf = find_address_conf(given_host, given_name, &ip, 
 #ifdef IPV6
@@ -198,26 +143,23 @@ mo_testline(struct Client *client_p, struct Client *source_p,
                  
   if (aconf != NULL)
   {
-    conf = unmap_conf_item(aconf);
-
     snprintf(userhost, sizeof(userhost), "%s@%s", aconf->user, aconf->host);
 
     if (aconf->status & CONF_CLIENT)
     {
       sendto_one(source_p, form_str(RPL_TESTLINE),
-                 me.name, source_p->name,
-                 'I', 0L, userhost,
+                 me.name, source_p->name, 'I', 0L, userhost,
                  aconf->class_ptr ? aconf->class_ptr->name : "<default>", "");
       ++matches;
     }
-    else if (aconf->status & CONF_KILL)
+    else if (aconf->status & CONF_KLINE)
     {
       sendto_one(source_p, form_str(RPL_TESTLINE),
                  me.name, source_p->name,
                  IsConfTemporary(aconf) ? 'k' : 'K',
                  IsConfTemporary(aconf) ? ((aconf->hold - CurrentTime) / 60)
                  : 0L,
-                 userhost, aconf->passwd ? aconf->passwd : "No reason",
+                 userhost, aconf->reason? aconf->reason : CONF_NOREASON,
                  aconf->oper_reason ? aconf->oper_reason : "");
       ++matches;
     }
@@ -227,23 +169,19 @@ mo_testline(struct Client *client_p, struct Client *source_p,
 
   if (conf != NULL)
   {
-    struct MatchItem *mconf;
-    mconf = (struct MatchItem *)map_to_conf(conf);
+    const struct MatchItem *mconf = map_to_conf(conf);
 
     sendto_one(source_p, form_str(RPL_TESTLINE),
-               me.name, source_p->name,
-               'Q', 0L,
+               me.name, source_p->name, 'Q', 0L,
                conf->name, 
-               mconf->reason ? mconf->reason : "No reason",
+               mconf->reason ? mconf->reason : CONF_NOREASON,
                mconf->oper_reason ? mconf->oper_reason : "");
     ++matches;
   }
 
   if (matches == 0)
     sendto_one(source_p, form_str(RPL_NOTESTLINE),
-               me.name, source_p->name, orig_parv1);
-
-  MyFree(orig_parv1);
+               me.name, source_p->name, parv1_copy);
 }
 
 /* mo_testgecos()
@@ -261,10 +199,9 @@ mo_testline(struct Client *client_p, struct Client *source_p,
  */
 static void
 mo_testgecos(struct Client *client_p, struct Client *source_p,
-            int parc, char *parv[])
+             int parc, char *parv[])
 {
   struct ConfItem *conf = NULL;
-  struct MatchItem *xconf = NULL;
 
   if (EmptyString(parv[1]))
   {
@@ -275,7 +212,7 @@ mo_testgecos(struct Client *client_p, struct Client *source_p,
 
   if ((conf = find_matching_name_conf(XLINE_TYPE, parv[1], NULL, NULL, 0)))
   {
-    xconf = map_to_conf(conf);
+    const struct MatchItem *xconf = map_to_conf(conf);
     sendto_one(source_p, form_str(RPL_TESTLINE),
                me.name, source_p->name, 'X', 0L,
                conf->name, xconf->reason ? xconf->reason : "X-lined",
@@ -285,3 +222,37 @@ mo_testgecos(struct Client *client_p, struct Client *source_p,
     sendto_one(source_p, form_str(RPL_NOTESTLINE),
                me.name, source_p->name, parv[1]);
 }
+
+static struct Message testline_msgtab = {
+  "TESTLINE", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  { m_unregistered, m_not_oper, m_ignore, m_ignore, mo_testline, m_ignore }
+};
+
+struct Message testgecos_msgtab = {
+  "TESTGECOS", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  { m_unregistered, m_not_oper, m_ignore, m_ignore, mo_testgecos, m_ignore }
+};
+
+static void
+module_init(void)
+{
+  mod_add_cmd(&testline_msgtab);
+  mod_add_cmd(&testgecos_msgtab);
+}
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&testline_msgtab);
+  mod_del_cmd(&testgecos_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1871 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = 0
+};
