@@ -19,67 +19,42 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_time.c,v 1.26.2.1 2004/06/16 04:55:54 erik Exp $
+ *  $Id: m_time.c 1309 2012-03-25 11:24:18Z michael $
  */
 
 #include "stdinc.h"
-#include "handlers.h"
 #include "client.h"
 #include "ircd.h"
 #include "numeric.h"
 #include "s_misc.h"
-#include "s_conf.h"
+#include "conf.h"
 #include "s_serv.h"
 #include "send.h"
-#include "msg.h"
 #include "parse.h"
 #include "modules.h"
 #include "packet.h"
 
-static void m_time(struct Client*, struct Client*, int, char**);
-static void mo_time(struct Client*, struct Client*, int, char**);
 
-struct Message time_msgtab = {
-  "TIME", 0, 0, 0, 0, MFLG_SLOW, 0,
-  {m_unregistered, m_time, mo_time, m_ignore, mo_time}
-};
-
-#ifndef STATIC_MODULES
-void
-_modinit(void)
-{
-  mod_add_cmd(&time_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&time_msgtab);
-}
-
-const char *_version = "$Revision: 1.26.2.1 $";
-#endif
 /*
  * m_time
  *      parv[0] = sender prefix
  *      parv[1] = servername
  */
-static void m_time(struct Client *client_p, struct Client *source_p,
-                  int parc, char *parv[])
+static void
+m_time(struct Client *client_p, struct Client *source_p,
+       int parc, char *parv[])
 {
   /* this is not rate limited, so end the grace period */
-  if(MyClient(source_p) && !IsFloodDone(source_p))
+  if (!IsFloodDone(source_p))
     flood_endgrace(source_p);
 
   /* This is safe enough to use during non hidden server mode */
-  if(!ConfigServerHide.disable_remote)
-    {
-      if (hunt_server(client_p,source_p,":%s TIME :%s",1,parc,parv) != HUNTED_ISME)
-        return;
-    }
+  if (!ConfigFileEntry.disable_remote)
+    if (hunt_server(client_p, source_p, ":%s TIME :%s", 1, parc, parv) != HUNTED_ISME)
+      return;
 
   sendto_one(source_p, form_str(RPL_TIME), me.name,
-             parv[0], me.name, date(0));
+             source_p->name, me.name, date(0));
 }
 
 /*
@@ -87,10 +62,38 @@ static void m_time(struct Client *client_p, struct Client *source_p,
  *      parv[0] = sender prefix
  *      parv[1] = servername
  */
-static void mo_time(struct Client *client_p, struct Client *source_p,
-                   int parc, char *parv[])
+static void
+mo_time(struct Client *client_p, struct Client *source_p,
+        int parc, char *parv[])
 {
-  if (hunt_server(client_p,source_p,":%s TIME :%s",1,parc,parv) == HUNTED_ISME)
+  if (hunt_server(client_p, source_p, ":%s TIME :%s", 1, parc, parv) == HUNTED_ISME)
     sendto_one(source_p, form_str(RPL_TIME), me.name,
-               parv[0], me.name, date(0));
+               source_p->name, me.name, date(0));
 }
+
+static struct Message time_msgtab = {
+  "TIME", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  { m_unregistered, m_time, mo_time, m_ignore, mo_time, m_ignore }
+};
+
+static void
+module_init(void)
+{
+  mod_add_cmd(&time_msgtab);
+}
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&time_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1309 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = 0
+};
