@@ -19,72 +19,40 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_trace.c 33 2005-10-02 20:50:00Z knight $
+ *  $Id: m_trace.c 1776 2013-01-24 17:26:55Z michael $
  */
 
 #include "stdinc.h"
-#include "handlers.h"
-#include "tools.h"
-#include "hook.h"
+#include "list.h"
 #include "client.h"
 #include "hash.h"
-#include "common.h"
 #include "irc_string.h"
 #include "ircd.h"
 #include "numeric.h"
-#include "fdlist.h"
 #include "s_bsd.h"
 #include "s_serv.h"
 #include "send.h"
-#include "msg.h"
 #include "parse.h"
 #include "modules.h"
-#include "s_conf.h"
-#include "irc_getnameinfo.h"
+#include "conf.h"
 
-static void m_trace(struct Client *, struct Client *, int, char **);
-static void ms_trace(struct Client*, struct Client*, int, char**);
-static void mo_trace(struct Client*, struct Client*, int, char**);
-static void do_actual_trace(struct Client *, int, char **);
 
-struct Message trace_msgtab = {
-  "TRACE", 0, 0, 0, 0, MFLG_SLOW, 0,
-  {m_unregistered, m_trace, ms_trace, m_ignore, mo_trace, m_ignore}
-};
+static void do_actual_trace(struct Client *, int, char *[]);
+static void report_this_status(struct Client *, struct Client *, int);
 
-#ifndef STATIC_MODULES
-const char *_version = "$Revision: 33 $";
-static struct Callback *trace_cb;
-
-static void *
-va_actual_trace(va_list args)
+static void
+trace_get_dependent(int *const server,
+                    int *const client, const struct Client *target_p)
 {
-  struct Client *source_p = va_arg(args, struct Client *);
-  int parc = va_arg(args, int);
-  char **parv = va_arg(args, char **);
+  const dlink_node *ptr = NULL;
 
-  do_actual_trace(source_p, parc, parv);
-  return NULL;
+  (*server)++;
+  (*client) += dlink_list_length(&target_p->serv->client_list);
+
+  DLINK_FOREACH(ptr, target_p->serv->server_list.head)
+    trace_get_dependent(server, client, ptr->data);
 }
 
-void
-_modinit(void)
-{
-  trace_cb = register_callback("doing_trace", va_actual_trace);
-  mod_add_cmd(&trace_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&trace_msgtab);
-  uninstall_hook(trace_cb, va_actual_trace);
-}
-#endif
-
-static int report_this_status(struct Client *source_p, struct Client *target_p,
-			      int dow, int link_u_p, int link_u_s);
-
 /*
  * m_trace()
  *
@@ -145,13 +113,13 @@ mo_trace(struct Client *client_p, struct Client *source_p,
     {
       struct Client *ac2ptr = NULL;
 
-      if ((ac2ptr = find_client(tname)) == NULL)
+      if ((ac2ptr = hash_find_client(tname)) == NULL)
       {
         DLINK_FOREACH(ptr, global_client_list.head)
         {
           ac2ptr = ptr->data;
 
-          if (match(tname, ac2ptr->name) || match(ac2ptr->name, tname))
+          if (match(tname, ac2ptr->name))
             break;
           else
             ac2ptr = NULL;
@@ -166,26 +134,39 @@ mo_trace(struct Client *client_p, struct Client *source_p,
                    ircd_version, tname, "ac2ptr_is_NULL!!");
       return;
     }
+
     case HUNTED_ISME:
-#ifdef STATIC_MODULES
       do_actual_trace(source_p, parc, parv);
-#else
-      execute_callback(trace_cb, source_p, parc, parv);
-#endif
       break;
     default:
       return;
   }
 }
 
+/*
+** ms_trace
+**      parv[0] = sender prefix
+**      parv[1] = servername
+*/
 static void
-do_actual_trace(struct Client *source_p, int parc, char **parv)
+ms_trace(struct Client *client_p, struct Client *source_p,
+         int parc, char *parv[])
+{
+  if (hunt_server(client_p, source_p, ":%s TRACE %s :%s", 2, parc, parv))
+    return;
+
+  if (HasUMode(source_p, UMODE_OPER))
+    mo_trace(client_p, source_p, parc, parv);
+}
+
+static void
+do_actual_trace(struct Client *source_p, int parc, char *parv[])
 {
   struct Client *target_p = NULL;
   struct ConfItem *conf;
   struct ClassItem *cltmp;
   int doall = 0;
-  int cnt = 0, wilds, dow;
+  int wilds, dow;
   dlink_node *ptr;
   const char *from, *to, *tname;
 
@@ -205,52 +186,51 @@ do_actual_trace(struct Client *source_p, int parc, char **parv)
     to = source_p->name;
   }
 
+  sendto_realops_flags(UMODE_SPY, L_ALL,
+                       "TRACE requested by %s (%s@%s) [%s]",
+                       source_p->name, source_p->username,
+                       source_p->host, source_p->servptr->name);
+
   if (match(tname, me.name))
-    doall = TRUE;
+    doall = 1;
   else if (!MyClient(source_p) && !strcmp(tname, me.id))
   {
-    doall = TRUE;
+    doall = 1;
     tname = me.name;
   }
 
-  wilds = !parv[1] || strchr(tname, '*') || strchr(tname, '?');
+  wilds = !parv[1] || has_wildcards(tname);
   dow = wilds || doall;
 
   set_time();
-  if (!IsOper(source_p) || !dow) /* non-oper traces must be full nicks */
+  if (!HasUMode(source_p, UMODE_OPER) || !dow) /* non-oper traces must be full nicks */
                               /* lets also do this for opers tracing nicks */
   {
     const char *name;
     const char *class_name;
-    char ipaddr[HOSTIPLEN];
 
-    target_p = find_client(tname);
+    target_p = hash_find_client(tname);
       
     if (target_p && IsClient(target_p)) 
     {
       name = get_client_name(target_p, HIDE_IP);
-      /* should we not use sockhost here? - stu */
-      irc_getnameinfo((struct sockaddr*)&target_p->localClient->ip,
-		      target_p->localClient->ip.ss_len, ipaddr, HOSTIPLEN, NULL, 0,
-		      NI_NUMERICHOST);
-
       class_name = get_client_class(target_p);
 
-      if (IsOper(target_p))
+      if (HasUMode(target_p, UMODE_OPER))
       {
         sendto_one(source_p, form_str(RPL_TRACEOPERATOR),
                    from, to, class_name, name, 
-                   IsIPSpoof(target_p) ? "255.255.255.255" : ipaddr,
-                   CurrentTime - target_p->lasttime,
-                   CurrentTime - target_p->localClient->last);
+                   IsIPSpoof(target_p) ? "255.255.255.255" : target_p->sockhost,
+                   CurrentTime - target_p->localClient->lasttime,
+                   idle_time_get(source_p, target_p));
       }
       else
       {
         sendto_one(source_p,form_str(RPL_TRACEUSER),
                    from, to, class_name, name, 
-		   IsIPSpoof(target_p) ? "255.255.255.255" : ipaddr,
-                   CurrentTime - target_p->lasttime,
-                   CurrentTime - target_p->localClient->last);
+		   IsIPSpoof(target_p) ? "255.255.255.255" : target_p->sockhost,
+                   CurrentTime - target_p->localClient->lasttime,
+                   idle_time_get(source_p, target_p));
       }
     }
       
@@ -264,16 +244,16 @@ do_actual_trace(struct Client *source_p, int parc, char **parv)
   {
     target_p = ptr->data;
 
-    if (IsInvisible(target_p) && dow &&
-	!(MyConnect(source_p) && IsOper(source_p)) &&
-	!IsOper(target_p) && (target_p != source_p))
+    if (HasUMode(target_p, UMODE_INVISIBLE) && dow &&
+	!(MyConnect(source_p) && HasUMode(source_p, UMODE_OPER)) &&
+	!HasUMode(target_p, UMODE_OPER) && (target_p != source_p))
       continue;
     if (!doall && wilds && !match(tname, target_p->name))
       continue;
     if (!dow && irccmp(tname, target_p->name))
       continue;
 
-    cnt = report_this_status(source_p, target_p, dow, 0, 0);
+    report_this_status(source_p, target_p, dow);
   }
 
   DLINK_FOREACH(ptr, serv_list.head)
@@ -285,8 +265,7 @@ do_actual_trace(struct Client *source_p, int parc, char **parv)
     if (!dow && irccmp(tname, target_p->name))
       continue;
 
-    cnt = report_this_status(source_p, target_p, dow,
-      target_p->serv->dep_users, target_p->serv->dep_servers);
+    report_this_status(source_p, target_p, dow);
   }
 
   /* This section is to report the unknowns */
@@ -299,38 +278,22 @@ do_actual_trace(struct Client *source_p, int parc, char **parv)
     if (!dow && irccmp(tname, target_p->name))
       continue;
 
-    cnt = report_this_status(source_p, target_p, dow, 0, 0);
+    report_this_status(source_p, target_p, dow);
   }
 
   DLINK_FOREACH(ptr, class_items.head)
   {
     conf = ptr->data;
-    cltmp = (struct ClassItem *)map_to_conf(conf);
-    if (CurrUserCount(cltmp) > 0)
+    cltmp = map_to_conf(conf);
+
+    if (cltmp->curr_user_count > 0)
       sendto_one(source_p, form_str(RPL_TRACECLASS),
-		 from, to, conf->name, CurrUserCount(cltmp));
+                 from, to, conf->name, cltmp->curr_user_count);
   }
 
   sendto_one(source_p, form_str(RPL_ENDOFTRACE), from, to, tname);
 }
 
-
-/*
-** ms_trace
-**      parv[0] = sender prefix
-**      parv[1] = servername
-*/
-static void
-ms_trace(struct Client *client_p, struct Client *source_p,
-         int parc, char *parv[])
-{
-  if (hunt_server(client_p, source_p, ":%s TRACE %s :%s", 2, parc, parv))
-    return;
-
-  if (IsOper(source_p))
-    mo_trace(client_p, source_p, parc, parv);
-}
-
 /* report_this_status()
  *
  * inputs	- pointer to client to report to
@@ -338,15 +301,12 @@ ms_trace(struct Client *client_p, struct Client *source_p,
  * output	- counter of number of hits
  * side effects - NONE
  */
-static int
-report_this_status(struct Client *source_p, struct Client *target_p,
-                   int dow, int link_u_p, int link_s_p)
+static void
+report_this_status(struct Client *source_p, struct Client *target_p, int dow)
 {
   const char *name;
   const char *class_name;
   const char *from, *to;
-  char ip[HOSTIPLEN];
-  int cnt = 0;
 
   if (!MyConnect(source_p) && IsCapable(source_p->from, CAP_TS6) && HasID(source_p))
   {
@@ -359,114 +319,130 @@ report_this_status(struct Client *source_p, struct Client *target_p,
     to = source_p->name;
   }
 
-  /* Should this be sockhost? - stu */
-  irc_getnameinfo((struct sockaddr*)&target_p->localClient->ip, 
-        target_p->localClient->ip.ss_len, ip, HOSTIPLEN, NULL, 0, 
-        NI_NUMERICHOST);
   name = get_client_name(target_p, HIDE_IP);
   class_name = get_client_class(target_p);
 
   set_time();
 
-  switch(target_p->status)
+  switch (target_p->status)
   {
     case STAT_CONNECTING:
       sendto_one(source_p, form_str(RPL_TRACECONNECTING),
                  from, to, class_name, 
-		 IsOperAdmin(source_p) ? name : target_p->name);
-		   
-      cnt++;
+		 HasUMode(source_p, UMODE_ADMIN) ? name : target_p->name);
       break;
     case STAT_HANDSHAKE:
       sendto_one(source_p, form_str(RPL_TRACEHANDSHAKE),
                  from, to, class_name, 
-                 IsOperAdmin(source_p) ? name : target_p->name);
-		   
-      cnt++;
+                 HasUMode(source_p, UMODE_ADMIN) ? name : target_p->name);
       break;
     case STAT_ME:
       break;
     case STAT_UNKNOWN:
       /* added time -Taner */
       sendto_one(source_p, form_str(RPL_TRACEUNKNOWN),
-		 from, to, class_name, name, ip,
-		 target_p->firsttime ? CurrentTime - target_p->firsttime : -1);
-      cnt++;
+		 from, to, class_name, name, target_p->sockhost,
+		 target_p->localClient->firsttime ? /* TBD: can't be 0 */
+                 CurrentTime - target_p->localClient->firsttime : -1);
       break;
     case STAT_CLIENT:
-      /* Only opers see users if there is a wildcard
+      /*
+       * Only opers see users if there is a wildcard
        * but anyone can see all the opers.
        */
-      if ((IsOper(source_p) &&
-	   (MyClient(source_p) || !(dow && IsInvisible(target_p))))
-	  || !dow || IsOper(target_p))
+      if ((HasUMode(source_p, UMODE_OPER) &&
+	   (MyClient(source_p) || !(dow && HasUMode(target_p, UMODE_INVISIBLE))))
+	  || !dow || HasUMode(target_p, UMODE_OPER))
 	{
-          if (IsAdmin(target_p) && !ConfigFileEntry.hide_spoof_ips)
+          if (HasUMode(target_p, UMODE_ADMIN) && !ConfigFileEntry.hide_spoof_ips)
 	    sendto_one(source_p, form_str(RPL_TRACEOPERATOR),
                        from, to, class_name, name,
-                       IsOperAdmin(source_p) ? ip : "255.255.255.255",
-                       CurrentTime - target_p->lasttime,
-                       CurrentTime - target_p->localClient->last);
+                       HasUMode(source_p, UMODE_ADMIN) ? target_p->sockhost : "255.255.255.255",
+                       CurrentTime - target_p->localClient->lasttime,
+                       idle_time_get(source_p, target_p));
 		       
-	  else if (IsOper(target_p))
+	  else if (HasUMode(target_p, UMODE_OPER))
           {
 	    if (ConfigFileEntry.hide_spoof_ips)
 	      sendto_one(source_p, form_str(RPL_TRACEOPERATOR),
 		         from, to, class_name, name, 
-		         IsIPSpoof(target_p) ? "255.255.255.255" : ip,
-		         CurrentTime - target_p->lasttime,
-		         CurrentTime - target_p->localClient->last);
+		         IsIPSpoof(target_p) ? "255.255.255.255" : target_p->sockhost,
+		         CurrentTime - target_p->localClient->lasttime,
+		         idle_time_get(source_p, target_p));
 	    else
               sendto_one(source_p, form_str(RPL_TRACEOPERATOR),
                          from, to, class_name, name,
-                         MyOper(source_p) ? ip :
-                         (IsIPSpoof(target_p) ? "255.255.255.255" : ip),
-                         CurrentTime - target_p->lasttime,
-                         CurrentTime - target_p->localClient->last);
+                         MyOper(source_p) ? target_p->sockhost :
+                         (IsIPSpoof(target_p) ? "255.255.255.255" : target_p->sockhost),
+                         CurrentTime - target_p->localClient->lasttime,
+                         idle_time_get(source_p, target_p));
 	  }		       
 	  else
           {
-	    const char *format_str=NULL;
-	    if (IsOper(source_p) && IsCaptured(target_p))
-	      format_str = form_str(RPL_TRACECAPTURED);
-	    else
-	      format_str = form_str(RPL_TRACEUSER);
-
             if (ConfigFileEntry.hide_spoof_ips)
-	      sendto_one(source_p, format_str,
+	      sendto_one(source_p, form_str(RPL_TRACEUSER),
 		         from, to, class_name, name,
-                         IsIPSpoof(target_p) ? "255.255.255.255" : ip,
-		         CurrentTime - target_p->lasttime,
-		         CurrentTime - target_p->localClient->last);
+                         IsIPSpoof(target_p) ? "255.255.255.255" : target_p->sockhost,
+		         CurrentTime - target_p->localClient->lasttime,
+		         idle_time_get(source_p, target_p));
 	    else
-              sendto_one(source_p, format_str,
+              sendto_one(source_p, form_str(RPL_TRACEUSER),
                          from, to, class_name, name,
-                         MyOper(source_p) ? ip :
-                         (IsIPSpoof(target_p) ? "255.255.255.255" : ip),
-                         CurrentTime - target_p->lasttime,
-                         CurrentTime - target_p->localClient->last);
+                         MyOper(source_p) ? target_p->sockhost :
+                         (IsIPSpoof(target_p) ? "255.255.255.255" : target_p->sockhost),
+                         CurrentTime - target_p->localClient->lasttime,
+                         idle_time_get(source_p, target_p));
 	  }
-	  cnt++;
 	}
       break;
     case STAT_SERVER:
-      if(!IsOperAdmin(source_p))
+    {
+      int clients = 0;
+      int servers = 0;
+
+      trace_get_dependent(&servers, &clients, target_p);
+
+      if (!HasUMode(source_p, UMODE_ADMIN))
         name = get_client_name(target_p, MASK_IP);
 
       sendto_one(source_p, form_str(RPL_TRACESERVER),
-		 from, to, class_name, link_s_p,
-		 link_u_p, name, *(target_p->serv->by) ?
+		 from, to, class_name, servers,
+		 clients, name, *(target_p->serv->by) ?
 		 target_p->serv->by : "*", "*",
-		 me.name, CurrentTime - target_p->lasttime);
-      cnt++;
+		 me.name, CurrentTime - target_p->localClient->lasttime);
       break;
+    }
       
     default: /* ...we actually shouldn't come here... --msa */
       sendto_one(source_p, form_str(RPL_TRACENEWTYPE),
-		 from, to, name);
-      cnt++;
+                 from, to, name);
       break;
   }
-
-  return cnt;
 }
+
+static struct Message trace_msgtab = {
+  "TRACE", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  { m_unregistered, m_trace, ms_trace, m_ignore, mo_trace, m_ignore }
+};
+
+static void
+module_init(void)
+{
+  mod_add_cmd(&trace_msgtab);
+}
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&trace_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1776 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = 0
+};
