@@ -19,26 +19,20 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_version.c 269 2005-11-14 19:39:09Z adx $
+ *  $Id: m_version.c 1495 2012-08-05 09:35:42Z michael $
  */
 
 #include "stdinc.h"
-#include "handlers.h"
 #include "client.h"
 #include "ircd.h"
 #include "numeric.h"
-#include "s_conf.h"
+#include "conf.h"
 #include "s_serv.h"
 #include "s_user.h"
 #include "send.h"
-#include "msg.h"
 #include "parse.h"
 #include "modules.h"
 
-static char *confopts(struct Client *);
-static void m_version(struct Client *, struct Client *, int, char *[]);
-static void ms_version(struct Client *, struct Client *, int, char *[]);
-static void mo_version(struct Client *, struct Client *, int, char *[]);
 
 /* Option string. */
 static const char serveropts[] = {
@@ -55,26 +49,43 @@ static const char serveropts[] = {
   '\0'
 };
 
-struct Message version_msgtab = {
-  "VERSION", 0, 0, 0, 0, MFLG_SLOW, 0,
-  { m_unregistered, m_version, ms_version, m_ignore, mo_version, m_ignore }
-};
-
-#ifndef STATIC_MODULES
-void
-_modinit(void)
+/* confopts()
+ *
+ * input  - client pointer
+ * output - ircd.conf option string
+ * side effects - none
+ */
+static char *
+confopts(struct Client *source_p)
 {
-  mod_add_cmd(&version_msgtab);
-}
+  static char result[12];
+  char *p = result;
 
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&version_msgtab);
-}
+  *p++ = 'e'; /* excepts */
 
-const char *_version = "$Revision: 269 $";
-#endif
+  if (ConfigFileEntry.glines)
+    *p++ = 'G';
+  *p++ = 'g';
+
+  /* might wanna hide this :P */
+  if (ServerInfo.hub &&
+      (!ConfigFileEntry.disable_remote || HasUMode(source_p, UMODE_OPER)))
+  {
+    *p++ = 'H';
+  }
+
+  *p++ = 'I'; /* invex */
+  *p++ = 'K'; /* knock */
+  *p++ = 'M';
+
+  if (ConfigFileEntry.ignore_bogus_ts)
+    *p++ = 'T';
+  *p++ = '6';
+
+  *p = '\0';
+
+  return result;
+}
 
 /*
  * m_version - VERSION command handler
@@ -94,15 +105,13 @@ m_version(struct Client *client_p, struct Client *source_p,
                me.name, source_p->name);
     return;
   }
-  else
-    last_used = CurrentTime;
+
+  last_used = CurrentTime;
 
   if (!ConfigFileEntry.disable_remote)
-  {
     if (hunt_server(client_p, source_p, ":%s VERSION :%s",
                     1, parc, parv) != HUNTED_ISME)
       return;
-  }
 
   sendto_one(source_p, form_str(RPL_VERSION),
              me.name, source_p->name, ircd_version, serno,
@@ -124,7 +133,8 @@ mo_version(struct Client *client_p, struct Client *source_p,
 		  1, parc, parv) != HUNTED_ISME)
     return;
 
-  sendto_one(source_p, form_str(RPL_VERSION), me.name, parv[0], ircd_version, 
+  sendto_one(source_p, form_str(RPL_VERSION), me.name,
+             source_p->name, ircd_version, 
   	     serno, me.name, confopts(source_p), serveropts);
 
   show_isupport(source_p);
@@ -151,48 +161,29 @@ ms_version(struct Client *client_p, struct Client *source_p,
   }
 }
 
-/* confopts()
- *
- * input  - client pointer
- * output - ircd.conf option string
- * side effects - none
- */
-static char *
-confopts(struct Client *source_p)
+static struct Message version_msgtab = {
+  "VERSION", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  { m_unregistered, m_version, ms_version, m_ignore, mo_version, m_ignore }
+};
+
+static void
+module_init(void)
 {
-  static char result[12];
-  char *p = result;
-
-  if (ConfigChannel.use_except)
-    *p++ = 'e';
-  if (ConfigFileEntry.glines)
-    *p++ = 'G';
-  *p++ = 'g';
-
-  /* might wanna hide this :P */
-  if (ServerInfo.hub && 
-      (!ConfigFileEntry.disable_remote || IsOper(source_p)))
-  {
-    *p++ = 'H';
-  }
-
-  if (ConfigChannel.use_invex)
-    *p++ = 'I';
-  if (ConfigChannel.use_knock)
-    *p++ = 'K';
-  *p++ = 'M';
-
-  if (ConfigFileEntry.ignore_bogus_ts)
-    *p++ = 'T';
-#ifdef USE_SYSLOG
-  *p++ = 'Y';
-#endif
-#ifdef HAVE_LIBZ
-  *p++ = 'Z';
-#endif
-  *p++ = '6';
-
-  *p = '\0';
-
-  return result;
+  mod_add_cmd(&version_msgtab);
 }
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&version_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1495 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = 0
+};
