@@ -19,46 +19,20 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_wallops.c 33 2005-10-02 20:50:00Z knight $
+ *  $Id: m_wallops.c 1474 2012-07-22 14:44:07Z michael $
  */
 
 #include "stdinc.h"
-#include "handlers.h"
 #include "client.h"
 #include "ircd.h"
 #include "irc_string.h"
 #include "numeric.h"
 #include "send.h"
 #include "s_user.h"
-#include "s_conf.h"
-#include "msg.h"
 #include "parse.h"
 #include "modules.h"
 #include "s_serv.h"
 
-static void ms_wallops(struct Client *, struct Client *, int, char **);
-static void mo_wallops(struct Client *, struct Client *, int, char **);
-
-struct Message wallops_msgtab = {
-  "WALLOPS", 0, 0, 2, 0, MFLG_SLOW, 0,
-  {m_unregistered, m_not_oper, ms_wallops, m_ignore, mo_wallops, m_ignore}
-};
-
-#ifndef STATIC_MODULES
-void
-_modinit(void)
-{
-  mod_add_cmd(&wallops_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&wallops_msgtab);
-}
- 
-const char *_version = "$Revision: 33 $";
-#endif
 
 /*
  * mo_wallops (write to *all* opers currently online)
@@ -79,9 +53,9 @@ mo_wallops(struct Client *client_p, struct Client *source_p,
   }
 
   sendto_wallops_flags(UMODE_OPERWALL, source_p, "OPERWALL - %s", message);
-  sendto_server(NULL, source_p, NULL, CAP_TS6, NOCAPS, LL_ICLIENT,
+  sendto_server(NULL, CAP_TS6, NOCAPS,
                 ":%s WALLOPS :%s", ID(source_p), message);
-  sendto_server(NULL, source_p, NULL, NOCAPS, CAP_TS6, LL_ICLIENT,
+  sendto_server(NULL, NOCAPS, CAP_TS6,
                 ":%s WALLOPS :%s", source_p->name, message);
 }
 
@@ -92,7 +66,7 @@ mo_wallops(struct Client *client_p, struct Client *source_p,
  */
 static void
 ms_wallops(struct Client *client_p, struct Client *source_p,
-	   int parc, char *parv[])
+           int parc, char *parv[])
 { 
   const char *message = parv[1];
 
@@ -104,9 +78,35 @@ ms_wallops(struct Client *client_p, struct Client *source_p,
   else
     sendto_wallops_flags(UMODE_WALLOP, source_p, "%s", message); 
 
-  sendto_server(client_p, source_p, NULL, CAP_TS6, NOCAPS, LL_ICLIENT,
+  sendto_server(client_p, CAP_TS6, NOCAPS,
                 ":%s WALLOPS :%s", ID(source_p), message);
-  sendto_server(client_p, source_p, NULL, NOCAPS, CAP_TS6, LL_ICLIENT,
+  sendto_server(client_p, NOCAPS, CAP_TS6,
                 ":%s WALLOPS :%s", source_p->name, message);
 }
 
+static struct Message wallops_msgtab = {
+  "WALLOPS", 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
+  {m_unregistered, m_not_oper, ms_wallops, m_ignore, mo_wallops, m_ignore}
+};
+
+static void
+module_init(void)
+{
+  mod_add_cmd(&wallops_msgtab);
+}
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&wallops_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1474 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = 0
+};
