@@ -19,12 +19,11 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_who.c 313 2005-12-23 10:55:47Z michael $
+ *  $Id: m_who.c 1549 2012-10-01 18:11:11Z michael $
  */
+
 #include "stdinc.h"
-#include "tools.h"
-#include "common.h"   
-#include "handlers.h"
+#include "list.h"
 #include "client.h"
 #include "channel.h"
 #include "channel_mode.h"
@@ -33,45 +32,17 @@
 #include "numeric.h"
 #include "s_serv.h"
 #include "send.h"
-#include "list.h"
 #include "irc_string.h"
 #include "sprintf_irc.h"
-#include "s_conf.h"
-#include "msg.h"
+#include "conf.h"
 #include "parse.h"
 #include "modules.h"
 
-static time_t last_used = 0;
-
-static void m_who(struct Client *, struct Client *, int, char **);
-static void ms_who(struct Client *, struct Client *, int, char **);
-
-struct Message who_msgtab = {
-  "WHO", 0, 0, 2, 0, MFLG_SLOW, 0,
-  {m_unregistered, m_who, ms_who, m_ignore, m_who, m_ignore}
-};
-
-#ifndef STATIC_MODULES
-void
-_modinit(void)
-{
-  mod_add_cmd(&who_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&who_msgtab);
-}
-
-const char *_version = "$Revision: 313 $";
-#endif
-
-static void who_global(struct Client *source_p, char *mask, int server_oper);
-static void do_who(struct Client *source_p, struct Client *target_p,
-                   const char *chname, const char *op_flags);
-static void do_who_on_channel(struct Client *source_p, struct Channel *chptr,
-                              const char *chname, int member, int server_oper);
+static void who_global(struct Client *, char *, int);
+static void do_who(struct Client *, struct Client *,
+                   const char *, const char *);
+static void do_who_on_channel(struct Client *, struct Channel *,
+                              const char *, int, int);
 
 /*
 ** m_who
@@ -88,25 +59,13 @@ m_who(struct Client *client_p, struct Client *source_p,
   dlink_node *lp;
   int server_oper = parc > 2 ? (*parv[2] == 'o') : 0; /* Show OPERS only */
   struct Channel *chptr;
-  const char *from, *to;
-
-  if (IsCapable(source_p->from, CAP_TS6) && HasID(source_p))
-  {
-    from = me.id;
-    to = source_p->id;
-  }
-  else
-  {
-    from = me.name;
-    to = source_p->name;
-  }
 
   /* See if mask is there, collapse it or return if not there */
-  if (mask == NULL || !*mask)
+  if (EmptyString(mask))
   {
     who_global(source_p, mask, server_oper);
     sendto_one(source_p, form_str(RPL_ENDOFWHO),
-               from, to, "*");
+               me.name, source_p->name, "*");
     return;
   }
 
@@ -114,16 +73,17 @@ m_who(struct Client *client_p, struct Client *source_p,
   collapse(mask);
 
   /* '/who *' */
-  if (mask[0] == '*' && !mask[1])
+  if (!strcmp(mask, "*"))
   {
     if ((lp = source_p->channel.head) != NULL)
     {
       struct Channel *mychannel = ((struct Membership *)lp->data)->chptr;
-      do_who_on_channel(source_p, mychannel, mychannel->chname, YES,
+      do_who_on_channel(source_p, mychannel, mychannel->chname, 1,
                         server_oper);
     }
 
-    sendto_one(source_p, form_str(RPL_ENDOFWHO), from, to, "*");
+    sendto_one(source_p, form_str(RPL_ENDOFWHO),
+               me.name, source_p->name, "*");
     return;
   }
 
@@ -134,23 +94,20 @@ m_who(struct Client *client_p, struct Client *source_p,
     if ((chptr = hash_find_channel(mask)) != NULL)
     {
       if (IsMember(source_p, chptr))
-        do_who_on_channel(source_p, chptr, chptr->chname, YES, server_oper);
+        do_who_on_channel(source_p, chptr, chptr->chname, 1, server_oper);
       else if (!SecretChannel(chptr))
-        do_who_on_channel(source_p, chptr, chptr->chname, NO, server_oper);
+        do_who_on_channel(source_p, chptr, chptr->chname, 0, server_oper);
     }
 
     sendto_one(source_p, form_str(RPL_ENDOFWHO),
-               from, to, mask);
+               me.name, source_p->name, mask);
     return;
   }
 
   /* '/who nick' */
-  if (((target_p = find_client(mask)) != NULL) &&
-      IsClient(target_p) && (!server_oper || IsOper(target_p)))
+  if (((target_p = hash_find_client(mask)) != NULL) &&
+      IsClient(target_p) && (!server_oper || HasUMode(target_p, UMODE_OPER)))
   {
-    if (IsServer(client_p))
-      client_burst_if_needed(client_p,target_p);
-
     DLINK_FOREACH(lp, target_p->channel.head)
     {
       chptr = ((struct Membership *) lp->data)->chptr;
@@ -160,24 +117,24 @@ m_who(struct Client *client_p, struct Client *source_p,
 
     if (lp != NULL)
       do_who(source_p, target_p, chptr->chname,
-             get_member_status(lp->data, NO));
+             get_member_status(lp->data, !!HasCap(source_p, CAP_MULTI_PREFIX)));
     else
       do_who(source_p, target_p, NULL, "");
 
     sendto_one(source_p, form_str(RPL_ENDOFWHO),
-               from, to, mask);
+               me.name, source_p->name, mask);
     return;
   }
 
   /* '/who 0' */
-  if (mask[0] == '0' && !mask[1])
+  if (!strcmp(mask, "0"))
     who_global(source_p, NULL, server_oper);
   else
     who_global(source_p, mask, server_oper);
 
- /* Wasn't a nick, wasn't a channel, wasn't a '*' so ... */
+  /* Wasn't a nick, wasn't a channel, wasn't a '*' so ... */
   sendto_one(source_p, form_str(RPL_ENDOFWHO),
-             from, to, mask);
+             me.name, source_p->name, mask);
 }
 
 /* who_common_channel
@@ -195,27 +152,28 @@ static void
 who_common_channel(struct Client *source_p, struct Channel *chptr,
                    char *mask, int server_oper, int *maxmatches)
 {
-  dlink_node *ptr;
-  struct Client *target_p;
+  dlink_node *ptr = NULL;
 
   DLINK_FOREACH(ptr, chptr->members.head)
   {
-    target_p = ((struct Membership *)ptr->data)->client_p;
+    struct Client *target_p = ((struct Membership *)ptr->data)->client_p;
 
-    if (!IsInvisible(target_p) || IsMarked(target_p))
+    if (!HasUMode(target_p, UMODE_INVISIBLE) || HasFlag(target_p, FLAGS_MARK))
       continue;
 
-    if (server_oper && !IsOper(target_p))
-      continue;
+    if (server_oper)
+      if (!HasUMode(target_p, UMODE_OPER) ||
+          (HasUMode(target_p, UMODE_HIDDEN) && !HasUMode(source_p, UMODE_OPER)))
+        continue;
 
-    SetMark(target_p);
+    AddFlag(target_p, FLAGS_MARK);
 
     assert(target_p->servptr != NULL);
 
     if ((mask == NULL) ||
       match(mask, target_p->name) || match(mask, target_p->username) ||
       match(mask, target_p->host) || 
-      ((!ConfigServerHide.hide_servers || IsOper(source_p)) &&
+      ((!ConfigServerHide.hide_servers || HasUMode(source_p, UMODE_OPER)) &&
        match(mask, target_p->servptr->name)) ||
       match(mask, target_p->info))
     {
@@ -249,18 +207,18 @@ who_global(struct Client *source_p, char *mask, int server_oper)
   dlink_node *gcptr;
   dlink_node *gcptr_next;
   int maxmatches = 500;
+  static time_t last_used = 0;
 
-  if (!IsOper(source_p))
+  if (!HasUMode(source_p, UMODE_OPER))
   {
     if ((last_used + ConfigFileEntry.pace_wait) > CurrentTime)
     {
       /* safe enough to give this on a local connect only */
-      sendto_one(source_p, form_str(RPL_LOAD2HI),
-        me.name, source_p->name);
+      sendto_one(source_p, form_str(RPL_LOAD2HI), me.name, source_p->name);
       return;
     }
-    else
-      last_used = CurrentTime;
+
+    last_used = CurrentTime;
   }
 
   /* first, list all matching invisible clients on common channels */
@@ -278,14 +236,16 @@ who_global(struct Client *source_p, char *mask, int server_oper)
     if (!IsClient(target_p))
       continue;
 
-    if (IsInvisible(target_p))
+    if (HasUMode(target_p, UMODE_INVISIBLE))
     {
-      ClearMark(target_p);
+      DelFlag(target_p, FLAGS_MARK);
       continue;
     }
 
-    if (server_oper && !IsOper(target_p))
-      continue;
+    if (server_oper)
+      if (!HasUMode(target_p, UMODE_OPER) ||
+          (HasUMode(target_p, UMODE_HIDDEN) && !HasUMode(source_p, UMODE_OPER)))
+        continue;
 
     assert(target_p->servptr != NULL);
 
@@ -329,11 +289,13 @@ do_who_on_channel(struct Client *source_p, struct Channel *chptr,
     ms = ptr->data;
     target_p = ms->client_p;
 
-    if (member || !IsInvisible(target_p))
+    if (member || !HasUMode(target_p, UMODE_INVISIBLE))
     {
-      if (server_oper && !IsOper(target_p))
-        continue;
-      do_who(source_p, target_p, chname, get_member_status(ms, NO));
+      if (server_oper)
+        if (!HasUMode(target_p, UMODE_OPER) ||
+            (HasUMode(target_p, UMODE_HIDDEN) && !HasUMode(source_p, UMODE_OPER)))
+          continue;
+      do_who(source_p, target_p, chname, get_member_status(ms, !!HasCap(source_p, CAP_MULTI_PREFIX)));
     }
   }
 }
@@ -351,61 +313,56 @@ static void
 do_who(struct Client *source_p, struct Client *target_p,
        const char *chname, const char *op_flags)
 {
-  char status[6];
-  const char *from, *to;
+  char status[7]; /* G*@%+\0 */
 
-  if (IsCapable(source_p->from, CAP_TS6) && HasID(source_p))
-  {
-    from = me.id;
-    to = source_p->id;
-  }
+  if (HasUMode(source_p, UMODE_OPER))
+    snprintf(status, sizeof(status), "%c%s%s", target_p->away[0] ? 'G' : 'H',
+             HasUMode(target_p, UMODE_OPER) ? "*" : "", op_flags);
   else
-  {
-    from = me.name;
-    to = source_p->name;
-  }
-
-  if (IsOper(source_p))
-    ircsprintf(status, "%c%s%s%s", 
-	       target_p->away ? 'G' : 'H',
-	       IsOper(target_p) ? "*" : "", IsCaptured(target_p) ? "#" : "", op_flags);
-  else
-    ircsprintf(status, "%c%s%s", target_p->away ? 'G' : 'H',
-	       IsOper(target_p) ? "*" : "", op_flags);
+    snprintf(status, sizeof(status), "%c%s%s", target_p->away[0] ? 'G' : 'H',
+             HasUMode(target_p, UMODE_OPER) &&
+             !HasUMode(target_p, UMODE_HIDDEN) ? "*" : "", op_flags);
 
   if (ConfigServerHide.hide_servers)
   {
-    sendto_one(source_p, form_str(RPL_WHOREPLY), from, to,
+    sendto_one(source_p, form_str(RPL_WHOREPLY), me.name, source_p->name,
 	       (chname) ? (chname) : "*",
-	       target_p->username, target_p->host,
-	       IsOper(source_p) ? target_p->servptr->name : "*",
+               target_p->username, target_p->host,
+	       HasUMode(source_p, UMODE_OPER) ? target_p->servptr->name : "*",
 	       target_p->name, status, 0, target_p->info);
   }
   else
   {
-    sendto_one(source_p, form_str(RPL_WHOREPLY), from, to,
-	       (chname) ? (chname) : "*",
-	       target_p->username,
+    sendto_one(source_p, form_str(RPL_WHOREPLY), me.name, source_p->name,
+	       (chname) ? (chname) : "*", target_p->username,
 	       target_p->host, target_p->servptr->name, target_p->name,
 	       status, target_p->hopcount, target_p->info);
   }
 }
 
-/*
-** ms_who()
-**      parv[0] = sender prefix
-**      parv[1] = nickname mask list
-**      parv[2] = additional selection flag, only 'o' for now.
-*/
+static struct Message who_msgtab = {
+  "WHO", 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
+  {m_unregistered, m_who, m_ignore, m_ignore, m_who, m_ignore}
+};
+
 static void
-ms_who(struct Client *client_p, struct Client *source_p,
-       int parc, char *parv[])
+module_init(void)
 {
-  /* If its running as a hub, and linked with lazy links
-   * then allow leaf to use normal client m_who()
-   * other wise, ignore it.
-   */
-  if (IsClient(source_p) && ServerInfo.hub &&
-      IsCapable(client_p->from, CAP_LL))
-    m_who(client_p, source_p, parc, parv);
+  mod_add_cmd(&who_msgtab);
 }
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&who_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1549 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = 0
+};
