@@ -19,14 +19,13 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_whowas.c 269 2005-11-14 19:39:09Z adx $
+ *  $Id: m_whowas.c 1358 2012-04-22 13:49:23Z michael $
  */
 
 #include "stdinc.h"
+#include "list.h"
 #include "whowas.h"
-#include "handlers.h"
 #include "client.h"
-#include "common.h"
 #include "hash.h"
 #include "irc_string.h"
 #include "ircd.h"
@@ -35,89 +34,19 @@
 #include "s_serv.h"
 #include "s_user.h"
 #include "send.h"
-#include "s_conf.h"
-#include "msg.h"
+#include "conf.h"
 #include "parse.h"
 #include "modules.h"
 
 
-static void m_whowas(struct Client *, struct Client *, int, char *[]);
-static void mo_whowas(struct Client *, struct Client *, int, char *[]);
-static void whowas_do(struct Client *, struct Client *, int, char *[]);
-
-struct Message whowas_msgtab = {
-  "WHOWAS", 0, 0, 0, 0, MFLG_SLOW, 0,
-  { m_unregistered, m_whowas, mo_whowas, m_ignore, mo_whowas, m_ignore }
-};
-
-#ifndef STATIC_MODULES
-void
-_modinit(void)
-{
-  mod_add_cmd(&whowas_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&whowas_msgtab);
-}
-
-const char *_version = "$Revision: 269 $";
-#endif
-
-/*
-** m_whowas
-**      parv[0] = sender prefix
-**      parv[1] = nickname queried
-*/
-static void
-m_whowas(struct Client *client_p, struct Client *source_p,
-         int parc, char *parv[])
-{
-  static time_t last_used = 0;
-
-  if (parc < 2 || *parv[1] == '\0')
-  {
-    sendto_one(source_p, form_str(ERR_NONICKNAMEGIVEN),
-               me.name, source_p->name);
-    return;
-  }
-
-  if ((last_used + ConfigFileEntry.pace_wait) > CurrentTime)
-  {
-    sendto_one(source_p,form_str(RPL_LOAD2HI),
-               me.name, source_p->name);
-    return;
-  }
-  else
-    last_used = CurrentTime;
-
-  whowas_do(client_p, source_p, parc, parv);
-}
-
-static void
-mo_whowas(struct Client *client_p, struct Client *source_p,
-          int parc, char *parv[])
-{
-  if (parc < 2 || *parv[1] == '\0')
-  {
-    sendto_one(source_p, form_str(ERR_NONICKNAMEGIVEN),
-               me.name, source_p->name);
-    return;
-  }
-
-  whowas_do(client_p, source_p, parc, parv);
-}
-
 static void
 whowas_do(struct Client *client_p, struct Client *source_p,
           int parc, char *parv[])
 {
-  struct Whowas *temp = NULL;
   int cur = 0;
   int max = -1;
   char *p = NULL, *nick = NULL;
+  const dlink_node *ptr = NULL;
 
   if (parc > 2)
   {
@@ -140,18 +69,18 @@ whowas_do(struct Client *client_p, struct Client *source_p,
   if (*nick == '\0')
     return;
 
-  temp  = WHOWASHASH[strhash(nick)];
-
-  for (; temp; temp = temp->next)
+  DLINK_FOREACH(ptr, WHOWASHASH[strhash(nick)].head)
   {
-    if (irccmp(nick, temp->name) == 0)
+    const struct Whowas *temp = ptr->data;
+
+    if (!irccmp(nick, temp->name))
     {
       sendto_one(source_p, form_str(RPL_WHOWASUSER),
                  me.name, source_p->name, temp->name,
                  temp->username, temp->hostname,
                  temp->realname);
 
-      if (ConfigServerHide.hide_servers && !IsOper(source_p))
+      if (ConfigServerHide.hide_servers && !HasUMode(source_p, UMODE_OPER))
         sendto_one(source_p, form_str(RPL_WHOISSERVER),
                    me.name, source_p->name, temp->name,
                    ServerInfo.network_name, myctime(temp->logoff));
@@ -159,7 +88,7 @@ whowas_do(struct Client *client_p, struct Client *source_p,
         sendto_one(source_p, form_str(RPL_WHOISSERVER),
                    me.name, source_p->name, temp->name,
                    temp->servername, myctime(temp->logoff));
-      cur++;
+      ++cur;
     }
 
     if (max > 0 && cur >= max)
@@ -173,3 +102,74 @@ whowas_do(struct Client *client_p, struct Client *source_p,
   sendto_one(source_p, form_str(RPL_ENDOFWHOWAS),
              me.name, source_p->name, parv[1]);
 }
+
+/*
+** m_whowas
+**      parv[0] = sender prefix
+**      parv[1] = nickname queried
+*/
+static void
+m_whowas(struct Client *client_p, struct Client *source_p,
+         int parc, char *parv[])
+{
+  static time_t last_used = 0;
+
+  if (parc < 2 || EmptyString(parv[1]))
+  {
+    sendto_one(source_p, form_str(ERR_NONICKNAMEGIVEN),
+               me.name, source_p->name);
+    return;
+  }
+
+  if ((last_used + ConfigFileEntry.pace_wait) > CurrentTime)
+  {
+    sendto_one(source_p,form_str(RPL_LOAD2HI),
+               me.name, source_p->name);
+    return;
+  }
+
+  last_used = CurrentTime;
+
+  whowas_do(client_p, source_p, parc, parv);
+}
+
+static void
+mo_whowas(struct Client *client_p, struct Client *source_p,
+          int parc, char *parv[])
+{
+  if (parc < 2 || EmptyString(parv[1]))
+  {
+    sendto_one(source_p, form_str(ERR_NONICKNAMEGIVEN),
+               me.name, source_p->name);
+    return;
+  }
+
+  whowas_do(client_p, source_p, parc, parv);
+}
+
+static struct Message whowas_msgtab = {
+  "WHOWAS", 0, 0, 0, MAXPARA, MFLG_SLOW, 0,
+  { m_unregistered, m_whowas, mo_whowas, m_ignore, mo_whowas, m_ignore }
+};
+
+static void
+module_init(void)
+{
+  mod_add_cmd(&whowas_msgtab);
+}
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&whowas_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1358 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = 0
+};
