@@ -19,14 +19,13 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: m_xline.c 382 2006-01-31 12:13:42Z michael $
+ *  $Id: m_xline.c 1309 2012-03-25 11:24:18Z michael $
  */
 
 #include "stdinc.h"
-#include "tools.h"
+#include "list.h"
 #include "channel.h"
 #include "client.h"
-#include "common.h"
 #include "irc_string.h"
 #include "sprintf_irc.h"
 #include "ircd.h"
@@ -34,25 +33,16 @@
 #include "numeric.h"
 #include "fdlist.h"
 #include "s_bsd.h"
-#include "s_conf.h"
-#include "s_log.h"
+#include "conf.h"
+#include "log.h"
 #include "s_misc.h"
 #include "send.h"
 #include "hash.h"
-#include "handlers.h"
 #include "s_serv.h"
-#include "msg.h"
 #include "parse.h"
 #include "modules.h"
 #include "resv.h"
-#include "list.h"
 
-static void mo_xline(struct Client *, struct Client *, int, char *[]);
-static void ms_xline(struct Client *, struct Client *, int, char *[]);
-static void me_xline(struct Client *, struct Client *, int, char *[]);
-
-static void mo_unxline(struct Client *, struct Client *, int, char *[]);
-static void ms_unxline(struct Client *, struct Client *, int, char *[]);
 
 static int valid_xline(struct Client *, char *, char *, int);
 static void write_xline(struct Client *, char *, char *, time_t);
@@ -61,35 +51,6 @@ static int remove_txline_match(const char *);
 
 static void relay_xline(struct Client *, char *[]);
 
-struct Message xline_msgtab = {
-  "XLINE", 0, 0, 2, 0, MFLG_SLOW, 0,
-  { m_unregistered, m_not_oper, ms_xline, me_xline, mo_xline, m_ignore }
-};
-
-struct Message unxline_msgtab = {
-  "UNXLINE", 0, 0, 2, 0, MFLG_SLOW, 0,
-  { m_unregistered, m_not_oper, ms_unxline, m_ignore, mo_unxline, m_ignore }
-};
-
-#ifndef STATIC_MODULES
-void
-_modinit(void)
-{
-  mod_add_cmd(&xline_msgtab);
-  mod_add_cmd(&unxline_msgtab);
-}
-
-void
-_moddeinit(void)
-{
-  mod_del_cmd(&xline_msgtab);
-  mod_del_cmd(&unxline_msgtab);
-}
-
-const char *_version = "$Revision: 382 $";
-#endif
-
-
 /* mo_xline()
  *
  * inputs	- pointer to server
@@ -111,7 +72,7 @@ mo_xline(struct Client *client_p, struct Client *source_p,
   char *target_server = NULL;
   time_t tkline_time = 0;
 
-  if (!IsOperX(source_p))
+  if (!HasOFlag(source_p, OPER_FLAG_X))
   {
     sendto_one(source_p, form_str(ERR_NOPRIVS),
                me.name, source_p->name, "xline");
@@ -238,7 +199,7 @@ relay_xline(struct Client *source_p, char *parv[])
   if (!match(parv[1], me.name))
     return;
 
-  if (find_matching_name_conf(ULINE_TYPE, source_p->servptr->name,
+  if (HasFlag(source_p, FLAGS_SERVICE) || find_matching_name_conf(ULINE_TYPE, source_p->servptr->name,
                               source_p->username, source_p->host,
                               SHARED_XLINE))
   {
@@ -273,7 +234,7 @@ mo_unxline(struct Client *client_p, struct Client *source_p,
   char *gecos = NULL;
   char *target_server = NULL;
 
-  if (!IsOperX(source_p))
+  if (!HasOFlag(source_p, OPER_FLAG_X))
   {
     sendto_one(source_p, form_str(ERR_NOPRIVS),
                me.name, source_p->name, "unxline");
@@ -323,7 +284,7 @@ ms_unxline(struct Client *client_p, struct Client *source_p,
   if (!match(parv[1], me.name))
     return;
 
-  if (find_matching_name_conf(ULINE_TYPE, source_p->servptr->name,
+  if (HasFlag(source_p, FLAGS_SERVICE) || find_matching_name_conf(ULINE_TYPE, source_p->servptr->name,
                               source_p->username, source_p->host,
                               SHARED_UNXLINE))
     remove_xline(source_p, parv[2]);
@@ -399,7 +360,7 @@ write_xline(struct Client *source_p, char *gecos, char *reason,
     sendto_one(source_p, ":%s NOTICE %s :Added temporary %d min. X-Line [%s]",
 	       MyConnect(source_p) ? me.name : ID_or_name(&me, source_p->from),
 	       source_p->name, (int)tkline_time/60, conf->name);
-    ilog(L_TRACE, "%s added temporary %d min. X-Line for [%s] [%s]",
+    ilog(LOG_TYPE_KLINE, "%s added temporary %d min. X-Line for [%s] [%s]",
 	 source_p->name, (int)tkline_time/60,
 	 conf->name, match_item->reason);
     match_item->hold = CurrentTime + tkline_time;
@@ -422,7 +383,7 @@ remove_xline(struct Client *source_p, char *gecos)
     sendto_realops_flags(UMODE_ALL, L_ALL,
                          "%s has removed the temporary X-Line for: [%s]",
                          get_oper_name(source_p), gecos);
-    ilog(L_NOTICE, "%s removed temporary X-Line for [%s]",
+    ilog(LOG_TYPE_KLINE, "%s removed temporary X-Line for [%s]",
          source_p->name, gecos);
     return;
   }
@@ -434,7 +395,7 @@ remove_xline(struct Client *source_p, char *gecos)
     sendto_realops_flags(UMODE_ALL, L_ALL,
                          "%s has removed the X-Line for: [%s]",
                          get_oper_name(source_p), gecos);
-    ilog(L_NOTICE, "%s removed X-Line for [%s]",
+    ilog(LOG_TYPE_KLINE, "%s removed X-Line for [%s]",
          get_oper_name(source_p), gecos);
   }
   else
@@ -470,3 +431,37 @@ remove_txline_match(const char *gecos)
 
   return 0;
 }
+
+static struct Message xline_msgtab = {
+  "XLINE", 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
+  { m_unregistered, m_not_oper, ms_xline, me_xline, mo_xline, m_ignore }
+};
+
+static struct Message unxline_msgtab = {
+  "UNXLINE", 0, 0, 2, MAXPARA, MFLG_SLOW, 0,
+  { m_unregistered, m_not_oper, ms_unxline, m_ignore, mo_unxline, m_ignore }
+};
+
+static void
+module_init(void)
+{
+  mod_add_cmd(&xline_msgtab);
+  mod_add_cmd(&unxline_msgtab);
+}
+
+static void
+module_exit(void)
+{
+  mod_del_cmd(&xline_msgtab);
+  mod_del_cmd(&unxline_msgtab);
+}
+
+struct module module_entry = {
+  .node    = { NULL, NULL, NULL },
+  .name    = NULL,
+  .version = "$Revision: 1309 $",
+  .handle  = NULL,
+  .modinit = module_init,
+  .modexit = module_exit,
+  .flags   = 0
+};
