@@ -27,7 +27,7 @@
 
 /*! \file balloc.c
  * \brief A block allocator
- * \version $Id: balloc.c 670 2006-06-12 12:20:55Z michael $
+ * \version $Id: balloc.c 1013 2009-10-18 14:26:49Z michael $
  * 
  * About the block allocator
  *
@@ -50,6 +50,7 @@
  * down after we no longer need it.
  */
 
+
 #include "stdinc.h"
 #ifdef HAVE_MMAP /* We've got mmap() that is good */
 #include <sys/mman.h>
@@ -62,10 +63,10 @@
 #endif /* MAP_ANONYMOUS */
 #endif
 
-#include "ircd.h"
+#include "list.h"
 #include "balloc.h"
+#include "memory.h"
 #include "irc_string.h"
-#include "tools.h"
 #include "client.h"
 #include "send.h"
 #include "numeric.h"
@@ -83,7 +84,7 @@ static void heap_garbage_collection(void *);
  * \param ptr  Pointer to memory to be freed
  * \param size The size of the memory space
  */
-static inline void
+static void
 free_block(void *ptr, size_t size)
 {
 #ifdef HAVE_MMAP
@@ -122,7 +123,7 @@ initBlockHeap(void)
  * \param size Size of block to allocate
  * \return Address pointer to allocated data space
  */
-static inline void *
+static void *
 get_block(size_t size)
 {
 #ifdef HAVE_MMAP
@@ -241,8 +242,6 @@ BlockHeapCreate(const char *const name, size_t elemsize, int elemsperblock)
      outofmemory();    /* die.. out of memory */
   }
 
-  assert(bh);
-
   bh->next = heap_list;
   heap_list = bh;
 
