@@ -21,36 +21,34 @@
 
 /*! \file channel.c
  * \brief Responsible for managing channels, members, bans and topics
- * \version $Id: channel.c 675 2006-06-13 16:51:10Z adx $
+ * \version $Id: channel.c 1913 2013-04-28 11:46:26Z michael $
  */
 
 #include "stdinc.h"
-#include "tools.h"
+#include "list.h"
 #include "channel.h"
 #include "channel_mode.h"
 #include "client.h"
-#include "common.h"
 #include "hash.h"
 #include "hostmask.h"
 #include "irc_string.h"
 #include "sprintf_irc.h"
 #include "ircd.h"
-#include "list.h"
 #include "numeric.h"
 #include "s_serv.h"             /* captab */
 #include "s_user.h"
 #include "send.h"
-#include "s_conf.h"             /* ConfigFileEntry, ConfigChannel */
+#include "conf.h"             /* ConfigFileEntry, ConfigChannel */
 #include "event.h"
 #include "memory.h"
 #include "balloc.h"
+#include "s_misc.h"
+
 
 struct config_channel_entry ConfigChannel;
 dlink_list global_channel_list = { NULL, NULL, 0 };
-dlink_list lazylink_channels = { NULL, NULL, 0 };
 BlockHeap *ban_heap;    /*! \todo ban_heap shouldn't be a global var */
 
-static BlockHeap *topic_heap = NULL;
 static BlockHeap *member_heap = NULL;
 static BlockHeap *channel_heap = NULL;
 
@@ -70,7 +68,6 @@ init_channels(void)
 
   channel_heap = BlockHeapCreate("channel", sizeof(struct Channel), CHANNEL_HEAP_SIZE);
   ban_heap = BlockHeapCreate("ban", sizeof(struct Ban), BAN_HEAP_SIZE);
-  topic_heap = BlockHeapCreate("topic", TOPICLEN+1 + USERHOST_REPLYLEN, TOPIC_HEAP_SIZE);
   member_heap = BlockHeapCreate("member", sizeof(struct Membership), CHANNEL_HEAP_SIZE*2);
 }
 
@@ -142,7 +139,7 @@ remove_user_from_channel(struct Membership *member)
 
   BlockHeapFree(member_heap, member);
 
-  if (dlink_list_length(&chptr->members) == 0)
+  if (chptr->members.head == NULL)
     destroy_channel(chptr);
 }
 
@@ -185,7 +182,7 @@ send_members(struct Client *client_p, struct Channel *chptr,
     /* space will be converted into CR, but we also need space for LF..
      * That's why we use '- 1' here
      * -adx */
-    if (t + tlen - buf > sizeof(buf) - 1)
+    if (t + tlen - buf > IRCD_BUFSIZE - 1)
     {
       *(t - 1) = '\0';  /* kill the space and terminate the string */
       sendto_one(client_p, "%s", buf);
@@ -291,19 +288,13 @@ send_mode_list(struct Client *client_p, struct Channel *chptr,
 void
 send_channel_modes(struct Client *client_p, struct Channel *chptr)
 {
-  if (chptr->chname[0] != '#')
-    return;
-
   *modebuf = *parabuf = '\0';
   channel_modes(chptr, client_p, modebuf, parabuf);
   send_members(client_p, chptr, modebuf, parabuf);
 
   send_mode_list(client_p, chptr, &chptr->banlist, 'b');
-
-  if (IsCapable(client_p, CAP_EX))
-    send_mode_list(client_p, chptr, &chptr->exceptlist, 'e');
-  if (IsCapable(client_p, CAP_IE))
-    send_mode_list(client_p, chptr, &chptr->invexlist, 'I');
+  send_mode_list(client_p, chptr, &chptr->exceptlist, 'e');
+  send_mode_list(client_p, chptr, &chptr->invexlist, 'I');
 }
 
 /*! \brief check channel name for invalid characters
@@ -315,7 +306,7 @@ int
 check_channel_name(const char *name, int local)
 {
   const char *p = name;
-  int max_length = local ? LOCAL_CHANNELLEN : CHANNELLEN;
+  const int max_length = local ? LOCAL_CHANNELLEN : CHANNELLEN;
   assert(name != NULL);
 
   if (!IsChanPrefix(*p))
@@ -409,16 +400,9 @@ destroy_channel(struct Channel *chptr)
   free_channel_list(&chptr->exceptlist);
   free_channel_list(&chptr->invexlist);
 
-  /* Free the topic */
-  free_topic(chptr);
-
   dlinkDelete(&chptr->node, &global_channel_list);
   hash_del_channel(chptr);
 
-  if (ServerInfo.hub)
-    if ((ptr = dlinkFindDelete(&lazylink_channels, chptr)))
-      free_dlink_node(ptr);
-
   BlockHeapFree(channel_heap, chptr);
 }
 
@@ -427,7 +411,7 @@ destroy_channel(struct Channel *chptr)
  * \return string pointer "=" if public, "@" if secret else "*"
  */
 static const char *
-channel_pub_or_secret(struct Channel *chptr)
+channel_pub_or_secret(const struct Channel *chptr)
 {
   if (SecretChannel(chptr))
     return "@";
@@ -453,7 +437,7 @@ channel_member_names(struct Client *source_p, struct Channel *chptr,
   char *t = NULL, *start = NULL;
   int tlen = 0;
   int is_member = IsMember(source_p, chptr);
-  int multi_prefix = (source_p->localClient->cap_active & CAP_MULTI_PREFIX) != 0;
+  int multi_prefix = HasCap(source_p, CAP_MULTI_PREFIX) != 0;
 
   if (PubChannel(chptr) || is_member)
   {
@@ -468,7 +452,7 @@ channel_member_names(struct Client *source_p, struct Channel *chptr,
       ms       = ptr->data;
       target_p = ms->client_p;
 
-      if (IsInvisible(target_p) && !is_member)
+      if (HasUMode(target_p, UMODE_INVISIBLE) && !is_member)
         continue;
 
       tlen = strlen(target_p->name) + 1;  /* nick + space */
@@ -608,7 +592,7 @@ find_bmask(const struct Client *who, const dlink_list *const list)
 
   DLINK_FOREACH(ptr, list->head)
   {
-    struct Ban *bp = ptr->data;
+    const struct Ban *bp = ptr->data;
 
     if (match(bp->name, who->name) && match(bp->username, who->username))
     {
@@ -645,10 +629,10 @@ find_bmask(const struct Client *who, const dlink_list *const list)
  * \return 0 if not banned, 1 otherwise
  */
 int
-is_banned(struct Channel *chptr, struct Client *who)
+is_banned(const struct Channel *chptr, const struct Client *who)
 {
   if (find_bmask(who, &chptr->banlist))
-    if (!ConfigChannel.use_except || !find_bmask(who, &chptr->exceptlist))
+    if (!find_bmask(who, &chptr->exceptlist))
       return 1;
 
   return 0;
@@ -667,12 +651,23 @@ can_join(struct Client *source_p, struct Channel *chptr, const char *key)
   if (is_banned(chptr, source_p))
     return ERR_BANNEDFROMCHAN;
 
+#ifdef HAVE_LIBCRYPTO
+  if ((chptr->mode.mode & MODE_SSLONLY) && !source_p->localClient->fd.ssl)
+    return ERR_SSLONLYCHAN;
+#endif
+
+  if ((chptr->mode.mode & MODE_REGONLY) && !HasUMode(source_p, UMODE_REGISTERED))
+    return ERR_NEEDREGGEDNICK;
+
+  if ((chptr->mode.mode & MODE_OPERONLY) && !HasUMode(source_p, UMODE_OPER))
+    return ERR_OPERONLYCHAN;
+
   if (chptr->mode.mode & MODE_INVITEONLY)
     if (!dlinkFind(&source_p->localClient->invited, chptr))
-      if (!ConfigChannel.use_invex || !find_bmask(source_p, &chptr->invexlist))
+      if (!find_bmask(source_p, &chptr->invexlist))
         return ERR_INVITEONLYCHAN;
 
-  if (chptr->mode.key[0] && (!key || irccmp(chptr->mode.key, key)))
+  if (chptr->mode.key[0] && (!key || strcmp(chptr->mode.key, key)))
     return ERR_BADCHANNELKEY;
 
   if (chptr->mode.limit && dlink_list_length(&chptr->members) >=
@@ -700,7 +695,7 @@ find_channel_link(struct Client *client_p, struct Channel *chptr)
 
   DLINK_FOREACH(ptr, client_p->channel.head)
     if (((struct Membership *)ptr->data)->chptr == chptr)
-      return (struct Membership *)ptr->data;
+      return ptr->data;
 
   return NULL;
 }
@@ -711,18 +706,18 @@ find_channel_link(struct Client *client_p, struct Channel *chptr)
  * \param ms       pointer to Membership struct (can be NULL)
  * \return CAN_SEND_OPV if op or voiced on channel\n
  *         CAN_SEND_NONOP if can send to channel but is not an op\n
- *         CAN_SEND_NO if they cannot send to channel\n
+ *         ERR_CANNOTSENDTOCHAN or ERR_NEEDREGGEDNICK if they cannot send to channel\n
  */
 int
 can_send(struct Channel *chptr, struct Client *source_p, struct Membership *ms)
 {
-  if (IsServer(source_p))
+  if (IsServer(source_p) || HasFlag(source_p, FLAGS_SERVICE))
     return CAN_SEND_OPV;
 
   if (MyClient(source_p) && !IsExemptResv(source_p))
-    if (!(IsOper(source_p) && ConfigFileEntry.oper_pass_resv))
+    if (!(HasUMode(source_p, UMODE_OPER) && ConfigFileEntry.oper_pass_resv))
       if (!hash_find_resv(chptr->chname) == ConfigChannel.restrict_channels)
-        return CAN_SEND_NO;
+        return ERR_CANNOTSENDTOCHAN;
 
   if (ms != NULL || (ms = find_channel_link(source_p, chptr)))
   {
@@ -733,14 +728,14 @@ can_send(struct Channel *chptr, struct Client *source_p, struct Membership *ms)
     if (ConfigChannel.quiet_on_ban && MyClient(source_p))
     {
       if (ms->flags & CHFL_BAN_SILENCED)
-        return CAN_SEND_NO;
+        return ERR_CANNOTSENDTOCHAN;
 
       if (!(ms->flags & CHFL_BAN_CHECKED))
       {
         if (is_banned(chptr, source_p))
         {
           ms->flags |= (CHFL_BAN_CHECKED|CHFL_BAN_SILENCED);
-          return CAN_SEND_NO;
+          return ERR_CANNOTSENDTOCHAN;
         }
 
         ms->flags |= CHFL_BAN_CHECKED;
@@ -748,10 +743,13 @@ can_send(struct Channel *chptr, struct Client *source_p, struct Membership *ms)
     }
   }
   else if (chptr->mode.mode & MODE_NOPRIVMSGS)
-    return CAN_SEND_NO;
+    return ERR_CANNOTSENDTOCHAN;
 
   if (chptr->mode.mode & MODE_MODERATED)
-    return CAN_SEND_NO;
+    return ERR_CANNOTSENDTOCHAN;
+
+  if ((chptr->mode.mode & MODE_REGONLY) && !HasUMode(source_p, UMODE_REGISTERED))
+    return ERR_NEEDREGGEDNICK;
 
   return CAN_SEND_NONOP;
 }
@@ -852,46 +850,6 @@ check_splitmode(void *unused)
   }
 }
 
-/*! \brief Allocates a new topic
- * \param chptr Channel to allocate a new topic for
- */
-static void
-allocate_topic(struct Channel *chptr)
-{
-  void *ptr = NULL;
-
-  if (chptr == NULL)
-    return;
-
-  ptr = BlockHeapAlloc(topic_heap);  
-
-  /* Basically we allocate one large block for the topic and
-   * the topic info.  We then split it up into two and shove it
-   * in the chptr 
-   */
-  chptr->topic       = ptr;
-  chptr->topic_info  = (char *)ptr + TOPICLEN+1;
-  *chptr->topic      = '\0';
-  *chptr->topic_info = '\0';
-}
-
-void
-free_topic(struct Channel *chptr)
-{
-  void *ptr = NULL;
-  assert(chptr);
-  if (chptr->topic == NULL)
-    return;
-
-  /*
-   * If you change allocate_topic you MUST change this as well
-   */
-  ptr = chptr->topic; 
-  BlockHeapFree(topic_heap, ptr);    
-  chptr->topic      = NULL;
-  chptr->topic_info = NULL;
-}
-
 /*! \brief Sets the channel topic for chptr
  * \param chptr      Pointer to struct Channel
  * \param topic      The topic string
@@ -900,24 +858,13 @@ free_topic(struct Channel *chptr)
  */
 void
 set_channel_topic(struct Channel *chptr, const char *topic,
-                  const char *topic_info, time_t topicts)
+                  const char *topic_info, time_t topicts, int local)
 {
-  if (!EmptyString(topic))
-  {
-    if (chptr->topic == NULL)
-      allocate_topic(chptr);
-
-    strlcpy(chptr->topic, topic, TOPICLEN+1);
-    strlcpy(chptr->topic_info, topic_info, USERHOST_REPLYLEN);
-    chptr->topic_time = topicts; 
-  }
+  if (local)
+    strlcpy(chptr->topic, topic, IRCD_MIN(sizeof(chptr->topic), ServerInfo.max_topic_length + 1));
   else
-  {
-    /*
-     * Do not reset chptr->topic_time here, it's required for
-     * bursting topics properly.
-     */
-    if (chptr->topic != NULL)
-      free_topic(chptr);
-  }
+    strlcpy(chptr->topic, topic, sizeof(chptr->topic));
+
+  strlcpy(chptr->topic_info, topic_info, sizeof(chptr->topic_info));
+  chptr->topic_time = topicts; 
 }
