@@ -19,32 +19,30 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: channel_mode.c 593 2006-05-12 05:47:32Z michael $
+ *  $Id: channel_mode.c 1771 2013-01-19 23:11:22Z michael $
  */
 
 #include "stdinc.h"
-#include "tools.h"
+#include "list.h"
 #include "channel.h"
 #include "channel_mode.h"
 #include "client.h"
-#include "common.h"
 #include "hash.h"
 #include "hostmask.h"
 #include "irc_string.h"
 #include "sprintf_irc.h"
 #include "ircd.h"
-#include "list.h"
 #include "numeric.h"
 #include "s_serv.h"             /* captab */
 #include "s_user.h"
 #include "send.h"
 #include "whowas.h"
-#include "s_conf.h"             /* ConfigFileEntry, ConfigChannel */
+#include "conf.h"             /* ConfigFileEntry, ConfigChannel */
 #include "event.h"
 #include "memory.h"
 #include "balloc.h"
-#include "s_log.h"
-#include "msg.h"
+#include "log.h"
+#include "parse.h"
 
 /* some small utility functions */
 static char *check_string(char *);
@@ -56,6 +54,9 @@ static void chm_nosuch(struct Client *, struct Client *,
 static void chm_simple(struct Client *, struct Client *, struct Channel *,
                        int, int *, char **, int *, int, int, char, void *,
                        const char *);
+static void chm_registered(struct Client *, struct Client *, struct Channel *,
+                       int, int *, char **, int *, int, int, char, void *,
+                       const char *);
 static void chm_limit(struct Client *, struct Client *, struct Channel *,
                       int, int *, char **, int *, int, int, char, void *,
                       const char *);
@@ -83,7 +84,7 @@ static void chm_invex(struct Client *, struct Client *, struct Channel *,
                       int, int *, char **, int *, int, int, char, void *,
                       const char *);
 static void send_cap_mode_changes(struct Client *, struct Client *,
-                                  struct Channel *, int, int);
+                                  struct Channel *, unsigned int, unsigned int);
 static void send_mode_changes(struct Client *, struct Client *,
                               struct Channel *, char *);
 
@@ -109,7 +110,6 @@ static struct ChCapCombo chcap_combos[NCHCAP_COMBOS];
 extern BlockHeap *ban_heap;
 
 
-/* XXX check_string is propably not longer required in add_id and del_id */
 /* check_string()
  *
  * inputs       - string to check
@@ -125,7 +125,7 @@ check_string(char *s)
   char *str = s;
   static char star[] = "*";
 
-  if (EmptyString(s))
+  if (EmptyString(str))
     return star;
 
   for (; *s; ++s)
@@ -137,7 +137,7 @@ check_string(char *s)
     }
   }
 
-  return str;
+  return EmptyString(str) ? star : str;
 }
 
 /*
@@ -155,7 +155,7 @@ add_id(struct Client *client_p, struct Channel *chptr, char *banid, int type)
   size_t len = 0;
   struct Ban *ban_p = NULL;
   unsigned int num_mask;
-  char name[NICKLEN];
+  char name[NICKLEN + 1];
   char user[USERLEN + 1];
   char host[HOSTLEN + 1];
   struct split_nuh_item nuh;
@@ -262,7 +262,7 @@ del_id(struct Channel *chptr, char *banid, int type)
   dlink_list *list;
   dlink_node *ban;
   struct Ban *banptr;
-  char name[NICKLEN];
+  char name[NICKLEN + 1];
   char user[USERLEN + 1];
   char host[HOSTLEN + 1];
   struct split_nuh_item nuh;
@@ -322,17 +322,17 @@ del_id(struct Channel *chptr, char *banid, int type)
   return 0;
 }
 
-static const struct mode_letter
-{
-  const unsigned int mode;
-  const unsigned char letter;
-} flags[] = {
+const struct mode_letter chan_modes[] = {
   { MODE_INVITEONLY, 'i' },
   { MODE_MODERATED,  'm' },
   { MODE_NOPRIVMSGS, 'n' },
   { MODE_PRIVATE,    'p' },
+  { MODE_REGISTERED, 'r' },
   { MODE_SECRET,     's' },
   { MODE_TOPICLIMIT, 't' },
+  { MODE_OPERONLY,   'O' },
+  { MODE_REGONLY,    'R' },
+  { MODE_SSLONLY,    'S' },
   { 0, '\0' }
 };
 
@@ -350,20 +350,20 @@ void
 channel_modes(struct Channel *chptr, struct Client *client_p,
               char *mbuf, char *pbuf)
 {
-  int i;
+  const struct mode_letter *tab = chan_modes;
 
   *mbuf++ = '+';
   *pbuf = '\0';
 
-  for (i = 0; flags[i].mode; ++i)
-    if (chptr->mode.mode & flags[i].mode)
-      *mbuf++ = flags[i].letter;
+  for (; tab->mode; ++tab)
+    if (chptr->mode.mode & tab->mode)
+      *mbuf++ = tab->letter;
 
   if (chptr->mode.limit)
   {
     *mbuf++ = 'l';
 
-    if (IsMember(client_p, chptr) || IsServer(client_p))
+    if (IsServer(client_p) || HasFlag(client_p, FLAGS_SERVICE) || IsMember(client_p, chptr))
       pbuf += ircsprintf(pbuf, "%d ", chptr->mode.limit);
   }
 
@@ -371,7 +371,7 @@ channel_modes(struct Channel *chptr, struct Client *client_p,
   {
     *mbuf++ = 'k';
 
-    if (*pbuf || IsMember(client_p, chptr) || IsServer(client_p))
+    if (IsServer(client_p) || HasFlag(client_p, FLAGS_SERVICE) || IsMember(client_p, chptr))
       ircsprintf(pbuf, "%s ", chptr->mode.key);
   }
 
@@ -438,6 +438,8 @@ fix_key_old(char *arg)
 #define SM_ERR_RPL_E        0x00000010
 #define SM_ERR_NOTONCHANNEL 0x00000020 /* Not on channel    */
 #define SM_ERR_RPL_I        0x00000040
+#define SM_ERR_NOTOPER      0x00000080
+#define SM_ERR_ONLYSERVER   0x00000100
 
 /* Now lets do some stuff to keep track of what combinations of
  * servers exist...
@@ -612,6 +614,138 @@ chm_simple(struct Client *client_p, struct Client *source_p, struct Channel *chp
   }
 }
 
+static void
+chm_registered(struct Client *client_p, struct Client *source_p, struct Channel *chptr,
+           int parc, int *parn, char **parv, int *errors, int alev, int dir,
+           char c, void *d, const char *chname)
+{
+  long mode_type;
+
+  mode_type = (long)d;
+
+
+  if (!IsServer(source_p) && !HasFlag(source_p, FLAGS_SERVICE))
+  {
+    if (!(*errors & SM_ERR_ONLYSERVER))
+      sendto_one(source_p, form_str(alev == CHACCESS_NOTONCHAN ?
+                                    ERR_NOTONCHANNEL : ERR_ONLYSERVERSCANCHANGE),
+                 me.name, source_p->name, chname);
+    *errors |= SM_ERR_ONLYSERVER;
+    return;
+  }
+
+  /* If have already dealt with this simple mode, ignore it */
+  if (simple_modes_mask & mode_type)
+    return;
+
+  simple_modes_mask |= mode_type;
+
+  /* setting + */
+  /* Apparently, (though no one has ever told the hybrid group directly) 
+   * admins don't like redundant mode checking. ok. It would have been nice 
+   * if you had have told us directly. I've left the original code snippets 
+   * in place. 
+   * 
+   * -Dianora 
+   */
+  if ((dir == MODE_ADD)) /* && !(chptr->mode.mode & mode_type)) */
+  {
+    chptr->mode.mode |= mode_type;
+
+    mode_changes[mode_count].letter = c;
+    mode_changes[mode_count].dir = MODE_ADD;
+    mode_changes[mode_count].caps = 0;
+    mode_changes[mode_count].nocaps = 0;
+    mode_changes[mode_count].id = NULL;
+    mode_changes[mode_count].mems = ALL_MEMBERS;
+    mode_changes[mode_count++].arg = NULL;
+  }
+  else if ((dir == MODE_DEL)) /* && (chptr->mode.mode & mode_type)) */
+  {
+    /* setting - */
+
+    chptr->mode.mode &= ~mode_type;
+
+    mode_changes[mode_count].letter = c;
+    mode_changes[mode_count].dir = MODE_DEL;
+    mode_changes[mode_count].caps = 0;
+    mode_changes[mode_count].nocaps = 0;
+    mode_changes[mode_count].mems = ALL_MEMBERS;
+    mode_changes[mode_count].id = NULL;
+    mode_changes[mode_count++].arg = NULL;
+  }
+}
+
+static void
+chm_operonly(struct Client *client_p, struct Client *source_p, struct Channel *chptr,
+            int parc, int *parn, char **parv, int *errors, int alev, int dir,
+            char c, void *d, const char *chname)
+{
+  long mode_type;
+
+  mode_type = (long)d;
+
+  if ((alev < CHACCESS_HALFOP) ||
+      ((mode_type == MODE_PRIVATE) && (alev < CHACCESS_CHANOP)))
+  {
+    if (!(*errors & SM_ERR_NOOPS))
+      sendto_one(source_p, form_str(alev == CHACCESS_NOTONCHAN ?
+                                    ERR_NOTONCHANNEL : ERR_CHANOPRIVSNEEDED),
+                 me.name, source_p->name, chname);
+    *errors |= SM_ERR_NOOPS;
+    return;
+  }
+  else if (MyClient(source_p) && !HasUMode(source_p, UMODE_OPER))
+  {
+    if (!(*errors & SM_ERR_NOTOPER))
+    {
+      if (alev == CHACCESS_NOTONCHAN)
+        sendto_one(source_p, form_str(ERR_NOTONCHANNEL),
+                   me.name, source_p->name, chname);
+      else
+        sendto_one(source_p, form_str(ERR_NOPRIVILEGES),
+                   me.name, source_p->name);
+    }
+
+    *errors |= SM_ERR_NOTOPER;
+    return;
+  }
+
+  /* If have already dealt with this simple mode, ignore it */
+  if (simple_modes_mask & mode_type)
+    return;
+
+  simple_modes_mask |= mode_type;
+
+  if ((dir == MODE_ADD)) /* && !(chptr->mode.mode & mode_type)) */
+  {
+    chptr->mode.mode |= mode_type;
+
+    mode_changes[mode_count].letter = c;
+    mode_changes[mode_count].dir = MODE_ADD;
+    mode_changes[mode_count].caps = 0;
+    mode_changes[mode_count].nocaps = 0;
+    mode_changes[mode_count].id = NULL;
+    mode_changes[mode_count].mems = ALL_MEMBERS;
+    mode_changes[mode_count].mems = ALL_MEMBERS;
+    mode_changes[mode_count++].arg = NULL;
+  }
+  else if ((dir == MODE_DEL)) /* && (chptr->mode.mode & mode_type)) */
+  {
+    /* setting - */
+
+    chptr->mode.mode &= ~mode_type;
+
+    mode_changes[mode_count].letter = c;
+    mode_changes[mode_count].dir = MODE_DEL;
+    mode_changes[mode_count].caps = 0;
+    mode_changes[mode_count].nocaps = 0;
+    mode_changes[mode_count].mems = ALL_MEMBERS;
+    mode_changes[mode_count].id = NULL;
+    mode_changes[mode_count++].arg = NULL;
+  }
+}
+
 static void
 chm_ban(struct Client *client_p, struct Client *source_p,
         struct Channel *chptr, int parc, int *parn,
@@ -671,16 +805,8 @@ chm_ban(struct Client *client_p, struct Client *source_p,
         return;
       break;
     case MODE_DEL:
-/* XXX grrrrrrr */
-#ifdef NO_BAN_COOKIE
       if (!del_id(chptr, mask, CHFL_BAN))
         return;
-#else
-     /* XXX this hack allows /mode * +o-b nick ban.cookie
-      * I'd like to see this hack go away in the future.
-      */
-      del_id(chptr, mask, CHFL_BAN);
-#endif
       break;
     default:
       assert(0);
@@ -703,20 +829,6 @@ chm_except(struct Client *client_p, struct Client *source_p,
 {
   char *mask = NULL;
 
-  /* if we have +e disabled, allow local clients to do anything but
-   * set the mode.  This prevents the abuse of +e when just a few
-   * servers support it. --fl
-   */
-  if (!ConfigChannel.use_except && MyClient(source_p) && 
-      ((dir == MODE_ADD) && (parc > *parn)))
-  {
-    if (*errors & SM_ERR_RPL_E)
-      return;
-
-    *errors |= SM_ERR_RPL_E;
-    return;
-  }
-
   if (alev < CHACCESS_HALFOP)
   {
     if (!(*errors & SM_ERR_NOOPS))
@@ -779,12 +891,7 @@ chm_except(struct Client *client_p, struct Client *source_p,
   mode_changes[mode_count].dir = dir;
   mode_changes[mode_count].caps = CAP_EX;
   mode_changes[mode_count].nocaps = 0;
-
-  if (ConfigChannel.use_except)
-    mode_changes[mode_count].mems = ONLY_CHANOPS;
-  else
-    mode_changes[mode_count].mems = ONLY_SERVERS;
-
+  mode_changes[mode_count].mems = ONLY_CHANOPS;
   mode_changes[mode_count].id = NULL;
   mode_changes[mode_count++].arg = mask;
 }
@@ -797,20 +904,6 @@ chm_invex(struct Client *client_p, struct Client *source_p,
 {
   char *mask = NULL;
 
-  /* if we have +I disabled, allow local clients to do anything but
-   * set the mode.  This prevents the abuse of +I when just a few
-   * servers support it --fl
-   */
-  if (!ConfigChannel.use_invex && MyClient(source_p) && 
-      (dir == MODE_ADD) && (parc > *parn))
-  {
-    if (*errors & SM_ERR_RPL_I)
-      return;
-    
-    *errors |= SM_ERR_RPL_I;
-    return;
-  }
-
   if (alev < CHACCESS_HALFOP)
   {
     if (!(*errors & SM_ERR_NOOPS))
@@ -873,12 +966,7 @@ chm_invex(struct Client *client_p, struct Client *source_p,
   mode_changes[mode_count].dir = dir;
   mode_changes[mode_count].caps = CAP_IE;
   mode_changes[mode_count].nocaps = 0;
-
-  if (ConfigChannel.use_invex)
-    mode_changes[mode_count].mems = ONLY_CHANOPS;
-  else
-    mode_changes[mode_count].mems = ONLY_SERVERS;
-
+  mode_changes[mode_count].mems = ONLY_CHANOPS;
   mode_changes[mode_count].id = NULL;
   mode_changes[mode_count++].arg = mask;
 }
@@ -902,6 +990,18 @@ clear_ban_cache(struct Channel *chptr)
   }
 }
 
+void
+clear_ban_cache_client(struct Client *client_p)
+{
+  dlink_node *ptr = NULL;
+
+  DLINK_FOREACH(ptr, client_p->channel.head)
+  {
+    struct Membership *ms = ptr->data;
+    ms->flags &= ~(CHFL_BAN_SILENCED|CHFL_BAN_CHECKED);
+  }
+}
+
 static void
 chm_op(struct Client *client_p, struct Client *source_p,
        struct Channel *chptr, int parc, int *parn,
@@ -952,8 +1052,11 @@ chm_op(struct Client *client_p, struct Client *source_p,
   {
 #ifdef HALFOPS
     if (has_member_flags(member, CHFL_HALFOP))
+    {
+      --*parn;
       chm_hop(client_p, source_p, chptr, parc, parn, parv, errors, alev,
               dir, c, d, chname);
+    }
 #endif
     return;
   }
@@ -1291,7 +1394,6 @@ struct ChannelMode
   void *d;
 };
 
-/* *INDENT-OFF* */
 static struct ChannelMode ModeTable[255] =
 {
   {chm_nosuch, NULL},
@@ -1309,11 +1411,11 @@ static struct ChannelMode ModeTable[255] =
   {chm_nosuch, NULL},                             /* L */
   {chm_nosuch, NULL},                             /* M */
   {chm_nosuch, NULL},                             /* N */
-  {chm_nosuch, NULL},                             /* O */
+  {chm_operonly, (void *) MODE_OPERONLY},         /* O */
   {chm_nosuch, NULL},                             /* P */
   {chm_nosuch, NULL},                             /* Q */
-  {chm_nosuch, NULL},                             /* R */
-  {chm_nosuch, NULL},                             /* S */
+  {chm_simple, (void *) MODE_REGONLY},            /* R */
+  {chm_simple, (void *) MODE_SSLONLY},            /* S */
   {chm_nosuch, NULL},                             /* T */
   {chm_nosuch, NULL},                             /* U */
   {chm_nosuch, NULL},                             /* V */
@@ -1348,7 +1450,7 @@ static struct ChannelMode ModeTable[255] =
   {chm_op, NULL},                                 /* o */
   {chm_simple, (void *) MODE_PRIVATE},            /* p */
   {chm_nosuch, NULL},                             /* q */
-  {chm_nosuch, NULL},                             /* r */
+  {chm_registered, (void *) MODE_REGISTERED},     /* r */
   {chm_simple, (void *) MODE_SECRET},             /* s */
   {chm_simple, (void *) MODE_TOPICLIMIT},         /* t */
   {chm_nosuch, NULL},                             /* u */
@@ -1358,7 +1460,6 @@ static struct ChannelMode ModeTable[255] =
   {chm_nosuch, NULL},                             /* y */
   {chm_nosuch, NULL},                             /* z */
 };
-/* *INDENT-ON* */
 
 /* get_channel_access()
  *
@@ -1415,7 +1516,7 @@ get_channel_access(struct Client *source_p, struct Membership *member)
 
 static void
 send_cap_mode_changes(struct Client *client_p, struct Client *source_p,
-                      struct Channel *chptr, int cap, int nocap)
+                      struct Channel *chptr, unsigned int cap, unsigned int nocap)
 {
   int i, mbl, pbl, arglen, nc, mc;
   int len;
@@ -1473,8 +1574,8 @@ send_cap_mode_changes(struct Client *client_p, struct Client *source_p,
         (pbl + arglen + BAN_FUDGE) >= MODEBUFLEN)
     {
       if (nc != 0)
-        sendto_server(client_p, source_p, chptr, cap, nocap,
-                      LL_ICHAN | LL_ICLIENT, "%s %s",
+        sendto_server(client_p, cap, nocap,
+                      "%s %s",
 		      modebuf, parabuf);
       nc = 0;
       mc = 0;
@@ -1515,8 +1616,8 @@ send_cap_mode_changes(struct Client *client_p, struct Client *source_p,
     parabuf[pbl - 1] = 0;
 
   if (nc != 0)
-    sendto_server(client_p, source_p, chptr, cap, nocap,
-                  LL_ICLIENT, "%s %s", modebuf, parabuf);
+    sendto_server(client_p, cap, nocap,
+                  "%s %s", modebuf, parabuf);
 }
 
 /* void send_mode_changes(struct Client *client_p,
@@ -1580,7 +1681,7 @@ send_mode_changes(struct Client *client_p, struct Client *source_p,
         modebuf[mbl - 1] = '\0';
 
       if (nc != 0)
-        sendto_channel_local(ALL_MEMBERS, NO, chptr, "%s %s", modebuf, parabuf);
+        sendto_channel_local(ALL_MEMBERS, 0, chptr, "%s %s", modebuf, parabuf);
 
       nc = 0;
       mc = 0;
@@ -1620,7 +1721,7 @@ send_mode_changes(struct Client *client_p, struct Client *source_p,
     parabuf[pbl - 1] = 0;
 
   if (nc != 0)
-    sendto_channel_local(ALL_MEMBERS, NO, chptr, "%s %s", modebuf, parabuf);
+    sendto_channel_local(ALL_MEMBERS, 0, chptr, "%s %s", modebuf, parabuf);
 
   nc = 0;
   mc = 0;
