@@ -19,28 +19,25 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: client.c 706 2006-07-02 11:25:55Z michael $
+ *  $Id: client.c 1784 2013-01-26 22:40:14Z michael $
  */
 
 #include "stdinc.h"
-#include "tools.h"
+#include "list.h"
 #include "client.h"
 #include "channel_mode.h"
-#include "common.h"
 #include "event.h"
 #include "fdlist.h"
 #include "hash.h"
 #include "irc_string.h"
-#include "sprintf_irc.h"
 #include "ircd.h"
-#include "list.h"
 #include "s_gline.h"
 #include "numeric.h"
 #include "packet.h"
 #include "s_auth.h"
 #include "s_bsd.h"
-#include "s_conf.h"
-#include "s_log.h"
+#include "conf.h"
+#include "log.h"
 #include "s_misc.h"
 #include "s_serv.h"
 #include "send.h"
@@ -53,6 +50,8 @@
 #include "listener.h"
 #include "irc_res.h"
 #include "userhost.h"
+#include "watch.h"
+#include "rng_mt.h"
 
 dlink_list listing_client_list = { NULL, NULL, 0 };
 /* Pointer to beginning of Client list */
@@ -76,7 +75,7 @@ static dlink_node *eac_next;  /* next aborted client to exit */
 
 static void check_pings_list(dlink_list *);
 static void check_unknowns_list(void);
-static void ban_them(struct Client *client_p, struct ConfItem *conf);
+static void ban_them(struct Client *, struct ConfItem *);
 
 
 /* init_client()
@@ -118,20 +117,24 @@ make_client(struct Client *from)
 
   if (from == NULL)
   {
-    client_p->from  = client_p; /* 'from' of local client is self! */
-    client_p->since = client_p->lasttime = client_p->firsttime = CurrentTime;
-
-    client_p->localClient = BlockHeapAlloc(lclient_heap);
+    client_p->from                      = client_p; /* 'from' of local client is self! */
+    client_p->localClient               = BlockHeapAlloc(lclient_heap);
+    client_p->localClient->since        = CurrentTime;
+    client_p->localClient->lasttime     = CurrentTime;
+    client_p->localClient->firsttime    = CurrentTime;
     client_p->localClient->registration = REG_INIT;
+
     /* as good a place as any... */
-    dlinkAdd(client_p, make_dlink_node(), &unknown_list);
+    dlinkAdd(client_p, &client_p->localClient->lclient_node, &unknown_list);
   }
   else
     client_p->from = from; /* 'from' of local client is self! */
 
+  client_p->idhnext = client_p;
   client_p->hnext  = client_p;
   client_p->status = STAT_UNKNOWN;
   strcpy(client_p->username, "unknown");
+  strcpy(client_p->svid, "0");
 
   return client_p;
 }
@@ -149,16 +152,19 @@ free_client(struct Client *client_p)
   assert(client_p != NULL);
   assert(client_p != &me);
   assert(client_p->hnext == client_p);
+  assert(client_p->idhnext == client_p);
   assert(client_p->channel.head == NULL);
   assert(dlink_list_length(&client_p->channel) == 0);
+  assert(dlink_list_length(&client_p->whowas) == 0);
+  assert(!IsServer(client_p) || (IsServer(client_p) && client_p->serv));
 
-  MyFree(client_p->away);
   MyFree(client_p->serv);
 
   if (MyConnect(client_p))
   {
     assert(client_p->localClient->invited.head == NULL);
     assert(dlink_list_length(&client_p->localClient->invited) == 0);
+    assert(dlink_list_length(&client_p->localClient->watches) == 0); 
     assert(IsClosing(client_p) && IsDead(client_p));
 
     MyFree(client_p->localClient->response);
@@ -246,45 +252,12 @@ check_pings_list(dlink_list *list)
       continue; 
     }
 
-    if (client_p->localClient->reject_delay > 0)
-    {
-      if (client_p->localClient->reject_delay <= CurrentTime)
-	exit_client(client_p, &me, "Rejected");
-      continue;
-    }
-
-    if (GlobalSetOptions.idletime && IsClient(client_p))
-    {
-      if (!IsExemptKline(client_p) && !IsOper(client_p) &&
-          !IsIdlelined(client_p) &&
-	  ((CurrentTime - client_p->localClient->last) > GlobalSetOptions.idletime))
-      {
-        struct ConfItem *conf;
-        struct AccessItem *aconf;
-
-        conf = make_conf_item(KLINE_TYPE);
-        aconf = (struct AccessItem *)map_to_conf(conf);
-
-        DupString(aconf->host, client_p->host);
-        DupString(aconf->reason, "idle exceeder");
-        DupString(aconf->user, client_p->username);
-        aconf->hold = CurrentTime + 60;
-        add_temp_line(conf);
-
-        sendto_realops_flags(UMODE_ALL, L_ALL,
-                             "Idle time limit exceeded for %s - temp k-lining",
-                             get_client_name(client_p, HIDE_IP));
-        exit_client(client_p, &me, aconf->reason);
-        continue;
-      }
-    }
-
     if (!IsRegistered(client_p))
       ping = CONNECTTIMEOUT, pingwarn = 0;
     else
       ping = get_client_ping(client_p, &pingwarn);
 
-    if (ping < CurrentTime - client_p->lasttime)
+    if (ping < CurrentTime - client_p->localClient->lasttime)
     {
       if (!IsPingSent(client_p))
       {
@@ -295,12 +268,12 @@ check_pings_list(dlink_list *list)
 	 */
 	SetPingSent(client_p);
 	ClearPingWarning(client_p);
-	client_p->lasttime = CurrentTime - ping;
+	client_p->localClient->lasttime = CurrentTime - ping;
 	sendto_one(client_p, "PING :%s", ID_or_name(&me, client_p));
       }
       else
       {
-        if (CurrentTime - client_p->lasttime >= 2 * ping)
+        if (CurrentTime - client_p->localClient->lasttime >= 2 * ping)
         {
           /*
            * If the client/server hasn't talked to us in 2*ping seconds
@@ -314,17 +287,17 @@ check_pings_list(dlink_list *list)
 	    sendto_realops_flags(UMODE_ALL, L_OPER,
 				 "No response from %s, closing link",
 				 get_client_name(client_p, MASK_IP));
-	    ilog(L_NOTICE, "No response from %s, closing link",
+	    ilog(LOG_TYPE_IRCD, "No response from %s, closing link",
 		 get_client_name(client_p, HIDE_IP));
 	  }
 
-          ircsprintf(scratch, "Ping timeout: %d seconds",
-                     (int)(CurrentTime - client_p->lasttime));
+          snprintf(scratch, sizeof(scratch), "Ping timeout: %d seconds",
+                   (int)(CurrentTime - client_p->localClient->lasttime));
           exit_client(client_p, &me, scratch);
         }
         else if (!IsPingWarning(client_p) && pingwarn > 0 &&
 	         (IsServer(client_p) || IsHandshake(client_p)) &&
-	         CurrentTime - client_p->lasttime >= ping + pingwarn)
+	         CurrentTime - client_p->localClient->lasttime >= ping + pingwarn)
         {
           /*
            * If the server hasn't replied in pingwarn seconds after sending
@@ -337,7 +310,7 @@ check_pings_list(dlink_list *list)
 	  sendto_realops_flags(UMODE_ALL, L_OPER,
 	                       "Warning, no response from %s in %d seconds",
 	                       get_client_name(client_p, MASK_IP), pingwarn);
-          ilog(L_NOTICE, "No response from %s in %d seconds",
+          ilog(LOG_TYPE_IRCD, "No response from %s in %d seconds",
 	       get_client_name(client_p, HIDE_IP), pingwarn);
         }
       }
@@ -360,18 +333,11 @@ check_unknowns_list(void)
   {
     struct Client *client_p = ptr->data;
 
-    if (client_p->localClient->reject_delay > 0)
-    {
-      if (client_p->localClient->reject_delay <= CurrentTime)
-        exit_client(client_p, &me, "Rejected");
-      continue;
-    }
-
     /*
      * Check UNKNOWN connections - if they have been in this state
      * for > 30s, close them.
      */
-    if (IsAuthFinished(client_p) && (CurrentTime - client_p->firsttime) > 30)
+    if (IsAuthFinished(client_p) && (CurrentTime - client_p->localClient->firsttime) > 30)
       exit_client(client_p, &me, "Registration timed out");
   }
 }
@@ -485,7 +451,6 @@ static void
 ban_them(struct Client *client_p, struct ConfItem *conf)
 {
   const char *user_reason = NULL;	/* What is sent to user */
-  const char *channel_reason = NULL;	/* What is sent to channel */
   struct AccessItem *aconf = NULL;
   struct MatchItem *xconf = NULL;
   const char *type_string = NULL;
@@ -520,20 +485,10 @@ ban_them(struct Client *client_p, struct ConfItem *conf)
       break;
   }
 
-  if (ConfigFileEntry.kline_with_reason)
-  {
-    if (aconf != NULL)
-      user_reason = aconf->reason ? aconf->reason : type_string;
-    if (xconf != NULL)
-      user_reason = xconf->reason ? xconf->reason : type_string;
-  }
-  else
-    user_reason = type_string;
-
-  if (ConfigFileEntry.kline_reason != NULL)
-    channel_reason = ConfigFileEntry.kline_reason;
-  else
-    channel_reason = user_reason;
+  if (aconf != NULL)
+    user_reason = aconf->reason ? aconf->reason : type_string;
+  if (xconf != NULL)
+    user_reason = xconf->reason ? xconf->reason : type_string;
 
   sendto_realops_flags(UMODE_ALL, L_ALL, "%s active for %s",
                        type_string, get_client_name(client_p, HIDE_IP));
@@ -542,7 +497,7 @@ ban_them(struct Client *client_p, struct ConfItem *conf)
     sendto_one(client_p, form_str(ERR_YOUREBANNEDCREEP),
 	       me.name, client_p->name, user_reason);
 
-  exit_client(client_p, &me, channel_reason);
+  exit_client(client_p, &me, user_reason);
 }
 
 /* update_client_exit_stats()
@@ -554,20 +509,18 @@ ban_them(struct Client *client_p, struct ConfItem *conf)
 static void
 update_client_exit_stats(struct Client *client_p)
 {
-  if (IsServer(client_p))
-  {
-    sendto_realops_flags(UMODE_EXTERNAL, L_ALL, 
-			 "Server %s split from %s",
-			 client_p->name, client_p->servptr->name);
-  }
-  else if (IsClient(client_p))
+  if (IsClient(client_p))
   {
+    assert(Count.total > 0);
     --Count.total;
-    if (IsOper(client_p))
+    if (HasUMode(client_p, UMODE_OPER))
       --Count.oper;
-    if (IsInvisible(client_p))
+    if (HasUMode(client_p, UMODE_INVISIBLE))
       --Count.invisi;
   }
+  else if (IsServer(client_p))
+    sendto_realops_flags(UMODE_EXTERNAL, L_ALL, "Server %s split from %s",
+                         client_p->name, client_p->servptr->name);
 
   if (splitchecking && !splitmode)
     check_splitmode(NULL);
@@ -579,23 +532,23 @@ update_client_exit_stats(struct Client *client_p)
  * output	- return client pointer
  * side effects - find person by (nick)name
  */
-/* XXX - ugly wrapper */
 struct Client *
 find_person(const struct Client *client_p, const char *name)
 {
-  struct Client *c2ptr;
+  struct Client *c2ptr = NULL;
 
   if (IsDigit(*name))
   {
     if ((c2ptr = hash_find_id(name)) != NULL)
     {
       /* invisible users shall not be found by UID guessing */
-      if (IsInvisible(c2ptr) && !IsServer(client_p))
-        c2ptr = NULL;
+      if (HasUMode(c2ptr, UMODE_INVISIBLE))
+        if (!IsServer(client_p) && !HasFlag(client_p, FLAGS_SERVICE))
+          c2ptr = NULL;
     }
   }
   else
-    c2ptr = find_client(name);
+    c2ptr = hash_find_client(name);
 
   return ((c2ptr != NULL && IsClient(c2ptr)) ? c2ptr : NULL);
 }
@@ -615,10 +568,10 @@ find_chasing(struct Client *client_p, struct Client *source_p, const char *user,
     *chasing = 0;
 
   if (who)
-    return(who);
+    return who;
 
   if (IsDigit(*user))
-    return(NULL);
+    return NULL;
 
   if ((who = get_history(user,
 			(time_t)ConfigFileEntry.kill_chase_time_limit))
@@ -626,13 +579,13 @@ find_chasing(struct Client *client_p, struct Client *source_p, const char *user,
   {
     sendto_one(source_p, form_str(ERR_NOSUCHNICK),
                me.name, source_p->name, user);
-    return(NULL);
+    return NULL;
   }
 
   if (chasing)
     *chasing = 1;
 
-  return(who);
+  return who;
 }
 
 /*
@@ -654,65 +607,60 @@ find_chasing(struct Client *client_p, struct Client *source_p, const char *user,
  *        to modify what it points!!!
  */
 const char *
-get_client_name(struct Client *client, int showip)
+get_client_name(const struct Client *client, enum addr_mask_type type)
 {
   static char nbuf[HOSTLEN * 2 + USERLEN + 5];
 
   assert(client != NULL);
 
-  if (irccmp(client->name, client->host) == 0)
-    return(client->name);
+  if (!MyConnect(client)) 
+    return client->name;
 
-  if (ConfigServerHide.hide_server_ips)
-    if (IsServer(client) || IsConnecting(client) || IsHandshake(client))
-      showip = MASK_IP;
-
-  if (ConfigFileEntry.hide_spoof_ips)
-    if (showip == SHOW_IP && IsIPSpoof(client))
-      showip = MASK_IP;
-
-  /* And finally, let's get the host information, ip or name */
-  switch (showip)
+  if (IsServer(client) || IsConnecting(client) || IsHandshake(client))
   {
-    case SHOW_IP:
-      if (MyConnect(client))
-      {
-        ircsprintf(nbuf, "%s[%s@%s]", client->name, client->username,
-                   client->sockhost);
-        break;
-      }
-    case MASK_IP:
-      ircsprintf(nbuf, "%s[%s@255.255.255.255]", client->name,
-                 client->username);
-      break;
-    default:
-      ircsprintf(nbuf, "%s[%s@%s]", client->name, client->username,
-                 client->host);
+    if (!irccmp(client->name, client->host))
+      return client->name;
+    else if (ConfigServerHide.hide_server_ips)
+      type = MASK_IP;
   }
 
-  return(nbuf);
+  if (ConfigFileEntry.hide_spoof_ips)
+    if (type == SHOW_IP && IsIPSpoof(client))
+      type = MASK_IP;
+
+  /* And finally, let's get the host information, ip or name */
+  switch (type)
+  {
+    case SHOW_IP:
+      snprintf(nbuf, sizeof(nbuf), "%s[%s@%s]",
+               client->name,
+               client->username, client->sockhost);
+      break;
+    case MASK_IP:
+      if (client->localClient->aftype == AF_INET)
+        snprintf(nbuf, sizeof(nbuf), "%s[%s@255.255.255.255]",
+                 client->name, client->username);
+      else
+        snprintf(nbuf, sizeof(nbuf), "%s[%s@ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff]",
+                 client->name, client->username);
+      break;
+    default:
+      snprintf(nbuf, sizeof(nbuf), "%s[%s@%s]",
+               client->name,
+               client->username, client->host);
+  }
+
+  return nbuf;
 }
 
 void
 free_exited_clients(void)
 {
-  dlink_node *ptr, *next;
-  struct Client *target_p;
+  dlink_node *ptr = NULL, *next = NULL;
   
   DLINK_FOREACH_SAFE(ptr, next, dead_list.head)
   {
-    target_p = ptr->data;
-
-    if (ptr->data == NULL)
-    {
-      sendto_realops_flags(UMODE_ALL, L_ALL,
-                           "Warning: null client on dead_list!");
-      dlinkDelete(ptr, &dead_list);
-      free_dlink_node(ptr);
-      continue;
-    }
-
-    free_client(target_p);
+    free_client(ptr->data);
     dlinkDelete(ptr, &dead_list);
     free_dlink_node(ptr);
   }
@@ -731,52 +679,44 @@ exit_one_client(struct Client *source_p, const char *quitmsg)
 
   assert(!IsMe(source_p));
 
-  if (IsServer(source_p))
-  {
-    dlinkDelete(&source_p->lnode, &source_p->servptr->serv->servers);
-
-    if ((lp = dlinkFindDelete(&global_serv_list, source_p)) != NULL)
-      free_dlink_node(lp);
-
-    if (!MyConnect(source_p))
-    {
-      source_p->from->serv->dep_servers--;
-      assert(source_p->from->serv->dep_servers > 0);
-    }
-  }
-  else if (IsClient(source_p))
+  if (IsClient(source_p))
   {
     if (source_p->servptr->serv != NULL)
-      dlinkDelete(&source_p->lnode, &source_p->servptr->serv->users);
+      dlinkDelete(&source_p->lnode, &source_p->servptr->serv->client_list);
 
-    /* If a person is on a channel, send a QUIT notice
-    ** to every client (person) on the same channel (so
-    ** that the client can show the "**signoff" message).
-    ** (Note: The notice is to the local clients *only*)
-    */
+    /*
+     * If a person is on a channel, send a QUIT notice
+     * to every client (person) on the same channel (so
+     * that the client can show the "**signoff" message).
+     * (Note: The notice is to the local clients *only*)
+     */
     sendto_common_channels_local(source_p, 0, ":%s!%s@%s QUIT :%s",
                                  source_p->name, source_p->username,
                                  source_p->host, quitmsg);
     DLINK_FOREACH_SAFE(lp, next_lp, source_p->channel.head)
       remove_user_from_channel(lp->data);
 
-    /* Clean up allow lists */
-    del_all_accepts(source_p);
     add_history(source_p, 0);
     off_history(source_p);
 
-    if (!MyConnect(source_p))
-    {
-      source_p->from->serv->dep_users--;
-      assert(source_p->from->serv->dep_users >= 0);
-    }
-    else
+    watch_check_hash(source_p, RPL_LOGOFF);
+
+    if (MyConnect(source_p))
     {
       /* Clean up invitefield */
       DLINK_FOREACH_SAFE(lp, next_lp, source_p->localClient->invited.head)
         del_invite(lp->data, source_p);
+
+      del_all_accepts(source_p);
     }
   }
+  else if (IsServer(source_p))
+  {
+    dlinkDelete(&source_p->lnode, &source_p->servptr->serv->server_list);
+
+    if ((lp = dlinkFindDelete(&global_serv_list, source_p)) != NULL)
+      free_dlink_node(lp);
+  }
 
   /* Remove source_p from the client lists */
   if (HasID(source_p))
@@ -816,33 +756,29 @@ exit_one_client(struct Client *source_p, const char *quitmsg)
 static void
 recurse_send_quits(struct Client *original_source_p, struct Client *source_p,
                    struct Client *from, struct Client *to, const char *comment,
-                   const char *splitstr, const char *myname)
+                   const char *splitstr)
 {
   dlink_node *ptr, *next;
   struct Client *target_p;
-  int hidden = match(myname, source_p->name);
 
   assert(to != source_p);  /* should be already removed from serv_list */
 
   /* If this server can handle quit storm (QS) removal
    * of dependents, just send the SQUIT
-   *
-   * Always check *all* dependent servers if some of them are
-   * hidden behind fakename. If so, send out the QUITs -adx
    */
-  if (hidden || !IsCapable(to, CAP_QS))
-    DLINK_FOREACH_SAFE(ptr, next, source_p->serv->users.head)
+  if (!IsCapable(to, CAP_QS))
+    DLINK_FOREACH_SAFE(ptr, next, source_p->serv->client_list.head)
     {
       target_p = ptr->data;
       sendto_one(to, ":%s QUIT :%s", target_p->name, splitstr);
     }
 
-  DLINK_FOREACH_SAFE(ptr, next, source_p->serv->servers.head)
+  DLINK_FOREACH_SAFE(ptr, next, source_p->serv->server_list.head)
     recurse_send_quits(original_source_p, ptr->data, from, to,
-                       comment, splitstr, myname);
+                       comment, splitstr);
 
-  if (!hidden && ((source_p == original_source_p && to != from) ||
-                  !IsCapable(to, CAP_QS)))
+  if ((source_p == original_source_p && to != from) ||
+                  !IsCapable(to, CAP_QS))
   {
     /* don't use a prefix here - we have to be 100% sure the message
      * will be accepted without Unknown prefix etc.. */
@@ -861,17 +797,14 @@ recurse_remove_clients(struct Client *source_p, const char *quitmsg)
 {
   dlink_node *ptr, *next;
 
-  DLINK_FOREACH_SAFE(ptr, next, source_p->serv->users.head)
+  DLINK_FOREACH_SAFE(ptr, next, source_p->serv->client_list.head)
     exit_one_client(ptr->data, quitmsg);
 
-  DLINK_FOREACH_SAFE(ptr, next, source_p->serv->servers.head)
+  DLINK_FOREACH_SAFE(ptr, next, source_p->serv->server_list.head)
   {
     recurse_remove_clients(ptr->data, quitmsg);
     exit_one_client(ptr->data, quitmsg);
   }
-
-  assert(source_p->serv->dep_servers == 1);
-  assert(source_p->serv->dep_users == 0);
 }
 
 /*
@@ -883,22 +816,11 @@ static void
 remove_dependents(struct Client *source_p, struct Client *from,
                   const char *comment, const char *splitstr)
 {
-  struct Client *to;
-  struct ConfItem *conf;
-  static char myname[HOSTLEN+1];
-  dlink_node *ptr;
+  dlink_node *ptr = NULL;
 
   DLINK_FOREACH(ptr, serv_list.head)
-  {
-    to = ptr->data;
-
-    if ((conf = to->serv->sconf) != NULL)
-      strlcpy(myname, my_name_for_link(conf), sizeof(myname));
-    else
-      strlcpy(myname, me.name, sizeof(myname));
-    recurse_send_quits(source_p, source_p, from, to,
-                       comment, splitstr, myname);
-  }
+    recurse_send_quits(source_p, source_p, from, ptr->data,
+                       comment, splitstr);
 
   recurse_remove_clients(source_p, splitstr);
 }
@@ -908,8 +830,8 @@ remove_dependents(struct Client *source_p, struct Client *from,
  * this on any struct Client, regardless of its state.
  *
  * Note, you shouldn't exit remote _users_ without first doing
- * SetKilled and propagating a kill or similar message. However,
- * it is perfectly correct to call exit_client to force a _server_
+ * AddFlag(x, FLAGS_KILLED) and propagating a kill or similar message.
+ * However, it is perfectly correct to call exit_client to force a _server_
  * quit (either local or remote one).
  *
  * inputs:       - a client pointer that is going to be exited
@@ -924,7 +846,7 @@ remove_dependents(struct Client *source_p, struct Client *from,
 void
 exit_client(struct Client *source_p, struct Client *from, const char *comment)
 {
-  dlink_node *m;
+  dlink_node *m = NULL;
 
   if (MyConnect(source_p))
   {
@@ -939,9 +861,14 @@ exit_client(struct Client *source_p, struct Client *from, const char *comment)
     if (IsIpHash(source_p))
       remove_one_ip(&source_p->localClient->ip);
 
-    delete_auth(source_p);
+    if (source_p->localClient->auth)
+    {
+      delete_auth(source_p->localClient->auth);
+      source_p->localClient->auth = NULL;
+    }
 
-    /* This source_p could have status of one of STAT_UNKNOWN, STAT_CONNECTING
+    /*
+     * This source_p could have status of one of STAT_UNKNOWN, STAT_CONNECTING
      * STAT_HANDSHAKE or STAT_UNKNOWN
      * all of which are lumped together into unknown_list
      *
@@ -949,54 +876,55 @@ exit_client(struct Client *source_p, struct Client *from, const char *comment)
      */
     if (!IsRegistered(source_p))
     {
-      if ((m = dlinkFindDelete(&unknown_list, source_p)) != NULL)
-        free_dlink_node(m);
+      assert(dlinkFind(&unknown_list, source_p));
+
+      dlinkDelete(&source_p->localClient->lclient_node, &unknown_list);
     }
     else if (IsClient(source_p))
     {
+      time_t on_for = CurrentTime - source_p->localClient->firsttime;
+      assert(Count.local > 0);
       Count.local--;
 
-      if (IsOper(source_p))
-      {
+      if (HasUMode(source_p, UMODE_OPER))
         if ((m = dlinkFindDelete(&oper_list, source_p)) != NULL)
           free_dlink_node(m);
-      }
 
+      assert(dlinkFind(&local_client_list, source_p));
       dlinkDelete(&source_p->localClient->lclient_node, &local_client_list);
+
       if (source_p->localClient->list_task != NULL)
         free_list_task(source_p->localClient->list_task, source_p);
 
+      watch_del_watch_list(source_p);
       sendto_realops_flags(UMODE_CCONN, L_ALL, "Client exiting: %s (%s@%s) [%s] [%s]",
                            source_p->name, source_p->username, source_p->host, comment,
                            ConfigFileEntry.hide_spoof_ips && IsIPSpoof(source_p) ?
                            "255.255.255.255" : source_p->sockhost);
+      sendto_realops_flags(UMODE_CCONN_FULL, L_ALL, "CLIEXIT: %s %s %s %s 0 %s",
+                           source_p->name,
+			   source_p->username,
+			   source_p->host,
+                           ConfigFileEntry.hide_spoof_ips && IsIPSpoof(source_p) ?
+                           "255.255.255.255" : source_p->sockhost,
+			   comment);
+      ilog(LOG_TYPE_USER, "%s (%3u:%02u:%02u): %s!%s@%s %llu/%llu",
+           myctime(source_p->localClient->firsttime), (unsigned int)(on_for / 3600),
+           (unsigned int)((on_for % 3600)/60), (unsigned int)(on_for % 60),
+           source_p->name, source_p->username, source_p->host,
+           source_p->localClient->send.bytes>>10,
+           source_p->localClient->recv.bytes>>10);
     }
-
-    /* As soon as a client is known to be a server of some sort
-     * it has to be put on the serv_list, or SJOIN's to this new server
-     * from the connect burst will not be seen.
-     */
-    if (IsServer(source_p) || IsConnecting(source_p) ||
-        IsHandshake(source_p))
+    else if (IsServer(source_p))
     {
-      if ((m = dlinkFindDelete(&serv_list, source_p)) != NULL)
-      {
-        free_dlink_node(m);
-        unset_chcap_usage_counts(source_p);
-      }
+      assert(Count.myserver > 0);
+      --Count.myserver;
 
-      if (IsServer(source_p))
-      {
-        Count.myserver--;
-        if (ServerInfo.hub)
-          remove_lazylink_flags(source_p->localClient->serverMask);
-        else
-          uplink = NULL;
-      }
+      assert(dlinkFind(&serv_list, source_p));
+      dlinkDelete(&source_p->localClient->lclient_node, &serv_list);
+      unset_chcap_usage_counts(source_p);
     }
 
-    log_user_exit(source_p);
-
     if (!IsDead(source_p))
     {
       if (IsServer(source_p))
@@ -1032,7 +960,8 @@ exit_client(struct Client *source_p, struct Client *from, const char *comment)
     assert(source_p->serv != NULL && source_p->servptr != NULL);
 
     if (ConfigServerHide.hide_servers)
-      /* set netsplit message to "*.net *.split" to still show 
+      /*
+       * Set netsplit message to "*.net *.split" to still show 
        * that its a split, but hide the servers splitting
        */
       strcpy(splitstr, "*.net *.split");
@@ -1046,20 +975,20 @@ exit_client(struct Client *source_p, struct Client *from, const char *comment)
     {
       sendto_realops_flags(UMODE_ALL, L_ALL,
                            "%s was connected for %d seconds.  %llu/%llu sendK/recvK.",
-                           source_p->name, (int)(CurrentTime - source_p->firsttime),
+                           source_p->name, (int)(CurrentTime - source_p->localClient->firsttime),
                            source_p->localClient->send.bytes >> 10,
                            source_p->localClient->recv.bytes >> 10);
-      ilog(L_NOTICE, "%s was connected for %d seconds.  %llu/%llu sendK/recvK.",
-           source_p->name, (int)(CurrentTime - source_p->firsttime), 
+      ilog(LOG_TYPE_IRCD, "%s was connected for %d seconds.  %llu/%llu sendK/recvK.",
+           source_p->name, (int)(CurrentTime - source_p->localClient->firsttime), 
            source_p->localClient->send.bytes >> 10,
            source_p->localClient->recv.bytes >> 10);
     }
   }
-  else if (IsClient(source_p) && !IsKilled(source_p))
+  else if (IsClient(source_p) && !HasFlag(source_p, FLAGS_KILLED))
   {
-    sendto_server(from->from, source_p, NULL, CAP_TS6, NOCAPS, NOFLAGS,
+    sendto_server(from->from, CAP_TS6, NOCAPS,
                   ":%s QUIT :%s", ID(source_p), comment);
-    sendto_server(from->from, source_p, NULL, NOCAPS, CAP_TS6, NOFLAGS,
+    sendto_server(from->from, NOCAPS, CAP_TS6,
                   ":%s QUIT :%s", source_p->name, comment);
   }
 
@@ -1118,7 +1047,7 @@ dead_link_on_read(struct Client *client_p, int error)
 
   if (IsServer(client_p) || IsHandshake(client_p))
   {
-    int connected = CurrentTime - client_p->firsttime;
+    int connected = CurrentTime - client_p->localClient->firsttime;
       
     if (error == 0)
     {
@@ -1132,7 +1061,7 @@ dead_link_on_read(struct Client *client_p, int error)
 			   "Server %s closed the connection",
 			   get_client_name(client_p, MASK_IP));
 
-      ilog(L_NOTICE, "Server %s closed the connection",
+      ilog(LOG_TYPE_IRCD, "Server %s closed the connection",
 	   get_client_name(client_p, SHOW_IP));
     }
     else
@@ -1155,8 +1084,8 @@ dead_link_on_read(struct Client *client_p, int error)
     strlcpy(errmsg, "Remote host closed the connection",
             sizeof(errmsg));
   else
-    ircsprintf(errmsg, "Read error: %s",
-               strerror(current_error));
+    snprintf(errmsg, sizeof(errmsg), "Read error: %s",
+             strerror(current_error));
 
   exit_client(client_p, &me, errmsg);
 }
@@ -1196,256 +1125,118 @@ exit_aborted_clients(void)
 
 /*
  * accept processing, this adds a form of "caller ID" to ircd
- * 
+ *
  * If a client puts themselves into "caller ID only" mode,
- * only clients that match a client pointer they have put on 
+ * only clients that match a client pointer they have put on
  * the accept list will be allowed to message them.
  *
- * [ source.on_allow_list ] -> [ target1 ] -> [ target2 ]
- *
- * [target.allow_list] -> [ source1 ] -> [source2 ]
- *
- * i.e. a target will have a link list of source pointers it will allow
- * each source client then has a back pointer pointing back
- * to the client that has it on its accept list.
- * This allows for exit_one_client to remove these now bogus entries
- * from any client having an accept on them. 
+ * Diane Bruce, "Dianora" db@db.net
  */
 
+void
+del_accept(struct split_nuh_item *accept_p, struct Client *client_p)
+{
+  dlinkDelete(&accept_p->node, &client_p->localClient->acceptlist);
+
+  MyFree(accept_p->nickptr);
+  MyFree(accept_p->userptr);
+  MyFree(accept_p->hostptr);
+  MyFree(accept_p);
+}
+
+struct split_nuh_item *
+find_accept(const char *nick, const char *user,
+            const char *host, struct Client *client_p, int do_match)
+{
+  dlink_node *ptr = NULL;
+  /* XXX We wouldn't need that if match() would return 0 on match */
+  int (*cmpfunc)(const char *, const char *) = do_match ? match : irccmp;
+
+  DLINK_FOREACH(ptr, client_p->localClient->acceptlist.head)
+  {
+    struct split_nuh_item *accept_p = ptr->data;
+
+    if (cmpfunc(accept_p->nickptr, nick) == do_match &&
+        cmpfunc(accept_p->userptr, user) == do_match &&
+        cmpfunc(accept_p->hostptr, host) == do_match)
+      return accept_p;
+  }
+
+  return NULL;
+}
+
 /* accept_message()
  *
- * inputs	- pointer to source client
- * 		- pointer to target client
- * output	- 1 if accept this message 0 if not
+ * inputs       - pointer to source client
+ *              - pointer to target client
+ * output       - 1 if accept this message 0 if not
  * side effects - See if source is on target's allow list
  */
 int
-accept_message(struct Client *source, struct Client *target)
+accept_message(struct Client *source,
+               struct Client *target)
 {
-  dlink_node *ptr;
+  dlink_node *ptr = NULL;
 
-  DLINK_FOREACH(ptr, target->allow_list.head)
-  {
-    struct Client *target_p = ptr->data;
+  if (source == target || find_accept(source->name, source->username,
+                                      source->host, target, 1))
+    return 1;
 
-    if (source == target_p)
-      return (1);
-  }
-
-  if (IsSoftCallerId(target))
-  {
+  if (HasUMode(target, UMODE_SOFTCALLERID))
     DLINK_FOREACH(ptr, target->channel.head)
       if (IsMember(source, ((struct Membership *)ptr->data)->chptr))
-        return (1);
-  }
+        return 1;
 
-  return (0);
-}
-
-/* del_from_accept()
- *
- * inputs	- pointer to source client
- * 		- pointer to target client
- * output	- NONE
- * side effects - Delete's source pointer to targets allow list
- *
- * Walk through the target's accept list, remove if source is found,
- * Then walk through the source's on_accept_list remove target if found.
- */
-void
-del_from_accept(struct Client *source, struct Client *target)
-{
-  dlink_node *ptr;
-  dlink_node *ptr2;
-  dlink_node *next_ptr;
-  dlink_node *next_ptr2;
-  struct Client *target_p;
-
-  DLINK_FOREACH_SAFE(ptr, next_ptr, target->allow_list.head)
-  {
-    target_p = ptr->data;
-
-    if (source == target_p)
-    {
-      dlinkDelete(ptr, &target->allow_list);
-      free_dlink_node(ptr);
-
-      DLINK_FOREACH_SAFE(ptr2, next_ptr2, source->on_allow_list.head)
-      {
-        target_p = ptr2->data;
-
-        if (target == target_p)
-        {
-          dlinkDelete(ptr2, &source->on_allow_list);
-          free_dlink_node(ptr2);
-        }
-      }
-    }
-  }
+  return 0;
 }
 
 /* del_all_accepts()
  *
- * inputs	- pointer to exiting client
- * output	- NONE
- * side effects - Walk through given clients allow_list and on_allow_list
- *                remove all references to this client
+ * inputs       - pointer to exiting client
+ * output       - NONE
+ * side effects - Walk through given clients acceptlist and remove all entries
  */
 void
 del_all_accepts(struct Client *client_p)
 {
-  dlink_node *ptr, *next_ptr;
+  dlink_node *ptr = NULL, *next_ptr = NULL;
 
-  DLINK_FOREACH_SAFE(ptr, next_ptr, client_p->allow_list.head)
-    del_from_accept(ptr->data, client_p);
-
-  DLINK_FOREACH_SAFE(ptr, next_ptr, client_p->on_allow_list.head)
-    del_from_accept(client_p, ptr->data);
+  DLINK_FOREACH_SAFE(ptr, next_ptr, client_p->localClient->acceptlist.head)
+    del_accept(ptr->data, client_p);
 }
 
-/* del_all_their_accepts()
- *
- * inputs	- pointer to exiting client
- * output	- NONE
- * side effects - Walk through given clients on_allow_list
- *                remove all references to this client,
- *		  allow this client to keep their own allow_list
- */
-void
-del_all_their_accepts(struct Client *client_p)
+unsigned int
+idle_time_get(struct Client *source_p, struct Client *target_p)
 {
-  dlink_node *ptr, *next_ptr;
+  unsigned int idle = 0;
+  unsigned int min_idle = 0;
+  unsigned int max_idle = 0;
+  const struct ClassItem *class = get_client_class_ptr(target_p);
 
-  DLINK_FOREACH_SAFE(ptr, next_ptr, client_p->on_allow_list.head)
-    del_from_accept(client_p, ptr->data);
-}
+  if (!(class->flags & CONF_FLAGS_FAKE_IDLE) || target_p == source_p)
+    return CurrentTime - target_p->localClient->last_privmsg;
+  if (HasUMode(source_p, UMODE_OPER) &&
+      !(class->flags & CONF_FLAGS_HIDE_IDLE_FROM_OPERS))
+    return CurrentTime - target_p->localClient->last_privmsg;
 
-/* set_initial_nick()
- *
- * inputs
- * output
- * side effects	-
- *
- * This function is only called to set up an initially registering
- * client. 
- */
-void
-set_initial_nick(struct Client *client_p, struct Client *source_p,
-                 const char *nick)
-{
- char buf[USERLEN + 1];
+  min_idle = class->min_idle;
+  max_idle = class->max_idle;
 
-  /* Client setting NICK the first time */
-  
-  /* This had to be copied here to avoid problems.. */
-  source_p->tsinfo = CurrentTime;
-  source_p->localClient->registration &= ~REG_NEED_NICK;
+  if (min_idle == max_idle)
+    return min_idle;
 
-  if (source_p->name[0])
-    hash_del_client(source_p);
-
-  strlcpy(source_p->name, nick, sizeof(source_p->name));
-  hash_add_client(source_p);
-
-  /* fd_desc is long enough */
-  fd_note(&client_p->localClient->fd, "Nick: %s", nick);
-  
-  /* They have the nick they want now.. */
-  client_p->llname[0] = '\0';
-
-  if (!source_p->localClient->registration)
-  {
-    strlcpy(buf, source_p->username, sizeof(buf));
-
-    /*
-     * USER already received, now we have NICK.
-     * *NOTE* For servers "NICK" *must* precede the
-     * user message (giving USER before NICK is possible
-     * only for local client connection!). register_user
-     * may reject the client and call exit_client for it
-     * --must test this and exit m_nick too!!!
-     */
-    register_local_user(client_p, source_p, nick, buf);
-  }
-}
-
-/* change_local_nick()
- *
- * inputs	- pointer to server
- *		- pointer to client
- *              - nick
- * output	- 
- * side effects	- changes nick of a LOCAL user
- */
-void
-change_local_nick(struct Client *client_p, struct Client *source_p, const char *nick)
-{
-  /*
-  ** Client just changing his/her nick. If he/she is
-  ** on a channel, send note of change to all clients
-  ** on that channel. Propagate notice to other servers.
-  */
-  if ((source_p->localClient->last_nick_change +
-       ConfigFileEntry.max_nick_time) < CurrentTime)
-    source_p->localClient->number_of_nick_changes = 0;
-  source_p->localClient->last_nick_change = CurrentTime;
-  source_p->localClient->number_of_nick_changes++;
-
-  if ((ConfigFileEntry.anti_nick_flood && 
-      (source_p->localClient->number_of_nick_changes
-       <= ConfigFileEntry.max_nick_changes)) ||
-     !ConfigFileEntry.anti_nick_flood || 
-     (IsOper(source_p) && ConfigFileEntry.no_oper_flood))
-  {
-    if (irccmp(source_p->name, nick))
-    {
-      /*
-       * Make sure everyone that has this client on its accept list
-       * loses that reference.
-       */
-      del_all_their_accepts(source_p);
-      source_p->tsinfo = CurrentTime;
-    }
-
-    /* XXX - the format of this notice should eventually be changed
-     * to either %s[%s@%s], or even better would be get_client_name() -bill
-     */
-    sendto_realops_flags(UMODE_NCHANGE, L_ALL, "Nick change: From %s to %s [%s@%s]",
-                         source_p->name, nick, source_p->username, source_p->host);
-    sendto_common_channels_local(source_p, 1, ":%s!%s@%s NICK :%s",
-                                 source_p->name, source_p->username,
-                                 source_p->host, nick);
-    add_history(source_p, 1);
-
-    /*
-     * Only hubs care about lazy link nicks not being sent on yet
-     * lazylink leafs/leafs always send their nicks up to hub,
-     * hence must always propagate nick changes.
-     * hubs might not propagate a nick change, if the leaf
-     * does not know about that client yet.
-     */
-    sendto_server(client_p, source_p, NULL, CAP_TS6, NOCAPS, NOFLAGS,
-                  ":%s NICK %s :%lu",
-                  ID(source_p), nick, (unsigned long)source_p->tsinfo);
-    sendto_server(client_p, source_p, NULL, NOCAPS, CAP_TS6, NOFLAGS,
-                  ":%s NICK %s :%lu",
-                  source_p->name, nick, (unsigned long)source_p->tsinfo);
-  }
+  if (class->flags & CONF_FLAGS_RANDOM_IDLE)
+    idle = genrand_int32();
   else
-  {
-    sendto_one(source_p, form_str(ERR_NICKTOOFAST),
-               me.name, source_p->name, source_p->name,
-               nick, ConfigFileEntry.max_nick_time);
-    return;
-  }
+    idle = CurrentTime - target_p->localClient->last_privmsg;
 
-  /* Finally, add to hash */
-  if (source_p->name[0])
-    hash_del_client(source_p);
+  if (max_idle == 0)
+    idle = 0;
+  else
+    idle %= max_idle;
 
-  strcpy(source_p->name, nick);
-  hash_add_client(source_p);
+  if (idle < min_idle)
+    idle = min_idle + (idle % (max_idle - min_idle));
 
-  /* fd_desc is long enough */
-  fd_note(&client_p->localClient->fd, "Nick: %s", nick);
+  return idle;
 }
