@@ -19,15 +19,13 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: dbuf.c 33 2005-10-02 20:50:00Z knight $
+ *  $Id: dbuf.c 1243 2011-09-30 10:47:53Z michael $
  */
 
 #include "stdinc.h"
-#include "balloc.h"
-#include "common.h"
-#include "dbuf.h"
 #include "list.h"
-#include "tools.h"
+#include "balloc.h"
+#include "dbuf.h"
 #include "memory.h"
 
 static BlockHeap *dbuf_heap;
