@@ -38,7 +38,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: event.c 33 2005-10-02 20:50:00Z knight $
+ *  $Id: event.c 1672 2012-11-21 20:10:32Z michael $
  */
 
 /*
@@ -50,13 +50,13 @@
  */
 
 #include "stdinc.h"
-
+#include "list.h"
 #include "ircd.h"
 #include "event.h"
 #include "client.h"
 #include "send.h"
 #include "memory.h"
-#include "s_log.h"
+#include "log.h"
 #include "numeric.h"
 
 static const char *last_event_ran = NULL;
@@ -96,7 +96,7 @@ eventAdd(const char *name, EVH *func, void *arg, time_t when)
     }
   }
   /* XXX if reach here, its an error */
-  ilog(L_ERROR, "Event table is full! (%d)", i);
+  ilog(LOG_TYPE_IRCD, "Event table is full! (%d)", i);
 }
 
 /*
@@ -281,6 +281,8 @@ set_back_events(time_t by)
 {
   int i;
 
+  event_time_min = -1;
+
   for (i = 0; i < MAX_EVENTS; i++)
   {
     if (event_table[i].when > by)
