@@ -19,21 +19,20 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: hash.c 33 2005-10-02 20:50:00Z knight $
+ *  $Id: hash.c 1397 2012-05-06 10:01:52Z michael $
  */
 
 #include "stdinc.h"
-#include "tools.h"
-#include "s_conf.h"
+#include "list.h"
+#include "balloc.h"
+#include "conf.h"
 #include "channel.h"
 #include "channel_mode.h"
 #include "client.h"
-#include "common.h"
-#include "handlers.h"
-#include "list.h"
 #include "modules.h"
 #include "hash.h"
 #include "resv.h"
+#include "rng_mt.h"
 #include "userhost.h"
 #include "irc_string.h"
 #include "ircd.h"
@@ -46,9 +45,8 @@
 
 static BlockHeap *userhost_heap = NULL;
 static BlockHeap *namehost_heap = NULL;
-static struct UserHost *find_or_add_userhost(const char *);
 
-static unsigned int ircd_random_key = 0;
+static unsigned int hashf_xor_key = 0;
 
 /* The actual hash tables, They MUST be of the same HASHSIZE, variable
  * size tables could be supported but the rehash routine should also
@@ -72,8 +70,6 @@ static struct ResvChannel *resvchannelTable[HASHSIZE];
 void
 init_hash(void)
 {
-  unsigned int i;
-
   /* Default the userhost/namehost sizes to CLIENT_HEAP_SIZE for now,
    * should be a good close approximation anyway
    * - Dianora
@@ -81,17 +77,7 @@ init_hash(void)
   userhost_heap = BlockHeapCreate("userhost", sizeof(struct UserHost), CLIENT_HEAP_SIZE);
   namehost_heap = BlockHeapCreate("namehost", sizeof(struct NameHost), CLIENT_HEAP_SIZE);
 
-  ircd_random_key = rand() % 256;  /* better than nothing --adx */
-
-  /* Clear the hash tables first */
-  for (i = 0; i < HASHSIZE; ++i)
-  {
-    idTable[i]          = NULL;
-    clientTable[i]      = NULL;
-    channelTable[i]     = NULL;
-    userhostTable[i]    = NULL;
-    resvchannelTable[i] = NULL;
-  }
+  hashf_xor_key = genrand_int32() % 256;  /* better than nothing --adx */
 }
 
 /*
@@ -113,10 +99,10 @@ strhash(const char *name)
   {
     hval += (hval << 1) + (hval <<  4) + (hval << 7) +
             (hval << 8) + (hval << 24);
-    hval ^= (ToLower(*p) ^ ircd_random_key);
+    hval ^= (ToLower(*p) ^ hashf_xor_key);
   }
 
-  return (hval >> FNV1_32_BITS) ^ (hval & ((1 << FNV1_32_BITS) -1));
+  return (hval >> FNV1_32_BITS) ^ (hval & ((1 << FNV1_32_BITS) - 1));
 }
 
 /************************** Externally visible functions ********************/
@@ -215,10 +201,8 @@ hash_del_id(struct Client *client_p)
     else
     {
       while (tmp->idhnext != client_p)
-      {
         if ((tmp = tmp->idhnext) == NULL)
           return;
-      }
 
       tmp->idhnext = tmp->idhnext->idhnext;
       client_p->idhnext = client_p;
@@ -248,10 +232,8 @@ hash_del_client(struct Client *client_p)
     else
     {
       while (tmp->hnext != client_p)
-      {
         if ((tmp = tmp->hnext) == NULL)
           return;
-      }
 
       tmp->hnext = tmp->hnext->hnext;
       client_p->hnext = client_p;
@@ -281,10 +263,8 @@ hash_del_userhost(struct UserHost *userhost)
     else
     {
       while (tmp->next != userhost)
-      {
         if ((tmp = tmp->next) == NULL)
           return;
-      }
 
       tmp->next = tmp->next->next;
       userhost->next = userhost;
@@ -315,10 +295,8 @@ hash_del_channel(struct Channel *chptr)
     else
     {
       while (tmp->hnextch != chptr)
-      {
         if ((tmp = tmp->hnextch) == NULL)
           return;
-      }
 
       tmp->hnextch = tmp->hnextch->hnextch;
       chptr->hnextch = chptr;
@@ -342,10 +320,8 @@ hash_del_resv(struct ResvChannel *chptr)
     else
     {
       while (tmp->hnext != chptr)
-      {
         if ((tmp = tmp->hnext) == NULL)
           return;
-      }
 
       tmp->hnext = tmp->hnext->hnext;
       chptr->hnext = chptr;
@@ -353,7 +329,7 @@ hash_del_resv(struct ResvChannel *chptr)
   }
 }
 
-/* find_client()
+/* hash_find_client()
  *
  * inputs       - pointer to name
  * output       - NONE
@@ -362,7 +338,7 @@ hash_del_resv(struct ResvChannel *chptr)
  *                it to the top of the list and returns it.
  */
 struct Client *
-find_client(const char *name)
+hash_find_client(const char *name)
 {
   unsigned int hashv = strhash(name);
   struct Client *client_p;
@@ -397,13 +373,13 @@ hash_find_id(const char *name)
 
   if ((client_p = idTable[hashv]) != NULL)
   {
-    if (irccmp(name, client_p->id))
+    if (strcmp(name, client_p->id))
     {
       struct Client *prev;
 
       while (prev = client_p, (client_p = client_p->idhnext) != NULL)
       {
-        if (!irccmp(name, client_p->id))
+        if (!strcmp(name, client_p->id))
         {
           prev->idhnext = client_p->idhnext;
           client_p->idhnext = idTable[hashv];
@@ -417,57 +393,16 @@ hash_find_id(const char *name)
   return client_p;
 }
 
-/*
- * Whats happening in this next loop ? Well, it takes a name like
- * foo.bar.edu and proceeds to earch for *.edu and then *.bar.edu.
- * This is for checking full server names against masks although
- * it isnt often done this way in lieu of using matches().
- *
- * Rewrote to do *.bar.edu first, which is the most likely case,
- * also made const correct
- * --Bleep
- */
-static struct Client *
-hash_find_masked_server(const char *name)
-{
-  char buf[HOSTLEN + 1];
-  char *p = buf;
-  char *s = NULL;
-  struct Client *server = NULL;
-
-  if (*name == '*' || *name == '.')
-    return NULL;
-
-  /*
-   * copy the damn thing and be done with it
-   */
-  strlcpy(buf, name, sizeof(buf));
-
-  while ((s = strchr(p, '.')) != NULL)
-  {
-    *--s = '*';
-
-    /* Dont need to check IsServer() here since nicknames cant
-     * have *'s in them anyway.
-     */
-    if ((server = find_client(s)) != NULL)
-      return server;
-    p = s + 2;
-  }
-
-  return NULL;
-}
-
 struct Client *
-find_server(const char *name)
+hash_find_server(const char *name)
 {
   unsigned int hashv = strhash(name);
   struct Client *client_p = NULL;
 
   if (IsDigit(*name) && strlen(name) == IRC_MAXSID)
-    client_p = hash_find_id(name);
+    return hash_find_id(name);
 
-  if ((client_p == NULL) && (client_p = clientTable[hashv]) != NULL)
+  if ((client_p = clientTable[hashv]) != NULL)
   {
     if ((!IsServer(client_p) && !IsMe(client_p)) ||
         irccmp(name, client_p->name))
@@ -488,7 +423,7 @@ find_server(const char *name)
     }
   }
 
-  return (client_p != NULL) ? client_p : hash_find_masked_server(name);
+  return client_p;
 }
 
 /* hash_find_channel()
@@ -671,6 +606,27 @@ count_user_host(const char *user, const char *host, int *global_p,
   }
 }
 
+/* find_or_add_userhost()
+ *
+ * inputs       - host name
+ * output       - none
+ * side effects - find UserHost * for given host name
+ */
+static struct UserHost *
+find_or_add_userhost(const char *host)
+{
+  struct UserHost *userhost;
+
+  if ((userhost = hash_find_userhost(host)) != NULL)
+    return userhost;
+
+  userhost = BlockHeapAlloc(userhost_heap);
+  strlcpy(userhost->host, host, sizeof(userhost->host));
+  hash_add_userhost(userhost);
+
+  return userhost;
+}
+
 /* add_user_host()
  *
  * inputs	- user name
@@ -703,12 +659,14 @@ add_user_host(const char *user, const char *host, int global)
     if (!irccmp(user, nameh->name))
     {
       nameh->gcount++;
+
       if (!global)
       {
-	if (hasident)
-	  nameh->icount++;
-	nameh->lcount++;
+        if (hasident)
+          nameh->icount++;
+        nameh->lcount++;
       }
+
       return;
     }
   }
@@ -717,6 +675,7 @@ add_user_host(const char *user, const char *host, int global)
   strlcpy(nameh->name, user, sizeof(nameh->name));
 
   nameh->gcount = 1;
+
   if (!global)
   {
     if (hasident)
@@ -762,22 +721,22 @@ delete_user_host(const char *user, const char *host, int global)
         nameh->gcount--;
       if (!global)
       {
-	if (nameh->lcount > 0)
-	  nameh->lcount--;
-	if (hasident && nameh->icount > 0)
-	  nameh->icount--;
+        if (nameh->lcount > 0)
+          nameh->lcount--;
+        if (hasident && nameh->icount > 0)
+          nameh->icount--;
       }
 
       if (nameh->gcount == 0 && nameh->lcount == 0)
       {
-	dlinkDelete(&nameh->node, &found_userhost->list);
-	BlockHeapFree(namehost_heap, nameh);
+        dlinkDelete(&nameh->node, &found_userhost->list);
+        BlockHeapFree(namehost_heap, nameh);
       }
 
       if (dlink_list_length(&found_userhost->list) == 0)
       {
-	hash_del_userhost(found_userhost);
-	BlockHeapFree(userhost_heap, found_userhost);
+        hash_del_userhost(found_userhost);
+        BlockHeapFree(userhost_heap, found_userhost);
       }
 
       return;
@@ -785,27 +744,6 @@ delete_user_host(const char *user, const char *host, int global)
   }
 }
 
-/* find_or_add_userhost()
- *
- * inputs	- host name
- * output	- none
- * side effects	- find UserHost * for given host name
- */
-static struct UserHost *
-find_or_add_userhost(const char *host)
-{
-  struct UserHost *userhost;
-
-  if ((userhost = hash_find_userhost(host)) != NULL)
-    return userhost;
-
-  userhost = BlockHeapAlloc(userhost_heap);
-  strlcpy(userhost->host, host, sizeof(userhost->host));
-  hash_add_userhost(userhost);
-
-  return userhost;
-}
-
 /*
  * Safe list code.
  *
@@ -897,13 +835,12 @@ list_allow_channel(const char *chname, struct ListTask *lt)
  */
 static void
 list_one_channel(struct Client *source_p, struct Channel *chptr,
-                 struct ListTask *list_task, int remote_request)
+                 struct ListTask *list_task)
 {
-  if ((remote_request && chptr->chname[0] == '&') ||
-      (SecretChannel(chptr) && !IsMember(source_p, chptr)))
+  if (SecretChannel(chptr) && !IsMember(source_p, chptr))
     return;
-  if ((unsigned int)dlink_list_length(&chptr->members) < list_task->users_min ||
-      (unsigned int)dlink_list_length(&chptr->members) > list_task->users_max ||
+  if (dlink_list_length(&chptr->members) < list_task->users_min ||
+      dlink_list_length(&chptr->members) > list_task->users_max ||
       (chptr->channelts != 0 &&
        ((unsigned int)chptr->channelts < list_task->created_min ||
         (unsigned int)chptr->channelts > list_task->created_max)) ||
@@ -916,7 +853,7 @@ list_one_channel(struct Client *source_p, struct Channel *chptr,
     return;
   sendto_one(source_p, form_str(RPL_LIST), me.name, source_p->name,
              chptr->chname, dlink_list_length(&chptr->members),
-             chptr->topic == NULL ? "" : chptr->topic);
+             chptr->topic);
 }
 
 /* safe_list_channels()
@@ -935,24 +872,24 @@ list_one_channel(struct Client *source_p, struct Channel *chptr,
  */
 void
 safe_list_channels(struct Client *source_p, struct ListTask *list_task,
-                   int only_unmasked_channels, int remote_request)
+                   int only_unmasked_channels)
 {
   struct Channel *chptr = NULL;
 
   if (!only_unmasked_channels)
   {
-    int i;
+    unsigned int i;
 
-    for (i = list_task->hash_index; i < HASHSIZE; i++)
+    for (i = list_task->hash_index; i < HASHSIZE; ++i)
     {
       if (exceeding_sendq(source_p->from))
       {
         list_task->hash_index = i;
-        return; /* still more to do */
+        return;    /* still more to do */
       }
 
       for (chptr = channelTable[i]; chptr; chptr = chptr->hnextch)
-        list_one_channel(source_p, chptr, list_task, remote_request);
+        list_one_channel(source_p, chptr, list_task);
     }
   }
   else
@@ -961,7 +898,7 @@ safe_list_channels(struct Client *source_p, struct ListTask *list_task,
 
     DLINK_FOREACH(dl, list_task->show_mask.head)
       if ((chptr = hash_find_channel(dl->data)) != NULL)
-        list_one_channel(source_p, chptr, list_task, remote_request);
+        list_one_channel(source_p, chptr, list_task);
   }
 
   free_list_task(list_task, source_p);
