@@ -2,7 +2,7 @@
  *  ircd-hybrid: an advanced Internet Relay Chat Daemon(ircd).
  *  hostmask.c: Code to efficiently find IP & hostmask based configs.
  *
- *  Copyright (C) 2002 by the past and present ircd coders, and others.
+ *  Copyright (C) 2005 by the past and present ircd coders, and others.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,24 +19,19 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: hostmask.c,v 7.73.2.3 2003/05/04 23:25:21 lusky Exp $
+ *  $Id: hostmask.c 1871 2013-04-26 08:53:04Z michael $
  */
 
 #include "stdinc.h"
 #include "memory.h"
 #include "ircd_defs.h"
-#include "s_conf.h"
+#include "list.h"
+#include "conf.h"
 #include "hostmask.h"
-#include "numeric.h"
 #include "send.h"
 #include "irc_string.h"
+#include "ircd.h"
 
-#ifdef IPV6
-static int try_parse_v6_netmask(const char *, struct irc_inaddr *, int *);
-static unsigned long hash_ipv6(struct irc_inaddr *, int);
-#endif
-static int try_parse_v4_netmask(const char *, struct irc_inaddr *, int *);
-static unsigned long hash_ipv4(struct irc_inaddr *, int);
 
 #define DigitParse(ch) do { \
                        if (ch >= '0' && ch <= '9') \
@@ -45,11 +40,11 @@ static unsigned long hash_ipv4(struct irc_inaddr *, int);
                          ch = ch - 'A' + 10; \
                        else if (ch >= 'a' && ch <= 'f') \
                          ch = ch - 'a' + 10; \
-                       } while(0);
+                       } while (0);
 
 /* The mask parser/type determination code... */
 
-/* int try_parse_v6_netmask(const char *, struct irc_inaddr *, int *);
+/* int try_parse_v6_netmask(const char *, struct irc_ssaddr *, int *);
  * Input: A possible IPV6 address as a string.
  * Output: An integer describing whether it is an IPV6 or hostmask,
  *         an address(if it is IPV6), a bitlength(if it is IPV6).
@@ -62,7 +57,7 @@ static unsigned long hash_ipv4(struct irc_inaddr *, int);
 */
 #ifdef IPV6
 static int
-try_parse_v6_netmask(const char *text, struct irc_inaddr *addr, int *b)
+try_parse_v6_netmask(const char *text, struct irc_ssaddr *addr, int *b)
 {
   const char *p;
   char c;
@@ -73,6 +68,7 @@ try_parse_v6_netmask(const char *text, struct irc_inaddr *addr, int *b)
   int bits = 128;
   int deficit = 0;
   short dc[8];
+  struct sockaddr_in6 *v6 = (struct sockaddr_in6 *)addr;
 
   for (p = text; (c = *p); p++)
     if (IsXDigit(c))
@@ -114,6 +110,7 @@ try_parse_v6_netmask(const char *text, struct irc_inaddr *addr, int *b)
       d[dp] = d[dp] >> 4 * nyble;
       dp++;
       bits = strtoul(p + 1, &after, 10);
+
       if (bits < 0 || *after)
         return HM_HOST;
       if (bits > dp * 4 && !(finsert >= 0 && bits <= 128))
@@ -124,12 +121,15 @@ try_parse_v6_netmask(const char *text, struct irc_inaddr *addr, int *b)
       return HM_HOST;
 
   d[dp] = d[dp] >> 4 * nyble;
+
   if (c == 0)
     dp++;
   if (finsert < 0 && bits == 0)
     bits = dp * 16;
+
   /* How many words are missing? -A1kmm */
   deficit = bits / 16 + ((bits % 16) ? 1 : 0) - dp;
+
   /* Now fill in the gaps(from ::) in the copied table... -A1kmm */
   for (dp = 0, nyble = 0; dp < 8; dp++)
   {
@@ -141,23 +141,26 @@ try_parse_v6_netmask(const char *text, struct irc_inaddr *addr, int *b)
     else
       dc[dp] = d[nyble++];
   }
+
   /* Set unused bits to 0... -A1kmm */
   if (bits < 128 && (bits % 16 != 0))
     dc[bits / 16] &= ~((1 << (15 - bits % 16)) - 1);
   for (dp = bits / 16 + (bits % 16 ? 1 : 0); dp < 8; dp++)
     dc[dp] = 0;
+
   /* And assign... -A1kmm */
   if (addr)
     for (dp = 0; dp < 8; dp++)
       /* The cast is a kludge to make netbsd work. */
-      ((unsigned short *)&addr->sins.sin6)[dp] = htons(dc[dp]);
+      ((unsigned short *)&v6->sin6_addr)[dp] = htons(dc[dp]);
+
   if (b != NULL)
     *b = bits;
   return HM_IPV6;
 }
 #endif
 
-/* int try_parse_v4_netmask(const char *, struct irc_inaddr *, int *);
+/* int try_parse_v4_netmask(const char *, struct irc_ssaddr *, int *);
  * Input: A possible IPV4 address as a string.
  * Output: An integer describing whether it is an IPV4 or hostmask,
  *         an address(if it is IPV4), a bitlength(if it is IPV4).
@@ -165,13 +168,14 @@ try_parse_v6_netmask(const char *text, struct irc_inaddr *addr, int *b)
  * Comments: Called from parse_netmask
  */
 static int
-try_parse_v4_netmask(const char *text, struct irc_inaddr *addr, int *b)
+try_parse_v4_netmask(const char *text, struct irc_ssaddr *addr, int *b)
 {
   const char *p;
   const char *digits[4];
   unsigned char addb[4];
   int n = 0, bits = 0;
   char c;
+  struct sockaddr_in *v4 = (struct sockaddr_in *)addr;
 
   digits[n++] = text;
 
@@ -182,12 +186,14 @@ try_parse_v4_netmask(const char *text, struct irc_inaddr *addr, int *b)
     {
       if (n >= 4)
         return HM_HOST;
+
       digits[n++] = p + 1;
     }
     else if (c == '*')
     {
       if (*(p + 1) || n == 0 || *(p - 1) != '.')
         return HM_HOST;
+
       bits = (n - 1) * 8;
       break;
     }
@@ -195,10 +201,12 @@ try_parse_v4_netmask(const char *text, struct irc_inaddr *addr, int *b)
     {
       char *after;
       bits = strtoul(p + 1, &after, 10);
-      if (!bits || *after)
+
+      if (bits < 0 || *after)
         return HM_HOST;
       if (bits > n * 8)
         return HM_HOST;
+
       break;
     }
     else
@@ -209,24 +217,27 @@ try_parse_v4_netmask(const char *text, struct irc_inaddr *addr, int *b)
   if (bits)
     while (n < 4)
       digits[n++] = "0";
+
   for (n = 0; n < 4; n++)
     addb[n] = strtoul(digits[n], NULL, 10);
+
   if (bits == 0)
     bits = 32;
+
   /* Set unused bits to 0... -A1kmm */
   if (bits < 32 && bits % 8)
     addb[bits / 8] &= ~((1 << (8 - bits % 8)) - 1);
   for (n = bits / 8 + (bits % 8 ? 1 : 0); n < 4; n++)
     addb[n] = 0;
   if (addr)
-    addr->sins.sin.s_addr =
+    v4->sin_addr.s_addr =
       htonl(addb[0] << 24 | addb[1] << 16 | addb[2] << 8 | addb[3]);
   if (b != NULL)
     *b = bits;
   return HM_IPV4;
 }
 
-/* int parse_netmask(const char *, struct irc_inaddr *, int *);
+/* int parse_netmask(const char *, struct irc_ssaddr *, int *);
  * Input: A hostmask, or an IPV4/6 address.
  * Output: An integer describing whether it is an IPV4, IPV6 address or a
  *         hostmask, an address(if it is an IP mask),
@@ -234,7 +245,7 @@ try_parse_v4_netmask(const char *text, struct irc_inaddr *addr, int *b)
  * Side effects: None
  */
 int
-parse_netmask(const char *text, struct irc_inaddr *addr, int *b)
+parse_netmask(const char *text, struct irc_ssaddr *addr, int *b)
 {
 #ifdef IPV6
   if (strchr(text, ':'))
@@ -246,45 +257,93 @@ parse_netmask(const char *text, struct irc_inaddr *addr, int *b)
 }
 
 /* The address matching stuff... */
-/* int match_ipv6(struct irc_inaddr *, struct irc_inaddr *, int)
+/* int match_ipv6(struct irc_ssaddr *, struct irc_ssaddr *, int)
  * Input: An IP address, an IP mask, the number of bits in the mask.
  * Output: if match, -1 else 0
  * Side effects: None
  */
 #ifdef IPV6
 int
-match_ipv6(struct irc_inaddr *addr, struct irc_inaddr *mask, int bits)
+match_ipv6(const struct irc_ssaddr *addr, const struct irc_ssaddr *mask, int bits)
 {
   int i, m, n = bits / 8;
+  const struct sockaddr_in6 *v6 = (const struct sockaddr_in6 *)addr;
+  const struct sockaddr_in6 *v6mask = (const struct sockaddr_in6 *)mask;
 
   for (i = 0; i < n; i++)
-    if (addr->sins.sin6.s6_addr[i] != mask->sins.sin6.s6_addr[i])
+    if (v6->sin6_addr.s6_addr[i] != v6mask->sin6_addr.s6_addr[i])
       return 0;
   if ((m = bits % 8) == 0)
     return -1;
-  if ((addr->sins.sin6.s6_addr[n] & ~((1 << (8 - m)) - 1)) ==
-      mask->sins.sin6.s6_addr[n])
+  if ((v6->sin6_addr.s6_addr[n] & ~((1 << (8 - m)) - 1)) ==
+      v6mask->sin6_addr.s6_addr[n])
     return -1;
   return 0;
 }
 #endif
 
-/* int match_ipv4(struct irc_inaddr *, struct irc_inaddr *, int)
+/* int match_ipv4(struct irc_ssaddr *, struct irc_ssaddr *, int)
  * Input: An IP address, an IP mask, the number of bits in the mask.
  * Output: if match, -1 else 0
  * Side Effects: None
  */
 int
-match_ipv4(struct irc_inaddr *addr, struct irc_inaddr *mask, int bits)
+match_ipv4(const struct irc_ssaddr *addr, const struct irc_ssaddr *mask, int bits)
 {
-  if ((ntohl(addr->sins.sin.s_addr) & ~((1 << (32 - bits)) - 1)) !=
-      ntohl(mask->sins.sin.s_addr))
+  const struct sockaddr_in *v4 = (const struct sockaddr_in *)addr;
+  const struct sockaddr_in *v4mask = (const struct sockaddr_in *)mask;
+
+  if ((ntohl(v4->sin_addr.s_addr) & ~((1 << (32 - bits)) - 1)) !=
+      ntohl(v4mask->sin_addr.s_addr))
     return 0;
   return -1;
 }
 
+/*
+ * mask_addr
+ *
+ * inputs       - pointer to the ip to mask
+ *              - bitlen
+ * output       - NONE
+ * side effects -
+ */
+void
+mask_addr(struct irc_ssaddr *ip, int bits)
+{
+  int mask;
+#ifdef IPV6
+  struct sockaddr_in6 *v6_base_ip;
+  int i, m, n;
+#endif
+  struct sockaddr_in *v4_base_ip;
+
+#ifdef IPV6
+  if (ip->ss.ss_family != AF_INET6)
+#endif
+  {
+    v4_base_ip = (struct sockaddr_in *)ip;
+
+    mask = ~((1 << (32 - bits)) - 1);
+    v4_base_ip->sin_addr.s_addr = htonl(ntohl(v4_base_ip->sin_addr.s_addr) & mask);
+  }
+#ifdef IPV6
+  else
+  {
+    n = bits / 8;
+    m = bits % 8;
+    v6_base_ip = (struct sockaddr_in6 *)ip;
+
+    mask = ~((1 << (8 - m)) -1 );
+    v6_base_ip->sin6_addr.s6_addr[n] = v6_base_ip->sin6_addr.s6_addr[n] & mask;
+
+    for (i = n + 1; i < 16; i++)
+      v6_base_ip->sin6_addr.s6_addr[i] = 0;
+  }
+#endif
+}
+
 /* Hashtable stuff...now external as its used in m_stats.c */
-struct AddressRec *atable[ATABLE_SIZE];
+dlink_list atable[ATABLE_SIZE];
 
 void
 init_host_hash(void)
@@ -292,38 +351,47 @@ init_host_hash(void)
   memset(&atable, 0, sizeof(atable));
 }
 
-/* unsigned long hash_ipv4(struct irc_inaddr*)
+/* unsigned long hash_ipv4(struct irc_ssaddr*)
  * Input: An IP address.
  * Output: A hash value of the IP address.
  * Side effects: None
  */
-static unsigned long
-hash_ipv4(struct irc_inaddr *addr, int bits)
+static uint32_t
+hash_ipv4(struct irc_ssaddr *addr, int bits)
 {
-  unsigned long av = ntohl(addr->sins.sin.s_addr) & ~((1 << (32 - bits)) - 1);
-  return (av ^ (av >> 12) ^ (av >> 24)) & (ATABLE_SIZE - 1);
+  if (bits != 0)
+  {
+    struct sockaddr_in *v4 = (struct sockaddr_in *)addr;
+    uint32_t av = ntohl(v4->sin_addr.s_addr) & ~((1 << (32 - bits)) - 1);
+
+    return (av ^ (av >> 12) ^ (av >> 24)) & (ATABLE_SIZE - 1);
+  }
+
+  return 0;
 }
 
-/* unsigned long hash_ipv6(struct irc_inaddr*)
+/* unsigned long hash_ipv6(struct irc_ssaddr*)
  * Input: An IP address.
  * Output: A hash value of the IP address.
  * Side effects: None
  */
 #ifdef IPV6
-static unsigned long
-hash_ipv6(struct irc_inaddr *addr, int bits)
+static uint32_t
+hash_ipv6(struct irc_ssaddr *addr, int bits)
 {
-  unsigned long v = 0, n;
+  uint32_t v = 0, n;
+  struct sockaddr_in6 *v6 = (struct sockaddr_in6 *)addr;
+
   for (n = 0; n < 16; n++)
   {
     if (bits >= 8)
     {
-      v ^= addr->sins.sin6.s6_addr[n];
+      v ^= v6->sin6_addr.s6_addr[n];
       bits -= 8;
     }
     else if (bits)
     {
-      v ^= addr->sins.sin6.s6_addr[n] & ~((1 << (8 - bits)) - 1);
+      v ^= v6->sin6_addr.s6_addr[n] & ~((1 << (8 - bits)) - 1);
       return v & (ATABLE_SIZE - 1);
     }
     else
@@ -338,17 +406,16 @@ hash_ipv6(struct irc_inaddr *addr, int bits)
  * Output: The hash of the string between 1 and (TH_MAX-1)
  * Side-effects: None.
  */
-static int
+static uint32_t
 hash_text(const char *start)
 {
   const char *p = start;
-  unsigned long h = 0;
+  uint32_t h = 0;
 
-  while (*p)
-  {
-    h = (h << 4) - (h + (unsigned char)ToLower(*p++));
-  }
-  return (h & (ATABLE_SIZE - 1));
+  for (; *p; ++p)
+    h = (h << 4) - (h + (unsigned char)ToLower(*p));
+
+  return h & (ATABLE_SIZE - 1);
 }
 
 /* unsigned long get_hash_mask(const char *)
@@ -357,38 +424,44 @@ hash_text(const char *start)
  *         wildcard in the string.
  * Side-effects: None.
  */
-static unsigned long
+static uint32_t
 get_mask_hash(const char *text)
 {
   const char *hp = "", *p;
 
   for (p = text + strlen(text) - 1; p >= text; p--)
-    if (*p == '*' || *p == '?')
+    if (IsMWildChar(*p))
       return hash_text(hp);
     else if (*p == '.')
       hp = p + 1;
   return hash_text(text);
 }
 
-/* struct ConfItem* find_conf_by_address(const char*, struct irc_inaddr*,
- *         int type, int fam, const char *username)
+/* struct AccessItem *find_conf_by_address(const char *, struct irc_ssaddr *,
+ *                                         int type, int fam, const char *username)
  * Input: The hostname, the address, the type of mask to find, the address
  *        family, the username.
  * Output: The matching value with the highest precedence.
  * Side-effects: None
  * Note: Setting bit 0 of the type means that the username is ignored.
+ * Warning: IsNeedPassword for everything that is not an auth{} entry
+ * should always be true (i.e. aconf->flags & CONF_FLAGS_NEED_PASSWORD == 0)
  */
-struct ConfItem *
-find_conf_by_address(const char *name, struct irc_inaddr *addr, int type,
-                     int fam, const char *username)
+struct AccessItem *
+find_conf_by_address(const char *name, struct irc_ssaddr *addr, unsigned int type,
+                     int fam, const char *username, const char *password, int do_match)
 {
-  unsigned long hprecv = 0;
-  struct ConfItem *hprec = NULL;
+  unsigned int hprecv = 0;
+  dlink_node *ptr = NULL;
+  struct AccessItem *hprec = NULL;
   struct AddressRec *arec;
   int b;
+  int (*cmpfunc)(const char *, const char *) = do_match ? match : irccmp;
 
   if (username == NULL)
     username = "";
+  if (password == NULL)
+    password = "";
 
   if (addr)
   {
@@ -398,17 +471,23 @@ find_conf_by_address(const char *name, struct irc_inaddr *addr, int type,
     {
       for (b = 128; b >= 0; b -= 16)
       {
-        for (arec = atable[hash_ipv6(addr, b)]; arec; arec = arec->next)
+        DLINK_FOREACH(ptr, atable[hash_ipv6(addr, b)].head)
+        {
+          arec = ptr->data; 
+
           if (arec->type == (type & ~0x1) &&
+              arec->precedence > hprecv &&
               arec->masktype == HM_IPV6 &&
               match_ipv6(addr, &arec->Mask.ipa.addr,
                          arec->Mask.ipa.bits) &&
-              (type & 0x1 || match(arec->username, username)) &&
-              arec->precedence > hprecv)
+              (type & 0x1 || cmpfunc(arec->username, username) == do_match) &&
+	      (IsNeedPassword(arec->aconf) || arec->aconf->passwd == NULL ||
+	       match_conf_password(password, arec->aconf)))
           {
             hprecv = arec->precedence;
             hprec = arec->aconf;
           }
+        }
       }
     }
     else
@@ -417,224 +496,235 @@ find_conf_by_address(const char *name, struct irc_inaddr *addr, int type,
     {
       for (b = 32; b >= 0; b -= 8)
       {
-        for (arec = atable[hash_ipv4(addr, b)]; arec; arec = arec->next)
+        DLINK_FOREACH(ptr, atable[hash_ipv4(addr, b)].head)
+        {
+          arec = ptr->data;
+
           if (arec->type == (type & ~0x1) &&
+              arec->precedence > hprecv &&
               arec->masktype == HM_IPV4 &&
               match_ipv4(addr, &arec->Mask.ipa.addr,
                          arec->Mask.ipa.bits) &&
-              (type & 0x1 || match(arec->username, username)) &&
-              arec->precedence > hprecv)
+              (type & 0x1 || cmpfunc(arec->username, username) == do_match) &&
+	      (IsNeedPassword(arec->aconf) || arec->aconf->passwd == NULL ||
+	       match_conf_password(password, arec->aconf)))
           {
             hprecv = arec->precedence;
             hprec = arec->aconf;
           }
+        }
       }
     }
   }
 
   if (name != NULL)
   {
-    const char *p;
-    /* And yes - we have to check p after strchr and p after increment for
-     * NULL -kre */
-    for (p = name; p != NULL; )
+    const char *p = name;
+
+    while (1)
     {
-      for (arec = atable[hash_text(p)]; arec; arec = arec->next)
-        if ((arec->type == (type & ~0x1)) &&
+        DLINK_FOREACH(ptr, atable[hash_text(p)].head)
+        {
+          arec = ptr->data;
+          if ((arec->type == (type & ~0x1)) &&
+            arec->precedence > hprecv &&
             (arec->masktype == HM_HOST) &&
-            match(arec->Mask.hostname, name) &&
-            (type & 0x1 || match(arec->username, username)) &&
-            (arec->precedence > hprecv))
+            cmpfunc(arec->Mask.hostname, name) == do_match &&
+            (type & 0x1 || cmpfunc(arec->username, username) == do_match) &&
+            (IsNeedPassword(arec->aconf) || arec->aconf->passwd == NULL ||
+             match_conf_password(password, arec->aconf)))
         {
           hprecv = arec->precedence;
           hprec = arec->aconf;
         }
-        p = strchr(p, '.');
-        if (p != NULL)
-          p++;
-        else
-          break;
+      }
+      p = strchr(p, '.');
+      if (p == NULL)
+        break;
+      p++;
     }
-    for (arec = atable[0]; arec; arec = arec->next)
+
+    DLINK_FOREACH(ptr, atable[0].head)
+    {
+      arec = ptr->data;
+
       if (arec->type == (type & ~0x1) &&
+          arec->precedence > hprecv &&
           arec->masktype == HM_HOST &&
-          match(arec->Mask.hostname, name) &&
-          (type & 0x1 || match(arec->username, username)) &&
-          arec->precedence > hprecv)
+          cmpfunc(arec->Mask.hostname, name) == do_match &&
+          (type & 0x1 || cmpfunc(arec->username, username) == do_match) &&
+          (IsNeedPassword(arec->aconf) || arec->aconf->passwd == NULL ||
+           match_conf_password(password, arec->aconf)))
       {
         hprecv = arec->precedence;
         hprec = arec->aconf;
       }
+    }
   }
+
   return hprec;
 }
 
-/* struct ConfItem* find_address_conf(const char*, const char*,
- * 	                               struct irc_inaddr*, int);
+/* struct AccessItem* find_address_conf(const char*, const char*,
+ * 	                               struct irc_ssaddr*, int, char *);
  * Input: The hostname, username, address, address family.
- * Output: The applicable ConfItem.
+ * Output: The applicable AccessItem.
  * Side-effects: None
  */
-struct ConfItem *
+struct AccessItem *
 find_address_conf(const char *host, const char *user,
-                  struct irc_inaddr *ip, int aftype)
+                  struct irc_ssaddr *ip, int aftype, char *password)
 {
-  struct ConfItem *iconf, *kconf;
+  struct AccessItem *iconf, *kconf;
 
-  /* Find the best I-line... If none, return NULL -A1kmm */
-  if (!(iconf = find_conf_by_address(host, ip, CONF_CLIENT, aftype, user)))
+  /* Find the best auth{} block... If none, return NULL -A1kmm */
+  if ((iconf = find_conf_by_address(host, ip, CONF_CLIENT, aftype, user,
+                                    password, 1)) == NULL)
     return NULL;
 
-  /* If they are exempt from K-lines, return the best I-line. -A1kmm */
+  /* If they are exempt from K-lines, return the best auth{} block. -A1kmm */
   if (IsConfExemptKline(iconf))
     return iconf;
 
   /* Find the best K-line... -A1kmm */
-  kconf = find_conf_by_address(host, ip, CONF_KILL, aftype, user);
+  kconf = find_conf_by_address(host, ip, CONF_KLINE, aftype, user, NULL, 1);
 
-  /* If they are K-lined, return the K-line. Otherwise, return the
-   * I-line. -A1kmm */
-  if (kconf)
+  /*
+   * If they are K-lined, return the K-line. Otherwise, return the
+   * auth{} block. -A1kmm
+   */
+  if (kconf != NULL)
     return kconf;
+
+  if (IsConfExemptGline(iconf))
+    return iconf;
+
+  kconf = find_conf_by_address(host, ip, CONF_GLINE, aftype, user, NULL, 1);
+  if (kconf != NULL)
+    return kconf;
+
   return iconf;
 }
 
-/*
- * find_kline_conf
- *
- * inputs	- pointer to hostname
- *		- pointer to username
- *		- incoming IP and type (IPv4 vs. IPv6)
- * outut	- pointer to kline conf if found NULL if not
- * side effects	-
- */
-struct ConfItem *
-find_kline_conf(const char *host, const char *user,
-		struct irc_inaddr *ip, int aftype)
-{
-  struct ConfItem *kconf;
-
-  /* Find the best K-line... -A1kmm */
-  kconf = find_conf_by_address(host, ip, CONF_KILL, aftype, user);
-
-  /* If they are K-lined, return the K-line. Otherwise, return the
-   * I-line. -A1kmm */
-  return (kconf);
-}
-
-/* struct ConfItem* find_dline_conf(struct irc_inaddr*, int)
+/* struct AccessItem* find_dline_conf(struct irc_ssaddr*, int)
  *
  * Input:	An address, an address family.
  * Output:	The best matching D-line or exempt line.
  * Side effects: None.
  */
-struct ConfItem *
-find_dline_conf(struct irc_inaddr *addr, int aftype)
+struct AccessItem *
+find_dline_conf(struct irc_ssaddr *addr, int aftype)
 {
-  struct ConfItem *eline;
+  struct AccessItem *eline;
+
   eline = find_conf_by_address(NULL, addr, CONF_EXEMPTDLINE | 1, aftype,
-                               NULL);
-  if (eline)
+                               NULL, NULL, 1);
+  if (eline != NULL)
     return eline;
-  return find_conf_by_address(NULL, addr, CONF_DLINE | 1, aftype, NULL);
+
+  return find_conf_by_address(NULL, addr, CONF_DLINE | 1, aftype, NULL, NULL, 1);
 }
 
-/* void add_conf_by_address(const char*, int, const char *,
- *         struct ConfItem *aconf)
+/* void add_conf_by_address(int, struct AccessItem *aconf)
  * Input: 
  * Output: None
  * Side-effects: Adds this entry to the hash table.
  */
 void
-add_conf_by_address(const char *address, int type, const char *username,
-                    struct ConfItem *aconf)
+add_conf_by_address(const unsigned int type, struct AccessItem *aconf)
 {
-  static unsigned long prec_value = 0xFFFFFFFF;
-  int masktype, bits;
-  unsigned long hv;
+  const char *address;
+  const char *username;
+  static unsigned int prec_value = 0xFFFFFFFF;
+  int bits = 0;
   struct AddressRec *arec;
 
-  if (address == NULL)
+  address = aconf->host;
+  username = aconf->user;
+
+  assert(type != 0); 
+  assert(aconf != NULL);
+
+  if (EmptyString(address))
     address = "/NOMATCH!/";
+
   arec = MyMalloc(sizeof(struct AddressRec));
-  masktype = parse_netmask(address, &arec->Mask.ipa.addr, &bits);
+  arec->masktype = parse_netmask(address, &arec->Mask.ipa.addr, &bits);
   arec->Mask.ipa.bits = bits;
-  arec->masktype = masktype;
-#ifdef IPV6
-  if (masktype == HM_IPV6)
-  {
-    /* We have to do this, since we do not re-hash for every bit -A1kmm. */
-    bits -= bits % 16;
-    arec->next = atable[(hv = hash_ipv6(&arec->Mask.ipa.addr, bits))];
-    atable[hv] = arec;
-  }
-  else
-#endif
-  if (masktype == HM_IPV4)
-  {
-    /* We have to do this, since we do not re-hash for every bit -A1kmm. */
-    bits -= bits % 8;
-    arec->next = atable[(hv = hash_ipv4(&arec->Mask.ipa.addr, bits))];
-    atable[hv] = arec;
-  }
-  else
-  {
-    arec->Mask.hostname = address;
-    arec->next = atable[(hv = get_mask_hash(address))];
-    atable[hv] = arec;
-  }
   arec->username = username;
   arec->aconf = aconf;
   arec->precedence = prec_value--;
   arec->type = type;
+
+  switch (arec->masktype)
+  {
+    case HM_IPV4:
+      /* We have to do this, since we do not re-hash for every bit -A1kmm. */
+      bits -= bits % 8;
+      dlinkAdd(arec, &arec->node, &atable[hash_ipv4(&arec->Mask.ipa.addr, bits)]);
+      break;
+#ifdef IPV6
+    case HM_IPV6:
+      /* We have to do this, since we do not re-hash for every bit -A1kmm. */
+      bits -= bits % 16;
+      dlinkAdd(arec, &arec->node, &atable[hash_ipv6(&arec->Mask.ipa.addr, bits)]);
+      break;
+#endif
+    default: /* HM_HOST */
+      arec->Mask.hostname = address;
+      dlinkAdd(arec, &arec->node, &atable[get_mask_hash(address)]);
+      break;
+  }
 }
 
-/* void delete_one_address(const char*, struct ConfItem*)
- * Input: An address string, the associated ConfItem.
+/* void delete_one_address(const char*, struct AccessItem*)
+ * Input: An address string, the associated AccessItem.
  * Output: None
- * Side effects: Deletes an address record. Frees the ConfItem if there
+ * Side effects: Deletes an address record. Frees the AccessItem if there
  *               is nothing referencing it, sets it as illegal otherwise.
  */
 void
-delete_one_address_conf(const char *address, struct ConfItem *aconf)
+delete_one_address_conf(const char *address, struct AccessItem *aconf)
 {
-  int masktype, bits;
-  unsigned long hv;
-  struct AddressRec *arec, *arecl = NULL;
-  struct irc_inaddr addr;
-  masktype = parse_netmask(address, &addr, &bits);
+  int bits = 0;
+  uint32_t hv = 0;
+  dlink_node *ptr = NULL, *ptr_next = NULL;
+  struct irc_ssaddr addr;
+
+  switch (parse_netmask(address, &addr, &bits))
+  {
+    case HM_IPV4:
+      /* We have to do this, since we do not re-hash for every bit -A1kmm. */
+      bits -= bits % 8;
+      hv = hash_ipv4(&addr, bits);
+      break;
 #ifdef IPV6
-  if (masktype == HM_IPV6)
-  {
-    /* We have to do this, since we do not re-hash for every bit -A1kmm. */
-    bits -= bits % 16;
-    hv = hash_ipv6(&addr, bits);
-  }
-  else
+    case HM_IPV6:
+      /* We have to do this, since we do not re-hash for every bit -A1kmm. */
+      bits -= bits % 16;
+      hv = hash_ipv6(&addr, bits);
+      break;
 #endif
-  if (masktype == HM_IPV4)
-  {
-    /* We have to do this, since we do not re-hash for every bit -A1kmm. */
-    bits -= bits % 8;
-    hv = hash_ipv4(&addr, bits);
+    default: /* HM_HOST */
+      hv = get_mask_hash(address);
+      break;
   }
-  else
-    hv = get_mask_hash(address);
-  for (arec = atable[hv]; arec; arec = arec->next)
+
+  DLINK_FOREACH_SAFE(ptr, ptr_next, atable[hv].head)
   {
+    struct AddressRec *arec = ptr->data;
+
     if (arec->aconf == aconf)
     {
-      if (arecl)
-        arecl->next = arec->next;
-      else
-        atable[hv] = arec->next;
+      dlinkDelete(&arec->node, &atable[hv]);
       aconf->status |= CONF_ILLEGAL;
-      if (aconf->clients == 0)
-        free_conf(aconf);
+
+      if (!aconf->clients)
+        free_access_item(aconf);
+
       MyFree(arec);
       return;
     }
-    arecl = arec;
   }
 }
 
@@ -642,162 +732,92 @@ delete_one_address_conf(const char *address, struct ConfItem *aconf)
  * Input: None
  * Output: None
  * Side effects: Clears out all address records in the hash table,
- *               frees them, and frees the ConfItems if nothing references
+ *               frees them, and frees the AccessItems if nothing references
  *               them, otherwise sets them as illegal.   
  */
 void
 clear_out_address_conf(void)
 {
-  int i;
-  struct AddressRec *arec;
-  struct AddressRec *last_arec;
-  struct AddressRec *next_arec;
+  unsigned int i = 0;
+  dlink_node *ptr = NULL, *ptr_next = NULL;
  
-  for (i = 0; i < ATABLE_SIZE; i++)
+  for (i = 0; i < ATABLE_SIZE; ++i)
   {
-    last_arec = NULL;
-    for (arec = atable[i]; arec; arec = next_arec)
+    DLINK_FOREACH_SAFE(ptr, ptr_next, atable[i].head)
     {
+      struct AddressRec *arec = ptr->data;
+
       /* We keep the temporary K-lines and destroy the
        * permanent ones, just to be confusing :) -A1kmm
        */  
-      next_arec = arec->next;
-
-      if (arec->aconf->flags & CONF_FLAGS_TEMPORARY)
-      {
-        last_arec = arec;
-      }
-      else   
+      if (!(arec->aconf->flags & CONF_FLAGS_TEMPORARY))
       {
+        dlinkDelete(&arec->node, &atable[i]);
         /* unlink it from link list - Dianora */
-  
-        if (last_arec == NULL)
-        {
-          atable[i] = next_arec;
-        }
-        else
-        {
-          last_arec->next = next_arec;
-        }
-
         arec->aconf->status |= CONF_ILLEGAL;
-        if (arec->aconf->clients == 0)
-          free_conf(arec->aconf);
+
+        if (!arec->aconf->clients)
+          free_access_item(arec->aconf);
         MyFree(arec);
       }
     }
   }
 }
 
-/*
- * show_iline_prefix()
- *
- * inputs       - pointer to struct Client requesting output
- *              - pointer to struct ConfItem 
- *              - name to which iline prefix will be prefixed to
- * output       - pointer to static string with prefixes listed in ascii form
- * side effects - NONE
- */
-char *
-show_iline_prefix(struct Client *sptr, struct ConfItem *aconf, char *name)
+static void
+hostmask_send_expiration(struct AddressRec *arec)
 {
-  static char prefix_of_host[USERLEN + 15];
-  char *prefix_ptr;
+  char ban_type = '\0';
 
-  prefix_ptr = prefix_of_host;
-  if (IsNoTilde(aconf))
-    *prefix_ptr++ = '-';
-  if (IsLimitIp(aconf))
-    *prefix_ptr++ = '!';
-  if (IsNeedIdentd(aconf))
-    *prefix_ptr++ = '+';
-  if (IsPassIdentd(aconf))
-    *prefix_ptr++ = '$';
-  if (IsNoMatchIp(aconf))
-    *prefix_ptr++ = '%';
-  if (IsConfDoSpoofIp(aconf))
-    *prefix_ptr++ = '=';
-  if (MyOper(sptr) && IsConfExemptKline(aconf))
-    *prefix_ptr++ = '^';
-  if (MyOper(sptr) && IsConfExemptLimits(aconf))
-    *prefix_ptr++ = '>';
-  if (MyOper(sptr) && IsConfIdlelined(aconf))
-    *prefix_ptr++ = '<';
-  *prefix_ptr = '\0';
-  strncpy(prefix_ptr, name, USERLEN);
-  return (prefix_of_host);
+  if (!ConfigFileEntry.tkline_expire_notices)
+    return;
+
+  switch (arec->type)
+  {
+    case CONF_KLINE:
+      ban_type = 'K';
+      break;
+    case CONF_DLINE:
+      ban_type = 'D';
+      break;
+    case CONF_GLINE:
+      ban_type = 'G';
+      break;
+  }
+  
+  sendto_realops_flags(UMODE_ALL, L_ALL,
+                       "Temporary %c-line for [%s@%s] expired", ban_type,
+                       (arec->aconf->user) ? arec->aconf->user : "*",
+                       (arec->aconf->host) ? arec->aconf->host : "*");
 }
 
-/* report_auth()
- *
- * Inputs: pointer to client to report to
- * Output: None
- * Side effects: Reports configured auth{} blocks to client_p
- */
 void
-report_auth(struct Client *client_p)
+hostmask_expire_temporary(void)
 {
-  char *name, *host, *pass, *user, *classname;
-  struct AddressRec *arec;
-  struct ConfItem *aconf;
-  int i, port;
+  unsigned int i = 0;
+  dlink_node *ptr = NULL, *ptr_next = NULL;
 
-  for (i = 0; i < ATABLE_SIZE; i++)
-    for (arec = atable[i]; arec; arec = arec->next)
-      if (arec->type == CONF_CLIENT)
+  for (i = 0; i < ATABLE_SIZE; ++i)
+  {
+    DLINK_FOREACH_SAFE(ptr, ptr_next, atable[i].head)
+    {
+      struct AddressRec *arec = ptr->data;
+
+      if (!IsConfTemporary(arec->aconf) || arec->aconf->hold > CurrentTime)
+        continue;
+
+      switch (arec->type)
       {
-        aconf = arec->aconf;
+        case CONF_KLINE:
+        case CONF_DLINE:
+        case CONF_GLINE:
+          hostmask_send_expiration(arec);
 
-        if (!MyOper(client_p) && IsConfDoSpoofIp(aconf))
-          continue;
-
-        get_printable_conf(aconf, &name, &host, &pass, &user, &port,
-                           &classname);
-
-        /* We are doing a partial list, based on what matches the u@h of the
-         * sender, so prepare the strings for comparing --fl_ */
-
-        sendto_one(client_p, form_str(RPL_STATSILINE), me.name,
-                   client_p->name, (IsConfRestricted(aconf)) ? 'i' : 'I', name,
-                   show_iline_prefix(client_p, aconf, user),
-#ifdef HIDE_SPOOF_IPS
-                   IsConfDoSpoofIp(aconf) ? "255.255.255.255" :
-#endif
-                   host, port, classname);
-      }
-}
-
-/* report_Klines()
- * Inputs: Client to report to,
- *	   type(==0 for perm, !=0 for temporary)
- *	   mask 
- * Output: None
- * Side effects: Reports configured K(or k)-lines to client_p.
- */
-void
-report_Klines(struct Client *client_p, int tkline)
-{
-  char *name, *host, *pass, *user, *classname, c;
-  struct AddressRec *arec;
-  struct ConfItem *aconf = NULL;
-  int i, port;
-
-  if (tkline)
-    c = 'k';
-  else
-    c = 'K';
-
-  for (i = 0; i < ATABLE_SIZE; i++)
-    for (arec = atable[i]; arec; arec = arec->next)
-      if (arec->type == CONF_KILL)
-      {
-        if ((tkline && !((aconf = arec->aconf)->flags & CONF_FLAGS_TEMPORARY))
-            || (!tkline
-                && ((aconf = arec->aconf)->flags & CONF_FLAGS_TEMPORARY)))
-          continue;
-        get_printable_conf(aconf, &name, &host, &pass, &user, &port,
-                           &classname);
-        sendto_one(client_p, form_str(RPL_STATSKLINE), me.name,
-                   client_p->name, c, host, user, pass);
+          dlinkDelete(&arec->node, &atable[i]);
+          free_access_item(arec->aconf);
+          MyFree(arec);
+          break;
       }
+    }
+  }
 }
