@@ -81,23 +81,21 @@
  */
 
 #include "stdinc.h"
-#include "irc_reslib.h"	
-#include "irc_res.h"	
 #include "ircd_defs.h"
-#include "fileio.h"
+#include "irc_res.h"
+#include "irc_reslib.h"
 #include "irc_string.h"
-#include "irc_getaddrinfo.h"
 
 #define NS_TYPE_ELT             0x40 /* EDNS0 extended label type */
 #define DNS_LABELTYPE_BITSTRING 0x41
 #define MAXLINE 128
 
-/* $Id: irc_reslib.c 708 2006-07-07 08:56:45Z michael $ */
+/* $Id: irc_reslib.c 1346 2012-04-09 17:35:40Z michael $ */
 
 struct irc_ssaddr irc_nsaddr_list[IRCD_MAXNS];
 int irc_nscount = 0;
-char irc_domain[HOSTLEN + 1];
 
+static const char digits[] = "0123456789";
 static const char digitvalue[256] = {
   -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, /*16*/
   -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, /*32*/
@@ -117,34 +115,56 @@ static const char digitvalue[256] = {
   -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, /*256*/
 };
 
-static int parse_resvconf(void);
-static void add_nameserver(char *arg);
-
-static const char digits[] = "0123456789";
 static int labellen(const unsigned char *lp);
 static int special(int ch);
 static int printable(int ch);
-static int irc_decode_bitstring(const char **cpp, char *dn, const char *eom);
+static int irc_decode_bitstring(const unsigned char **cpp, char *dn, const char *eom);
 static int irc_ns_name_compress(const char *src, unsigned char *dst, size_t dstsiz,
     const unsigned char **dnptrs, const unsigned char **lastdnptr);
 static int irc_dn_find(const unsigned char *, const unsigned char *, const unsigned char * const *,
                        const unsigned char * const *);
 static int irc_encode_bitsring(const char **, const char *, unsigned char **, unsigned char **, 
-                               const char *);
+                               const unsigned char *);
 static int irc_ns_name_uncompress(const unsigned char *, const unsigned char *,
                                   const unsigned char *, char *, size_t);
 static int irc_ns_name_unpack(const unsigned char *, const unsigned char *,
                               const unsigned char *, unsigned char *,
                               size_t);
-static int irc_ns_name_ntop(const char *, char *, size_t);
+static int irc_ns_name_ntop(const unsigned char *, char *, size_t);
 static int irc_ns_name_skip(const unsigned char **, const unsigned char *);
 static int mklower(int ch);
   
-int
-irc_res_init(void)
+
+/* add_nameserver()
+ *
+ * input        - either an IPV4 address in dotted quad
+ *                or an IPV6 address in : format
+ * output       - NONE
+ * side effects - entry in irc_nsaddr_list is filled in as needed
+ */
+static void
+add_nameserver(const char *arg)
 {
-  irc_nscount = 0;
-  return parse_resvconf();
+  struct addrinfo hints, *res;
+
+  /* Done max number of nameservers? */
+  if (irc_nscount >= IRCD_MAXNS)
+    return;
+
+  memset(&hints, 0, sizeof(hints));
+  hints.ai_family   = PF_UNSPEC;
+  hints.ai_socktype = SOCK_DGRAM;
+  hints.ai_flags    = AI_PASSIVE | AI_NUMERICHOST;
+
+  if (getaddrinfo(arg, "domain", &hints, &res))
+    return;
+
+  if (res == NULL)
+    return;
+
+  memcpy(&irc_nsaddr_list[irc_nscount].ss, res->ai_addr, res->ai_addrlen);
+  irc_nsaddr_list[irc_nscount++].ss_len = res->ai_addrlen;
+  freeaddrinfo(res);
 }
 
 /* parse_resvconf()
@@ -153,22 +173,22 @@ irc_res_init(void)
  * output - -1 if failure 0 if success
  * side effects - fills in irc_nsaddr_list
  */
-static int
+static void
 parse_resvconf(void)
 {
   char *p;
   char *opt;
   char *arg;
   char input[MAXLINE];
-  FBFILE *file;
+  FILE *file;
 
-  /* XXX "/etc/resolv.conf" should be from a define in setup.h perhaps
+  /* XXX "/etc/resolv.conf" should be from a define in config.h perhaps
    * for cygwin support etc. this hardcodes it to unix for now -db
    */
-  if ((file = fbopen("/etc/resolv.conf", "r")) == NULL)
-    return -1;
+  if ((file = fopen("/etc/resolv.conf", "r")) == NULL)
+    return;
 
-  while (fbgets(input, sizeof(input), file) != NULL)
+  while (fgets(input, sizeof(input), file) != NULL)
   {
     /* blow away any newline */
     if ((p = strpbrk(input, "\r\n")) != NULL)
@@ -209,46 +229,23 @@ parse_resvconf(void)
     if ((p = strpbrk(arg, " \t")) != NULL)
       *p = '\0';  /* take the first word */
 
-    if (irccmp(opt, "domain") == 0)
-      strlcpy(irc_domain, arg, HOSTLEN);
-    else if (irccmp(opt, "nameserver") == 0)
+    if (!irccmp(opt, "nameserver"))
       add_nameserver(arg);
   }
 
-  fbclose(file);
-  return 0;
+  fclose(file);
 }
 
-/* add_nameserver()
- *
- * input        - either an IPV4 address in dotted quad
- *                or an IPV6 address in : format
- * output       - NONE
- * side effects - entry in irc_nsaddr_list is filled in as needed
- */
-static void
-add_nameserver(char *arg)
+void
+irc_res_init(void)
 {
-  struct addrinfo hints, *res;
-  /* Done max number of nameservers? */
-  if ((irc_nscount) >= IRCD_MAXNS)
-    return;
+  irc_nscount = 0;
+  memset(irc_nsaddr_list, 0, sizeof(irc_nsaddr_list));
 
-  memset(&hints, 0, sizeof(hints));
-  hints.ai_family   = PF_UNSPEC;
-  hints.ai_socktype = SOCK_DGRAM;
-  hints.ai_flags    = AI_PASSIVE | AI_NUMERICHOST;
+  parse_resvconf();
 
-  if (irc_getaddrinfo(arg, "domain", &hints, &res))
-    return;
-
-  if (res == NULL)
-    return;
-
-  memcpy(&irc_nsaddr_list[irc_nscount].ss, res->ai_addr, res->ai_addrlen);
-  irc_nsaddr_list[irc_nscount].ss_len = res->ai_addrlen;
-  irc_nscount++;
-  irc_freeaddrinfo(res);
+  if (!irc_nscount)
+    add_nameserver("127.0.0.1");
 }
 
 /*
@@ -267,7 +264,7 @@ irc_dn_expand(const unsigned char *msg, const unsigned char *eom,
   if (n > 0 && dst[0] == '.')
     dst[0] = '\0';
   return(n);
-}
+} /*2*/
 
 /*
  * irc_ns_name_uncompress(msg, eom, src, dst, dstsiz)
@@ -286,10 +283,11 @@ irc_ns_name_uncompress(const unsigned char *msg, const unsigned char *eom,
 
   if ((n = irc_ns_name_unpack(msg, eom, src, tmp, sizeof tmp)) == -1)
     return(-1);
-  if (irc_ns_name_ntop((char*)tmp, dst, dstsiz) == -1)
+  if (irc_ns_name_ntop(tmp, dst, dstsiz) == -1)
     return(-1);
   return(n);
-}
+} /*2*/
+
 /*
  * irc_ns_name_unpack(msg, eom, src, dst, dstsiz)
  *	Unpack a domain name from a message, source may be compressed.
@@ -369,7 +367,7 @@ irc_ns_name_unpack(const unsigned char *msg, const unsigned char *eom,
 	if (len < 0)
 		len = srcp - src;
 	return (len);
-}
+} /*2*/
 
 /*
  * irc_ns_name_ntop(src, dst, dstsiz)
@@ -381,9 +379,9 @@ irc_ns_name_unpack(const unsigned char *msg, const unsigned char *eom,
  *	All other domains are returned in non absolute form
  */
 static int
-irc_ns_name_ntop(const char *src, char *dst, size_t dstsiz)
+irc_ns_name_ntop(const unsigned char *src, char *dst, size_t dstsiz)
 {
-	const char *cp;
+	const unsigned char *cp;
 	char *dn, *eom;
 	unsigned char c;
 	unsigned int n;
@@ -406,7 +404,7 @@ irc_ns_name_ntop(const char *src, char *dst, size_t dstsiz)
 			}
 			*dn++ = '.';
 		}
-		if ((l = labellen((unsigned char*)(cp - 1))) < 0) {
+		if ((l = labellen((cp - 1))) < 0) {
 			errno = EMSGSIZE; /* XXX */
 			return(-1);
 		}
@@ -470,21 +468,7 @@ irc_ns_name_ntop(const char *src, char *dst, size_t dstsiz)
 	}
 	*dn++ = '\0';
 	return (dn - dst);
-}
-
-/*
- * Pack domain name 'exp_dn' in presentation form into 'comp_dn'.
- * Return the size of the compressed name or -1.
- * 'length' is the size of the array pointed to by 'comp_dn'.
- */
-static int
-irc_dn_comp(const char *src, unsigned char *dst, int dstsiz,
-            unsigned char **dnptrs, unsigned char **lastdnptr)
-{
-  return(irc_ns_name_compress(src, dst, (size_t)dstsiz,
-                              (const unsigned char **)dnptrs,
-                              (const unsigned char **)lastdnptr));
-}
+} /*2*/
 
 /*
  * Skip over a compressed domain name. Return the size or -1.
@@ -496,7 +480,7 @@ irc_dn_skipname(const unsigned char *ptr, const unsigned char *eom) {
   if (irc_ns_name_skip(&ptr, eom) == -1)
     return(-1);
   return(ptr - saveptr);
-}
+} /*2*/
 
 /*
  * ns_name_skip(ptrptr, eom)
@@ -549,7 +533,7 @@ irc_ns_name_skip(const unsigned char **ptrptr, const unsigned char *eom)
 
   *ptrptr = cp;
   return(0);
-}
+} /*2*/
 
 unsigned int
 irc_ns_get16(const unsigned char *src)
@@ -608,7 +592,7 @@ special(int ch)
     default:
       return(0);
   }
-}
+} /*2*/
 
 static int
 labellen(const unsigned char *lp)
@@ -635,7 +619,7 @@ labellen(const unsigned char *lp)
   }
 
   return(l);
-}
+} /*2*/
 
 
 /*
@@ -649,12 +633,12 @@ static int
 printable(int ch)
 {
   return(ch > 0x20 && ch < 0x7f);
-}
+} /*2*/
 
 static int
-irc_decode_bitstring(const char **cpp, char *dn, const char *eom)
+irc_decode_bitstring(const unsigned char **cpp, char *dn, const char *eom)
 {
-        const char *cp = *cpp;
+        const unsigned char *cp = *cpp;
         char *beg = dn, tc;
         int b, blen, plen;
 
@@ -681,7 +665,7 @@ irc_decode_bitstring(const char **cpp, char *dn, const char *eom)
 
         *cpp = cp;
         return(dn - beg);
-}
+} /*2*/
 
 /*
  * irc_ns_name_pton(src, dst, dstsiz)
@@ -717,7 +701,7 @@ irc_ns_name_pton(const char *src, unsigned char *dst, size_t dstsiz)
                cp + 2,
                &label,
                &bp,
-               (const char *)eom))
+               eom))
             != 0) {
           errno = e;
           return(-1);
@@ -821,7 +805,7 @@ irc_ns_name_pton(const char *src, unsigned char *dst, size_t dstsiz)
   }
 
   return (0);
-}
+} /*2*/
 
 /*
  * irc_ns_name_pack(src, dst, dstsiz, dnptrs, lastdnptr)
@@ -930,7 +914,7 @@ cleanup:
     return (-1);
   }
   return(dstp - dst);
-}
+} /*2*/
 
 static int
 irc_ns_name_compress(const char *src, unsigned char *dst, size_t dstsiz,
@@ -945,11 +929,12 @@ irc_ns_name_compress(const char *src, unsigned char *dst, size_t dstsiz,
 
 static int
 irc_encode_bitsring(const char **bp, const char *end, unsigned char **labelp,
-                    unsigned char **dst, const char *eom)
+                    unsigned char **dst, const unsigned char *eom)
 {
   int afterslash = 0;
   const char *cp = *bp;
-  char *tp, c;
+  unsigned char *tp;
+  char c;
   const char *beg_blen;
   char *end_blen = NULL;
   int value = 0, count = 0, tbcount = 0, blen = 0;
@@ -966,7 +951,7 @@ irc_encode_bitsring(const char **bp, const char *end, unsigned char **labelp,
   if (!isxdigit((*cp) & 0xff)) /* reject '\[x/BLEN]' */
     return(EINVAL);
 
-  for (tp = (char*)(dst + 1); cp < end && tp < eom; cp++) {
+  for (tp = *dst + 1; cp < end && tp < eom; cp++) {
     switch((c = *cp)) {
     case ']': /* end of the bitstring */
       if (afterslash) {
@@ -1043,10 +1028,10 @@ irc_encode_bitsring(const char **bp, const char *end, unsigned char **labelp,
   **dst = blen;
 
   *bp = cp;
-  *dst = (unsigned char*)tp;
+  *dst = tp;
 
   return(0);
-}
+} /*2*/
 
 /*
  * dn_find(domain, msg, dnptrs, lastdnptr)
@@ -1115,7 +1100,7 @@ irc_dn_find(const unsigned char *domain, const unsigned char *msg,
   }
   errno = ENOENT;
   return (-1);
-}
+} /*2*/
 
 /*
  *  *  Thinking in noninternationalized USASCII (per the DNS spec),
@@ -1128,7 +1113,7 @@ mklower(int ch)
     return(ch + 0x20);
 
   return(ch);
-}
+} /*2*/
 
 /* From resolv/mkquery.c */
 
@@ -1146,7 +1131,7 @@ irc_res_mkquery(
 	HEADER *hp;
 	unsigned char *cp;
 	int n;
-	unsigned char *dnptrs[20], **dpp, **lastdnptr;
+	const unsigned char *dnptrs[20], **dpp, **lastdnptr;
 
 	/*
 	 * Initialize header fields.
@@ -1169,7 +1154,8 @@ irc_res_mkquery(
 
 	if ((buflen -= QFIXEDSZ) < 0)
 	  return (-1);
-	if ((n = irc_dn_comp(dname, cp, buflen, dnptrs, lastdnptr)) < 0)
+	if ((n = irc_ns_name_compress(dname, cp, buflen, dnptrs,
+                                                         lastdnptr)) < 0)
 	  return (-1);
 
 	cp += n;
