@@ -19,10 +19,11 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: listener.c 563 2006-04-23 09:07:41Z michael $
+ *  $Id: listener.c 1309 2012-03-25 11:24:18Z michael $
  */
 
 #include "stdinc.h"
+#include "list.h"
 #include "listener.h"
 #include "client.h"
 #include "fdlist.h"
@@ -31,18 +32,10 @@
 #include "ircd.h"
 #include "ircd_defs.h"
 #include "s_bsd.h"
-#include "irc_getnameinfo.h"
-#include "irc_getaddrinfo.h"
 #include "numeric.h"
-#include "s_conf.h"
-#include "s_stats.h"
+#include "conf.h"
 #include "send.h"
 #include "memory.h"
-#include "tools.h"
-#ifdef HAVE_LIBCRYPTO
-#include <openssl/bio.h>
-#endif
-
 
 static PF accept_connection;
 
@@ -53,9 +46,7 @@ static struct Listener *
 make_listener(int port, struct irc_ssaddr *addr)
 {
   struct Listener *listener = MyMalloc(sizeof(struct Listener));
-  assert(listener != 0);
 
-  listener->name = me.name;
   listener->port = port;
   memcpy(&listener->addr, addr, sizeof(struct irc_ssaddr));
 
@@ -67,30 +58,22 @@ free_listener(struct Listener *listener)
 {
   assert(listener != NULL);
 
-  if (listener == NULL)
-    return;
-
   dlinkDelete(&listener->listener_node, &ListenerPollList);
   MyFree(listener);
 }
 
 /*
  * get_listener_name - return displayable listener name and port
- * returns "host.foo.org:6667" for a given listener
+ * returns "host.foo.org/6667" for a given listener
  */
 const char *
-get_listener_name(const struct Listener *listener)
+get_listener_name(const struct Listener *const listener)
 {
   static char buf[HOSTLEN + HOSTLEN + PORTNAMELEN + 4];
 
-  assert(listener != NULL);
-
-  if (listener == NULL)
-    return(NULL);
-
-  ircsprintf(buf, "%s[%s/%u]",
-             me.name, listener->name, listener->port);
-  return(buf);
+  snprintf(buf, sizeof(buf), "%s[%s/%u]", me.name,
+           listener->name, listener->port);
+  return buf;
 }
 
 /* show_ports()
@@ -102,7 +85,7 @@ get_listener_name(const struct Listener *listener)
 void 
 show_ports(struct Client *source_p)
 {
-  char buf[4];
+  char buf[6];
   char *p = NULL;
   dlink_node *ptr;
 
@@ -112,17 +95,19 @@ show_ports(struct Client *source_p)
     p = buf;
 
     if (listener->flags & LISTENER_HIDDEN) {
-      if (!IsAdmin(source_p))
+      if (!HasUMode(source_p, UMODE_ADMIN))
         continue;
       *p++ = 'H';
     }
 
+    if (listener->flags & LISTENER_SERVER)
+      *p++ = 'S';
     if (listener->flags & LISTENER_SSL)
       *p++ = 's';
     *p = '\0';
     sendto_one(source_p, form_str(RPL_STATSPLINE),
                me.name, source_p->name, 'P', listener->port,
-               IsAdmin(source_p) ? listener->name : me.name,
+               HasUMode(source_p, UMODE_ADMIN) ? listener->name : me.name,
                listener->ref_count, buf,
                listener->active ? "active" : "disabled");
   }
@@ -147,6 +132,12 @@ inetport(struct Listener *listener)
   struct irc_ssaddr lsin;
   socklen_t opt = 1;
 
+  memset(&lsin, 0, sizeof(lsin));
+  memcpy(&lsin, &listener->addr, sizeof(lsin));
+
+  getnameinfo((struct sockaddr *)&lsin, lsin.ss_len, listener->name,
+              sizeof(listener->name), NULL, 0, NI_NUMERICHOST);
+
   /*
    * At first, open a new socket
    */
@@ -158,26 +149,12 @@ inetport(struct Listener *listener)
     return 0;
   }
 
-  memset(&lsin, 0, sizeof(lsin));
-  memcpy(&lsin, &listener->addr, sizeof(struct irc_ssaddr));
-  
-  irc_getnameinfo((struct sockaddr*)&lsin, lsin.ss_len, listener->vhost, 
-        HOSTLEN, NULL, 0, NI_NUMERICHOST);
-  listener->name = listener->vhost;
-
-  /*
-   * XXX - we don't want to do all this crap for a listener
-   * set_sock_opts(listener);
-   */
   if (setsockopt(listener->fd.fd, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(opt)))
   {
-#ifdef _WIN32
-    errno = WSAGetLastError();
-#endif
     report_error(L_ALL, "setting SO_REUSEADDR for listener %s:%s",
                  get_listener_name(listener), errno);
     fd_close(&listener->fd);
-    return(0);
+    return 0;
   }
 
   /*
@@ -188,24 +165,18 @@ inetport(struct Listener *listener)
 
   if (bind(listener->fd.fd, (struct sockaddr *)&lsin, lsin.ss_len))
   {
-#ifdef _WIN32
-    errno = WSAGetLastError();
-#endif
     report_error(L_ALL, "binding listener socket %s:%s",
                  get_listener_name(listener), errno);
     fd_close(&listener->fd);
-    return(0);
+    return 0;
   }
 
   if (listen(listener->fd.fd, HYBRID_SOMAXCONN))
   {
-#ifdef _WIN32
-    errno = WSAGetLastError();
-#endif
     report_error(L_ALL, "listen failed for %s:%s",
                  get_listener_name(listener), errno);
     fd_close(&listener->fd);
-    return(0);
+    return 0;
   }
 
   /* Listen completion events are READ events .. */
@@ -275,15 +246,15 @@ add_listener(int port, const char *vhost_ip, unsigned int flags)
 #ifdef IPV6
   if (ServerInfo.can_use_v6)
   {
-    snprintf(portname, PORTNAMELEN, "%d", port);
-    irc_getaddrinfo("::", portname, &hints, &res);
+    snprintf(portname, sizeof(portname), "%d", port);
+    getaddrinfo("::", portname, &hints, &res);
     vaddr.ss.ss_family = AF_INET6;
     assert(res != NULL);
 
     memcpy((struct sockaddr*)&vaddr, res->ai_addr, res->ai_addrlen);
     vaddr.ss_port = port;
     vaddr.ss_len = res->ai_addrlen;
-    irc_freeaddrinfo(res);
+    freeaddrinfo(res);
   }
   else
 #endif
@@ -299,15 +270,15 @@ add_listener(int port, const char *vhost_ip, unsigned int flags)
 
   if (vhost_ip)
   {
-    if (irc_getaddrinfo(vhost_ip, portname, &hints, &res))
-        return;
+    if (getaddrinfo(vhost_ip, portname, &hints, &res))
+      return;
 
     assert(res != NULL);
 
     memcpy((struct sockaddr*)&vaddr, res->ai_addr, res->ai_addrlen);
     vaddr.ss_port = port;
     vaddr.ss_len = res->ai_addrlen;
-    irc_freeaddrinfo(res);
+    freeaddrinfo(res);
   }
 #ifdef IPV6
   else if (pass == 0 && ServerInfo.can_use_v6)
@@ -366,8 +337,7 @@ close_listener(struct Listener *listener)
 void 
 close_listeners(void)
 {
-  dlink_node *ptr;
-  dlink_node *next_ptr;
+  dlink_node *ptr = NULL, *next_ptr = NULL;
 
   /* close all 'extra' listening ports we have */
   DLINK_FOREACH_SAFE(ptr, next_ptr, ListenerPollList.head)
@@ -389,8 +359,6 @@ accept_connection(fde_t *pfd, void *data)
   memset(&addr, 0, sizeof(addr));
 
   assert(listener != NULL);
-  if (listener == NULL)
-    return;
 
   /* There may be many reasons for error return, but
    * in otherwise correctly working environment the
@@ -409,7 +377,7 @@ accept_connection(fde_t *pfd, void *data)
      */
     if (number_fd > hard_fdlimit - 10)
     {
-      ++ServerStats->is_ref;
+      ++ServerStats.is_ref;
 
       /*
        * slow down the whining to opers bit
@@ -423,11 +391,8 @@ accept_connection(fde_t *pfd, void *data)
 
       if (!(listener->flags & LISTENER_SSL))
         send(fd, "ERROR :All connections in use\r\n", 32, 0);
-#ifdef _WIN32
-      closesocket(fd);
-#else
+
       close(fd);
-#endif
       break;    /* jump out and re-register a new io request */
     }
 
@@ -437,7 +402,8 @@ accept_connection(fde_t *pfd, void *data)
      */
     if ((pe = conf_connect_allowed(&addr, addr.ss.ss_family)) != 0)
     {
-      ++ServerStats->is_ref;
+      ++ServerStats.is_ref;
+
       if (!(listener->flags & LISTENER_SSL))
         switch (pe)
         {
@@ -449,15 +415,11 @@ accept_connection(fde_t *pfd, void *data)
             break;
         }
 
-#ifdef _WIN32
-      closesocket(fd);
-#else
       close(fd);
-#endif
       continue;    /* drop the one and keep on clearing the queue */
     }
 
-    ++ServerStats->is_ac;
+    ++ServerStats.is_ac;
     add_connection(listener, &addr, fd);
   }
 
