@@ -19,26 +19,21 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: parse.c 522 2006-03-12 13:42:20Z michael $
+ *  $Id: parse.c 1682 2012-12-18 18:45:27Z michael $
  */
 
 #include "stdinc.h"
-#include "parse.h"
 #include "client.h"
+#include "parse.h"
 #include "channel.h"
-#include "handlers.h"
-#include "common.h"
 #include "hash.h"
 #include "irc_string.h"
 #include "sprintf_irc.h"
 #include "ircd.h"
 #include "numeric.h"
-#include "s_log.h"
-#include "s_stats.h"
+#include "log.h"
 #include "send.h"
-#include "ircd_handler.h"
-#include "msg.h"
-#include "s_conf.h"
+#include "conf.h"
 #include "memory.h"
 #include "s_user.h"
 #include "s_serv.h"
@@ -107,70 +102,14 @@ static struct MessageTree msg_tree;
  * NOTE: parse() should not be called recursively by other functions!
  */
 static char *sender;
-static char *para[MAXPARA + 1];
+static char *para[MAXPARA + 2]; /* <prefix> + <params> + NULL */
 static char buffer[1024];
 
 static int cancel_clients(struct Client *, struct Client *, char *);
 static void remove_unknown(struct Client *, char *, char *);
-static void do_numeric(char[], struct Client *, struct Client *, int, char **);
-static void handle_command(struct Message *, struct Client *, struct Client *, unsigned int, char **);
-static void recurse_report_messages(struct Client *source_p, struct MessageTree *mtree);
-static void add_msg_element(struct MessageTree *mtree_p, struct Message *msg_p, const char *cmd);
-static void del_msg_element(struct MessageTree *mtree_p, const char *cmd);
+static void handle_numeric(char[], struct Client *, struct Client *, int, char *[]);
+static void handle_command(struct Message *, struct Client *, struct Client *, unsigned int, char *[]);
 
-/* turn a string into a parc/parv pair */
-static inline int
-string_to_array(char *string, char *parv[MAXPARA])
-{
-  char *p;
-  char *buf = string;
-  int x = 1;
-
-  parv[x] = NULL;
-
-  while (*buf == ' ') /* skip leading spaces */
-    buf++;
-
-  if (*buf == '\0') /* ignore all-space args */
-    return(x);
-
-  do
-  {
-    if (*buf == ':') /* Last parameter */
-    {
-      buf++;
-      parv[x++] = buf;
-      parv[x]   = NULL;
-      return(x);
-    }
-    else
-    {
-      parv[x++] = buf;
-      parv[x]   = NULL;
-
-      if ((p = strchr(buf, ' ')) != NULL)
-      {
-        *p++ = '\0';
-        buf  = p;
-      }
-      else
-        return(x);
-    }       
-
-    while (*buf == ' ')
-      buf++;
-
-    if (*buf == '\0')
-      return(x);
-  } while (x < MAXPARA - 1);
-
-  if (*p == ':')
-    p++;
-
-  parv[x++] = p;
-  parv[x]   = NULL;
-  return(x);
-}
 
 /*
  * parse a buffer.
@@ -181,13 +120,12 @@ void
 parse(struct Client *client_p, char *pbuffer, char *bufend)
 {
   struct Client *from = client_p;
-  char *ch;
-  char *s;
-  char *numeric = 0;
-  unsigned int i = 0;
-  int paramcount;
-  int mpara = 0;
-  struct Message *mptr = NULL;
+  struct Message *msg_ptr = NULL;
+  char *ch = NULL;
+  char *s = NULL;
+  char *numeric = NULL;
+  unsigned int parc = 0;
+  unsigned int paramcount;
 
   if (IsDefunct(client_p))
     return;
@@ -195,48 +133,28 @@ parse(struct Client *client_p, char *pbuffer, char *bufend)
   assert(client_p->localClient->fd.flags.open);
   assert((bufend - pbuffer) < 512);
 
-  for (ch = pbuffer; *ch == ' '; ch++) /* skip spaces */
+  for (ch = pbuffer; *ch == ' '; ++ch) /* skip spaces */
     /* null statement */ ;
 
-  para[0] = from->name;
-
   if (*ch == ':')
   {
-    ch++;
-
-    /* Copy the prefix to 'sender' assuming it terminates
+    /*
+     * Copy the prefix to 'sender' assuming it terminates
      * with SPACE (or NULL, which is an error, though).
      */
-    sender = ch;
+    sender = ++ch;
 
     if ((s = strchr(ch, ' ')) != NULL)
     {
       *s = '\0';
-      s++;
-      ch = s;
+      ch = ++s;
     }
 
     if (*sender && IsServer(client_p))
     {
-      /*
-       * XXX it could be useful to know which of these occurs most frequently.
-       * the ID check should always come first, though, since it is so easy.
-       */
       if ((from = find_person(client_p, sender)) == NULL)
-      {
-        from = find_server(sender);
+        from = hash_find_server(sender);
 
-	if (from == NULL && IsCapable(client_p, CAP_TS6) &&
-	  client_p->name[0] == '*' && IsDigit(*sender) && strlen(sender) == 3)
-	{
-          /* Dirty hack to allow messages from masked SIDs (i.e. the ones
-           * hidden by fakename="..."). It shouldn't break anything, since
-           * unknown SIDs don't happen during normal ircd work --adx
-           */
-	  from = client_p;
-	}
-      }
-      
       /* Hmm! If the client corresponding to the
        * prefix is not found--what is the correct
        * action??? Now, I will ignore the message
@@ -245,28 +163,26 @@ parse(struct Client *client_p, char *pbuffer, char *bufend)
        */
       if (from == NULL)
       {
-        ServerStats->is_unpf++;
+        ++ServerStats.is_unpf;
         remove_unknown(client_p, sender, pbuffer);
         return;
       }
 
-      para[0] = from->name;
-
       if (from->from != client_p)
       {
-        ServerStats->is_wrdi++;
+        ++ServerStats.is_wrdi;
         cancel_clients(client_p, from, pbuffer);
         return;
       }
     }
 
     while (*ch == ' ')
-      ch++;
+      ++ch;
   }
 
   if (*ch == '\0')
   {
-    ServerStats->is_empt++;
+    ++ServerStats.is_empt;
     return;
   }
 
@@ -281,21 +197,20 @@ parse(struct Client *client_p, char *pbuffer, char *bufend)
   if (*(ch + 3) == ' ' && /* ok, lets see if its a possible numeric.. */
       IsDigit(*ch) && IsDigit(*(ch + 1)) && IsDigit(*(ch + 2)))
   {
-    mptr = NULL;
     numeric = ch;
     paramcount = MAXPARA;
-    ServerStats->is_num++;
+    ++ServerStats.is_num;
     s = ch + 3;  /* I know this is ' ' from above if            */
     *s++ = '\0'; /* blow away the ' ', and point s to next part */
   }
   else
   { 
-    int ii = 0;
+    unsigned int ii = 0;
 
     if ((s = strchr(ch, ' ')) != NULL)
       *s++ = '\0';
 
-    if ((mptr = find_command(ch)) == NULL)
+    if ((msg_ptr = find_command(ch)) == NULL)
     {
       /* Note: Give error message *only* to recognized
        * persons. It's a nightmare situation to have
@@ -307,38 +222,71 @@ parse(struct Client *client_p, char *pbuffer, char *bufend)
        * Hm, when is the buffer empty -- if a command
        * code has been found ?? -Armin
        */
-      if (pbuffer[0] != '\0')
+      if (*pbuffer != '\0')
       {
         if (IsClient(from))
           sendto_one(from, form_str(ERR_UNKNOWNCOMMAND),
                      me.name, from->name, ch);
       }
 
-      ServerStats->is_unco++;
+      ++ServerStats.is_unco;
       return;
     }
 
-    assert(mptr->cmd != NULL);
-
-    paramcount = mptr->parameters;
-    mpara      = mptr->maxpara;
+    assert(msg_ptr->cmd != NULL);
 
+    paramcount = msg_ptr->args_max;
     ii = bufend - ((s) ? s : ch);
-    mptr->bytes += ii;
+    msg_ptr->bytes += ii;
   }
 
-  if (s != NULL)
-    i = string_to_array(s, para);
-  else
+  /*
+   * Must the following loop really be so devious? On surface it
+   * splits the message to parameters from blank spaces. But, if
+   * paramcount has been reached, the rest of the message goes into
+   * this last parameter (about same effect as ":" has...) --msa
+   */
+
+  /* Note initially true: s==NULL || *(s-1) == '\0' !! */
+
+  para[parc] = from->name;
+
+  if (s)
   {
-    i = 0;
-    para[1] = NULL;
+    if (paramcount > MAXPARA)
+      paramcount = MAXPARA;
+
+    while (1)
+    {
+       while (*s == ' ')
+         *s++ = '\0';
+
+       if (*s == '\0')
+         break;
+
+       if (*s == ':')
+       {
+         /* The rest is a single parameter */
+         para[++parc] = s + 1;
+         break;
+       }
+
+       para[++parc] = s;
+
+       if (parc >= paramcount)
+         break;
+
+       while (*s && *s != ' ')
+         ++s;
+    }
   }
 
-  if (mptr == NULL)
-    do_numeric(numeric, client_p, from, i, para);
+  para[++parc] = NULL;
+
+  if (msg_ptr != NULL)
+    handle_command(msg_ptr, client_p, from, parc, para);
   else
-    handle_command(mptr, client_p, from, i, para);
+    handle_numeric(numeric, client_p, from, parc, para);
 }
 
 /* handle_command()
@@ -353,7 +301,7 @@ parse(struct Client *client_p, char *pbuffer, char *bufend)
  */
 static void
 handle_command(struct Message *mptr, struct Client *client_p,
-               struct Client *from, unsigned int i, char *hpara[MAXPARA])
+               struct Client *from, unsigned int i, char *hpara[])
 {
   MessageHandler handler = 0;
 
@@ -362,35 +310,23 @@ handle_command(struct Message *mptr, struct Client *client_p,
 
   mptr->count++;
 
-  /* New patch to avoid server flooding from unregistered connects
-   * - Pie-Man 07/27/2000 */
-  if (!IsRegistered(client_p))
-  {
-    /* if its from a possible server connection
-     * ignore it.. more than likely its a header thats sneaked through
-     */
-    if ((IsHandshake(client_p) || IsConnecting(client_p) ||
-        IsServer(client_p)) && !(mptr->flags & MFLG_UNREG))
-      return;
-  }
-
   handler = mptr->handlers[client_p->handler];
 
   /* check right amount of params is passed... --is */
-  if (i < mptr->parameters)
+  if (i < mptr->args_min)
   {
     if (!IsServer(client_p))
     {
-      sendto_one(client_p, form_str(ERR_NEEDMOREPARAMS),
-                   me.name, EmptyString(hpara[0]) ? "*" : hpara[0], mptr->cmd);
+      sendto_one(client_p, form_str(ERR_NEEDMOREPARAMS), me.name,
+                 EmptyString(hpara[0]) ? "*" : hpara[0], mptr->cmd);
     }
     else
     {
       sendto_realops_flags(UMODE_ALL, L_ALL, 
                            "Dropping server %s due to (invalid) command '%s' "
                            "with only %d arguments (expecting %d).",
-                           client_p->name, mptr->cmd, i, mptr->parameters);
-      ilog(L_CRIT, "Insufficient parameters (%d) for command '%s' from %s.",
+                           client_p->name, mptr->cmd, i, mptr->args_min);
+      ilog(LOG_TYPE_IRCD, "Insufficient parameters (%d) for command '%s' from %s.",
            i, mptr->cmd, client_p->name);
       exit_client(client_p, client_p,
                   "Not enough arguments to server command.");
@@ -400,19 +336,6 @@ handle_command(struct Message *mptr, struct Client *client_p,
     (*handler)(client_p, from, i, hpara);
 }
 
-/* clear_tree_parse()
- *
- * inputs       - NONE
- * output       - NONE
- * side effects - MUST MUST be called at startup ONCE before
- *                any other keyword routine is used.
- */
-void
-clear_tree_parse(void)
-{
-  memset(&msg_tree, 0, sizeof(msg_tree));
-}
-
 /* add_msg_element()
  *
  * inputs	- pointer to MessageTree
@@ -434,15 +357,15 @@ clear_tree_parse(void)
  * in the parent.
  */
 static void
-add_msg_element(struct MessageTree *mtree_p, 
-		struct Message *msg_p, const char *cmd)
+add_msg_element(struct MessageTree *mtree_p, struct Message *msg_p,
+                const char *cmd)
 {
   struct MessageTree *ntree_p;
 
   if (*cmd == '\0')
   {
     mtree_p->msg = msg_p;
-    mtree_p->links++;		/* Have msg pointer, so up ref count */
+    mtree_p->links++;  /* Have msg pointer, so up ref count */
   }
   else
   {
@@ -452,14 +375,15 @@ add_msg_element(struct MessageTree *mtree_p,
      * Thus 'A' -> 0x1 'B' -> 0x2 'c' -> 0x3 etc.
      */
 
-    if ((ntree_p = mtree_p->pointers[*cmd & (MAXPTRLEN-1)]) == NULL)
+    if ((ntree_p = mtree_p->pointers[*cmd & (MAXPTRLEN - 1)]) == NULL)
     {
-      ntree_p = (struct MessageTree *)MyMalloc(sizeof(struct MessageTree));
-      mtree_p->pointers[*cmd & (MAXPTRLEN-1)] = ntree_p;
+      ntree_p = MyMalloc(sizeof(struct MessageTree));
+      mtree_p->pointers[*cmd & (MAXPTRLEN - 1)] = ntree_p;
 
-      mtree_p->links++;		/* Have new pointer, so up ref count */
+      mtree_p->links++;  /* Have new pointer, so up ref count */
     }
-    add_msg_element(ntree_p, msg_p, cmd+1);
+
+    add_msg_element(ntree_p, msg_p, cmd + 1);
   }
 }
 
@@ -491,11 +415,11 @@ del_msg_element(struct MessageTree *mtree_p, const char *cmd)
 {
   struct MessageTree *ntree_p;
 
-  /* In case this is called for a nonexistent command
+  /*
+   * In case this is called for a nonexistent command
    * check that there is a msg pointer here, else links-- goes -ve
    * -db
    */
-
   if ((*cmd == '\0') && (mtree_p->msg != NULL))
   {
     mtree_p->msg = NULL;
@@ -503,14 +427,15 @@ del_msg_element(struct MessageTree *mtree_p, const char *cmd)
   }
   else
   {
-    if ((ntree_p = mtree_p->pointers[*cmd & (MAXPTRLEN-1)]) != NULL)
+    if ((ntree_p = mtree_p->pointers[*cmd & (MAXPTRLEN - 1)]) != NULL)
     {
-      del_msg_element(ntree_p, cmd+1);
+      del_msg_element(ntree_p, cmd + 1);
+
       if (ntree_p->links == 0)
       {
-	mtree_p->pointers[*cmd & (MAXPTRLEN-1)] = NULL;
-	mtree_p->links--;
-	MyFree(ntree_p);
+        mtree_p->pointers[*cmd & (MAXPTRLEN - 1)] = NULL;
+        mtree_p->links--;
+        MyFree(ntree_p);
       }
     }
   }
@@ -524,9 +449,9 @@ del_msg_element(struct MessageTree *mtree_p, const char *cmd)
  * side effects	- none
  */
 static struct Message *
-msg_tree_parse(const char *cmd, struct MessageTree *root)
+msg_tree_parse(const char *cmd)
 {
-  struct MessageTree *mtree = root;
+  struct MessageTree *mtree = &msg_tree;
   assert(cmd && *cmd);
 
   while (IsAlpha(*cmd) && (mtree = mtree->pointers[*cmd & (MAXPTRLEN - 1)]))
@@ -547,16 +472,10 @@ msg_tree_parse(const char *cmd, struct MessageTree *root)
 void
 mod_add_cmd(struct Message *msg)
 {
-  struct Message *found_msg;
-
-  if (msg == NULL)
-    return;
-
-  /* someone loaded a module with a bad messagetab */
-  assert(msg->cmd != NULL);
+  assert(msg && msg->cmd);
 
   /* command already added? */
-  if ((found_msg = msg_tree_parse(msg->cmd, &msg_tree)) != NULL)
+  if (msg_tree_parse(msg->cmd))
     return;
 
   add_msg_element(&msg_tree, msg, msg->cmd);
@@ -572,10 +491,7 @@ mod_add_cmd(struct Message *msg)
 void
 mod_del_cmd(struct Message *msg)
 {
-  assert(msg != NULL);
-
-  if (msg == NULL)
-    return;
+  assert(msg && msg->cmd);
 
   del_msg_element(&msg_tree, msg->cmd);
 }
@@ -589,7 +505,23 @@ mod_del_cmd(struct Message *msg)
 struct Message *
 find_command(const char *cmd)
 {
-  return(msg_tree_parse(cmd, &msg_tree));
+  return msg_tree_parse(cmd);
+}
+
+static void
+recurse_report_messages(struct Client *source_p, const struct MessageTree *mtree)
+{
+  unsigned int i;
+
+  if (mtree->msg != NULL)
+    sendto_one(source_p, form_str(RPL_STATSCOMMANDS),
+               me.name, source_p->name, mtree->msg->cmd,
+               mtree->msg->count, mtree->msg->bytes,
+               mtree->msg->rcount);
+
+  for (i = 0; i < MAXPTRLEN; ++i)
+    if (mtree->pointers[i] != NULL)
+      recurse_report_messages(source_p, mtree->pointers[i]);
 }
 
 /* report_messages()
@@ -601,34 +533,12 @@ find_command(const char *cmd)
 void
 report_messages(struct Client *source_p)
 {
-  struct MessageTree *mtree = &msg_tree;
-  int i;
+  const struct MessageTree *mtree = &msg_tree;
+  unsigned int i;
 
-  for (i = 0; i < MAXPTRLEN; i++)
-  {
+  for (i = 0; i < MAXPTRLEN; ++i)
     if (mtree->pointers[i] != NULL)
       recurse_report_messages(source_p, mtree->pointers[i]);
-  }
-}
-
-static void
-recurse_report_messages(struct Client *source_p, struct MessageTree *mtree)
-{
-  int i;
-
-  if (mtree->msg != NULL)
-  {
-    sendto_one(source_p, form_str(RPL_STATSCOMMANDS),
-               me.name, source_p->name, mtree->msg->cmd,
-               mtree->msg->count, mtree->msg->bytes,
-               mtree->msg->rcount);
-  }
-
-  for (i = 0; i < MAXPTRLEN; i++)
-  {
-    if (mtree->pointers[i] != NULL)
-      recurse_report_messages(source_p, mtree->pointers[i]);
-  }
 }
 
 /* cancel_clients()
@@ -671,7 +581,7 @@ cancel_clients(struct Client *client_p, struct Client *source_p, char *cmd)
     sendto_realops_flags(UMODE_DEBUG, L_ALL,
                          "Not dropping server %s (%s) for Fake Direction",
                          client_p->name, source_p->name);
-    return(-1);
+    return -1;
     /* return exit_client(client_p, client_p, &me, "Fake Direction");*/
   }
 
@@ -693,7 +603,7 @@ cancel_clients(struct Client *client_p, struct Client *source_p, char *cmd)
                        source_p->name, source_p->username, source_p->host,
                        source_p->from->name, get_client_name(client_p, MASK_IP));
 
-  return(0);
+  return 0;
 }
 
 /* remove_unknown()
@@ -745,8 +655,8 @@ remove_unknown(struct Client *client_p, char *lsender, char *lbuffer)
  *      a ping pong error message...
  */
 static void
-do_numeric(char numeric[], struct Client *client_p, struct Client *source_p,
-           int parc, char *parv[])
+handle_numeric(char numeric[], struct Client *client_p, struct Client *source_p,
+               int parc, char *parv[])
 {
   struct Client *target_p;
   struct Channel *chptr;
@@ -772,10 +682,10 @@ do_numeric(char numeric[], struct Client *client_p, struct Client *source_p,
     t += tl;
   }
 
-  ircsprintf(t," :%s", parv[parc-1]);
+  ircsprintf(t, " :%s", parv[parc-1]);
 
   if (((target_p = find_person(client_p, parv[1])) != NULL) ||
-      ((target_p = find_server(parv[1])) != NULL))
+      ((target_p = hash_find_server(parv[1])) != NULL))
   {
     if (IsMe(target_p)) 
     {
@@ -800,15 +710,29 @@ do_numeric(char numeric[], struct Client *client_p, struct Client *source_p,
        * unfortunately, it did not work. --Dianora
        */
 
+      /*
+       * Also ignoring ERR_NOTREGISTERED for now. A connecting server may send this
+       * in response to the "NOTICE AUTH ..." message upon registration. Normally
+       * the !IsServer() test in handle_numeric() would take care of this situation,
+       * but due to the fact that read_packet() in s_auth.c:release_auth_client() is
+       * called for the very first time _after_ ident/dns registration, it looks like
+       * the numeric (451) messages are queued up whereas meanwhile the serverlink
+       * succesfully establishes. Right after that, after IsServer() is true, the queued
+       * numeric messages will then be processed, and a notice is falsely sent to operators.
+       * The I/O engine will be rewritten soon (a mixture of libevent), so maybe this also
+       * will fix described behavior above, but for now we're just going to drop this message
+       * (no problem at all, we propably shouldn't process numeric destined to servers anyway)
+       * -Michael
+       */
       /* Yes, a good compiler would have optimised this, but
        * this is probably easier to read. -db
        */
       num = atoi(numeric);
 
-      if ((num != ERR_NOSUCHNICK))
+      if ((num != ERR_NOSUCHNICK) && (num != ERR_NOTREGISTERED))
         sendto_realops_flags(UMODE_ALL, L_ADMIN,
-			     "*** %s(via %s) sent a %s numeric to me: %s",
-			     source_p->name, client_p->name, numeric, buffer);
+                             "*** %s(via %s) sent a %s numeric to me: %s",
+                             source_p->name, client_p->name, numeric, buffer);
       return;
     }
     else if (target_p->from == client_p) 
@@ -825,19 +749,17 @@ do_numeric(char numeric[], struct Client *client_p, struct Client *source_p,
     
     /* Fake it for server hiding, if its our client */
     if (ConfigServerHide.hide_servers &&
-        MyClient(target_p) && !IsOper(target_p))
+        MyClient(target_p) && !HasUMode(target_p, UMODE_OPER))
       sendto_one(target_p, ":%s %s %s%s", me.name, numeric, target_p->name, buffer);
-    else if (!MyClient(target_p) && IsCapable(target_p->from, CAP_TS6) && HasID(source_p))
-      sendto_one(target_p, ":%s %s %s%s", source_p->id, numeric, target_p->id, buffer);
-    else /* either it is our client, or a client linked throuh a non-ts6 server. must use names! */
-      sendto_one(target_p, ":%s %s %s%s", source_p->name, numeric, target_p->name, buffer);
+    else
+      sendto_one(target_p, ":%s %s %s%s", ID_or_name(source_p, target_p->from),
+                 numeric, ID_or_name(target_p, target_p->from), buffer);
     return;
   }
   else if ((chptr = hash_find_channel(parv[1])) != NULL)
-    sendto_channel_local(ALL_MEMBERS, NO, chptr,
-			 ":%s %s %s %s",
-			 source_p->name,
-			 numeric, chptr->chname, buffer);
+    sendto_channel_local(ALL_MEMBERS, 0, chptr, ":%s %s %s %s",
+                         source_p->name,
+                         numeric, chptr->chname, buffer);
 }
 
 /* m_not_oper()
@@ -850,32 +772,23 @@ m_not_oper(struct Client *client_p, struct Client *source_p,
            int parc, char *parv[])
 {
   sendto_one(source_p, form_str(ERR_NOPRIVILEGES),
-             me.name, parv[0]);
+             me.name, source_p->name);
 }
 
 void
 m_unregistered(struct Client *client_p, struct Client *source_p,
                int parc, char *parv[])
 {
-  /* bit of a hack.
-   * I don't =really= want to waste a bit in a flag
-   * number_of_nick_changes is only really valid after the client
-   * is fully registered..
-   */
-  if (client_p->localClient->number_of_nick_changes == 0)
-  {
-    sendto_one(client_p, ":%s %d * %s :Register first.",
-               me.name, ERR_NOTREGISTERED, parv[0]);
-    client_p->localClient->number_of_nick_changes++;
-  }
+  sendto_one(source_p, form_str(ERR_NOTREGISTERED), me.name,
+             source_p->name[0] ? source_p->name : "*");
 }
 
 void
 m_registered(struct Client *client_p, struct Client *source_p,
              int parc, char *parv[])
 {
-  sendto_one(client_p, form_str(ERR_ALREADYREGISTRED),   
-             me.name, parv[0]); 
+  sendto_one(source_p, form_str(ERR_ALREADYREGISTRED),   
+             me.name, source_p->name);
 }
 
 void
@@ -885,3 +798,32 @@ m_ignore(struct Client *client_p, struct Client *source_p,
   return;
 }
 
+void
+rfc1459_command_send_error(struct Client *client_p, struct Client *source_p,
+                           int parc, char *parv[])
+{
+  const char *in_para;
+
+  in_para = (parc > 1 && *parv[1] != '\0') ? parv[1] : "<>";
+
+  ilog(LOG_TYPE_IRCD, "Received ERROR message from %s: %s",
+       source_p->name, in_para);
+
+  if (client_p == source_p)
+  {
+    sendto_realops_flags(UMODE_ALL, L_ADMIN, "ERROR :from %s -- %s",
+                         get_client_name(client_p, HIDE_IP), in_para);
+    sendto_realops_flags(UMODE_ALL, L_OPER,  "ERROR :from %s -- %s",
+                         get_client_name(client_p, MASK_IP), in_para);
+  }
+  else
+  {
+    sendto_realops_flags(UMODE_ALL, L_OPER, "ERROR :from %s via %s -- %s",
+                         source_p->name, get_client_name(client_p, MASK_IP), in_para);
+    sendto_realops_flags(UMODE_ALL, L_ADMIN, "ERROR :from %s via %s -- %s",
+                         source_p->name, get_client_name(client_p, HIDE_IP), in_para);
+  }
+
+  if (MyClient(source_p))
+    exit_client(source_p, source_p, "ERROR");
+}
