@@ -19,18 +19,17 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: restart.c 588 2006-05-08 13:22:59Z michael $
+ *  $Id: restart.c 1341 2012-04-07 18:33:13Z michael $
  */
 
 #include "stdinc.h"
-#include "tools.h"
+#include "list.h"
 #include "restart.h"
-#include "common.h"
 #include "fdlist.h"
 #include "ircd.h"
 #include "irc_string.h"
 #include "send.h"
-#include "s_log.h"
+#include "log.h"
 #include "client.h" /* for UMODE_ALL */
 #include "memory.h"
 
@@ -39,11 +38,10 @@ restart(const char *mesg)
 {
   static int was_here = 0; /* redundant due to restarting flag below */
 
-  if (was_here)
+  if (was_here++)
     abort();
-  was_here = 1;
 
-  server_die(mesg, YES);
+  server_die(mesg, 1);
 }
 
 void
@@ -79,7 +77,7 @@ server_die(const char *mesg, int rboot)
     sendto_one(target_p, ":%s ERROR :%s", me.name, buffer);
   }
 
-  ilog(L_NOTICE, buffer);
+  ilog(LOG_TYPE_IRCD, "%s", buffer);
 
   send_queued_all();
   close_fds(NULL);
