@@ -19,13 +19,11 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: resv.c 33 2005-10-02 20:50:00Z knight $
+ *  $Id: resv.c 1685 2012-12-19 20:24:52Z michael $
  */
 
 #include "stdinc.h"
-#include "tools.h"
-#include "common.h"
-#include "fdlist.h"
+#include "list.h"
 #include "ircd.h"
 #include "send.h"
 #include "client.h"   
@@ -35,7 +33,7 @@
 #include "hash.h"
 #include "irc_string.h"
 #include "ircd_defs.h"
-#include "s_conf.h"
+#include "conf.h"
 
 dlink_list resv_channel_list = { NULL, NULL, 0 };
 
@@ -51,8 +49,8 @@ dlink_list resv_channel_list = { NULL, NULL, 0 };
 struct ConfItem *
 create_channel_resv(char *name, char *reason, int in_conf)
 {
-  struct ConfItem *conf;
-  struct ResvChannel *resv_p;
+  struct ConfItem *conf = NULL;
+  struct ResvChannel *resv_p = NULL;
 
   if (name == NULL || reason == NULL)
     return NULL;
@@ -67,6 +65,7 @@ create_channel_resv(char *name, char *reason, int in_conf)
   resv_p = map_to_conf(conf);
 
   strlcpy(resv_p->name, name, sizeof(resv_p->name));
+  DupString(conf->name, name);
   DupString(resv_p->reason, reason);
   resv_p->conf = in_conf;
 
@@ -87,8 +86,8 @@ create_channel_resv(char *name, char *reason, int in_conf)
 struct ConfItem *
 create_nick_resv(char *name, char *reason, int in_conf)
 {
-  struct ConfItem *conf;
-  struct MatchItem *resv_p;
+  struct ConfItem *conf = NULL;
+  struct MatchItem *resv_p = NULL;
 
   if (name == NULL || reason == NULL)
     return NULL;
@@ -118,15 +117,10 @@ create_nick_resv(char *name, char *reason, int in_conf)
 void
 clear_conf_resv(void)
 {
-  dlink_node *ptr;
-  dlink_node *next_ptr;
-  struct ResvChannel *resv_cp;
+  dlink_node *ptr = NULL, *next_ptr = NULL;
 
   DLINK_FOREACH_SAFE(ptr, next_ptr, resv_channel_list.head)
-  {
-    resv_cp = ptr->data;
-    delete_channel_resv(resv_cp);
-  }
+    delete_channel_resv(ptr->data);
 }
 
 /* delete_channel_resv()
@@ -138,7 +132,7 @@ clear_conf_resv(void)
 int
 delete_channel_resv(struct ResvChannel *resv_p)
 {
-  struct ConfItem *conf;
+  struct ConfItem *conf = NULL;
   assert(resv_p != NULL);
 
   hash_del_resv(resv_p);
@@ -195,7 +189,7 @@ report_resv(struct Client *source_p)
     resv_cp = ptr->data;
     sendto_one(source_p, form_str(RPL_STATSQLINE),
                me.name, source_p->name,
-	       resv_cp->conf ? 'Q' : 'q',
+	       resv_cp->conf ? 'Q' : 'q', resv_cp->count,
 	       resv_cp->name, resv_cp->reason);
   }
 
@@ -206,7 +200,7 @@ report_resv(struct Client *source_p)
 
     sendto_one(source_p, form_str(RPL_STATSQLINE),
                me.name, source_p->name,
-	       resv_np->action ? 'Q' : 'q',
+	       resv_np->action ? 'Q' : 'q', resv_np->count,
 	       conf->name, resv_np->reason);
   }
 }
