@@ -19,37 +19,30 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: s_bsd.c 683 2006-06-15 14:28:59Z michael $
+ *  $Id: s_bsd.c 1871 2013-04-26 08:53:04Z michael $
  */
 
 #include "stdinc.h"
-#ifndef _WIN32
 #include <netinet/in_systm.h>
 #include <netinet/ip.h>
 #include <netinet/tcp.h>
-#endif
+#include "list.h"
 #include "fdlist.h"
 #include "s_bsd.h"
 #include "client.h"
-#include "common.h"
 #include "dbuf.h"
 #include "event.h"
 #include "irc_string.h"
-#include "irc_getnameinfo.h"
-#include "irc_getaddrinfo.h"
 #include "ircd.h"
-#include "list.h"
 #include "listener.h"
 #include "numeric.h"
 #include "packet.h"
 #include "irc_res.h"
-#include "inet_misc.h"
 #include "restart.h"
 #include "s_auth.h"
-#include "s_conf.h"
-#include "s_log.h"
+#include "conf.h"
+#include "log.h"
 #include "s_serv.h"
-#include "s_stats.h"
 #include "send.h"
 #include "memory.h"
 #include "s_user.h"
@@ -61,12 +54,11 @@ static const char *comm_err_str[] = { "Comm OK", "Error during bind()",
 
 struct Callback *setup_socket_cb = NULL;
 
-static void comm_connect_callback(fde_t *fd, int status);
+static void comm_connect_callback(fde_t *, int);
 static PF comm_connect_timeout;
-static void comm_connect_dns_callback(void *vptr, struct DNSReply *reply);
+static void comm_connect_dns_callback(void *, const struct irc_ssaddr *, const char *);
 static PF comm_connect_tryconnect;
 
-extern void init_netio(void);
 
 /* check_can_use_v6()
  *  Check if the system can open AF_INET6 sockets
@@ -82,11 +74,7 @@ check_can_use_v6(void)
   else
   {
     ServerInfo.can_use_v6 = 1;
-#ifdef _WIN32
-    closesocket(v6);
-#else
     close(v6);
-#endif
   }
 #else
   ServerInfo.can_use_v6 = 0;
@@ -102,16 +90,12 @@ check_can_use_v6(void)
 int
 get_sockerr(int fd)
 {
-#ifndef _WIN32
   int errtmp = errno;
-#else
-  int errtmp = WSAGetLastError();
-#endif
 #ifdef SO_ERROR
   int err = 0;
   socklen_t len = sizeof(err);
 
-  if (-1 < fd && !getsockopt(fd, SOL_SOCKET, SO_ERROR, (char*) &err, (socklen_t *)&len))
+  if (-1 < fd && !getsockopt(fd, SOL_SOCKET, SO_ERROR, &err, &len))
   {
     if (err)
       errtmp = err;
@@ -146,8 +130,7 @@ report_error(int level, const char* text, const char* who, int error)
   who = (who) ? who : "";
 
   sendto_realops_flags(UMODE_DEBUG, level, text, who, strerror(error));
-  log_oper_action(LOG_IOERR_TYPE, NULL, "%s %s %s\n", who, text, strerror(error));
-  ilog(L_ERROR, text, who, strerror(error));
+  ilog(LOG_TYPE_IRCD, text, who, strerror(error));
 }
 
 /*
@@ -161,16 +144,14 @@ setup_socket(va_list args)
   int fd = va_arg(args, int);
   int opt = 1;
 
-  setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, (char *) &opt, sizeof(opt));
+  setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, &opt, sizeof(opt));
 
 #ifdef IPTOS_LOWDELAY
   opt = IPTOS_LOWDELAY;
-  setsockopt(fd, IPPROTO_IP, IP_TOS, (char *) &opt, sizeof(opt));
+  setsockopt(fd, IPPROTO_IP, IP_TOS, &opt, sizeof(opt));
 #endif
 
-#ifndef _WIN32
   fcntl(fd, F_SETFL, fcntl(fd, F_GETFL, 0) | O_NONBLOCK);
-#endif
 
   return NULL;
 }
@@ -195,11 +176,11 @@ init_comm(void)
 void
 close_connection(struct Client *client_p)
 {
-  struct ConfItem *conf;
   struct AccessItem *aconf;
   struct ClassItem *aclass;
+  dlink_node *ptr = NULL;
 
-  assert(NULL != client_p);
+  assert(client_p);
 
   if (!IsDead(client_p))
   {
@@ -212,49 +193,38 @@ close_connection(struct Client *client_p)
     send_queued_write(client_p);
   }
 
-  if (IsServer(client_p))
+  if (IsClient(client_p))
   {
-    ServerStats->is_sv++;
-    ServerStats->is_sbs += client_p->localClient->send.bytes;
-    ServerStats->is_sbr += client_p->localClient->recv.bytes;
-    ServerStats->is_sti += CurrentTime - client_p->firsttime;
+    ++ServerStats.is_cl;
+    ServerStats.is_cbs += client_p->localClient->send.bytes;
+    ServerStats.is_cbr += client_p->localClient->recv.bytes;
+    ServerStats.is_cti += CurrentTime - client_p->localClient->firsttime;
+  }
+  else if (IsServer(client_p))
+  {
+    ++ServerStats.is_sv;
+    ServerStats.is_sbs += client_p->localClient->send.bytes;
+    ServerStats.is_sbr += client_p->localClient->recv.bytes;
+    ServerStats.is_sti += CurrentTime - client_p->localClient->firsttime;
 
-    /* XXX Does this even make any sense at all anymore?
-     * scheduling a 'quick' reconnect could cause a pile of
-     * nick collides under TSora protocol... -db
-     */
-    /*
-     * If the connection has been up for a long amount of time, schedule
-     * a 'quick' reconnect, else reset the next-connect cycle.
-     */
-    if ((conf = find_conf_exact(SERVER_TYPE,
-				  client_p->name, client_p->username,
-				  client_p->host)))
+    DLINK_FOREACH(ptr, server_items.head)
     {
+      struct ConfItem *conf = ptr->data;
+
+      if (irccmp(conf->name, client_p->name))
+        continue;
+
       /*
-       * Reschedule a faster reconnect, if this was a automatically
-       * connected configuration entry. (Note that if we have had
-       * a rehash in between, the status has been changed to
-       * CONF_ILLEGAL). But only do this if it was a "good" link.
+       * Reset next-connect cycle of all connect{} blocks that match
+       * this servername.
        */
-      aconf = (struct AccessItem *)map_to_conf(conf);
-      aclass = (struct ClassItem *)map_to_conf(aconf->class_ptr);
-      aconf->hold = time(NULL);
-      aconf->hold += (aconf->hold - client_p->since > HANGONGOODLINK) ?
-        HANGONRETRYDELAY : ConFreq(aclass);
-      if (nextconnect > aconf->hold)
-        nextconnect = aconf->hold;
+      aconf  = map_to_conf(conf);
+      aclass = map_to_conf(aconf->class_ptr);
+      aconf->hold = CurrentTime + aclass->con_freq;
     }
   }
-  else if (IsClient(client_p))
-  {
-    ServerStats->is_cl++;
-    ServerStats->is_cbs += client_p->localClient->send.bytes;
-    ServerStats->is_cbr += client_p->localClient->recv.bytes;
-    ServerStats->is_cti += CurrentTime - client_p->firsttime;
-  }
   else
-    ServerStats->is_ni++;
+    ++ServerStats.is_ni;
 
 #ifdef HAVE_LIBCRYPTO
   if (client_p->localClient->fd.ssl)
@@ -268,12 +238,6 @@ close_connection(struct Client *client_p)
   if (client_p->localClient->fd.flags.open)
     fd_close(&client_p->localClient->fd);
 
-  if (HasServlink(client_p))
-  {
-    if (client_p->localClient->ctrlfd.flags.open)
-      fd_close(&client_p->localClient->ctrlfd);
-  }
-
   dbuf_clear(&client_p->localClient->buf_sendq);
   dbuf_clear(&client_p->localClient->buf_recvq);
   
@@ -324,50 +288,42 @@ ssl_handshake(int fd, struct Client *client_p)
 void
 add_connection(struct Listener *listener, struct irc_ssaddr *irn, int fd)
 {
-  struct Client *new_client;
-
-  assert(NULL != listener);
-
-  new_client = make_client(NULL);
+  struct Client *new_client = make_client(NULL);
 
   fd_open(&new_client->localClient->fd, fd, 1,
           (listener->flags & LISTENER_SSL) ?
 	  "Incoming SSL connection" : "Incoming connection");
 
-  /* 
+  /*
    * copy address to 'sockhost' as a string, copy it to host too
    * so we have something valid to put into error messages...
    */
   memcpy(&new_client->localClient->ip, irn, sizeof(struct irc_ssaddr));
 
-  irc_getnameinfo((struct sockaddr*)&new_client->localClient->ip,
-        new_client->localClient->ip.ss_len,  new_client->sockhost, 
-        HOSTIPLEN, NULL, 0, NI_NUMERICHOST);
+  getnameinfo((struct sockaddr *)&new_client->localClient->ip,
+              new_client->localClient->ip.ss_len, new_client->sockhost, 
+              sizeof(new_client->sockhost), NULL, 0, NI_NUMERICHOST);
   new_client->localClient->aftype = new_client->localClient->ip.ss.ss_family;
-#ifdef IPV6
-  if (new_client->sockhost[0] == ':')
-    strlcat(new_client->host, "0", HOSTLEN+1);
 
-  if (new_client->localClient->aftype == AF_INET6 && 
-      ConfigFileEntry.dot_in_ip6_addr == 1)
+  if (new_client->sockhost[0] == ':' && new_client->sockhost[1] == ':')
   {
-    strlcat(new_client->host, new_client->sockhost,HOSTLEN+1);
-    strlcat(new_client->host, ".", HOSTLEN+1);
+    strlcpy(new_client->host, "0", sizeof(new_client->host));
+    strlcpy(new_client->host+1, new_client->sockhost, sizeof(new_client->host)-1);
+    memmove(new_client->sockhost+1, new_client->sockhost, sizeof(new_client->sockhost)-1);
+    new_client->sockhost[0] = '0';
   }
   else
-#endif
-    strlcat(new_client->host, new_client->sockhost,HOSTLEN+1);
+    strlcpy(new_client->host, new_client->sockhost, sizeof(new_client->host));
 
-  new_client->connect_id = ++connect_id;
   new_client->localClient->listener = listener;
   ++listener->ref_count;
 
 #ifdef HAVE_LIBCRYPTO
   if (listener->flags & LISTENER_SSL)
   {
-    if ((new_client->localClient->fd.ssl = SSL_new(ServerInfo.ctx)) == NULL)
+    if ((new_client->localClient->fd.ssl = SSL_new(ServerInfo.server_ctx)) == NULL)
     {
-      ilog(L_CRIT, "SSL_new() ERROR! -- %s",
+      ilog(LOG_TYPE_IRCD, "SSL_new() ERROR! -- %s",
            ERR_error_string(ERR_get_error(), NULL));
 
       SetDead(new_client);
@@ -510,7 +466,7 @@ comm_connect_tcp(fde_t *fd, const char *host, unsigned short port,
                  void *data, int aftype, int timeout)
 {
   struct addrinfo hints, *res;
-  char portname[PORTNAMELEN+1];
+  char portname[PORTNAMELEN + 1];
 
   assert(callback);
   fd->connect.callback = callback;
@@ -542,15 +498,15 @@ comm_connect_tcp(fde_t *fd, const char *host, unsigned short port,
   hints.ai_socktype = SOCK_STREAM;
   hints.ai_flags = AI_PASSIVE | AI_NUMERICHOST;
 
-  snprintf(portname, PORTNAMELEN, "%d", port);
+  snprintf(portname, sizeof(portname), "%d", port);
 
-  if (irc_getaddrinfo(host, portname, &hints, &res))
+  if (getaddrinfo(host, portname, &hints, &res))
   {
     /* Send the DNS request, for the next level */
-    fd->dns_query = MyMalloc(sizeof(struct DNSQuery));
-    fd->dns_query->ptr = fd;
-    fd->dns_query->callback = comm_connect_dns_callback;
-    gethost_byname(host, fd->dns_query);
+    if (aftype == AF_INET6)
+      gethost_byname_type(comm_connect_dns_callback, fd, host, T_AAAA);
+    else
+      gethost_byname_type(comm_connect_dns_callback, fd, host, T_A);
   }
   else
   {
@@ -560,7 +516,7 @@ comm_connect_tcp(fde_t *fd, const char *host, unsigned short port,
     memcpy(&fd->connect.hostaddr, res->ai_addr, res->ai_addrlen);
     fd->connect.hostaddr.ss_len = res->ai_addrlen;
     fd->connect.hostaddr.ss.ss_family = res->ai_family;
-    irc_freeaddrinfo(res);
+    freeaddrinfo(res);
     comm_settimeout(fd, timeout*1000, comm_connect_timeout, NULL);
     comm_connect_tryconnect(fd, NULL);
   }
@@ -608,14 +564,12 @@ comm_connect_timeout(fde_t *fd, void *notused)
  * otherwise we initiate the connect()
  */
 static void
-comm_connect_dns_callback(void *vptr, struct DNSReply *reply)
+comm_connect_dns_callback(void *vptr, const struct irc_ssaddr *addr, const char *name)
 {
   fde_t *F = vptr;
 
-  if (reply == NULL)
+  if (name == NULL)
   {
-    MyFree(F->dns_query);
-    F->dns_query = NULL;
     comm_connect_callback(F, COMM_ERR_DNS);
     return;
   }
@@ -629,15 +583,13 @@ comm_connect_dns_callback(void *vptr, struct DNSReply *reply)
    * the DNS record around, and the DNS cache is gone anyway.. 
    *     -- adrian
    */
-  memcpy(&F->connect.hostaddr, &reply->addr, reply->addr.ss_len);
+  memcpy(&F->connect.hostaddr, addr, addr->ss_len);
   /* The cast is hacky, but safe - port offset is same on v4 and v6 */
   ((struct sockaddr_in *) &F->connect.hostaddr)->sin_port =
     F->connect.hostaddr.ss_port;
-  F->connect.hostaddr.ss_len = reply->addr.ss_len;
+  F->connect.hostaddr.ss_len = addr->ss_len;
 
   /* Now, call the tryconnect() routine to try a connect() */
-  MyFree(F->dns_query);
-  F->dns_query = NULL;
   comm_connect_tryconnect(F, NULL);
 }
 
@@ -665,9 +617,6 @@ comm_connect_tryconnect(fde_t *fd, void *notused)
   /* Error? */
   if (retval < 0)
   {
-#ifdef _WIN32
-    errno = WSAGetLastError();
-#endif
     /*
      * If we get EISCONN, then we've already connect()ed the socket,
      * which is a good thing.
@@ -726,12 +675,7 @@ comm_open(fde_t *F, int family, int sock_type, int proto, const char *note)
    */
   fd = socket(family, sock_type, proto);
   if (fd < 0)
-  {
-#ifdef _WIN32
-    errno = WSAGetLastError();
-#endif
     return -1; /* errno will be passed through, yay.. */
-  }
 
   execute_callback(setup_socket_cb, fd);
 
@@ -764,14 +708,9 @@ comm_accept(struct Listener *lptr, struct irc_ssaddr *pn)
    * reserved fd limit, but we can deal with that when comm_open()
    * also does it. XXX -- adrian
    */
-  newfd = accept(lptr->fd.fd, (struct sockaddr *)pn, (socklen_t *)&addrlen);
+  newfd = accept(lptr->fd.fd, (struct sockaddr *)pn, &addrlen);
   if (newfd < 0)
-  {
-#ifdef _WIN32
-    errno = WSAGetLastError();
-#endif
     return -1;
-  }
 
 #ifdef IPV6
   remove_ipv6_mapping(pn);
@@ -787,8 +726,7 @@ comm_accept(struct Listener *lptr, struct irc_ssaddr *pn)
 
 /* 
  * remove_ipv6_mapping() - Removes IPv4-In-IPv6 mapping from an address
- * This function should really inspect the struct itself rather than relying
- * on inet_pton and inet_ntop.  OSes with IPv6 mapping listening on both
+ * OSes with IPv6 mapping listening on both
  * AF_INET and AF_INET6 map AF_INET connections inside AF_INET6 structures
  * 
  */
@@ -798,15 +736,15 @@ remove_ipv6_mapping(struct irc_ssaddr *addr)
 {
   if (addr->ss.ss_family == AF_INET6)
   {
-    struct sockaddr_in6 *v6;
-
-    v6 = (struct sockaddr_in6*)addr;
-    if (IN6_IS_ADDR_V4MAPPED(&v6->sin6_addr))
+    if (IN6_IS_ADDR_V4MAPPED(&((struct sockaddr_in6 *)addr)->sin6_addr))
     {
-      char v4ip[HOSTIPLEN];
-      struct sockaddr_in *v4 = (struct sockaddr_in*)addr;
-      inetntop(AF_INET6, &v6->sin6_addr, v4ip, HOSTIPLEN);
-      inet_pton(AF_INET, v4ip, &v4->sin_addr);
+      struct sockaddr_in6 v6; 
+      struct sockaddr_in *v4 = (struct sockaddr_in *)addr;
+
+      memcpy(&v6, addr, sizeof(v6));
+      memset(v4, 0, sizeof(struct sockaddr_in));
+      memcpy(&v4->sin_addr, &v6.sin6_addr.s6_addr[12], sizeof(v4->sin_addr));
+
       addr->ss.ss_family = AF_INET;
       addr->ss_len = sizeof(struct sockaddr_in);
     }
