@@ -20,15 +20,17 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: s_bsd_select.c 33 2005-10-02 20:50:00Z knight $
+ *  $Id: s_bsd_select.c 1309 2012-03-25 11:24:18Z michael $
  */
 
 #include "stdinc.h"
+#if USE_IOPOLL_MECHANISM == __IOPOLL_MECHANISM_SELECT
+#include "list.h"
 #include "fdlist.h"
 #include "hook.h"
 #include "ircd.h"
 #include "s_bsd.h"
-#include "s_log.h"
+#include "log.h"
 
 /*
  * Note that this is only a single list - multiple lists is kinda pointless
@@ -199,3 +201,4 @@ comm_select(void)
       comm_setselect(F, 0, NULL, NULL, 0);
     }
 }
+#endif
