@@ -19,33 +19,27 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: s_gline.c 33 2005-10-02 20:50:00Z knight $
+ *  $Id: s_gline.c 1871 2013-04-26 08:53:04Z michael $
  */
 
 #include "stdinc.h"
-#include "tools.h"
-#include "handlers.h"
+#include "list.h"
 #include "client.h"
-#include "common.h"
 #include "irc_string.h"
 #include "ircd.h"
 #include "hostmask.h"
-#include "fdlist.h"
-#include "s_bsd.h"
-#include "s_conf.h"
+#include "conf.h"
 #include "s_misc.h"
 #include "send.h"
-#include "msg.h"
-#include "fileio.h"
 #include "s_serv.h"
 #include "s_gline.h"
 #include "event.h"
-#include "list.h"
 #include "memory.h"
 
-dlink_list pending_glines = { NULL, NULL, 0 };
+dlink_list pending_glines[GLINE_PENDING_ADD_TYPE + 1] = { { NULL, NULL, 0 },
+                                                          { NULL, NULL, 0 } };
 
-static void expire_pending_glines(void);
+static void expire_pending_glines(struct gline_pending *);
 
 
 struct AccessItem *
@@ -71,8 +65,8 @@ find_is_glined(const char *host, const char *user)
     piphost = NULL;
   }
 
-  aconf = find_conf_by_address(host, piphost, CONF_GLINE, t, user, NULL);
-  return(aconf);
+  aconf = find_conf_by_address(host, piphost, CONF_GLINE, t, user, NULL, 0);
+  return aconf;
 }
 
 /* cleanup_glines()
@@ -85,7 +79,7 @@ find_is_glined(const char *host, const char *user)
 void
 cleanup_glines(void *unused)
 {
-  expire_pending_glines();
+  expire_pending_glines(unused);
 }
 
 /* expire_pending_glines()
@@ -98,21 +92,23 @@ cleanup_glines(void *unused)
  * enough "votes" in the time period allowed
  */
 static void
-expire_pending_glines(void)
+expire_pending_glines(struct gline_pending *in)
 {
-  dlink_node *ptr;
-  dlink_node *next_ptr;
-  struct gline_pending *glp_ptr;
+  dlink_node *ptr = NULL, *next_ptr = NULL;
+  unsigned int idx = 0;
 
-  DLINK_FOREACH_SAFE(ptr, next_ptr, pending_glines.head)
+  for (; idx < GLINE_PENDING_ADD_TYPE + 1; ++idx)
   {
-    glp_ptr = ptr->data;
-
-    if (((glp_ptr->last_gline_time + GLINE_PENDING_EXPIRE) <= CurrentTime) ||
-        find_is_glined(glp_ptr->host, glp_ptr->user))
+    DLINK_FOREACH_SAFE(ptr, next_ptr, pending_glines[idx].head)
     {
-      dlinkDelete(&glp_ptr->node, &pending_glines);
-      MyFree(glp_ptr);
+      struct gline_pending *glp_ptr = ptr->data;
+
+      if ((glp_ptr->last_gline_time + ConfigFileEntry.gline_request_time) <= CurrentTime ||
+          glp_ptr == in)
+      {
+        dlinkDelete(&glp_ptr->node, &pending_glines[idx]);
+        MyFree(glp_ptr);
+      }
     }
   }
 }
