@@ -19,13 +19,12 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: s_misc.c 33 2005-10-02 20:50:00Z knight $
+ *  $Id: s_misc.c 1309 2012-03-25 11:24:18Z michael $
  */
 
 #include "stdinc.h"
 #include "s_misc.h"
 #include "client.h"
-#include "common.h"
 #include "irc_string.h"
 #include "sprintf_irc.h"
 #include "ircd.h"
@@ -33,7 +32,7 @@
 #include "irc_res.h"
 #include "fdlist.h"
 #include "s_bsd.h"
-#include "s_conf.h"
+#include "conf.h"
 #include "s_serv.h"
 #include "send.h"
 #include "memory.h"
@@ -86,10 +85,10 @@ date(time_t lclock)
   if (minswest < 0)
     minswest = -minswest;
 
-  ircsprintf(buf, "%s %s %d %d -- %02u:%02u:%02u %c%02u:%02u",
-             weekdays[lt->tm_wday], months[lt->tm_mon],lt->tm_mday,
-             lt->tm_year + 1900, lt->tm_hour, lt->tm_min, lt->tm_sec,
-             plus, minswest/60, minswest%60);
+  snprintf(buf, sizeof(buf), "%s %s %d %d -- %02u:%02u:%02u %c%02u:%02u",
+           weekdays[lt->tm_wday], months[lt->tm_mon],lt->tm_mday,
+           lt->tm_year + 1900, lt->tm_hour, lt->tm_min, lt->tm_sec,
+           plus, minswest/60, minswest%60);
   return buf;
 }
 
@@ -108,62 +107,24 @@ smalldate(time_t lclock)
   gm = &gmbuf; 
   lt = localtime(&lclock);
   
-  ircsprintf(buf, "%d/%d/%d %02d.%02d",
-             lt->tm_year + 1900, lt->tm_mon + 1, lt->tm_mday,
-             lt->tm_hour, lt->tm_min);
+  snprintf(buf, sizeof(buf), "%d/%d/%d %02d.%02d",
+           lt->tm_year + 1900, lt->tm_mon + 1, lt->tm_mday,
+           lt->tm_hour, lt->tm_min);
 
   return buf;
 }
 
-/* small_file_date()
- * Make a small YYYYMMDD formatted string suitable for a
- * dated file stamp. 
- */
-char *
-small_file_date(time_t lclock)
-{
-  static char timebuffer[MAX_DATE_STRING];
-  struct tm *tmptr;
-
-  if (!lclock)
-    time(&lclock);
-
-  tmptr = localtime(&lclock);
-  strftime(timebuffer, MAX_DATE_STRING, "%Y%m%d", tmptr);
-
-  return timebuffer;
-}
-
 #ifdef HAVE_LIBCRYPTO
 char *
-ssl_get_cipher(SSL *ssl)
+ssl_get_cipher(const SSL *ssl)
 {
-  static char buffer[128];
-  const char *name = NULL;
-  int bits;
-
-  switch (ssl->session->ssl_version)
-  {
-    case SSL2_VERSION:
-      name = "SSLv2";
-      break;
-
-    case SSL3_VERSION:
-      name = "SSLv3";
-      break;
-
-    case TLS1_VERSION:
-      name = "TLSv1";
-      break;
-
-    default:
-      name = "UNKNOWN";
-  }
+  static char buffer[IRCD_BUFSIZE / 4];
+  int bits = 0;
 
   SSL_CIPHER_get_bits(SSL_get_current_cipher(ssl), &bits);
 
-  snprintf(buffer, sizeof(buffer), "%s %s-%d",
-           name, SSL_get_cipher(ssl), bits);
+  snprintf(buffer, sizeof(buffer), "%s %s-%d", SSL_get_version(ssl),
+           SSL_get_cipher(ssl), bits);
   
   return buffer;
 }
