@@ -19,56 +19,51 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: s_user.c 716 2006-07-15 10:22:58Z adx $
+ *  $Id: s_user.c 1901 2013-04-27 19:17:36Z michael $
  */
 
 #include "stdinc.h"
-#include "tools.h"
+#include "list.h"
 #include "s_user.h"
 #include "s_misc.h"
 #include "channel.h"
 #include "channel_mode.h"
 #include "client.h"
-#include "common.h"
 #include "fdlist.h"
 #include "hash.h"
 #include "irc_string.h"
 #include "sprintf_irc.h"
 #include "s_bsd.h"
-#include "irc_getnameinfo.h"
 #include "ircd.h"
-#include "list.h"
 #include "listener.h"
 #include "motd.h"
 #include "numeric.h"
-#include "s_conf.h"
-#include "s_log.h"
+#include "conf.h"
+#include "log.h"
 #include "s_serv.h"
-#include "s_stats.h"
 #include "send.h"
 #include "supported.h"
 #include "whowas.h"
 #include "memory.h"
 #include "packet.h"
+#include "rng_mt.h"
 #include "userhost.h"
 #include "hook.h"
 #include "s_misc.h"
-#include "msg.h"
-#include "pcre.h"
+#include "parse.h"
+#include "watch.h"
+
 
-int MaxClientCount     = 1;
-int MaxConnectionCount = 1;
 struct Callback *entering_umode_cb = NULL;
 struct Callback *umode_cb = NULL;
-struct Callback *uid_get_cb = NULL;
 
 static char umode_buffer[IRCD_BUFSIZE];
 
 static void user_welcome(struct Client *);
 static void report_and_set_user_flags(struct Client *, const struct AccessItem *);
 static int check_xline(struct Client *);
-static void introduce_client(struct Client *, struct Client *);
-static void *uid_get(va_list);
+static void introduce_client(struct Client *);
+static const char *uid_get(void);
 
 /* Used for building up the isupport string,
  * used with init_isupport, add_isupport, delete_isupport
@@ -95,12 +90,12 @@ unsigned int user_modes[256] =
   0,                  /* @ */
   0,                  /* A */
   0,                  /* B */
-  0,                  /* C */
+  UMODE_CCONN_FULL,   /* C */
   UMODE_DEAF,         /* D */
   0,                  /* E */
   0,                  /* F */
   UMODE_SOFTCALLERID, /* G */
-  0,                  /* H */
+  UMODE_HIDDEN,       /* H */
   0,                  /* I */
   0,                  /* J */
   0,                  /* K */
@@ -110,7 +105,7 @@ unsigned int user_modes[256] =
   0,                  /* O */
   0,                  /* P */
   0,                  /* Q */
-  0,                  /* R */
+  UMODE_REGONLY,      /* R */
   0,                  /* S */
   0,                  /* T */
   0,                  /* U */
@@ -125,12 +120,12 @@ unsigned int user_modes[256] =
   UMODE_BOTS,         /* b */
   UMODE_CCONN,        /* c */
   UMODE_DEBUG,        /* d */
-  0,                  /* e */
+  UMODE_EXTERNAL,     /* e */
   UMODE_FULL,         /* f */
   UMODE_CALLERID,     /* g */
   0,                  /* h */
   UMODE_INVISIBLE,    /* i */
-  0,                  /* j */
+  UMODE_REJ,          /* j */
   UMODE_SKILL,        /* k */
   UMODE_LOCOPS,       /* l */
   0,                  /* m */
@@ -138,13 +133,13 @@ unsigned int user_modes[256] =
   UMODE_OPER,         /* o */
   0,                  /* p */
   0,                  /* q */
-  UMODE_REJ,          /* r */
+  UMODE_REGISTERED,   /* r */
   UMODE_SERVNOTICE,   /* s */
   0,                  /* t */
   UMODE_UNAUTH,       /* u */
   0,                  /* v */
   UMODE_WALLOP,       /* w */
-  UMODE_EXTERNAL,     /* x */
+  0,                  /* x */
   UMODE_SPY,          /* y */
   UMODE_OPERWALL,     /* z      0x7A */
   0,0,0,0,0,          /* 0x7B - 0x7F */
@@ -165,7 +160,7 @@ assemble_umode_buffer(void)
   unsigned int idx = 0;
   char *umode_buffer_ptr = umode_buffer;
 
-  for (; idx < (sizeof(user_modes) / sizeof(int)); ++idx)
+  for (; idx < (sizeof(user_modes) / sizeof(user_modes[0])); ++idx)
     if (user_modes[idx])
       *umode_buffer_ptr++ = idx;
 
@@ -194,7 +189,7 @@ show_lusers(struct Client *source_p)
     to = source_p->name;
   }
 
-  if (!ConfigServerHide.hide_servers || IsOper(source_p))
+  if (!ConfigServerHide.hide_servers || HasUMode(source_p, UMODE_OPER))
     sendto_one(source_p, form_str(RPL_LUSERCLIENT),
                from, to, (Count.total-Count.invisi),
                Count.invisi, dlink_list_length(&global_serv_list));
@@ -214,36 +209,33 @@ show_lusers(struct Client *source_p)
     sendto_one(source_p, form_str(RPL_LUSERCHANNELS),
                from, to, dlink_list_length(&global_channel_list));
 
-  if (!ConfigServerHide.hide_servers || IsOper(source_p))
+  if (!ConfigServerHide.hide_servers || HasUMode(source_p, UMODE_OPER))
   {
     sendto_one(source_p, form_str(RPL_LUSERME),
                from, to, Count.local, Count.myserver);
     sendto_one(source_p, form_str(RPL_LOCALUSERS),
-               from, to, Count.local, Count.max_loc,
-               Count.local, Count.max_loc);
+               from, to, Count.local, Count.max_loc);
   }
   else
   {
     sendto_one(source_p, form_str(RPL_LUSERME),
                from, to, Count.total, 0);
     sendto_one(source_p, form_str(RPL_LOCALUSERS), 
-               from, to, Count.total, Count.max_tot,
-               Count.total, Count.max_tot);
+               from, to, Count.total, Count.max_tot);
   }
 
   sendto_one(source_p, form_str(RPL_GLOBALUSERS),
-             from, to, Count.total, Count.max_tot,
-             Count.total, Count.max_tot);
+             from, to, Count.total, Count.max_tot);
 
-  if (!ConfigServerHide.hide_servers || IsOper(source_p))
+  if (!ConfigServerHide.hide_servers || HasUMode(source_p, UMODE_OPER))
     sendto_one(source_p, form_str(RPL_STATSCONN), from, to,
-               MaxConnectionCount, MaxClientCount, Count.totalrestartcount);
+               Count.max_loc_con, Count.max_loc_cli, Count.totalrestartcount);
 
-  if (Count.local > MaxClientCount)
-    MaxClientCount = Count.local;
+  if (Count.local > Count.max_loc_cli)
+    Count.max_loc_cli = Count.local;
 
-  if ((Count.local + Count.myserver) > MaxConnectionCount)
-    MaxConnectionCount = Count.local + Count.myserver; 
+  if ((Count.local + Count.myserver) > Count.max_loc_con)
+    Count.max_loc_con = Count.local + Count.myserver; 
 }
 
 /* show_isupport()
@@ -282,28 +274,28 @@ show_isupport(struct Client *source_p)
 **         nick from local user or kill him/her...
 */
 void
-register_local_user(struct Client *client_p, struct Client *source_p, 
-                    const char *nick, const char *username)
+register_local_user(struct Client *source_p)
 {
+  const char *id = NULL;
   const struct AccessItem *aconf = NULL;
-  char ipaddr[HOSTIPLEN];
   dlink_node *ptr = NULL;
-  dlink_node *m = NULL;
 
   assert(source_p != NULL);
+  assert(source_p == source_p->from);
   assert(MyConnect(source_p));
-  assert(source_p->username != username);
   assert(!source_p->localClient->registration);
 
-  ClearCap(client_p, CAP_TS6);
+  ClearCap(source_p, CAP_TS6);
 
   if (ConfigFileEntry.ping_cookie)
   {
-    if (!IsPingSent(source_p) &&
-       source_p->localClient->random_ping == 0)
+    if (!IsPingSent(source_p) && source_p->localClient->random_ping == 0)
     {
-      source_p->localClient->random_ping = (unsigned long)rand();
-      sendto_one(source_p, "PING :%lu",
+      do
+        source_p->localClient->random_ping = genrand_int32();
+      while (!source_p->localClient->random_ping);
+
+      sendto_one(source_p, "PING :%u",
                  source_p->localClient->random_ping);
       SetPingSent(source_p);
       return;
@@ -313,11 +305,11 @@ register_local_user(struct Client *client_p, struct Client *source_p,
       return;
   }
 
-  source_p->localClient->last = CurrentTime;
+  source_p->localClient->last_privmsg = CurrentTime;
   /* Straight up the maximum rate of flooding... */
   source_p->localClient->allow_read = MAX_FLOOD_BURST;
 
-  if (!execute_callback(client_check_cb, source_p, username))
+  if (!execute_callback(client_check_cb, source_p, source_p->username))
     return;
 
   if (valid_hostname(source_p->host) == 0)
@@ -333,29 +325,27 @@ register_local_user(struct Client *client_p, struct Client *source_p,
 
   if (!IsGotId(source_p))
   {
-    const char *p = NULL;
+    char username[USERLEN + 1];
+    const char *p = username;
     unsigned int i = 0;
 
     if (IsNeedIdentd(aconf))
     {
-      ServerStats->is_ref++;
+      ++ServerStats.is_ref;
       sendto_one(source_p, ":%s NOTICE %s :*** Notice -- You need to install "
                  "identd to use this server", me.name, source_p->name);
       exit_client(source_p, &me, "Install identd");
       return;
     }
 
-    p = username;
+    strlcpy(username, source_p->username, sizeof(username));
 
     if (!IsNoTilde(aconf))
       source_p->username[i++] = '~';
 
-    while (*p && i < USERLEN)
-    {
+    for (; *p && i < USERLEN; ++p)
       if (*p != '[')
         source_p->username[i++] = *p;
-      p++;
-    }
 
     source_p->username[i] = '\0';
   }
@@ -367,7 +357,7 @@ register_local_user(struct Client *client_p, struct Client *source_p,
 
     if (!match_conf_password(pass, aconf))
     {
-      ServerStats->is_ref++;
+      ++ServerStats.is_ref;
       sendto_one(source_p, form_str(ERR_PASSWDMISMATCH),
                  me.name, source_p->name);
       exit_client(source_p, &me, "Bad Password");
@@ -383,6 +373,9 @@ register_local_user(struct Client *client_p, struct Client *source_p,
   /* report if user has &^>= etc. and set flags as needed in source_p */
   report_and_set_user_flags(source_p, aconf);
 
+  if (IsDead(source_p))
+    return;
+
   /* Limit clients -
    * We want to be able to have servers and F-line clients
    * connect, so save room for "buffer" connections.
@@ -396,8 +389,8 @@ register_local_user(struct Client *client_p, struct Client *source_p,
   {
     sendto_realops_flags(UMODE_FULL, L_ALL,
                          "Too many clients, rejecting %s[%s].",
-                         nick, source_p->host);
-    ServerStats->is_ref++;
+                         source_p->name, source_p->host);
+    ++ServerStats.is_ref;
     exit_client(source_p, &me, "Sorry, server is full - try later");
     return;
   }
@@ -408,51 +401,48 @@ register_local_user(struct Client *client_p, struct Client *source_p,
     char tmpstr2[IRCD_BUFSIZE];
 
     sendto_realops_flags(UMODE_REJ, L_ALL, "Invalid username: %s (%s@%s)",
-                         nick, source_p->username, source_p->host);
-    ServerStats->is_ref++;
-    ircsprintf(tmpstr2, "Invalid username [%s]", source_p->username);
+                         source_p->name, source_p->username, source_p->host);
+    ++ServerStats.is_ref;
+    snprintf(tmpstr2, sizeof(tmpstr2), "Invalid username [%s]",
+             source_p->username);
     exit_client(source_p, &me, tmpstr2);
     return;
   }
 
-  assert(source_p == client_p);
-
   if (check_xline(source_p))
     return;
 
-  if (IsDead(client_p))
-    return;
+  while (hash_find_id((id = uid_get())) != NULL)
+    ;
 
-  if (me.id[0])
-  {
-    const char *id = execute_callback(uid_get_cb, source_p);
-
-    while (hash_find_id(id) != NULL)
-      id = uid_get(NULL);
-
-    strlcpy(source_p->id, id, sizeof(source_p->id));
-    hash_add_id(source_p);
-  }
-
-  irc_getnameinfo((struct sockaddr *)&source_p->localClient->ip,
-                  source_p->localClient->ip.ss_len, ipaddr,
-                  HOSTIPLEN, NULL, 0, NI_NUMERICHOST);
+  strlcpy(source_p->id, id, sizeof(source_p->id));
+  hash_add_id(source_p);
 
   sendto_realops_flags(UMODE_CCONN, L_ALL,
-                       "Client connecting: %s (%s@%s) [%s] {%s} [%s]",
-                       nick, source_p->username, source_p->host,
+                       "Client connecting: %s (%s@%s) [%s] {%s} [%s] <%s>",
+                       source_p->name, source_p->username, source_p->host,
                        ConfigFileEntry.hide_spoof_ips && IsIPSpoof(source_p) ?
-                       "255.255.255.255" : ipaddr, get_client_class(source_p),
+                       "255.255.255.255" : source_p->sockhost,
+                       get_client_class(source_p),
+                       source_p->info, source_p->id);
+
+  sendto_realops_flags(UMODE_CCONN_FULL, L_ALL,
+                       "CLICONN %s %s %s %s %s %s %s 0 %s",
+                       source_p->name, source_p->username, source_p->host,
+                       ConfigFileEntry.hide_spoof_ips && IsIPSpoof(source_p) ?
+                       "255.255.255.255" : source_p->sockhost,
+		       get_client_class(source_p),
+		       ConfigFileEntry.hide_spoof_ips && IsIPSpoof(source_p) ?
+                           "<hidden>" : source_p->localClient->client_host,
+		       ConfigFileEntry.hide_spoof_ips && IsIPSpoof(source_p) ?
+                           "<hidden>" : source_p->localClient->client_server,
                        source_p->info);
 
-  /* If they have died in send_* don't do anything. */
-  if (IsDead(source_p))
-    return;
 
   if (ConfigFileEntry.invisible_on_connect)
   {
-    source_p->umodes |= UMODE_INVISIBLE;
-    Count.invisi++;
+    AddUMode(source_p, UMODE_INVISIBLE);
+    ++Count.invisi;
   }
 
   if ((++Count.local) > Count.max_loc)
@@ -464,36 +454,32 @@ register_local_user(struct Client *client_p, struct Client *source_p,
                            Count.max_loc);
   }
 
-  SetClient(source_p);
-
-  source_p->servptr = &me;
-  dlinkAdd(source_p, &source_p->lnode, &source_p->servptr->serv->users);
-
   /* Increment our total user count here */
   if (++Count.total > Count.max_tot)
     Count.max_tot = Count.total;
-  Count.totalrestartcount++;
+  ++Count.totalrestartcount;
+
+  assert(source_p->servptr == &me);
+  SetClient(source_p);
+  dlinkAdd(source_p, &source_p->lnode, &source_p->servptr->serv->client_list);
 
   source_p->localClient->allow_read = MAX_FLOOD_BURST;
 
-  if ((m = dlinkFindDelete(&unknown_list, source_p)) != NULL)
-  {
-    free_dlink_node(m);
-    dlinkAdd(source_p, &source_p->localClient->lclient_node, &local_client_list);
-  }
-  else assert(0);
+  assert(dlinkFind(&unknown_list, source_p));
+
+  dlink_move_node(&source_p->localClient->lclient_node,
+                  &unknown_list, &local_client_list);
 
   user_welcome(source_p);
   add_user_host(source_p->username, source_p->host, 0);
   SetUserHost(source_p);
 
-  introduce_client(client_p, source_p);
+  introduce_client(source_p);
 }
 
 /* register_remote_user()
  *
- * inputs       - client_p directly connected client
- *              - source_p remote or directly connected client
+ * inputs       - source_p remote or directly connected client
  *              - username to register as
  *              - host name to register as
  *              - server name
@@ -503,7 +489,7 @@ register_local_user(struct Client *client_p, struct Client *source_p,
  *		  is introduced by a server.
  */
 void
-register_remote_user(struct Client *client_p, struct Client *source_p,
+register_remote_user(struct Client *source_p,
                      const char *username, const char *host, const char *server,
                      const char *realname)
 {
@@ -513,13 +499,12 @@ register_remote_user(struct Client *client_p, struct Client *source_p,
   assert(source_p->username != username);
 
   strlcpy(source_p->host, host, sizeof(source_p->host)); 
-  strlcpy(source_p->info, realname, sizeof(source_p->info));
   strlcpy(source_p->username, username, sizeof(source_p->username));
 
   /*
    * coming from another server, take the servers word for it
    */
-  source_p->servptr = find_server(server);
+  source_p->servptr = hash_find_server(server);
 
   /* Super GhostDetect:
    * If we can't find the server the user is supposed to be on,
@@ -531,9 +516,9 @@ register_remote_user(struct Client *client_p, struct Client *source_p,
                          "No server %s for user %s[%s@%s] from %s",
                          server, source_p->name, source_p->username,
                          source_p->host, source_p->from->name);
-    kill_client(client_p, source_p, "%s (Server doesn't exist)", me.name);
+    kill_client(source_p->from, source_p, "%s (Server doesn't exist)", me.name);
 
-    SetKilled(source_p);
+    AddFlag(source_p, FLAGS_KILLED);
     exit_client(source_p, &me, "Ghosted Client");
     return;
   }
@@ -542,42 +527,46 @@ register_remote_user(struct Client *client_p, struct Client *source_p,
   {
     sendto_realops_flags(UMODE_DEBUG, L_ALL,
                          "Bad User [%s] :%s USER %s@%s %s, != %s[%s]",
-                         client_p->name, source_p->name, source_p->username,
+                         source_p->from->name, source_p->name, source_p->username,
                          source_p->host, source_p->servptr->name,
                          target_p->name, target_p->from->name);
-    kill_client(client_p, source_p,
+    kill_client(source_p->from, source_p,
                 "%s (NICK from wrong direction (%s != %s))",
                 me.name, source_p->servptr->name, target_p->from->name);
-    SetKilled(source_p);
+    AddFlag(source_p, FLAGS_KILLED);
     exit_client(source_p, &me, "USER server wrong direction");
     return;
   }
 
+  /*
+   * If the nick has been introduced by a services server,
+   * make it a service as well.
+   */
+  if (HasFlag(source_p->servptr, FLAGS_SERVICE))
+    AddFlag(source_p, FLAGS_SERVICE);
+
   /* Increment our total user count here */
   if (++Count.total > Count.max_tot)
     Count.max_tot = Count.total;
 
-  ++source_p->from->serv->dep_users;
-
   SetClient(source_p);
-  dlinkAdd(source_p, &source_p->lnode, &source_p->servptr->serv->users);
+  dlinkAdd(source_p, &source_p->lnode, &source_p->servptr->serv->client_list);
   add_user_host(source_p->username, source_p->host, 1);
   SetUserHost(source_p);
 
-  introduce_client(client_p, source_p);
+  introduce_client(source_p);
 }
 
 /* introduce_client()
  *
- * inputs	- client_p
- *              - source_p
+ * inputs	- source_p
  * output	- NONE
  * side effects - This common function introduces a client to the rest
  *		  of the net, either from a local client connect or
  *		  from a remote connect.
  */
 static void
-introduce_client(struct Client *client_p, struct Client *source_p)
+introduce_client(struct Client *source_p)
 {
   dlink_node *server_node = NULL;
   static char ubuf[12];
@@ -587,58 +576,44 @@ introduce_client(struct Client *client_p, struct Client *source_p)
   else
     send_umode(NULL, source_p, 0, SEND_UMODES, ubuf);
 
+  watch_check_hash(source_p, RPL_LOGON);
+
   if (*ubuf == '\0')
   {
     ubuf[0] = '+';
     ubuf[1] = '\0';
   }
 
-  /* arghhh one could try not introducing new nicks to ll leafs
-   * but then you have to introduce them "on the fly" in SJOIN
-   * not fun.
-   * Its not going to cost much more bandwidth to simply let new
-   * nicks just ride on through.
-   */
-
-  /* We now introduce nicks "on the fly" in SJOIN anyway --
-   * you _need_ to if you aren't going to burst everyone initially.
-   *
-   * Only send to non CAP_LL servers, unless we're a lazylink leaf,
-   * in that case just send it to the uplink.
-   * -davidt
-   * rewritten to cope with SIDs .. eww eww eww --is
-   */
-  if (!ServerInfo.hub && uplink && IsCapable(uplink, CAP_LL) &&
-      client_p != uplink)
+  DLINK_FOREACH(server_node, serv_list.head)
   {
-    if (IsCapable(uplink, CAP_TS6) && HasID(source_p))
+    struct Client *server = server_node->data;
+
+    if (server == source_p->from)
+        continue;
+
+    if (IsCapable(server, CAP_SVS))
     {
-      sendto_one(uplink, ":%s UID %s %d %lu %s %s %s %s %s :%s",
-                 source_p->servptr->id,
-                 source_p->name, source_p->hopcount+1,
-                 (unsigned long)source_p->tsinfo,
-                 ubuf, source_p->username, source_p->host,
-                 (MyClient(source_p) && IsIPSpoof(source_p)) ?
-                 "0" : source_p->sockhost, source_p->id, source_p->info);
+      if (IsCapable(server, CAP_TS6) && HasID(source_p))
+        sendto_one(server, ":%s UID %s %d %lu %s %s %s %s %s %s :%s",
+                   source_p->servptr->id,
+                   source_p->name, source_p->hopcount+1,
+                   (unsigned long)source_p->tsinfo,
+                   ubuf, source_p->username, source_p->host,
+                   (MyClient(source_p) && IsIPSpoof(source_p)) ?
+                   "0" : source_p->sockhost, source_p->id,
+                   source_p->svid,
+                   source_p->info);
+      else
+        sendto_one(server, "NICK %s %d %lu %s %s %s %s %s :%s",
+                   source_p->name, source_p->hopcount+1,
+                   (unsigned long)source_p->tsinfo,
+                   ubuf, source_p->username, source_p->host,
+                   source_p->servptr->name, source_p->svid,
+                   source_p->info);
+
     }
     else
     {
-      sendto_one(uplink, "NICK %s %d %lu %s %s %s %s :%s",
-                 source_p->name, source_p->hopcount+1,
-                 (unsigned long)source_p->tsinfo,
-                 ubuf, source_p->username, source_p->host,
-                 source_p->servptr->name, source_p->info);
-    }
-  }
-  else
-  {
-    DLINK_FOREACH(server_node, serv_list.head)
-    {
-      struct Client *server = server_node->data;
-
-      if (IsCapable(server, CAP_LL) || server == client_p)
-        continue;
-
       if (IsCapable(server, CAP_TS6) && HasID(source_p))
         sendto_one(server, ":%s UID %s %d %lu %s %s %s %s %s :%s",
                    source_p->servptr->id,
@@ -673,17 +648,14 @@ valid_hostname(const char *hostname)
 
   assert(p != NULL);
 
-  if ('.' == *p || ':' == *p)
+  if (*p == '.' || *p == ':')
     return 0;
 
-  while (*p)
-  {
+  for (; *p; ++p)
     if (!IsHostChar(*p))
       return 0;
-    p++;
-  }
 
-  return 1;
+  return p - hostname <= HOSTLEN;
 }
 
 /* valid_username()
@@ -705,7 +677,7 @@ valid_username(const char *username)
 
   assert(p != NULL);
 
-  if ('~' == *p)
+  if (*p == '~')
     ++p;
 
   /* reject usernames that don't start with an alphanum
@@ -719,11 +691,9 @@ valid_username(const char *username)
   {
     if ((*p == '.') && ConfigFileEntry.dots_in_ident)
     {
-      dots++;
-
-      if (dots > ConfigFileEntry.dots_in_ident)
+      if (++dots > ConfigFileEntry.dots_in_ident)
         return 0;
-      if (!IsUserChar(p[1]))
+      if (!IsUserChar(*(p + 1)))
         return 0;
     }
     else if (!IsUserChar(*p))
@@ -733,6 +703,31 @@ valid_username(const char *username)
   return 1;
 }
 
+/* clean_nick_name()
+ *
+ * input        - nickname
+ *              - whether it's a local nick (1) or remote (0)
+ * output       - none
+ * side effects - walks through the nickname, returning 0 if erroneous
+ */
+int
+valid_nickname(const char *nickname, const int local)
+{
+  const char *p = nickname;
+  assert(nickname && *nickname);
+
+  /* nicks can't start with a digit or - or be 0 length */
+  /* This closer duplicates behaviour of hybrid-6 */
+  if (*p == '-' || (IsDigit(*p) && local) || *p == '\0')
+    return 0;
+
+  for (; *p; ++p)
+    if (!IsNickChar(*p))
+      return 0;
+
+  return p - nickname <= NICKLEN;
+}
+
 /* report_and_set_user_flags()
  *
  * inputs       - pointer to source_p
@@ -787,15 +782,6 @@ report_and_set_user_flags(struct Client *source_p, const struct AccessItem *acon
                me.name,source_p->name);
   }
 
-  /* If this user is exempt from idle time outs */
-  if (IsConfIdlelined(aconf))
-  {
-    SetIdlelined(source_p);
-    sendto_one(source_p,
-               ":%s NOTICE %s :*** You are exempt from idle limits. congrats.",
-               me.name, source_p->name);
-  }
-
   if (IsConfCanFlood(aconf))
   {
     SetCanFlood(source_p);
@@ -805,52 +791,6 @@ report_and_set_user_flags(struct Client *source_p, const struct AccessItem *acon
   }
 }
 
-/* do_local_user()
- *
- * inputs	-
- * output	- NONE
- * side effects -
- */
-void
-do_local_user(const char *nick, struct Client *client_p, struct Client *source_p,
-              const char *username, const char *host, const char *server,
-              const char *realname)
-{
-  assert(source_p != NULL);
-  assert(source_p->username != username);
-
-  if (source_p == NULL)
-    return;
-
-  if (!IsUnknown(source_p))
-  {
-    sendto_one(source_p, form_str(ERR_ALREADYREGISTRED),
-               me.name, nick);
-    return;
-  }
-
-  source_p->localClient->registration &= ~REG_NEED_USER;
-
-  /*
-   * don't take the clients word for it, ever
-   */
-  source_p->servptr = &me;
-
-  strlcpy(source_p->info, realname, sizeof(source_p->info));
-
-  if (!IsGotId(source_p)) 
-  {
-    /* save the username in the client
-     * If you move this you'll break ping cookies..you've been warned 
-     */
-    strlcpy(source_p->username, username, sizeof(source_p->username));
-  }
-
-  if (!source_p->localClient->registration)
-    /* NICK already received, now I have USER... */
-    register_local_user(client_p, source_p, source_p->name, username);
-}
-
 /* change_simple_umode()
  *
  * this callback can be hooked to allow special handling of
@@ -870,9 +810,9 @@ change_simple_umode(va_list args)
   flag = va_arg(args, unsigned int);
 
   if (what == MODE_ADD)
-    source_p->umodes |= flag;
+    AddUMode(source_p, flag);
   else
-    source_p->umodes &= ~flag;
+    DelUMode(source_p, flag);
 
   return NULL;
 }
@@ -910,7 +850,7 @@ set_user_mode(struct Client *client_p, struct Client *source_p,
      return;
   }
 
-  if (source_p != target_p || target_p->from != source_p->from)
+  if (source_p != target_p)
   {
      sendto_one(source_p, form_str(ERR_USERSDONTMATCH),
                 me.name, source_p->name);
@@ -923,7 +863,7 @@ set_user_mode(struct Client *client_p, struct Client *source_p,
     *m++ = '+';
 
     for (i = 0; i < 128; i++)
-      if (source_p->umodes & user_modes[i])
+      if (HasUMode(source_p, user_modes[i]))
         *m++ = (char)i;
     *m = '\0';
 
@@ -953,7 +893,7 @@ set_user_mode(struct Client *client_p, struct Client *source_p,
         case 'o':
           if (what == MODE_ADD)
           {
-            if (IsServer(client_p) && !IsOper(source_p))
+            if (IsServer(client_p) && !HasUMode(source_p, UMODE_OPER))
             {
               ++Count.oper;
               SetOper(source_p);
@@ -964,11 +904,10 @@ set_user_mode(struct Client *client_p, struct Client *source_p,
             /* Only decrement the oper counts if an oper to begin with
              * found by Pat Szuta, Perly , perly@xnet.com 
              */
-            if (!IsOper(source_p))
+            if (!HasUMode(source_p, UMODE_OPER))
               break;
 
             ClearOper(source_p);
-            source_p->umodes &= ~ConfigFileEntry.oper_only_umodes;
             Count.oper--;
 
             if (MyConnect(source_p))
@@ -976,7 +915,8 @@ set_user_mode(struct Client *client_p, struct Client *source_p,
               dlink_node *dm;
 
               detach_conf(source_p, OPER_TYPE);
-              ClearOperFlags(source_p);
+              ClrOFlag(source_p);
+              DelUMode(source_p, ConfigFileEntry.oper_only_umodes);
 
               if ((dm = dlinkFindDelete(&oper_list, source_p)) != NULL)
                 free_dlink_node(dm);
@@ -988,6 +928,7 @@ set_user_mode(struct Client *client_p, struct Client *source_p,
         /* we may not get these,
          * but they shouldnt be in default
          */
+        case 'r':
         case ' ' :
         case '\n':
         case '\r':
@@ -997,7 +938,7 @@ set_user_mode(struct Client *client_p, struct Client *source_p,
         default:
           if ((flag = user_modes[(unsigned char)*m]))
           {
-            if (MyConnect(source_p) && !IsOper(source_p) &&
+            if (MyConnect(source_p) && !HasUMode(source_p, UMODE_OPER) &&
                 (ConfigFileEntry.oper_only_umodes & flag))
             {
               badflag = 1;
@@ -1020,24 +961,24 @@ set_user_mode(struct Client *client_p, struct Client *source_p,
     sendto_one(source_p, form_str(ERR_UMODEUNKNOWNFLAG),
                me.name, source_p->name);
 
-  if ((source_p->umodes & UMODE_NCHANGE) && !IsOperN(source_p))
+  if (HasUMode(source_p, UMODE_NCHANGE) && !HasOFlag(source_p, OPER_FLAG_N))
+  {
+    sendto_one(source_p, ":%s NOTICE %s :*** You have no nchange flag;",
+               me.name, source_p->name);
+    DelUMode(source_p, UMODE_NCHANGE);
+  }
+
+  if (MyConnect(source_p) && HasUMode(source_p, UMODE_ADMIN) &&
+      !HasOFlag(source_p, OPER_FLAG_ADMIN))
   {
     sendto_one(source_p, ":%s NOTICE %s :*** You have no admin flag;",
                me.name, source_p->name);
-    source_p->umodes &= ~UMODE_NCHANGE; /* only tcm's really need this */
+    DelUMode(source_p, UMODE_ADMIN);
   }
 
-  if (MyConnect(source_p) && (source_p->umodes & UMODE_ADMIN) &&
-      !IsOperAdmin(source_p) && !IsOperHiddenAdmin(source_p))
-  {
-    sendto_one(source_p, ":%s NOTICE %s :*** You have no admin flag;",
-               me.name, source_p->name);
-    source_p->umodes &= ~UMODE_ADMIN;
-  }
-
-  if (!(setflags & UMODE_INVISIBLE) && IsInvisible(source_p))
+  if (!(setflags & UMODE_INVISIBLE) && HasUMode(source_p, UMODE_INVISIBLE))
     ++Count.invisi;
-  if ((setflags & UMODE_INVISIBLE) && !IsInvisible(source_p))
+  if ((setflags & UMODE_INVISIBLE) && !HasUMode(source_p, UMODE_INVISIBLE))
     --Count.invisi;
 
   /*
@@ -1050,15 +991,22 @@ set_user_mode(struct Client *client_p, struct Client *source_p,
 /* send_umode()
  * send the MODE string for user (user) to connection client_p
  * -avalon
+ *
+ * inputs	- client_p
+ *		- source_p
+ *		- int old
+ *		- sendmask mask of modes to send
+ * 		- suplied umode_buf
+ * output	- NONE
  */
 void
 send_umode(struct Client *client_p, struct Client *source_p,
            unsigned int old, unsigned int sendmask, char *umode_buf)
 {
+  char *m = umode_buf;
   int what = 0;
   unsigned int i;
   unsigned int flag;
-  char *m = umode_buf;
 
   /*
    * build a string in umode_buf to represent the change in the user's
@@ -1073,7 +1021,7 @@ send_umode(struct Client *client_p, struct Client *source_p,
     if (MyClient(source_p) && !(flag & sendmask))
       continue;
 
-    if ((flag & old) && !(source_p->umodes & flag))
+    if ((flag & old) && !HasUMode(source_p, flag))
     {
       if (what == MODE_DEL)
         *m++ = (char)i;
@@ -1084,7 +1032,7 @@ send_umode(struct Client *client_p, struct Client *source_p,
         *m++ = (char)i;
       }
     }
-    else if (!(flag & old) && (source_p->umodes & flag))
+    else if (!(flag & old) && HasUMode(source_p, flag))
     {
       if (what == MODE_ADD)
         *m++ = (char)i;
@@ -1118,8 +1066,7 @@ send_umode_out(struct Client *client_p, struct Client *source_p,
   char buf[IRCD_BUFSIZE] = { '\0' };
   dlink_node *ptr = NULL;
 
-  send_umode(NULL, source_p, old, IsOperHiddenAdmin(source_p) ?
-             SEND_UMODES & ~UMODE_ADMIN : SEND_UMODES, buf);
+  send_umode(NULL, source_p, old, SEND_UMODES, buf);
 
   if (buf[0])
   {
@@ -1128,14 +1075,9 @@ send_umode_out(struct Client *client_p, struct Client *source_p,
       struct Client *target_p = ptr->data;
 
       if ((target_p != client_p) && (target_p != source_p))
-      {
-        if ((!(ServerInfo.hub && IsCapable(target_p, CAP_LL))) ||
-            (target_p->localClient->serverMask &
-             source_p->lazyLinkClientExists))
-          sendto_one(target_p, ":%s MODE %s :%s",
-                     ID_or_name(source_p, target_p),
-                     ID_or_name(source_p, target_p), buf);
-      }
+        sendto_one(target_p, ":%s MODE %s :%s",
+                   ID_or_name(source_p, target_p),
+                   ID_or_name(source_p, target_p), buf);
     }
   }
 
@@ -1143,6 +1085,25 @@ send_umode_out(struct Client *client_p, struct Client *source_p,
     send_umode(client_p, source_p, old, 0xffffffff, buf);
 }
 
+void
+user_set_hostmask(struct Client *target_p, const char *hostname)
+{
+  if (!valid_hostname(hostname))
+    return;
+
+  if (IsUserHostIp(target_p))
+    delete_user_host(target_p->username, target_p->host, !MyConnect(target_p));
+
+  strlcpy(target_p->host, hostname, sizeof(target_p->host));
+  SetIPSpoof(target_p);
+
+  add_user_host(target_p->username, target_p->host, !MyConnect(target_p));
+  SetUserHost(target_p);
+
+  if (MyClient(target_p))
+    clear_ban_cache_client(target_p);
+}
+
 /* user_welcome()
  *
  * inputs	- client pointer to client to welcome
@@ -1183,11 +1144,11 @@ user_welcome(struct Client *source_p)
 
   if (ConfigFileEntry.short_motd)
   {
-    sendto_one(source_p, "NOTICE %s :*** Notice -- motd was last changed at %s",
-               source_p->name, ConfigFileEntry.motd.lastChangedDate);
+    sendto_one(source_p, ":%s NOTICE %s :*** Notice -- motd was last changed at %s",
+               me.name, source_p->name, ConfigFileEntry.motd.lastChangedDate);
     sendto_one(source_p,
-               "NOTICE %s :*** Notice -- Please read the motd if you haven't "
-               "read it", source_p->name);
+               ":%s NOTICE %s :*** Notice -- Please read the motd if you haven't "
+               "read it", me.name, source_p->name);
     sendto_one(source_p, form_str(RPL_MOTDSTART),
                me.name, source_p->name, me.name);
     sendto_one(source_p, form_str(RPL_MOTD),
@@ -1222,7 +1183,7 @@ check_xline(struct Client *source_p)
     if (reg->reason != NULL)
       reason = reg->reason;
     else
-      reason = "No Reason";
+      reason = CONF_NOREASON;
 
     sendto_realops_flags(UMODE_REJ, L_ALL,
                          "X-line Rejecting [%s] [%s], user %s [%s]",
@@ -1230,16 +1191,8 @@ check_xline(struct Client *source_p)
                          get_client_name(source_p, HIDE_IP),
                          source_p->sockhost);
 
-    ServerStats->is_ref++;
-    if (REJECT_HOLD_TIME > 0)
-    {
-      sendto_one(source_p, ":%s NOTICE %s :Bad user info",
-                 me.name, source_p->name);
-      source_p->localClient->reject_delay = CurrentTime + REJECT_HOLD_TIME;
-      SetCaptured(source_p);
-    }
-    else
-      exit_client(source_p, &me, "Bad user info");
+    ++ServerStats.is_ref;
+    exit_client(source_p, &me, "Bad user info");
     return 1;
   }
 
@@ -1257,8 +1210,7 @@ check_xline(struct Client *source_p)
 void
 oper_up(struct Client *source_p)
 {
-  unsigned int old = source_p->umodes;
-  const char *operprivs = "";
+  const unsigned int old = source_p->umodes;
   const struct AccessItem *oconf = NULL;
 
   assert(source_p->localClient->confs.head);
@@ -1268,32 +1220,29 @@ oper_up(struct Client *source_p)
   SetOper(source_p);
 
   if (oconf->modes)
-    source_p->umodes |= oconf->modes;
+    AddUMode(source_p, oconf->modes);
   else if (ConfigFileEntry.oper_umodes)
-    source_p->umodes |= ConfigFileEntry.oper_umodes;
-  else
-    source_p->umodes |= (UMODE_SERVNOTICE|UMODE_OPERWALL|
-                         UMODE_WALLOP|UMODE_LOCOPS);
+    AddUMode(source_p, ConfigFileEntry.oper_umodes);
+
+  if (!(old & UMODE_INVISIBLE) && HasUMode(source_p, UMODE_INVISIBLE))
+    ++Count.invisi;
+  if ((old & UMODE_INVISIBLE) && !HasUMode(source_p, UMODE_INVISIBLE))
+    --Count.invisi;
 
   assert(dlinkFind(&oper_list, source_p) == NULL);
   dlinkAdd(source_p, make_dlink_node(), &oper_list);
 
-  operprivs = oper_privs_as_string(oconf->port);
+  AddOFlag(source_p, oconf->port);
 
-  SetOFlag(source_p, oconf->port);
+  if (HasOFlag(source_p, OPER_FLAG_ADMIN))
+    AddUMode(source_p, UMODE_ADMIN);
+  if (!HasOFlag(source_p, OPER_FLAG_N))
+    DelUMode(source_p, UMODE_NCHANGE);
 
-  if (IsOperAdmin(source_p) || IsOperHiddenAdmin(source_p))
-    source_p->umodes |= UMODE_ADMIN;
-  if (!IsOperN(source_p))
-    source_p->umodes &= ~UMODE_NCHANGE;
-
-  sendto_realops_flags(UMODE_ALL, L_ALL, "%s (%s@%s) is now an operator",
-                       source_p->name, source_p->username, source_p->host);
+  sendto_realops_flags(UMODE_ALL, L_ALL, "%s is now an operator",
+                       get_oper_name(source_p));
   send_umode_out(source_p, source_p, old);
   sendto_one(source_p, form_str(RPL_YOUREOPER), me.name, source_p->name);
-  sendto_one(source_p, ":%s NOTICE %s :*** Oper privs are %s",
-             me.name, source_p->name, operprivs);
-  send_message_file(source_p, &ConfigFileEntry.opermotd);
 }
 
 static char new_uid[TOTALSIDUID + 1];     /* allow for \0 */
@@ -1301,7 +1250,6 @@ static char new_uid[TOTALSIDUID + 1];     /* allow for \0 */
 int
 valid_sid(const char *sid)
 {
-
   if (strlen(sid) == IRC_MAXSID)
     if (IsDigit(*sid))
       if (IsAlNum(*(sid + 1)) && IsAlNum(*(sid + 2)))
@@ -1326,15 +1274,10 @@ init_uid(void)
 
   memset(new_uid, 0, sizeof(new_uid));
 
-  if (ServerInfo.sid != NULL)
-  {
+  if (!EmptyString(ServerInfo.sid))
     strlcpy(new_uid, ServerInfo.sid, sizeof(new_uid));
-    strlcpy(me.id, ServerInfo.sid, sizeof(me.id));
 
-    hash_add_id(&me);
-  }
-
-  for (i = 0; i < IRC_MAXSID; i++)
+  for (i = 0; i < IRC_MAXSID; ++i)
     if (new_uid[i] == '\0') 
       new_uid[i] = 'A';
 
@@ -1347,7 +1290,6 @@ init_uid(void)
 
   entering_umode_cb = register_callback("entering_umode", NULL);
   umode_cb = register_callback("changing_umode", change_simple_umode);
-  uid_get_cb = register_callback("uid_get", uid_get);
 }
 
 /*
@@ -1390,8 +1332,8 @@ add_one_to_uid(int i)
  * output       - new UID is returned to caller
  * side effects - new_uid is incremented by one.
  */
-static void *
-uid_get(va_list args)
+static const char *
+uid_get(void)
 {
   add_one_to_uid(TOTALSIDUID - 1);    /* index from 0 */
   return new_uid;
@@ -1414,7 +1356,6 @@ init_isupport(void)
   add_isupport("DEAF", "D", -1);
   add_isupport("KICKLEN", NULL, KICKLEN);
   add_isupport("MODES", NULL, MAXMODEPARAMS);
-  add_isupport("NICKLEN", NULL, NICKLEN-1);
 #ifdef HALFOPS
   add_isupport("PREFIX", "(ohv)@%+", -1);
   add_isupport("STATUSMSG", "@%+", -1);
@@ -1422,7 +1363,6 @@ init_isupport(void)
   add_isupport("PREFIX", "(ov)@+", -1);
   add_isupport("STATUSMSG", "@+", -1);
 #endif
-  add_isupport("TOPICLEN", NULL, TOPICLEN);
 }
 
 /*
