@@ -19,41 +19,32 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: send.c 572 2006-04-30 16:57:48Z michael $
+ *  $Id: send.c 1762 2013-01-18 19:21:54Z michael $
  */
 
 #include "stdinc.h"
-#include "tools.h"
+#include "list.h"
 #include "send.h"
 #include "channel.h"
 #include "client.h"
-#include "common.h"
 #include "dbuf.h"
 #include "irc_string.h"
 #include "ircd.h"
-#include "handlers.h"
 #include "numeric.h"
 #include "fdlist.h"
 #include "s_bsd.h"
 #include "s_serv.h"
 #include "sprintf_irc.h"
-#include "s_conf.h"
-#include "list.h"
-#include "s_log.h"
+#include "conf.h"
+#include "log.h"
 #include "memory.h"
 #include "hook.h"
-#include "irc_getnameinfo.h"
 #include "packet.h"
 
-#define LOG_BUFSIZE 2048
 
 struct Callback *iosend_cb = NULL;
-struct Callback *iosendctrl_cb = NULL;
+static unsigned int current_serial = 0;
 
-static void send_message(struct Client *, char *, int);
-static void send_message_remote(struct Client *, struct Client *, char *, int);
-
-static unsigned long current_serial = 0L;
 
 /* send_format()
  *
@@ -87,7 +78,7 @@ send_format(char *lsendbuf, int bufsize, const char *pattern, va_list args)
 
   lsendbuf[len++] = '\r';
   lsendbuf[len++] = '\n';
-  return (len);
+  return len;
 }
 
 /*
@@ -119,7 +110,7 @@ send_message(struct Client *to, char *buf, int len)
   {
     if (IsServer(to))
       sendto_realops_flags(UMODE_ALL, L_ALL,
-                           "Max SendQ limit exceeded for %s: %lu > %lu",
+                           "Max SendQ limit exceeded for %s: %lu > %u",
                            get_client_name(to, HIDE_IP),
                            (unsigned long)(dbuf_length(&to->localClient->buf_sendq) + len),
                            get_sendq(to));
@@ -167,13 +158,6 @@ send_message_remote(struct Client *to, struct Client *from,
     return;
   }
 
-  if (ServerInfo.hub && IsCapable(to, CAP_LL))
-  {
-    if (((from->lazyLinkClientExists &
-          to->localClient->serverMask) == 0))
-      client_burst_if_needed(to, from);
-  }
-
   /* Optimize by checking if (from && to) before everything */
   /* we set to->from up there.. */
 
@@ -193,16 +177,16 @@ send_message_remote(struct Client *to, struct Client *from,
                          from->name, from->username, from->host,
                          to->from->name);
 
-    sendto_server(NULL, to, NULL, CAP_TS6, NOCAPS, NOFLAGS,
+    sendto_server(NULL, CAP_TS6, NOCAPS,
                   ":%s KILL %s :%s (%s[%s@%s] Ghosted %s)",
                   me.id, to->name, me.name, to->name,
                   to->username, to->host, to->from->name);
-    sendto_server(NULL, to, NULL, NOCAPS, CAP_TS6, NOFLAGS,
+    sendto_server(NULL, NOCAPS, CAP_TS6,
                   ":%s KILL %s :%s (%s[%s@%s] Ghosted %s)",
                   me.name, to->name, me.name, to->name,
                   to->username, to->host, to->from->name);
 
-    SetKilled(to);
+    AddFlag(to, FLAGS_KILLED);
 
     if (IsClient(from))
       sendto_one(from, form_str(ERR_GHOSTEDCLIENT),
@@ -236,17 +220,6 @@ sendq_unblocked(fde_t *fd, struct Client *client_p)
 #endif
 }
 
-/*
- ** slinkq_unblocked
- **      Called when a server control socket is ready for writing.
- */
-static void
-slinkq_unblocked(fde_t *fd, struct Client *client_p)
-{
-  ClearSlinkqBlocked(client_p);
-  send_queued_slink_write(client_p);
-}
-
 /*
  ** send_queued_write
  **      This is called when there is a chance that some output would
@@ -301,12 +274,7 @@ send_queued_write(struct Client *to)
         retlen = send(to->localClient->fd.fd, first->data, first->size, 0);
 
       if (retlen <= 0)
-      {
-#ifdef _WIN32
-        errno = WSAGetLastError();
-#endif
         break;
-      }
 
       dbuf_delete(&to->localClient->buf_sendq, retlen);
 
@@ -330,72 +298,6 @@ send_queued_write(struct Client *to)
   }
 }
 
-/*
- ** send_queued_slink_write
- **      This is called when there is a chance the some output would
- **      be possible. This attempts to empty the send queue as far as
- **      possible, and then if any data is left, a write is rescheduled.
- */
-void
-send_queued_slink_write(struct Client *to)
-{
-  int retlen;
-
-  /*
-   ** Once socket is marked dead, we cannot start writing to it,
-   ** even if the error is removed...
-   */
-  if (IsDead(to) || IsSlinkqBlocked(to))
-    return;  /* no use calling send() now */
-
-  /* Next, lets try to write some data */
-  if (to->localClient->slinkq != NULL)
-  {
-    retlen = send(to->localClient->ctrlfd.fd,
-                   to->localClient->slinkq + to->localClient->slinkq_ofs,
-                   to->localClient->slinkq_len, 0);
-    if (retlen < 0)
-    {
-      /* If we have a fatal error */
-      if (!ignoreErrno(errno))
-      {
-	dead_link_on_write(to, errno);
-	return;
-      }
-    }
-    else if (retlen == 0)
-    {
-      /* 0 bytes is an EOF .. */
-      dead_link_on_write(to, 0);
-      return;
-    }
-    else
-    {
-      execute_callback(iosendctrl_cb, to, retlen,
-        to->localClient->slinkq + to->localClient->slinkq_ofs);
-      to->localClient->slinkq_len -= retlen;
-
-      assert(to->localClient->slinkq_len >= 0);
-      if (to->localClient->slinkq_len)
-        to->localClient->slinkq_ofs += retlen;
-      else
-      {
-        to->localClient->slinkq_ofs = 0;
-        MyFree(to->localClient->slinkq);
-        to->localClient->slinkq = NULL;
-      }
-    }
-
-    /* Finally, if we have any more data, reschedule a write */
-    if (to->localClient->slinkq_len)
-    {
-      SetSlinkqBlocked(to);
-      comm_setselect(&to->localClient->ctrlfd, COMM_SELECT_WRITE,
-                     (PF *)slinkq_unblocked, (void *)to, 0);
-    }
-  }
-}
-
 /* send_queued_all()
  *
  * input        - NONE
@@ -466,7 +368,7 @@ sendto_one(struct Client *to, const char *pattern, ...)
  */
 void
 sendto_channel_butone(struct Client *one, struct Client *from,
-                      struct Channel *chptr, const char *command,
+                      struct Channel *chptr, unsigned int type,
                       const char *pattern, ...)
 {
   va_list alocal, aremote, auid;
@@ -474,21 +376,18 @@ sendto_channel_butone(struct Client *one, struct Client *from,
   char remote_buf[IRCD_BUFSIZE];
   char uid_buf[IRCD_BUFSIZE];
   int local_len, remote_len, uid_len;
-  dlink_node *ptr;
-  dlink_node *ptr_next;
-  struct Client *target_p;
+  dlink_node *ptr = NULL, *ptr_next = NULL;
 
   if (IsServer(from))
-    local_len = ircsprintf(local_buf, ":%s %s %s ",
-                           from->name, command, chptr->chname);
+    local_len = ircsprintf(local_buf, ":%s ",
+                           from->name);
   else
-    local_len = ircsprintf(local_buf, ":%s!%s@%s %s %s ",
-                           from->name, from->username, from->host,
-                           command, chptr->chname);
-  remote_len = ircsprintf(remote_buf, ":%s %s %s ",
-                          from->name, command, chptr->chname);
-  uid_len = ircsprintf(uid_buf, ":%s %s %s ",
-                       ID(from), command, chptr->chname);
+    local_len = ircsprintf(local_buf, ":%s!%s@%s ",
+                           from->name, from->username, from->host);
+  remote_len = ircsprintf(remote_buf, ":%s ",
+                          from->name);
+  uid_len = ircsprintf(uid_buf, ":%s ",
+                       ID(from));
 
   va_start(alocal, pattern);
   va_start(aremote, pattern);
@@ -507,18 +406,23 @@ sendto_channel_butone(struct Client *one, struct Client *from,
 
   DLINK_FOREACH_SAFE(ptr, ptr_next, chptr->members.head)
   {
-    target_p = ((struct Membership *)ptr->data)->client_p;
-    assert(target_p != NULL);
+    struct Membership *ms = ptr->data;
+    struct Client *target_p = ms->client_p;
 
-    if (IsDefunct(target_p) || IsDeaf(target_p) || target_p->from == one)
+    assert(IsClient(target_p));
+
+    if (IsDefunct(target_p) || HasUMode(target_p, UMODE_DEAF) || target_p->from == one)
       continue;
 
-    if (MyClient(target_p))
+    if (type != 0 && (ms->flags & type) == 0)
+      continue;
+
+    if (MyConnect(target_p))
     {
-      if (target_p->serial != current_serial)
+      if (target_p->localClient->serial != current_serial)
       {
         send_message(target_p, local_buf, local_len);
-        target_p->serial = current_serial;
+        target_p->localClient->serial = current_serial;
       }
     }
     else
@@ -526,13 +430,13 @@ sendto_channel_butone(struct Client *one, struct Client *from,
       /* Now check whether a message has been sent to this
        * remote link already
        */
-      if (target_p->from->serial != current_serial)
+      if (target_p->from->localClient->serial != current_serial)
       {
         if (IsCapable(target_p->from, CAP_TS6))
           send_message_remote(target_p->from, from, uid_buf, uid_len);
         else
           send_message_remote(target_p->from, from, remote_buf, remote_len);
-        target_p->from->serial = current_serial;
+        target_p->from->localClient->serial = current_serial;
       }
     }
   }
@@ -541,24 +445,15 @@ sendto_channel_butone(struct Client *one, struct Client *from,
 /* sendto_server()
  * 
  * inputs       - pointer to client to NOT send to
- *              - pointer to source client required by LL (if any)
- *              - pointer to channel required by LL (if any)
+ *              - pointer to channel
  *              - caps or'd together which must ALL be present
  *              - caps or'd together which must ALL NOT be present
- *              - LL flags: LL_ICLIENT | LL_ICHAN
  *              - printf style format string
  *              - args to format string
  * output       - NONE
  * side effects - Send a message to all connected servers, except the
  *                client 'one' (if non-NULL), as long as the servers
  *                support ALL capabs in 'caps', and NO capabs in 'nocaps'.
- *                If the server is a lazylink client, then it must know
- *                about source_p if non-NULL (unless LL_ICLIENT is specified,
- *                when source_p will be introduced where required) and
- *                chptr if non-NULL (unless LL_ICHANNEL is specified, when
- *                chptr will be introduced where required).
- *                Note: nothing will be introduced to a LazyLeaf unless
- *                the message is actually sent.
  *            
  * This function was written in an attempt to merge together the other
  * billion sendto_*serv*() functions, which sprung up with capabs,
@@ -566,22 +461,15 @@ sendto_channel_butone(struct Client *one, struct Client *from,
  * -davidt
  */
 void 
-sendto_server(struct Client *one, struct Client *source_p,
-              struct Channel *chptr, unsigned long caps,
-              unsigned long nocaps, unsigned long llflags,
+sendto_server(struct Client *one,
+              const unsigned int caps,
+              const unsigned int nocaps,
               const char *format, ...)
 {
   va_list args;
-  struct Client *client_p;
-  dlink_node *ptr;
+  dlink_node *ptr = NULL;
   char buffer[IRCD_BUFSIZE];
-  int len;
-
-  if (chptr != NULL)
-  {
-    if (chptr->chname[0] != '#')
-      return;
-  }
+  int len = 0;
 
   va_start(args, format);
   len = send_format(buffer, IRCD_BUFSIZE, format, args);
@@ -589,7 +477,7 @@ sendto_server(struct Client *one, struct Client *source_p,
 
   DLINK_FOREACH(ptr, serv_list.head)
   {
-    client_p = ptr->data;
+    struct Client *client_p = ptr->data;
 
     /* If dead already skip */
     if (IsDead(client_p))
@@ -604,35 +492,6 @@ sendto_server(struct Client *one, struct Client *source_p,
     if ((client_p->localClient->caps & nocaps) != 0)
       continue;
 
-    if (ServerInfo.hub && IsCapable(client_p, CAP_LL))
-    {
-      /* check LL channel */
-      if (chptr != NULL &&
-          ((chptr->lazyLinkChannelExists &
-            client_p->localClient->serverMask) == 0))
-      {
-        /* Only introduce the channel if we really will send this message */
-        if (!(llflags & LL_ICLIENT) && source_p &&
-            ((source_p->lazyLinkClientExists &
-              client_p->localClient->serverMask) == 0))
-          continue; /* we can't introduce the unknown source_p, skip */
-
-        if (llflags & LL_ICHAN)
-          burst_channel(client_p, chptr);
-        else
-          continue; /* we can't introduce the unknown chptr, skip */
-      }
-      /* check LL client */
-      if (source_p &&
-          ((source_p->lazyLinkClientExists &
-            client_p->localClient->serverMask) == 0))
-      {
-        if (llflags & LL_ICLIENT)
-          client_burst_if_needed(client_p,source_p);
-        else
-          continue; /* we can't introduce the unknown source_p, skip */
-      }
-    }
     send_message(client_p, buffer, len);
   }
 }
@@ -677,16 +536,16 @@ sendto_common_channels_local(struct Client *user, int touser,
       assert(target_p != NULL);
 
       if (!MyConnect(target_p) || target_p == user || IsDefunct(target_p) ||
-          target_p->serial == current_serial)
+          target_p->localClient->serial == current_serial)
         continue;
 
-      target_p->serial = current_serial;
+      target_p->localClient->serial = current_serial;
       send_message(target_p, buffer, len);
     }
   }
 
   if (touser && MyConnect(user) && !IsDead(user) &&
-      user->serial != current_serial)
+      user->localClient->serial != current_serial)
     send_message(user, buffer, len);
 }
 
@@ -724,7 +583,7 @@ sendto_channel_local(int type, int nodeaf, struct Channel *chptr,
       continue;
 
     if (!MyConnect(target_p) || IsDefunct(target_p) ||
-        (nodeaf && IsDeaf(target_p)))
+        (nodeaf && HasUMode(target_p, UMODE_DEAF)))
       continue;
 
     send_message(target_p, buffer, len);
@@ -767,7 +626,7 @@ sendto_channel_local_butone(struct Client *one, int type,
       continue;
 
     if (!MyConnect(target_p) || target_p == one ||
-        IsDefunct(target_p) || IsDeaf(target_p))
+        IsDefunct(target_p) || HasUMode(target_p, UMODE_DEAF))
       continue;
     send_message(target_p, buffer, len);
   }
@@ -786,8 +645,9 @@ sendto_channel_local_butone(struct Client *one, int type,
  *		  remote to this server.
  */
 void
-sendto_channel_remote(struct Client *one, struct Client *from, int type, int caps,
-                      int nocaps, struct Channel *chptr, const char *pattern, ...)
+sendto_channel_remote(struct Client *one, struct Client *from, int type,
+                      const unsigned int caps, const unsigned int nocaps,
+                      struct Channel *chptr, const char *pattern, ...)
 {
   va_list args;
   char buffer[IRCD_BUFSIZE];
@@ -818,10 +678,10 @@ sendto_channel_remote(struct Client *one, struct Client *from, int type, int cap
         ((target_p->from->localClient->caps & caps) != caps) ||
         ((target_p->from->localClient->caps & nocaps) != 0))
       continue;
-    if (target_p->from->serial != current_serial)
+    if (target_p->from->localClient->serial != current_serial)
     {
       send_message(target_p, buffer, len);
-      target_p->from->serial = current_serial;
+      target_p->from->localClient->serial = current_serial;
     }
   } 
 }
@@ -847,9 +707,9 @@ static int
 match_it(const struct Client *one, const char *mask, int what)
 {
   if (what == MATCH_HOST)
-    return(match(mask, one->host));
+    return match(mask, one->host);
 
-  return(match(mask, one->servptr->name));
+  return match(mask, one->servptr->name);
 }
 
 /* sendto_match_butone()
@@ -947,7 +807,7 @@ sendto_match_servs(struct Client *source_p, const char *mask, int cap,
   vsnprintf(buffer, sizeof(buffer), pattern, args);
   va_end(args);
 
-  current_serial++;
+  ++current_serial;
 
   DLINK_FOREACH(ptr, global_serv_list.head)
   {
@@ -957,7 +817,7 @@ sendto_match_servs(struct Client *source_p, const char *mask, int cap,
     if (IsMe(target_p) || target_p->from == source_p->from)
       continue;
 
-    if (target_p->from->serial == current_serial)
+    if (target_p->from->localClient->serial == current_serial)
       continue;
 
     if (match(mask, target_p->name))
@@ -966,7 +826,7 @@ sendto_match_servs(struct Client *source_p, const char *mask, int cap,
        * if we set the serial here, then we'll never do a
        * match() again, if !IsCapable()
        */
-      target_p->from->serial = current_serial;
+      target_p->from->localClient->serial = current_serial;
       found++;
 
       if (!IsCapable(target_p->from, cap))
@@ -1034,9 +894,8 @@ sendto_anywhere(struct Client *to, struct Client *from,
 void
 sendto_realops_flags(unsigned int flags, int level, const char *pattern, ...)
 {
-  struct Client *client_p;
+  dlink_node *ptr = NULL;
   char nbuf[IRCD_BUFSIZE];
-  dlink_node *ptr;
   va_list args;
 
   va_start(args, pattern);
@@ -1045,18 +904,47 @@ sendto_realops_flags(unsigned int flags, int level, const char *pattern, ...)
 
   DLINK_FOREACH(ptr, oper_list.head)
   {
-    client_p = ptr->data;
+    struct Client *client_p = ptr->data;
+    assert(HasUMode(client_p, UMODE_OPER));
+
+    /* If we're sending it to opers and theyre an admin, skip.
+     * If we're sending it to admins, and theyre not, skip.
+     */
+    if (((level == L_ADMIN) && !HasUMode(client_p, UMODE_ADMIN)) ||
+	((level == L_OPER) && HasUMode(client_p, UMODE_ADMIN)))
+      continue;
+
+    if (HasUMode(client_p, flags))
+      sendto_one(client_p, ":%s NOTICE %s :*** Notice -- %s",
+                 me.name, client_p->name, nbuf);
+  }
+}
+
+void
+sendto_globops_flags(unsigned int flags, int level, const char *pattern, ...)
+{
+  dlink_node *ptr = NULL;
+  char nbuf[IRCD_BUFSIZE];
+  va_list args;
+
+  va_start(args, pattern);
+  vsnprintf(nbuf, IRCD_BUFSIZE, pattern, args);
+  va_end(args);
+
+  DLINK_FOREACH(ptr, oper_list.head)
+  {
+    struct Client *client_p = ptr->data;
     assert(client_p->umodes & UMODE_OPER);
 
     /* If we're sending it to opers and theyre an admin, skip.
      * If we're sending it to admins, and theyre not, skip.
      */
-    if (((level == L_ADMIN) && !IsAdmin(client_p)) ||
-	((level == L_OPER) && IsAdmin(client_p)))
+    if (((level == L_ADMIN) && !HasUMode(client_p, UMODE_ADMIN)) ||
+        ((level == L_OPER) && HasUMode(client_p, UMODE_ADMIN)))
       continue;
 
-    if (client_p->umodes & flags)
-      sendto_one(client_p, ":%s NOTICE %s :*** Notice -- %s",
+    if (HasUMode(client_p, flags))
+      sendto_one(client_p, ":%s NOTICE %s :*** Global -- %s",
                  me.name, client_p->name, nbuf);
   }
 }
@@ -1073,8 +961,7 @@ void
 sendto_wallops_flags(unsigned int flags, struct Client *source_p,
                      const char *pattern, ...)
 {
-  struct Client *client_p;
-  dlink_node *ptr;
+  dlink_node *ptr = NULL;
   va_list args;
   char buffer[IRCD_BUFSIZE];
   int len;
@@ -1091,10 +978,10 @@ sendto_wallops_flags(unsigned int flags, struct Client *source_p,
 
   DLINK_FOREACH(ptr, oper_list.head)
   {
-    client_p = ptr->data;
+    struct Client *client_p = ptr->data;
     assert(client_p->umodes & UMODE_OPER);
 
-    if ((client_p->umodes & flags) && !IsDefunct(client_p))
+    if (HasUMode(client_p, flags) && !IsDefunct(client_p))
       send_message(client_p, buffer, len);
   }
 }
@@ -1110,7 +997,7 @@ void
 ts_warn(const char *pattern, ...)
 {
   va_list args;
-  char buffer[LOG_BUFSIZE];
+  char buffer[IRCD_BUFSIZE];
   static time_t last = 0;
   static int warnings = 0;
 
@@ -1133,11 +1020,11 @@ ts_warn(const char *pattern, ...)
   }
 
   va_start(args, pattern);
-  vsprintf_irc(buffer, pattern, args);
+  vsnprintf(buffer, sizeof(buffer), pattern, args);
   va_end(args);
 
   sendto_realops_flags(UMODE_ALL, L_ALL, "%s", buffer);
-  ilog(L_CRIT, "%s", buffer);
+  ilog(LOG_TYPE_IRCD, "%s", buffer);
 }
 
 /* kill_client()
@@ -1187,12 +1074,11 @@ kill_client_ll_serv_butone(struct Client *one, struct Client *source_p,
 {
   va_list args;
   int have_uid = 0;
-  struct Client *client_p;
-  dlink_node *ptr;
+  dlink_node *ptr = NULL;
   char buf_uid[IRCD_BUFSIZE], buf_nick[IRCD_BUFSIZE];
-  int len_uid = 0, len_nick;
+  int len_uid = 0, len_nick = 0;
 
-  if (HasID(source_p) && (me.id[0] != '\0'))
+  if (HasID(source_p))
   {
     have_uid = 1;
     va_start(args, pattern);
@@ -1210,22 +1096,16 @@ kill_client_ll_serv_butone(struct Client *one, struct Client *source_p,
 
   DLINK_FOREACH(ptr, serv_list.head)
   {
-    client_p = ptr->data;
+    struct Client *client_p = ptr->data;
 
     if (one != NULL && (client_p == one->from))
       continue;
     if (IsDefunct(client_p))
       continue;
 
-    /* XXX perhaps IsCapable should test for localClient itself ? -db */
-    if (client_p->localClient == NULL || !IsCapable(client_p, CAP_LL) ||
-        !ServerInfo.hub ||
-        (source_p->lazyLinkClientExists & client_p->localClient->serverMask))
-    {
-      if (have_uid && IsCapable(client_p, CAP_TS6))
-        send_message(client_p, buf_uid, len_uid);
-      else
-        send_message(client_p, buf_nick, len_nick);
-    }
+    if (have_uid && IsCapable(client_p, CAP_TS6))
+      send_message(client_p, buf_uid, len_uid);
+    else
+      send_message(client_p, buf_nick, len_nick);
   }
 } 
