@@ -18,38 +18,22 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
  *  USA
  *
- *  $Id: version.c 685 2006-06-16 12:22:31Z michael $
+ *  $Id: version.c 1748 2013-01-15 18:16:36Z michael $
  */
 
 #include "patchlevel.h"
 #include "serno.h"
 #include "ircd.h"
-#ifndef _WIN32
-#include <sys/utsname.h>
-#else
-#include "s_bsd.h"
-#endif
 
 const char *ircd_version = PATCHLEVEL;
 const char *serno = SERIALNUM;
 
-char *
-get_ircd_platform(char *str)
-{
-  struct utsname uts;
-
-  uname(&uts);
-  snprintf(str, PLATFORMLEN, "%s %s %s %s %s", uts.sysname, uts.nodename,
-           uts.release, uts.version, uts.machine);
-  return str;
-}
-
 const char *infotext[] =
 {
   "IRC --",
   "Based on the original code written by Jarkko Oikarinen",
   "Copyright 1988, 1989, 1990, 1991 University of Oulu, Computing Center",
-  "Copyright (c) 1997-2006 Hybrid Development Team", 
+  "Copyright (c) 1997-2013 Hybrid Development Team",
   "",
   "This program is free software; you can redistribute it and/or",
   "modify it under the terms of the GNU General Public License as",
@@ -69,14 +53,11 @@ const char *infotext[] =
   "",
   "The core team as, of this major release:",
   "",
-  "adx, Piotr Nizynski <adx@irc7.pl>",
   "billy-jon, William Bierman III <bill@mu.org>",
   "cryogen, Stuart Walsh <stu@ipng.org.uk>",
   "Dianora, Diane Bruce <db@db.net>",
-  "joshk, Joshua Kwan <joshk@triplehelix.org>",
-  "knight, Alan LeVee <alan.levee@prometheus-designs.net>",
   "metalrock, Jack Low <jclow@csupomona.edu>",
-  "Michael, Michael Wobst <michael.wobst@gmail.com>",
+  "Michael, Michael Wobst <michael@wobst.at>",
   "Rodder, Jon Lusky <lusky@blown.net>",
   "Wohali, Joan Touzet <joant@ieee.org>",
   "",
@@ -84,19 +65,23 @@ const char *infotext[] =
   "recent releases of Hybrid, in nick alphabetical order:",
   "",
   "A1kmm, Andrew Miller <a1kmm@mware.virtualave.net>",
+  "adx, Piotr Nizynski <nizynski@sysplex.pl>",
   "AndroSyn, Aaron Sethman <androsyn@ratbox.org>",
   "bane, Dragan Dosen <bane@idolnet.org>",
   "bysin, Ben Kittridge <bkittridge@cfl.rr.com>",
   "cosine, Patrick Alken <wnder@uwns.underworld.net>",
   "David-T, David Taylor <davidt@yadt.co.uk>",
+  "fgeek, Henri Salo <henri@nerv.fi>",
   "fl, Lee Hardy <lee@leeh.co.uk>",
   "Garion, Joost Vunderink <garion@efnet.nl>",
   "Habeeb, David Supuran <habeeb@cfl.rr.com>",
   "Hwy101, W. Campbell <wcampbel@botbay.net>",
   "jmallett, Juli Mallett <jmallett@FreeBSD.org>",
+  "joshk, Joshua Kwan <joshk@triplehelix.org>",
   "jv, Jakub Vlasek <jv@pilsedu.cz>",
   "k9, Jeremy Chadwick <ircd@jdc.parodius.com>",
   "kire, Erik Small <smalle@hawaii.edu>",
+  "knight, Alan LeVee <alan.levee@prometheus-designs.net>",
   "kre, Dinko Korunic <kreator@fly.srk.fer.hr>",
   "madmax, Paul Lomax <madmax@efnet.org>",
   "nenolod, William Pitcock <nenolod@nenolod.net>",
